local Text = require("conf.Text")
local IDX = require("conf.TextIndex")

local cc = cc
local ccs = ccs
local ipairs = ipairs
local pairs = pairs
local math = math
local P={}
P.UI_COLOR3B = {
    WHITE = cc.c3b(255,255,255)
    , BLACK = cc.c3b(0,0,0)
    , RED = cc.c3b(255,0,0)
    , GREEN = cc.c3b(0,255,0)
    , YEELOW = cc.c3b(255,96,0)
    , GREY = cc.c3b(64,64,64)
    , BLUEVIOLET = cc.c3b(138,43,226)
    , BROWN = cc.c3b(139,69,19)
    , TREE_GREEN = cc.c3b(56,94,15)
    , BLUE = cc.c3b(0,0,255)
}

P.UI_COLOR4B = {
    NONE = cc.c4b(255,255,255,0)
    , WHITE = cc.c4b(255,255,255,255)
    , BLACK = cc.c4b(0,0,0,255)
    , RED = cc.c4b(255,0,0,255)
    , YEELOW = cc.c4b(255,255,0,255)
    , YEELOW2 = cc.c4b(255,96,0,255)
    , GREEN= cc.c4b(0,255,0,255)
    , CRIMSON = cc.c4b(220,20,60,255)--猩红
    , MEDIUMBULE = cc.c4b(0,0,205,255)--深蓝
    , STEELBULE = cc.c4b(70,130,180,255)--钢蓝
    , ROYALBLUE = cc.c4b(65,105,225,255)--皇家蓝
    , DODGERBLUE = cc.c4b(30,144,255,255)--宝石蓝
    , BLUEVIOLET = cc.c4b(138,43,226,255)--紫罗蓝
    , GRAY = cc.c4b(166, 166, 166, 220)--灰色
    , GRAY2 = cc.c4b(166, 166, 166, 120)--灰色
}
P.UI_ANCHOR = {
    CENTER = 1,
    TOP_LEFT = 2,
    TOP_RIGHT = 3,
    BOTTOM_LEFT = 4,
    BOTTOM_RIGHT = 5,
    CENTER_TOP = 6,
    CENTER_LEFT = 7,
    CENTER_RIGHT = 8,
    CENTER_BOTTOM = 9
}

P.UI_ANCHOR_POINT = {
    ZERO = {x=0, y=0},
    CENTER = {x=0.5 , y=0.5},
    TOP_LEFT = {x=0, y=1},
    TOP_RIGHT = {x=1, y=1},
    BOTTOM_LEFT = {x=0, y=0},
    BOTTOM_RIGHT = {x=1, y=0},
    CENTER_TOP = {x=0.5, y=1},
    CENTER_LEFT = {x=0, y=0.5},
    CENTER_RIGHT = {x=1, y=0.5},
    CENTER_BOTTOM = {x=0.5, y=0}
}

P.FONT = {
    MINI = 15,--detail
    SMALL = 18,--Lv...
    SMALL_PLUS = 22,
    MIDDLE = 25,--name...
    LARGE = 30,--top tab
    SUPER = 40,--title
}

P.FONT_NAME_NORMAL = "Arial"

P.TRANSITION_TYPE = {
    FADE = 1,
    FADE_BL = 2,



    FLIP_X = 8,
}

P.IMG_IDX = {
    IMG91 = 91,
    IMG92 = 92,
    IMG93 = 93,
}

P.IN_IMG_IDX = {
    QUESTION = 9,--问号"?"
    SOULSTONE = 2,--魂石
    MONEY = 1,--金币
}

---根据param设置获取子UI中心点相对于父UI左下角偏移坐标
--@param offsetX 相对九点锚点中的某点x轴偏移量
--@param offsetY 相对九点锚点中的某点y轴偏移量
--@param uiWidth ui自身width值
--@param uiHeight    ui自身height值
--@param superWidth  将要成为父节点UI的width值
--@param superHeight 将要成为父节点UI的height值
--@param anchor  九点锚点中的某一点［见common.Enum UI_ANCHOR］
--@return x,y   子UI中心点相对于父UI左下角偏移坐标
function P.offsetWithSuperSize(offsetX, offsetY ,uiWidth, uiHeight, superWidth, superHeight, anchor)
    local x = 0
    local y = 0

    --cclog("UILib offsetWithSuperSize %d,%d,%d,%d,%d,%d,%d",offsetX, offsetY, uiWidth, uiHeight, superWidth, superHeight, anchor)

    local switch = {
        [P.UI_ANCHOR.CENTER] = function()
            x = superWidth*.5 + offsetX
            y = superHeight*.5 + offsetY
        end,
        [P.UI_ANCHOR.TOP_LEFT] = function()
            x = uiWidth*.5 + offsetX;
            y = superHeight + offsetY - uiHeight*.5
        end,
        [P.UI_ANCHOR.TOP_RIGHT] = function()
            x = superWidth + offsetX - uiWidth*.5
            y = superHeight + offsetY - uiHeight*.5
        end,
        [P.UI_ANCHOR.BOTTOM_LEFT] = function()
            x = offsetX + uiWidth*.5
            y = offsetY + uiHeight*.5
        end,
        [P.UI_ANCHOR.BOTTOM_RIGHT] = function()
            x = superWidth + offsetX - uiWidth*.5
            y = offsetY + uiHeight*.5
        end,
        [P.UI_ANCHOR.CENTER_TOP] = function()
            x = superWidth*.5 + offsetX;
            y = superHeight + offsetY - uiHeight*.5
        end,
        [P.UI_ANCHOR.CENTER_LEFT] = function()
            x = offsetX + uiWidth*.5;
            y = superHeight*.5 + offsetY;
        end,
        [P.UI_ANCHOR.CENTER_RIGHT] = function()
            x = superWidth + offsetX - uiWidth*.5
            y = superHeight*.5 + offsetY
        end,
        [P.UI_ANCHOR.CENTER_BOTTOM] = function()
            x = superWidth*.5 + offsetX
            y = offsetY + uiHeight*.5
        end
    }

    local f = switch[anchor]
    if(f) then
        f()
    else                -- for case default
        cclog("UILib offset unknow anchor:%d",anchor)
    end

    return cc.p(x,y)
end


---根据param设置获取子UI中心点相对于屏幕左下角偏移坐标
--@param offsetX 相对九点锚点中的某点x轴偏移量
--@param offsetY 相对九点锚点中的某点y轴偏移量
--@param uiWidth ui自身width值
--@param uiHeight    ui自身height值
--@param anchor  九点锚点中的某一点［见common.Enum UI_ANCHOR］
--@return x,y   子UI中心点相对于屏幕左下角偏移坐标
function P.offset(offsetX, offsetY ,uiWidth, uiHeight, anchor)
    local winSize = cc.Director:getInstance():getWinSize()
    local p = P.offsetWithSuperSize(offsetX,offsetY,uiWidth,uiHeight,winSize.width,winSize.height,anchor)
    local x, y = p.x ,p.y

    local visibleSize = cc.Director:getInstance():getVisibleSize()

    if (P.UI_ANCHOR.TOP_LEFT == anchor) or
        (P.UI_ANCHOR.CENTER_TOP == anchor) or
        (P.UI_ANCHOR.TOP_RIGHT == anchor) then
        y = y - (winSize.height - visibleSize.height)*.5
    elseif (P.UI_ANCHOR.BOTTOM_LEFT) or
        (P.UI_ANCHOR.BOTTOM_RIGHT) or
        (P.UI_ANCHOR.CENTER_BOTTOM) then
        y = y + (winSize.height - visibleSize.height)*.5
    end

    return cc.p(x,y)
end

---创建Armature对象
--@param #string dir 资源所在目录
--@param #string resName armature资源名称
--@return #Armature armature
function P.createArmature(dir, resName)
    local armature = ccs.Armature:create()
    
    local configFilePath = dir..resName..".ExportJson"
    ccs.ArmatureDataManager:getInstance():addArmatureFileInfo(configFilePath)
    armature:init(resName)
    
    return armature
end

function P.createClipSprite(imgIdx, iconInImgIdx)
    local imgName = nil
    if imgIdx == P.IMG_IDX.IMG91 then
    	imgName = "p_91.pngHK"
    elseif imgIdx == P.IMG_IDX.IMG92 then
        imgName = "p_92.pngHK"
    elseif imgIdx == P.IMG_IDX.IMG93 then
        imgName = "p_93.pngHK"
    end
    if imgName == nil then
    	return nil
    end
    local mIconInImageX = math.fmod(iconInImgIdx-1, 10) * 35
    local mIconInImageY = math.floor((iconInImgIdx-1) / 10) * 35
    local sprite = cc.Sprite:create(imgName,cc.rect(mIconInImageX,mIconInImageY,35,35))
    return sprite
end

---从rootNode中查找第一个符合tag值的node
--@param #Node rootNode 根节点Node
--@param #int tag Tag
--@return #Node node
function P.getNodeByTag(rootNode, tag)
    if rootNode:getTag() == tag then
        return rootNode
    end
    for k, node in ipairs(rootNode:getChildren()) do
            if node ~= nil then
                local childNode = node:getChildByTag(tag)
                if childNode ~= nil then
                    return childNode
                end
                childNode = P.getNodeByTag(node,tag)
                if childNode ~= nil then
                    return childNode
                end
            end
    end
    
    return nil
end


function P.addImageTo(parentNode, imgName, pos)
    local imageName = string.format("%s.%s",imgName,Text.get(IDX.PNG_TYPE))
    local isExist = cc.FileUtils:getInstance():isFileExist(imageName)
    if isExist ~= true then
        imageName = string.format("%s.pngHK",imgName)
    end
    local textImg = ccui.ImageView:create(imageName)
    textImg:setPosition(pos)
    parentNode:addChild(textImg)
    
    return textImg
end

---创建描边Label
--@param string     文本 
--@param fontName   文本字体类型 
--@param fontSize   文本大小 
--@param tColor3     文本颜色 
--@param sColor3     描边颜色
--@param lineWidth  所描边的宽度 
function P.textAddStroke(string, fontName, fontSize, tColor3, sColor3, lineWidth)
    local center = CCLabelTTF:create(string, fontName, fontSize)
    center:setColor(tColor3)
    
    --描边CCLabelTTF 上  
    local up = CCLabelTTF:create(string, fontName, fontSize) 
    up:setTag(1) 
    up:setColor(sColor3)
    up:setPosition(cc.p(center:getContentSize().width*0.5, center:getContentSize().height*0.5+lineWidth)) 
    center:addChild(up,-1)  

    --描边CCLabelTTF 下  
    local down = CCLabelTTF:create(string, fontName, fontSize) 
    down:setTag(1)
    down:setColor(sColor3)
    down:setPosition(cc.p(center:getContentSize().width*0.5, center:getContentSize().height*0.5-lineWidth))  
    center:addChild(down,-1)  
      
    --描边CCLabelTTF 左  
    local left = CCLabelTTF:create(string, fontName, fontSize) 
    left:setTag(1)
    left:setPosition(cc.p(center:getContentSize().width*0.5-lineWidth, center:getContentSize().height*0.5))  
    left:setColor(sColor3)
    center:addChild(left,-1)  
      
    --描边CCLabelTTF 右  
    local right = CCLabelTTF:create(string, fontName, fontSize) 
    right:setTag(1) 
    right:setColor(sColor3)  
    right:setPosition(cc.p(center:getContentSize().width*0.5+lineWidth,center:getContentSize().height*0.5))  
    center:addChild(right,-1) 

    return center
end

function P.setStrokeString(label, string)
    label:setString(string)
    for key, var in ipairs(label:getChildren()) do
    	if var:getTag() == 1 then
    	    local oPx = var:getPositionX()
            local oWidth = var:getContentSize().width
    		var:setString(string)
    		local nWidth = var:getContentSize().width
    		var:setPositionX(oPx-(oWidth-nWidth)/2)
    	end
    end
end

function P.setStrokeColor(label, sColor3)
    for key, var in ipairs(label:getChildren()) do
        if var:getTag() == 1 then
            var:setColor(sColor3)
        end
    end
end

return P
