--local print=function()end
local tconcat = table.concat
local tinsert = table.insert
local srep = string.rep
local type = type
local pairs = pairs
local tostring = tostring
local next = next
local math = math

local P={}

require("common.json")

local json = json

function P.calDistance(source, target)
  local len1 = math.pow(math.abs(source.x-target.x),2)
  local len2 = math.pow(math.abs(source.y-target.y),2)
  local dis = math.sqrt(len1+len2)
  return dis
end

local function lshift(v, n)
  return v * 2^ n
end

local function rshift(v, n)
  return math.ceil (v / (2^ n)  - 1)
end

function P.posToNumber(x, y)
  return lshift(x,16) + y
end
function P.numberToPos(n)
  local x = rshift(n,16)
  local y = n - lshift(x,16)
  return {x=x, y=y}

end

function P.json2Table(j)
  if(j == nil or j == '[]' or j=='')then
    return {}
  else
    return json.decode(j)
  end
end


function P.tableToJson(t)
  return json.encode(t)

end
------------------------
function P.printTable(root, depthMax)

  local cache = {  [root] = "." }
  local depth = 0
  local function _dump(t,space,name)
    depth = depth + 1
    local temp = {}
    for k,v in pairs(t) do
      local key = tostring(k)
      --      if cache[v] then
      --        tinsert(temp,"+" .. key .. " {" .. cache[v].."}")
      if type(v) == "table" and depth < depthMax then
        local new_key = name .. "." .. key
        cache[v] = new_key
        tinsert(temp,"+" .. key .. _dump(v,space .. (next(t,k) and "|" or " " ).. srep(" ",#key),new_key))
      else
        tinsert(temp,"+" .. key .. " [" .. tostring(v).."]")
      end
    end
    depth = depth - 1
    local rst = tconcat(temp,"\n"..space)
    --    print ("---------"..depth.."----------")
    --    print(rst)
    return rst
  end
  print(_dump(root, "",""))

end

function P.copyTab(tab)
    local newTab = {}
    for k, v in pairs(tab or {}) do
        if type(v) ~= "table" then
            newTab[k] = v
        else
            newTab[k] = P.copyTab(v)
        end
    end
    return newTab
end
--print (P.tableToJson({A="1",{"21","22",{"231","232","233"},"24"},"3","4",{"51","52","53"}, "6"}))
--print (P.tableToJson({}))
--print (P.json2Table("[]"))
--print( lshift(1,0))
--print( lshift(1,3))
--print( rshift(1,0))
--print( rshift(2048,3))
--
--local pos = P.posToNumber(13123,54345)
--print (pos)
--local xy = P.numberToPos(pos)
--print (xy.x)
--print (xy.y)

function P.insert(value, insert, place)

    if place == nil then
        place = string.len(value)+1
    end

    return string.sub(value, 1,place-1) .. string.gsub(string.sub(value, place,place), string.sub(value, place,place), insert) .. string.sub(value, place, string.len(value))

end

function P.playMusic(musicName, isLoop)
    local bgMusicPath = cc.FileUtils:getInstance():fullPathForFilename(musicName) 
    cc.SimpleAudioEngine:getInstance():playMusic(bgMusicPath, isLoop)
end

--[[拆分出单个字符--]]
function P.tab_cutText(str)
    local list = {}
    local len = string.len(str)
    local i = 1 
    while i <= len do
        local c = string.byte(str, i)
        local shift = 1
        if c > 0 and c <= 127 then
            shift = 1
        elseif (c >= 192 and c <= 223) then
            shift = 2
        elseif (c >= 224 and c <= 239) then
            shift = 3
        elseif (c >= 240 and c <= 247) then
            shift = 4
        end
        local char = string.sub(str, i, i+shift-1)
        i = i + shift
        table.insert(list, char)
    end
    return list, len
end

--[[解析16进制颜色rgb值--]]
function  P.getTextColor(xStr)
    if string.len(xStr) == 6 then
        local tmp = {}
        for i = 0,5 do
            local str =  string.sub(xStr,i+1,i+1)
            if(str >= '0' and str <= '9') then
                tmp[6-i] = str - '0'
            elseif(str == 'A' or str == 'a') then
                tmp[6-i] = 10
            elseif(str == 'B' or str == 'b') then
                tmp[6-i] = 11
            elseif(str == 'C' or str == 'c') then
                tmp[6-i] = 12
            elseif(str == 'D' or str == 'd') then
                tmp[6-i] = 13
            elseif(str == 'E' or str == 'e') then
                tmp[6-i] = 14
            elseif(str == 'F' or str == 'f') then
                tmp[6-i] = 15
            else
                print("Wrong color value.")
                tmp[6-i] = 0
            end
        end
        local r = tmp[6] * 16 + tmp[5]
        local g = tmp[4] * 16 + tmp[3]
        local b = tmp[2] * 16 + tmp[1]
        return cc.c3b(r,g,b)
    end
    
    return cc.c3b(255,255,255)
end

return P
