require "cocos.init"
local UILib = require("common.UILib")
local Enum = require("common.Enum")
local LKConfig = require("conf.LKConfig")

local Text = require("conf.Text")
local IDX = require("conf.TextIndex")

local cc = cc
local ccs = ccs
local class = class

local TAG = {
    ENTER = 22,--攻击 or 事件确认
    SUMMON = 23,--召
    ASSISTANT = 25,--附
    MAGIC_A = 26,--术A
    MAGIC_B = 27,--术B
    CATCH = 34,--捕捉
    CATCH_PANEL = 35,--捕捉容器(包含CATCH)
}

local BattleControllerUI = class("BattleControllerUI",
    function()
        local node = cc.Node:create()
        return node
    end
)

-- joe
local ControllerHolder = class("ControllerHolder")
function ControllerHolder:ctor()
    self.controller  = nil
    self._holderNode = nil

    self._leftJoystick = nil
    self._rightJoystick = nil

    self._dpadLeft = nil
    self._dpadRight = nil
    self._dpadUp = nil
    self._dpadDown = nil

    self._buttonX = nil
    self._buttonY = nil
    self._buttonA = nil
    self._buttonB = nil

    self._buttonL1 = nil
    self._buttonR1 = nil
    self._buttonL2 = nil
    self._buttonR2 = nil

    self.deviceLabel = nil
    self.externalKeyLabel = nil
end
--

BattleControllerUI.__index = BattleControllerUI

BattleControllerUI.uiAction = nil
BattleControllerUI.enterBut = nil
BattleControllerUI.isTouching = false
BattleControllerUI.lastBeganTime = nil

local catchPanel =nil

function BattleControllerUI:create()
    local battleControllerUI = BattleControllerUI.new()
    battleControllerUI:init()

    return battleControllerUI
end

function BattleControllerUI:init()
    --战斗操控控件
    self:setName("BattleControllerUI")
    local widget = ccs.GUIReader:getInstance():widgetFromBinaryFile("BattleControllerUI_1/BattleControllerUI_1.csb")
    local wSize = widget:getContentSize()
    self:setContentSize(wSize)
    self:setAnchorPoint(UILib.UI_ANCHOR_POINT.CENTER)
    self:addChild(widget)
    
    local summonKey = UILib.getNodeByTag(widget,TAG.SUMMON)
    local assistantKey = UILib.getNodeByTag(widget,TAG.ASSISTANT)
    local magicKeyA = UILib.getNodeByTag(widget,TAG.MAGIC_A)
    local magicKeyB = UILib.getNodeByTag(widget,TAG.MAGIC_B)
    
    local function doSoulKeyAction(sender, eventType) 
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            local key = sender:getTag()
            cclog("touch key = "..key)
            lk.LakooManager:getInstance():doSoulKeyAction(key)
        end
    end
    
    local keys = {summonKey, assistantKey, magicKeyA, magicKeyB}
    local imgNames = {"p_142", "p_143", "p_144", "p_144"}
    for var=1, #keys do
    	if keys[var] ~= nil then
    		lk.LakooManager:getInstance():addSoulKeyNode(keys[var], var-1)
    		keys[var]:setTag(var-1)
    		keys[var]:addTouchEventListener(doSoulKeyAction)
    		
    		--添加“召”、“附”、“术”文字图片
    		local imgName = string.format("%s.%s",imgNames[var],Text.get(IDX.PNG_TYPE))
    		local textImg = ccui.ImageView:create(imgName)
            local pos = UILib.offsetWithSuperSize(5,-5,textImg:getContentSize().width,textImg:getContentSize().height
            ,keys[var]:getContentSize().width,keys[var]:getContentSize().height,UILib.UI_ANCHOR.TOP_LEFT)
    		textImg:setPosition(pos)
    		keys[var]:addChild(textImg)
    	end
    end

    catchPanel = UILib.getNodeByTag(widget,TAG.CATCH_PANEL)
    if catchPanel ~= nil then
        catchPanel:setVisible(false)
    end 
    local function doCatchAction(sender, eventType) 
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            lk.LakooManager:getInstance():doCatchSoulAction()
        end
    end  
    local catchBut = UILib.getNodeByTag(widget,TAG.CATCH)
    if catchBut ~= nil then
        catchBut:addTouchEventListener(doCatchAction)
    end 
    --播放捉宠按钮动画
    self.uiAction = ccs.ActionManagerEx:getInstance():playActionByName("BattleControllerUI_1.csb","Animation0")
    
    self.lastBeganTime = nil
    local comboIndex = 0--0:单击;1:双击;2:三击
    local function touchButton(sender, eventType) 
        if eventType == ccui.TouchEventType.began then
            --处理npc响应事件
            if lk.LakooManager:getInstance():handleEventAction() then
                return
            end
            self.isTouching = true
            local beganTime = lk.LakooManager:getInstance():getCurrentTime()
            if self.lastBeganTime == nil then
            	self.lastBeganTime = beganTime
            	comboIndex = 0
            	lk.LakooManager:getInstance():setPlayerCombo(1)
            else
                if beganTime - self.lastBeganTime < LKConfig.COMBO_INTERVAL_TIME then
                    comboIndex = comboIndex + 1
                    lk.LakooManager:getInstance():setPlayerCombo(comboIndex + 1)
                    if comboIndex > 2 then
                        comboIndex = 0
                        lk.LakooManager:getInstance():setPlayerCombo(1)
                    end
                else
                    comboIndex = 0
                end
                --cclog("lastBeganTime:"..lastBeganTime.."beganTime:"..beganTime.."  dis:"..beganTime - lastBeganTime)
                self.lastBeganTime = beganTime
            end
            
            --cclog("comboIndex===:" ..comboIndex)
        end
        if eventType == ccui.TouchEventType.moved then
            --cclog("moved---")
        end
        if eventType == ccui.TouchEventType.ended 
            or eventType == ccui.TouchEventType.canceled then
            if self.isTouching ~= true then
            	return
            end
            self.isTouching = false
            lk.LakooManager:getInstance():setPlayerPowerPercent(0)--清除powerbar 
            local endTime = lk.LakooManager:getInstance():getCurrentTime()
            local touchTime = endTime - self.lastBeganTime
            --cclog("endTime:"..endTime.."  timeDis:"..touchTime)
            if touchTime >= LKConfig.PRESS_TIME then
                --pressTouch  
                lk.LakooManager:getInstance():changePlayerStatus(Enum.ACTOR_STATUS.KEEP_POWER_ATK)
            elseif lk.LakooManager:getInstance():getPlayerStatus() ~= Enum.ACTOR_STATUS.ATTACK  then 
                lk.LakooManager:getInstance():changePlayerStatus(Enum.ACTOR_STATUS.ATTACK)  
            end
        end
    end
    
    --战斗按钮或事件确认按钮
    self.enterBut = UILib.getNodeByTag(widget,TAG.ENTER)
    if self.enterBut ~= nil then
        self.enterBut:addTouchEventListener(touchButton)
    end
    
    local function update()
        if self:isVisible() ~= true then
            return
        end
        self:updatePowerBar()
      
        if catchPanel ~= nil then
            local isCanCatch = lk.LakooManager:getInstance():isCanCatchSoul()
            if isCanCatch then
                catchPanel:setVisible(true)
        	else
                catchPanel:setVisible(false)
        	end
        end
    end
    
    local function onNodeEvent(event)
        if "enterTransitionFinish" == event then
            self.scheduleID = cc.Director:getInstance():getScheduler():scheduleScriptFunc(update, 0, false)
        end
        if "exit" == event then
            self.uiAction:stop()
            ccs.ActionManagerEx:destroyInstance()
            self.uiAction = nil
            if self.scheduleID ~= nil then
                cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.scheduleID)
            end
        end
    end
    self:registerScriptHandler(onNodeEvent)
    
    --joe
    local osType = Enum.OS_TYPE.IOS
    osType = lk.LakooManager:getInstance():getOSType()
    if osType == Enum.OS_TYPE.ANDROID then
        self._firstHolder   = ControllerHolder.new()
        self._firstHolder.controller = nil
        self:registerControllerListener(keys)
    end
    --
end

function BattleControllerUI:updatePowerBar()
    if self.isTouching == false then
    	return
    end
    local now = lk.LakooManager:getInstance():getCurrentTime()
    local disTime = now - self.lastBeganTime
    if disTime < LKConfig.COMBO_INTERVAL_TIME then
    	return
    end
    local precent = (disTime - LKConfig.COMBO_INTERVAL_TIME)/(LKConfig.PRESS_TIME - LKConfig.COMBO_INTERVAL_TIME)*100
    if precent > 100 then
    	precent = 100
    end
    lk.LakooManager:getInstance():setPlayerAnimationByType(Enum.ANI_TYPE.KEEY_POWER)
    lk.LakooManager:getInstance():setPlayerPowerPercent(precent)
end

--joe
local comboIndex = 0--0:单击;1:双击;2:三击
function BattleControllerUI:registerControllerListener(keys)
    local function onConnectController(controller, event)
        if controller == nil or controller == self._firstHolder.controller then
            return
        end

        local deviceInfo = string.format("%s id:%d", controller:getDeviceName(), controller:getDeviceId())
        cclog("BattleControllerUI deviceInfo ="..deviceInfo)

        if self._firstHolder.controller == nil then
            self._firstHolder.controller = controller
        end
    end

    local function onDisconnectedController(controller, event)
        if self._firstHolder.controller == controller then
            self._firstHolder.controller = nil
        end
    end
    

    local function showButtonState(controller, keyCode, isPressed)
        onConnectController(controller, nil)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else    
            return
        end
        
        if lk.LakooManager:getInstance():getIsHasPopView() == true then
            return
        end
        
        local scene = cc.Director:getInstance():getRunningScene()
        local child = scene:getChildren()
        
        local topSence = child[#child]
        if topSence:getName() ~= "WorldScene" then 
            return
        end

        if isPressed then
            cclog("BattleControllerUI keydown ="..keyCode) 
            if keyCode == cc.ControllerKey.BUTTON_A then
                if self.enterBut:isVisible() then
                    --处理npc响应事件
                    if lk.LakooManager:getInstance():handleEventAction() then
                        return
                    end
                    self.isTouching = true
                    local beganTime = lk.LakooManager:getInstance():getCurrentTime()
                    if self.lastBeganTime == nil then
                        self.lastBeganTime = beganTime
                        comboIndex = 0
                        lk.LakooManager:getInstance():setPlayerCombo(1)
                    else
                        if beganTime - self.lastBeganTime < LKConfig.COMBO_INTERVAL_TIME then
                            comboIndex = comboIndex + 1
                            lk.LakooManager:getInstance():setPlayerCombo(comboIndex + 1)
                            if comboIndex > 2 then
                                comboIndex = 0
                                lk.LakooManager:getInstance():setPlayerCombo(1)
                            end
                        else
                            comboIndex = 0
                        end
                        --cclog("lastBeganTime:"..lastBeganTime.."beganTime:"..beganTime.."  dis:"..beganTime - lastBeganTime)
                        self.lastBeganTime = beganTime
                    end
                end
                
            elseif keyCode == cc.ControllerKey.BUTTON_B then
                if keys ~= nil then
                    local btn = keys[4]
                    if btn ~= nil then
                        if btn:isVisible() then
                            lk.LakooManager:getInstance():doSoulKeyAction(3)
                        end
                    end
                end
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key Down:%d",keyCode)
            end
        else
            cclog("BattleControllerUI keyup ="..keyCode) 
            if keyCode == cc.ControllerKey.BUTTON_A then
                if self.enterBut:isVisible() then
                    if self.isTouching ~= true then
                        return
                    end
                    self.isTouching = false
                    lk.LakooManager:getInstance():setPlayerPowerPercent(0)--清除powerbar 
                    local endTime = lk.LakooManager:getInstance():getCurrentTime()
                    local touchTime = endTime - self.lastBeganTime
                    --cclog("endTime:"..endTime.."  timeDis:"..touchTime)
                    if touchTime >= LKConfig.PRESS_TIME then
                        --pressTouch  
                        lk.LakooManager:getInstance():changePlayerStatus(Enum.ACTOR_STATUS.KEEP_POWER_ATK)
                    elseif lk.LakooManager:getInstance():getPlayerStatus() ~= Enum.ACTOR_STATUS.ATTACK  then 
                        lk.LakooManager:getInstance():changePlayerStatus(Enum.ACTOR_STATUS.ATTACK)
                    end
                end
                
            elseif keyCode == cc.ControllerKey.BUTTON_B then
                
            elseif keyCode == cc.ControllerKey.BUTTON_X then
                if self.isTouching ~= true then
                    return
                end
                if keys ~= nil then
                    local btn = keys[3]
                    if btn ~= nil then
                        if btn:isVisible() then
                            lk.LakooManager:getInstance():doSoulKeyAction(2)
                        end
                    end
                end
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
                if self.isTouching ~= true then
                    return
                end
                if keys ~= nil then
                    local btn = keys[2]
                    if btn ~= nil then
                        if btn:isVisible() then
                            lk.LakooManager:getInstance():doSoulKeyAction(1)
                        end
                    end
                end
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then

            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then

            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
                if self.isTouching ~= true then
                    return
                end
                if keys ~= nil then
                    local btn = keys[1]
                    if btn ~= nil then
                        if btn:isVisible() then
                            lk.LakooManager:getInstance():doSoulKeyAction(0)
                        end
                    end
                end
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
                
            else
                local ketStatus = string.format("Key up:%d",keyCode)
            end
        end
    end 

    local function onKeyDown(controller, keyCode, event)
        showButtonState(controller, keyCode, true)
    end

    local function onKeyUp(controller, keyCode, event)
        showButtonState(controller, keyCode, false)
    end

    local function onAxisEvent(controller, keyCode, event)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end

        local ketStatus = controller:getKeyStatus(keyCode)
        if keyCode == cc.ControllerKey.JOYSTICK_LEFT_X then
            if(ketStatus.value == -1) then
            elseif(ketStatus.value == 1) then

            end
        elseif keyCode == cc.ControllerKey.JOYSTICK_LEFT_Y then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_X then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_Y then
        elseif keyCode == cc.ControllerKey.AXIS_LEFT_TRIGGER then
            if(ketStatus.value == 0) then
               
            end
        elseif keyCode == cc.ControllerKey.AXIS_RIGHT_TRIGGER then
            if(ketStatus.value == 0) then
                if catchPanel:isVisible()== true then
                    lk.LakooManager:getInstance():doCatchSoulAction()
                end
            end
        else
        --do nothing
        end
    end

    local controllerListener = cc.EventListenerController:create()
    controllerListener:registerScriptHandler(onConnectController, cc.Handler.EVENT_CONTROLLER_CONNECTED)
    controllerListener:registerScriptHandler(onDisconnectedController, cc.Handler.EVENT_CONTROLLER_DISCONNECTED)

    controllerListener:registerScriptHandler(onKeyDown, cc.Handler.EVENT_CONTROLLER_KEYDOWN)
    controllerListener:registerScriptHandler(onKeyUp, cc.Handler.EVENT_CONTROLLER_KEYUP)
    controllerListener:registerScriptHandler(onAxisEvent, cc.Handler.EVENT_CONTROLLER_AXIS)

    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(controllerListener, self)

    cc.Controller:startDiscoveryController()
end
--

return BattleControllerUI