require "cocos.init"
local UILib = require("common.UILib")
local Enum = require("common.Enum")
local MathUtil = require("common.MathUtil")
local LKConfig = require("conf.LKConfig")
local Android = require("conf.Android")
local cc = cc
local ccs = ccs
local class = class
local table = table
local ipairs = ipairs
local lk = lk

local TAG = {
    TRACE_ICON = 5,
}

-- joe
local isUseController = false
local ControllerHolder = class("ControllerHolder")
function ControllerHolder:ctor()
    self.controller  = nil
    self._holderNode = nil

    self._leftJoystick = nil
    self._rightJoystick = nil

    self._dpadLeft = nil
    self._dpadRight = nil
    self._dpadUp = nil
    self._dpadDown = nil

    self._buttonX = nil
    self._buttonY = nil
    self._buttonA = nil
    self._buttonB = nil

    self._buttonL1 = nil
    self._buttonR1 = nil
    self._buttonL2 = nil
    self._buttonR2 = nil

    self.deviceLabel = nil
    self.externalKeyLabel = nil
end
--

local DirControllerUI = class("DirControllerUI",
    function()
        return cc.Node:create()
    end
)

DirControllerUI.__index = DirControllerUI

DirControllerUI.BUT_TAG = {UP = 7, RIGHT = 8, DOWN = 9, LEFT = 10}
DirControllerUI.isSwallow = false
DirControllerUI.dirButList = {}
DirControllerUI.traceIcon = nil
DirControllerUI.traceOrginPoint = cc.p(0,0)
DirControllerUI.touchPoint = cc.p(-1,-1)
DirControllerUI.eventTagetSize = cc.size(0,0)
DirControllerUI.isTouchEnd = false

function DirControllerUI:create()
    local dirControllerUI = DirControllerUI.new()
    dirControllerUI:init()

    return dirControllerUI
end

function DirControllerUI:init()
    self:setName("DirControllerUI")
    --方向操控控件
    local widget = ccs.GUIReader:getInstance():widgetFromBinaryFile("Controller_1/Controller_1.csb")
    self:addChild(widget)

    local dirControlUI = cc.LayerColor:create(UILib.UI_COLOR4B.NONE)
    dirControlUI:setContentSize(cc.size(140,140))
    widget:addChild(dirControlUI)

    local listener = cc.EventListenerTouchOneByOne:create()
    local function onTouchBegan(touch, event)
        self.isTouchEnd = false
        
        local target = event:getCurrentTarget()
        local locationInNode = target:convertToNodeSpace(touch:getLocation())

        local tSize = target:getContentSize();
        local rect = cc.rect(0, 0, tSize.width, tSize.height);

        if (cc.rectContainsPoint(rect,locationInNode)) then
            self.isSwallow = true               
        else
            self.isSwallow = false
        end
        
        if self:isVisible() == false then
        	self.isSwallow = false
        end
        
        listener:setSwallowTouches(self.isSwallow) 
        
        self.touchPoint = locationInNode
        self.eventTagetSize = target:getContentSize()
        
        return self.isSwallow
    end
    local function onTouchMoved(touch, event)
        local target = event:getCurrentTarget()
        local locationInNode = target:convertToNodeSpace(touch:getLocation())
        self.touchPoint = locationInNode
        self.eventTagetSize = target:getContentSize()
    end
    local function onTouchEnded(touch, event)
        self.isTouchEnd = true
    end

    listener:registerScriptHandler(onTouchBegan, cc.Handler.EVENT_TOUCH_BEGAN)
    listener:registerScriptHandler(onTouchMoved, cc.Handler.EVENT_TOUCH_MOVED)
    listener:registerScriptHandler(onTouchEnded, cc.Handler.EVENT_TOUCH_ENDED)
    local eventDispatcher = dirControlUI:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listener, dirControlUI)
    --joe
    local osType = Enum.OS_TYPE.IOS
    osType = lk.LakooManager:getInstance():getOSType()
    if osType == Enum.OS_TYPE.ANDROID and Android.CHANNEL == 4 then
        self._firstHolder = ControllerHolder.new()
        self._firstHolder.controller = nil
        self:registerControllerListener()
        widget:setVisible(false)
    end
    --
    
    for key, var in pairs(self.BUT_TAG) do
        local but = UILib.getNodeByTag(widget,var)
        table.insert(self.dirButList, but)
    end
    
    self.traceIcon = UILib.getNodeByTag(widget,TAG.TRACE_ICON)
    self.traceOrginPoint.x = self.traceIcon:getPositionX()
    self.traceOrginPoint.y = self.traceIcon:getPositionY()
    
    local function update()
        if self:isVisible() ~= true then
            self.traceIcon:setPosition(self.traceOrginPoint)
            lk.LakooManager:getInstance():setPlayerMoveDir(Enum.DIR.LK_DIR_NONE)
        	return
        end
        
        if isUseController == false then
            self:updateControlEvent()
            self:updateControlButList()
        end
    end
    local function onNodeEvent(event)
        --cclog("DirControllerUI Event:"..event)
        if "enterTransitionFinish" == event then
            self.scheduleID = cc.Director:getInstance():getScheduler():scheduleScriptFunc(update, 0, false)
        end
        if "exit" == event then
            if self.scheduleID ~= nil then
                cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.scheduleID)
            end
            for var=1, #self.dirButList do
            	self.dirButList[var] = nil
            end
        end
    end
    self:registerScriptHandler(onNodeEvent)
end


function DirControllerUI:updateControlEvent()
    if self.touchPoint.x == -1 or self.touchPoint.y == -1 or self.isSwallow == false then
        self.traceIcon:setPosition(self.traceOrginPoint)
        lk.LakooManager:getInstance():setPlayerMoveDir(Enum.DIR.LK_DIR_NONE)
    	return
    end
    local dir = self:getDir()
    lk.LakooManager:getInstance():setPlayerMoveDir(dir)

    local RADIUS = LKConfig.CONTROLER_RADIUS
    local len = MathUtil.calDistance(self.traceOrginPoint,self.touchPoint)
    if len < RADIUS then
        self.traceIcon:setPosition(self.touchPoint)
    else
        local point = MathUtil.getIntersectPoint(self.traceOrginPoint,RADIUS,self.touchPoint)
        self.traceIcon:setPosition(point)
    end
    
    if self.isTouchEnd then
        self.touchPoint = cc.p(-1,-1)
        self.isTouchEnd = false
    end
end


function DirControllerUI:updateControlButList()
    local currentDir = lk.LakooManager:getInstance():getPlayerMoveDir()
    for i, var in ipairs(self.dirButList) do
        repeat
            if var == nil then
                break
            end

            if currentDir == Enum.DIR.LK_DIR_UP then
                if var:getTag() == self.BUT_TAG.UP then
                    var:setHighlighted(true)
                else
                    var:setHighlighted(false)
                end
            elseif currentDir == Enum.DIR.LK_DIR_RIGHT then
                if var:getTag() == self.BUT_TAG.RIGHT then
                    var:setHighlighted(true)
                else
                    var:setHighlighted(false)
                end                
            elseif currentDir == Enum.DIR.LK_DIR_DOWN then
                if var:getTag() == self.BUT_TAG.DOWN then
                    var:setHighlighted(true)
                else
                    var:setHighlighted(false)
                end                
            elseif currentDir == Enum.DIR.LK_DIR_LEFT then
                if var:getTag() == self.BUT_TAG.LEFT then
                    var:setHighlighted(true)
                else
                    var:setHighlighted(false)
                end               
            else
                var:setHighlighted(false)
            end
        until true
    end
end


function DirControllerUI:getDir()
    local locationInNode = self.touchPoint
    local tSize = self.eventTagetSize

    local pA = cc.p(0, 0)
    local pB = cc.p(tSize.width*1.5, 0)
    local pC = cc.p(tSize.width*1.5, tSize.height*1.5)
    local pD = cc.p(0, tSize.height*1.5)
    local centerPoint = cc.p(tSize.width/2, tSize.height/2)
    local result = MathUtil.triangleContainsPoint(pA,pB,centerPoint,locationInNode)
    if result == MathUtil.TCPOINT.POINT_A or
        result == MathUtil.TCPOINT.SIDE_CA then
        return Enum.DIR.LK_DIR_LEFT
    elseif result == MathUtil.TCPOINT.POINT_B or
        result == MathUtil.TCPOINT.SIDE_BC or
        result == MathUtil.TCPOINT.SIDE_AB or
        result == MathUtil.TCPOINT.INSIDE then
        return Enum.DIR.LK_DIR_DOWN
    end
    
    result = MathUtil.triangleContainsPoint(pB,pC,centerPoint,locationInNode)
    if result == MathUtil.TCPOINT.POINT_B or
        result == MathUtil.TCPOINT.SIDE_BC or
        result == MathUtil.TCPOINT.SIDE_AB or
        result == MathUtil.TCPOINT.INSIDE then
        return Enum.DIR.LK_DIR_RIGHT
    end
    
    result = MathUtil.triangleContainsPoint(pC,pD,centerPoint,locationInNode)
    if result == MathUtil.TCPOINT.POINT_B or
        result == MathUtil.TCPOINT.SIDE_BC or
        result == MathUtil.TCPOINT.SIDE_AB or
        result == MathUtil.TCPOINT.INSIDE then
        return Enum.DIR.LK_DIR_UP
    end
    
    result = MathUtil.triangleContainsPoint(pD,pA,centerPoint,locationInNode)
    if  result == MathUtil.TCPOINT.SIDE_AB or
        result == MathUtil.TCPOINT.INSIDE then
        return Enum.DIR.LK_DIR_LEFT
    end
    
    return Enum.DIR.LK_DIR_NONE
end

--joe
function DirControllerUI:registerControllerListener()
    local function onConnectController(controller, event)
        if controller == nil or controller == self._firstHolder.controller then
            isUseController = true
            return
        end

        local deviceInfo = string.format("%s id:%d", controller:getDeviceName(), controller:getDeviceId())
        cclog("DirControllerUI deviceInfo ="..deviceInfo)

        if self._firstHolder.controller == nil then
            self._firstHolder.controller = controller
            isUseController = true
        end
    end

    local function onDisconnectedController(controller, event)
        if self._firstHolder.controller == controller then
            self._firstHolder.controller = nil
        end
    end

    local function showButtonState(controller, keyCode, isPressed)
        onConnectController(controller, nil)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else    
            return
        end
        
        if lk.LakooManager:getInstance():getIsHasPopView() == true then
            return
        end
        
        local scene = cc.Director:getInstance():getRunningScene()
        local child = scene:getChildren()
        
        local topSence = child[#child]
        if topSence:getName() ~= "WorldScene" then 
            return
        end

        if isPressed then
            if keyCode == cc.ControllerKey.BUTTON_A then
            elseif keyCode == cc.ControllerKey.BUTTON_B then
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
                lk.LakooManager:getInstance():setPlayerMoveDir(Enum.DIR.LK_DIR_UP)
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
                lk.LakooManager:getInstance():setPlayerMoveDir(Enum.DIR.LK_DIR_DOWN)
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
                lk.LakooManager:getInstance():setPlayerMoveDir(Enum.DIR.LK_DIR_LEFT)
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
                lk.LakooManager:getInstance():setPlayerMoveDir(Enum.DIR.LK_DIR_RIGHT)
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key Down:%d",keyCode)
            end
        else
            cclog("DirControllerUI keydown ="..keyCode)
            if keyCode == cc.ControllerKey.BUTTON_A then
                lk.LakooManager:getInstance():setPlayerMoveDir(Enum.DIR.LK_DIR_NONE)
            elseif keyCode == cc.ControllerKey.BUTTON_B then
                lk.LakooManager:getInstance():setPlayerMoveDir(Enum.DIR.LK_DIR_NONE)
            elseif keyCode == cc.ControllerKey.BUTTON_X then
                lk.LakooManager:getInstance():setPlayerMoveDir(Enum.DIR.LK_DIR_NONE)
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
                lk.LakooManager:getInstance():setPlayerMoveDir(Enum.DIR.LK_DIR_NONE)
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
                lk.LakooManager:getInstance():setPlayerMoveDir(Enum.DIR.LK_DIR_NONE)
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
                lk.LakooManager:getInstance():setPlayerMoveDir(Enum.DIR.LK_DIR_NONE)
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
                lk.LakooManager:getInstance():setPlayerMoveDir(Enum.DIR.LK_DIR_NONE)
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
                lk.LakooManager:getInstance():setPlayerMoveDir(Enum.DIR.LK_DIR_NONE)
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
                lk.LakooManager:getInstance():setPlayerMoveDir(Enum.DIR.LK_DIR_NONE)
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
                lk.LakooManager:getInstance():setPlayerMoveDir(Enum.DIR.LK_DIR_NONE)
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key up:%d",keyCode)
            end
        end
    end 

    local function onKeyDown(controller, keyCode, event)
        showButtonState(controller, keyCode, true)
    end

    local function onKeyUp(controller, keyCode, event)
        showButtonState(controller, keyCode, false)
    end

    local function onAxisEvent(controller, keyCode, event)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end
        
        if lk.LakooManager:getInstance():getIsHasPopView() == true then
            return
        end

        local ketStatus = controller:getKeyStatus(keyCode)
        if keyCode == cc.ControllerKey.JOYSTICK_LEFT_X then
            if(ketStatus.value == -1) then
            elseif(ketStatus.value == 1) then

            end
        elseif keyCode == cc.ControllerKey.JOYSTICK_LEFT_Y then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_X then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_Y then
        elseif keyCode == cc.ControllerKey.AXIS_LEFT_TRIGGER then
            lk.LakooManager:getInstance():setPlayerMoveDir(Enum.DIR.LK_DIR_NONE)
        elseif keyCode == cc.ControllerKey.AXIS_RIGHT_TRIGGER then
             lk.LakooManager:getInstance():setPlayerMoveDir(Enum.DIR.LK_DIR_NONE)
        else
        --do nothing
        end
    end

    local controllerListener = cc.EventListenerController:create()
    controllerListener:registerScriptHandler(onConnectController, cc.Handler.EVENT_CONTROLLER_CONNECTED)
    controllerListener:registerScriptHandler(onDisconnectedController, cc.Handler.EVENT_CONTROLLER_DISCONNECTED)

    controllerListener:registerScriptHandler(onKeyDown, cc.Handler.EVENT_CONTROLLER_KEYDOWN)
    controllerListener:registerScriptHandler(onKeyUp, cc.Handler.EVENT_CONTROLLER_KEYUP)
    controllerListener:registerScriptHandler(onAxisEvent, cc.Handler.EVENT_CONTROLLER_AXIS)

    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(controllerListener, self)

    cc.Controller:startDiscoveryController()
end
--

return DirControllerUI
