require "cocos.init"
local UILib = require("common.UILib")
local Enum = require("common.Enum")
local LKConfig = require("conf.LKConfig")

local LayerManager = require("manager.LayerManager")

local WarningLayer = require("gameUI.layer.WarningLayer")

local GamePool = require("model.GamePool")

local ControlHelperLayer = require("gameUI.layer.ControlHelperLayer")

local cc = cc
local ccs = ccs
local class = class

local TAG = {
    NUM_LV = 18,
    NUM_MONEY = 13,
    NUM_STONE = 17,
    HP = 6,
    MP = 7,
    EXP = 8,
    BUT_SAVE = 9,
    BUT_MAIN_POP = 4,
    BUT_MONEY = 10,
    BUT_STONE = 11,
    BUT_SOUL = 12,
}

local ControllerHolder = class("ControllerHolder")
function ControllerHolder:ctor()
    self.controller  = nil
    self._holderNode = nil

    self._leftJoystick = nil
    self._rightJoystick = nil

    self._dpadLeft = nil
    self._dpadRight = nil
    self._dpadUp = nil
    self._dpadDown = nil

    self._buttonX = nil
    self._buttonY = nil
    self._buttonA = nil
    self._buttonB = nil

    self._buttonL1 = nil
    self._buttonR1 = nil
    self._buttonL2 = nil
    self._buttonR2 = nil

    self.deviceLabel = nil
    self.externalKeyLabel = nil
end

local HeadUI = class("HeadUI",
    function()
        return cc.Node:create()
    end
)

HeadUI.__index = HeadUI

HeadUI.lvLabel = nil
HeadUI.moneyLabel = nil
HeadUI.stoneLabel = nil
HeadUI.hpBar = nil
HeadUI.mpBar = nil
HeadUI.expBar = nil
HeadUI.saveBut = nil
HeadUI.openBut = nil
HeadUI.openSoulBut = nil
HeadUI.countDownLabel = nil
HeadUI.lvUpdateTipsLabel = nil

function HeadUI:create()
    local headUI = HeadUI.new()
    headUI:init()

    return headUI
end

function HeadUI:init()
    self:setName("HeadUI")
    local widget = ccs.GUIReader:getInstance():widgetFromBinaryFile("HeadUI_1/HeadUI_1.csb")
    local wSize = widget:getContentSize()
    self:setContentSize(wSize)
    self:setAnchorPoint(UILib.UI_ANCHOR_POINT.CENTER)
    self:addChild(widget)
    
    --lv number
    self.lvLabel = UILib.getNodeByTag(widget,TAG.NUM_LV)
    
    --money number
    self.moneyLabel = UILib.getNodeByTag(widget,TAG.NUM_MONEY)
    
    --stone number
    self.stoneLabel = UILib.getNodeByTag(widget,TAG.NUM_STONE)
    
    --hp bar 
    self.hpBar = UILib.getNodeByTag(widget,TAG.HP)
    self.hpBar:setVisible(false)
    self.hpBarMax = ccui.LoadingBar:create("res/HeadUI_1/RES_ui/p_161_1.png", 100)
    self.hpBarMax:setPosition(self.hpBar:getPosition())
    self.hpBar:getParent():addChild(self.hpBarMax)
    
    --mp bar
    self.mpBar = UILib.getNodeByTag(widget,TAG.MP)
    
    --exp bar
    self.expBar = UILib.getNodeByTag(widget,TAG.EXP)
    
    self:updateUIValue()
    
    local function touchButton(sender, eventType) 
        if eventType == ccui.TouchEventType.began then
            if sender:getTag() == TAG.BUT_SAVE then
                local layer = WarningLayer:create()
                GamePool.getCurrentScene():addChild(layer)
                layer:setOnActionFunc(self.doSave)
                local msg = lk.LakooManager:getInstance():getString(Enum.TEXT.T_WHETHER_SAVE)
                local okText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_YES)
                local cancelText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_NO)
                layer:setString(msg, okText, cancelText)
            end
        end
        if eventType == ccui.TouchEventType.moved then
            --cclog("moved---")
        end
        if eventType == ccui.TouchEventType.ended then
        end
    end
    
    --存档按钮
    self.saveBut = UILib.getNodeByTag(widget,TAG.BUT_SAVE)
    if self.saveBut ~= nil then
        self.saveBut:addTouchEventListener(touchButton)
    end
    
    local function doButtonAction(sender, eventType) 
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            local tag = sender:getTag()
            if tag == TAG.BUT_MAIN_POP then
                LayerManager.showLayerWithType(Enum.LAYER_TYPE.MAIN_POP)
            elseif tag == TAG.BUT_MONEY or tag == TAG.BUT_STONE then
                LayerManager.showLayerWithType(Enum.LAYER_TYPE.SINGLE_POP_MAIN_SHOP)
            elseif tag == TAG.BUT_SOUL then
                LayerManager.showLayerWithType(Enum.LAYER_TYPE.MAIN_POP_SOUL)
            end
            
        end
    end

    --打开主菜单按钮
    self.openBut = UILib.getNodeByTag(widget,TAG.BUT_MAIN_POP)
    if self.openBut ~= nil then
        self.openBut:addTouchEventListener(doButtonAction)
    end
    
    --countDown is 0,初始化由事件Event触化,初始化值600(10分钟)
    local countDown = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.OTHER_3)
    self.countDownLabel = cc.LabelAtlas:_create(countDown, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
    local pos = cc.p(self.openBut:getPositionX()-self.openBut:getContentSize().width/4,self.openBut:getPositionY()-self.openBut:getContentSize().height/2-self.countDownLabel:getContentSize().height/2)
    self.countDownLabel:setPosition(pos)
    self:addChild(self.countDownLabel)
    self.countDownLabel:setVisible(false)
    
    --打开商店
    local shopBut1 = UILib.getNodeByTag(widget,TAG.BUT_MONEY)
    if shopBut1 ~= nil then
        shopBut1:addTouchEventListener(doButtonAction)
    end
    local shopBut2 = UILib.getNodeByTag(widget,TAG.BUT_STONE)
    if shopBut2 ~= nil then
        shopBut2:addTouchEventListener(doButtonAction)
    end
    
    --打开“妖魂”》“召唤”背包界面
    self.openSoulBut = UILib.getNodeByTag(widget,TAG.BUT_SOUL)
    if self.openSoulBut ~= nil then
        self.openSoulBut:addTouchEventListener(doButtonAction)
    end
    
    local function update()
        if self:isVisible() ~= true then
            return
        end
        self:updateUIValue()
        local isCanSave = lk.LakooManager:getInstance():isCanSave()
        local isInArena = lk.LakooManager:getInstance():isPlayerInArena()
        if isCanSave and (isInArena == false) then
            self.saveBut:setTouchEnabled(true)
            self.saveBut:setBright(true)
            self.saveBut:setHighlighted(false)
        else
            self.saveBut:setTouchEnabled(false)
            self.saveBut:setBright(false)
            self.saveBut:setHighlighted(true)
        end
        
        if isInArena then
            self.openBut:setTouchEnabled(false)
            self.openSoulBut:setTouchEnabled(false)
            
            self.countDownLabel:setVisible(true)
            local countDown = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.OTHER_3)
            self.countDownLabel:setString(countDown)
        else
            self.openBut:setTouchEnabled(true)
            self.openSoulBut:setTouchEnabled(true)
            
            self.countDownLabel:setVisible(false)
        end
        
        self:UpdateLVTips()
    end
    
    local function onNodeEvent(event)
        if "enterTransitionFinish" == event then
            self.scheduleID = cc.Director:getInstance():getScheduler():scheduleScriptFunc(update, 0, false)
        end
        if "exit" == event then
            if self.scheduleID ~= nil then
                cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.scheduleID)
            end
        end
    end
    self:registerScriptHandler(onNodeEvent)
    
    local osType = Enum.OS_TYPE.IOS
    osType = lk.LakooManager:getInstance():getOSType()
    if osType == Enum.OS_TYPE.ANDROID then
        --joe
        self._firstHolder   = ControllerHolder.new()
        self._firstHolder.controller = nil
        self:registerControllerListener()
        --
    end
end

function HeadUI:updateUIValue()
    if self.lvLabel ~= nil then
        local lv = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.LEVEL)
        self.lvLabel:setString(lv)
    end
    if self.moneyLabel ~= nil then
        local money = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.MONEY)
        self.moneyLabel:setString(money)
    end
    if self.stoneLabel ~= nil then
        local stone = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.SOUL_STONE)
        self.stoneLabel:setString(stone)
    end
    if self.hpBar ~= nil then
        --[[满血时HP Bar变成黄色--]]
        local hpPer = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.HP_PERCENT)
        if hpPer >= 100 then
            self.hpBar:setVisible(false)
            self.hpBarMax:setVisible(true)
            self.hpBarMax:setPercent(hpPer)
        else
            self.hpBarMax:setVisible(false)
            self.hpBar:setVisible(true)
            self.hpBar:setPercent(hpPer)       
        end
        
    end
    if self.mpBar ~= nil then
        local mpPer = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.MP_PERCENT)
        self.mpBar:setPercent(mpPer)
    end
    if self.expBar ~= nil then
        local exp = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.EXP)
        local nextExp = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.NEXT_EXP)
        local expPer = 0
        if nextExp ~= 0 then
        	expPer = exp / nextExp * 100
        end
        self.expBar:setPercent(expPer)
    end
end

function HeadUI:UpdateLVTips()
    local lvUpdateTipsStatus = lk.LakooManager:getInstance():getUpdateTipsStatus()
    if lvUpdateTipsStatus == Enum.LVUP_TIPS_STATUS.LVUP_NONE then
        if self.lvUpdateTipsLabel ~= nil then
        	self.lvUpdateTipsLabel:setVisible(false)
        end
        return
    end
    
    local gameCount = lk.LakooManager:getInstance():getGameCount()
    if math.fmod(gameCount,2) ~= 0 then
    	return
    end
    
    if self.lvUpdateTipsLabel ~= nil then
        self.lvUpdateTipsLabel:setVisible(true)
    end
    if self.lvUpdateTipsLabel == nil then
        local lvUpTips = lk.LakooManager:getInstance():getString(Enum.TEXT.T_UPGRADE_TIPS)
    	self.lvUpdateTipsLabel = UILib.textAddStroke(lvUpTips,UILib.FONT_NAME_NORMAL,UILib.FONT.MINI,UILib.UI_COLOR3B.WHITE,UILib.UI_COLOR3B.BLUE,1)
    	self:addChild(self.lvUpdateTipsLabel)
    	local pos = UILib.offsetWithSuperSize(20,-1,self.lvUpdateTipsLabel:getContentSize().width,self.lvUpdateTipsLabel:getContentSize().height,self:getContentSize().width,self:getContentSize().height,UILib.UI_ANCHOR.TOP_LEFT)
        self.lvUpdateTipsLabel:setPosition(pos)
    end
    
    if self.lvUpdateTipsLabel == nil then
    	return
    end
    
    if lvUpdateTipsStatus == Enum.LVUP_TIPS_STATUS.LVUP_ING or lvUpdateTipsStatus == Enum.LVUP_TIPS_STATUS.LVUP_DRAW_2 then
        UILib.setStrokeColor(self.lvUpdateTipsLabel, UILib.UI_COLOR3B.BLUE) 
        lk.LakooManager:getInstance():setUpdateTipsStatus(Enum.LVUP_TIPS_STATUS.LVUP_DRAW_1)
    elseif lvUpdateTipsStatus == Enum.LVUP_TIPS_STATUS.LVUP_DRAW_1 then  
        UILib.setStrokeColor(self.lvUpdateTipsLabel, UILib.UI_COLOR3B.RED) 
        lk.LakooManager:getInstance():setUpdateTipsStatus(Enum.LVUP_TIPS_STATUS.LVUP_DRAW_2)	
    end
end

function HeadUI.doSave()
    local result = lk.LakooManager:getInstance():doSave(Enum.FILE.GAME_CHUNK_NAME)
    local msg = nil
    if result > 0 then
        msg = lk.LakooManager:getInstance():getString(Enum.TEXT.T_SAVE_OK)        
    else
        msg = lk.LakooManager:getInstance():getString(Enum.TEXT.T_SAVE_FAILED)
    end
    if msg ~= nil then
        lk.LakooManager:getInstance():addPopTip(msg)
    end
end

--joe
function HeadUI:registerControllerListener()
    local function onConnectController(controller, event)
        if controller == nil or controller == self._firstHolder.controller then
            return
        end

        local deviceInfo = string.format("%s id:%d", controller:getDeviceName(), controller:getDeviceId())
        cclog("HeadUI deviceInfo ="..deviceInfo)

        if self._firstHolder.controller == nil then
            self._firstHolder.controller = controller
        end
    end

    local function onDisconnectedController(controller, event)
        if self._firstHolder.controller == controller then
            self._firstHolder.controller = nil
        end
    end
    
    local function showButtonState(controller, keyCode, isPressed)
        onConnectController(controller, nil)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else    
            return
        end
        
        if lk.LakooManager:getInstance():getIsHasPopView() == true then
            cclog("HeadUI lk.LakooManager:getInstance():getIsHasPopView() == true")
            return
        end
        
        local scene = cc.Director:getInstance():getRunningScene()
        local child = scene:getChildren()
        
        local topSence = child[#child]
        if topSence:getName() ~= "WorldScene" then 
            return
        end

        if isPressed then
            if keyCode == cc.ControllerKey.BUTTON_A then
            elseif keyCode == cc.ControllerKey.BUTTON_B then
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_START then
            else
                local ketStatus = string.format("Key Down:%d",keyCode)
            end
        else
            if keyCode == cc.ControllerKey.BUTTON_A then
            elseif keyCode == cc.ControllerKey.BUTTON_B then
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then

            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then

            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
                if self:isVisible() ~= true then
                    return
                end
                if self.saveBut:isTouchEnabled() == true then
                    local layer = WarningLayer:create()
                    GamePool.getCurrentScene():addChild(layer)
                    layer:setOnActionFunc(self.doSave)
                    local msg = lk.LakooManager:getInstance():getString(Enum.TEXT.T_WHETHER_SAVE)
                    local okText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_YES)
                    local cancelText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_NO)
                    layer:setString(msg, okText, cancelText)
                end
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
                if self:isVisible() ~= true then
                    return
                end
                local layer = ControlHelperLayer:create()
                GamePool.getCurrentScene():addChild(layer)
                
            elseif keyCode == cc.ControllerKey.BUTTON_START then
                if self:isVisible() ~= true then
                    return
                end
                LayerManager.showLayerWithType(Enum.LAYER_TYPE.MAIN_POP)
            else
                local ketStatus = string.format("Key up:%d",keyCode)
            end
        end
    end 

    local function onKeyDown(controller, keyCode, event)
        showButtonState(controller, keyCode, true)
    end

    local function onKeyUp(controller, keyCode, event)
        showButtonState(controller, keyCode, false)
    end

    local function onAxisEvent(controller, keyCode, event)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end
        
        if lk.LakooManager:getInstance():getIsHasPopView() == true then
            return
        end
        
        local scene = cc.Director:getInstance():getRunningScene()
        local child = scene:getChildren()
        
        local topSence = child[#child]
        if topSence:getName() ~= "WorldScene" then 
            return
        end

        local ketStatus = controller:getKeyStatus(keyCode)
        if keyCode == cc.ControllerKey.JOYSTICK_LEFT_X then
            
        elseif keyCode == cc.ControllerKey.JOYSTICK_LEFT_Y then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_X then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_Y then
        elseif keyCode == cc.ControllerKey.AXIS_LEFT_TRIGGER then
            if(ketStatus.value == 1) then
                if self:isVisible() ~= true then
                    return
                end
             LayerManager.showLayerWithType(Enum.LAYER_TYPE.SINGLE_POP_MAIN_SHOP)
            end
        elseif keyCode == cc.ControllerKey.AXIS_RIGHT_TRIGGER then
        else
        --do nothing
        end
    end

    local controllerListener = cc.EventListenerController:create()
    controllerListener:registerScriptHandler(onConnectController, cc.Handler.EVENT_CONTROLLER_CONNECTED)
    controllerListener:registerScriptHandler(onDisconnectedController, cc.Handler.EVENT_CONTROLLER_DISCONNECTED)

    controllerListener:registerScriptHandler(onKeyDown, cc.Handler.EVENT_CONTROLLER_KEYDOWN)
    controllerListener:registerScriptHandler(onKeyUp, cc.Handler.EVENT_CONTROLLER_KEYUP)
    controllerListener:registerScriptHandler(onAxisEvent, cc.Handler.EVENT_CONTROLLER_AXIS)

    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(controllerListener, self)
    cc.Controller:startDiscoveryController()
end
--

return HeadUI