require "cocos.init"

local cc = cc
local ccui = ccui
local class = class

local IrregularButton = class("IrregularButton",
    function()
        return ccui.Layout:create()
    end
)

IrregularButton.__index = IrregularButton

IrregularButton.delegate = nil
IrregularButton.shapeBtn = nil
IrregularButton.isSwallow = false
IrregularButton.image = nil

function IrregularButton:create(_normalImage, _delegate)
    local irregularButton = IrregularButton.new()
    irregularButton:init(_normalImage, _delegate)

    return irregularButton
end

function IrregularButton:init(_normalImage, _delegate)

    self.delegate = _delegate
    
    self.image = cc.Image:new()
    self.image:retain()
    self.image:initWithImageFile(_normalImage)
    local tempTexture = cc.Texture2D:new()
    tempTexture:initWithImage(self.image)
    
    self.shapeBtn = cc.Sprite:createWithTexture(tempTexture)
    self:addChild(self.shapeBtn)
    
    self:setSize(self.shapeBtn:getContentSize())
    self.shapeBtn:setPosition(self:getSize().width / 2, self:getSize().height / 2)

    local listener = cc.EventListenerTouchOneByOne:create()

    -- handling touch events
    local function onTouchBegan(touch, event)
        local target = event:getCurrentTarget()
        local locationInNode = target:convertToNodeSpace(touch:getLocation())
            
        local pt = cc.p(locationInNode.x, target:getContentSize().height - locationInNode.y);
        
        local s = target:getContentSize();
        local rect = cc.rect(0, 0, s.width, s.height);
        
        if (cc.rectContainsPoint(rect,locationInNode)) then
            --cclog("sprite began... x = %f, y = %f", locationInNode.x, locationInNode.y)
                
            local alpha = self.image:getAlphaWithPoint(pt.x, pt.y)
            --cclog("IrregularButton alpha[%d]",alpha)
            if (alpha <= 50) then
                self.isSwallow = false
            else
                self.isSwallow = true
            end                
        else
                self.isSwallow = false
        end
            
            listener:setSwallowTouches(self.isSwallow)            
            return self.isSwallow
    end
    
    local function onTouchEnded(touch, event)
        if self.delegate ~= nil and self.isSwallow and self:isVisible() then
        	self.delegate:buttonCallBack(self)
        end
    end

    listener:registerScriptHandler(onTouchBegan,cc.Handler.EVENT_TOUCH_BEGAN)
    listener:registerScriptHandler(onTouchEnded,cc.Handler.EVENT_TOUCH_ENDED)
    local eventDispatcher = self.shapeBtn:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listener, self.shapeBtn)
    
    local function onNodeEvent(event)
        if "exit" == event then
            self.image:release()
        end
    end
    self:registerScriptHandler(onNodeEvent)
end

function IrregularButton:setButtonTag(tag)
    self.shapeBtn:setTag(tag)
    self:setTag(tag)
end

return IrregularButton