require "cocos.init"

local UILib = require("common.UILib")
local Enum = require("common.Enum")
local PayUtil = require("common.PayUtil")

local Text = require("conf.Text")
local IDX = require("conf.TextIndex")

local GamePool = require("model.GamePool")
local LayerStage = require("manager.LayerStage")
local NetworkManager = require("manager.NetworkManager")

local EquipSoulLayer = require("gameUI.layer.EquipSoulLayer")
local WarningLayer = require("gameUI.layer.WarningLayer")

local cc = cc
local ccui = ccui
local class = class

local TAG = {
    EQUIP = 1,
    STRENGTHEN = 2,
    USE = 3,
    EQUIP_SOUL = 4,
    UNEQUIP_SOUL = 5,
    MERGE_SOUL = 6,
    GET_SOUL = 7,
    MERGE_WEAPON = 8,
    GET_WEAPON = 9,
    BUY = 10,
    BACK = 11,
    SELL = 12,
    RESTORE = 13,
}

local ControllerHolder = class("ControllerHolder")
function ControllerHolder:ctor()
    self.controller  = nil
    self._holderNode = nil

    self._leftJoystick = nil
    self._rightJoystick = nil

    self._dpadLeft = nil
    self._dpadRight = nil
    self._dpadUp = nil
    self._dpadDown = nil

    self._buttonX = nil
    self._buttonY = nil
    self._buttonA = nil
    self._buttonB = nil

    self._buttonL1 = nil
    self._buttonR1 = nil
    self._buttonL2 = nil
    self._buttonR2 = nil

    self.deviceLabel = nil
    self.externalKeyLabel = nil
end

local ItemInfoView = class("ItemInfoView",
    function(bgImgName)
        return ccui.ImageView:create(bgImgName)
    end
)

ItemInfoView.__index = ItemInfoView

ItemInfoView.bagTag = Enum.BAG_TAG.EQUIPMENT
ItemInfoView.item = nil
ItemInfoView.itemIndex = 0
ItemInfoView.scrollView = nil

ItemInfoView.enableShowStrengBut = false

ItemInfoView.equipButton = nil
ItemInfoView.strengButton = nil
ItemInfoView.useButton = nil
ItemInfoView.equipSoulButton = nil
ItemInfoView.unEquipSoulButton = nil
ItemInfoView.smeltSoulButton = nil
ItemInfoView.getSoulButton = nil
ItemInfoView.smeltWeaponButton = nil
ItemInfoView.getWeaponButton = nil
ItemInfoView.buyButton = nil
ItemInfoView.restoreButton = nil
ItemInfoView.backButton = nil
ItemInfoView.sellButton = nil
ItemInfoView.nextTimeLabel = nil

ItemInfoView.updateBagFun = nil
ItemInfoView.isHasDoGetMsg = false
ItemInfoView.currentOpType = -1
ItemInfoView.closeCallback = nil

ItemInfoView.focusAni = nil
ItemInfoView.itemSign = false
ItemInfoView.firstTap = nil
ItemInfoView.secondTap = nil
ItemInfoView.tapSelected = -1

ItemInfoView.tapTable = {}

function ItemInfoView:create(_currentBagType, _updateBag, _closeCallback)
    local itemInfoView = ItemInfoView.new("p_46.pngHK")
    itemInfoView:init(_currentBagType, _updateBag, _closeCallback)

    return itemInfoView
end

function ItemInfoView:init(_currentBagType, _updateBag, _closeCallback)
    self:setName("ItemInfoView")
    self.bagTag = _currentBagType
    cclog("ItemInfoView_currentBagType = ".._currentBagType)
    self.updateBagFun = _updateBag
    self.closeCallback = _closeCallback
    
    -- lk.LakooManager:getInstance():initRewardVideo(Enum.ADMOB_REWARD_VIDEO_LIST.REWARD_STONE)

    self.scrollView = ccui.ScrollView:create()
    self.scrollView:setClippingEnabled(true)
    self.scrollView:setContentSize(cc.size(self:getContentSize().width-15*2,self:getContentSize().height-20))
    self.scrollView:setPosition(cc.p(15,5))
    self:addChild(self.scrollView)

    --    local layer = cc.LayerColor:create(UILib.UI_COLOR4B.BLACK)
    --    layer:setPosition(cc.p(0,0))
    --    layer:setContentSize(self.scrollView:getContentSize())
    --    self.scrollView:addChild(layer)

    self.adsFreeGemTimes = lk.LakooManager:getInstance():getFreeGemTimes()
    self.adsFreeGemCD = lk.LakooManager:getInstance():getFreeGemCD()*60

    self.adsTimes = lk.LakooManager:getInstance():getAdsTimes("1")
    self.adsNextTime = lk.LakooManager:getInstance():getAdsNextTime("1")
    if self.adsNextTime > self.adsFreeGemCD then
        lk.LakooManager:getInstance():resetAds("1")
    end

    local function doButtonAction(sender, eventType)
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            local tag = sender:getTag()
            cclog("doButtonAction tag ="..tag)
            if tag == TAG.EQUIP then
                self:doEquip()
            elseif tag == TAG.STRENGTHEN then
                self:doStrengthen()
            elseif tag == TAG.USE then
                self:doUse()
            elseif tag == TAG.EQUIP_SOUL then
                self:doEquipSoul()
            elseif tag == TAG.UNEQUIP_SOUL then
                self:doUnEquipSoul()
            elseif tag == TAG.MERGE_SOUL then
                self:doMergeSoul()
            elseif tag == TAG.GET_SOUL then
                self:doGet()
            elseif tag == TAG.MERGE_WEAPON then
                self:doMergeWeapon()
            elseif tag == TAG.GET_WEAPON then
                self:doGet()
            elseif tag == TAG.BUY or tag == TAG.RESTORE then
                self:doBuy()
            elseif tag == TAG.BACK then
                self:doBack()
            elseif tag == TAG.SELL then
                self:doSell()
            end
        end
    end

    local osType = Enum.OS_TYPE.IOS
    osType = lk.LakooManager:getInstance():getOSType()
    if osType == Enum.OS_TYPE.ANDROID then
        --joe
        if self._firstHolder == nil then
            self._firstHolder = ControllerHolder.new()
            self._firstHolder.controller = nil
            self:registerControllerListener(doButtonAction)
            cclog("ItemInfoView registerControllerListener")
        end
        --
    end

--    if self.focusAni == nil then
--        ccs.ArmatureDataManager:getInstance():addArmatureFileInfo("FocusAni.csb")
--        ItemInfoView.focusAni = ccs.Armature:create("FocusAni")
--        self:addChild(self.focusAni)
--        self.focusAni:setLocalZOrder(99)
--        self.focusAni:getAnimation():playWithIndex(0)
--        self.focusAni:setVisible(false)
--    end
    --装备武器
    local imgName = string.format("%s.%s",Enum.TEXT_IMG.EQUIP,Text.get(IDX.PNG_TYPE))
    self.equipButton = ccui.Button:create(imgName,imgName,imgName)
    local pos1 = UILib.offsetWithSuperSize(0,-10,self.equipButton:getContentSize().width,self.equipButton:getContentSize().height,self:getContentSize().width,self:getContentSize().height,UILib.UI_ANCHOR.TOP_RIGHT)
    self.equipButton:setPosition(pos1)
    self:addChild(self.equipButton)
    self.equipButton:setTag(TAG.EQUIP)
    self.equipButton:addTouchEventListener(doButtonAction)

    --装备妖魂
    self.equipSoulButton= ccui.Button:create(imgName,imgName,imgName)
    self.equipSoulButton:setPosition(pos1)
    self:addChild(self.equipSoulButton)
    self.equipSoulButton:setTag(TAG.EQUIP_SOUL)
    self.equipSoulButton:addTouchEventListener(doButtonAction)

    --强化武器
    imgName = string.format("%s.%s",Enum.TEXT_IMG.STRENGTHEN,Text.get(IDX.PNG_TYPE))
    self.strengButton = ccui.Button:create(imgName,imgName,imgName)
    local pos2 = UILib.offsetWithSuperSize(0,5,self.strengButton:getContentSize().width,self.strengButton:getContentSize().height,self:getContentSize().width,self:getContentSize().height,UILib.UI_ANCHOR.BOTTOM_RIGHT)
    self.strengButton:setPosition(pos2)
    self:addChild(self.strengButton)
    self.strengButton:setTag(TAG.STRENGTHEN)
    self.strengButton:addTouchEventListener(doButtonAction)

    --使用药品
    imgName = string.format("%s.%s",Enum.TEXT_IMG.USE,Text.get(IDX.PNG_TYPE))
    self.useButton = ccui.Button:create(imgName,imgName,imgName)
    self.useButton:setPosition(pos1)
    self:addChild(self.useButton)
    self.useButton:setTag(TAG.USE)
    self.useButton:addTouchEventListener(doButtonAction)

    --卸下妖魂
    imgName = string.format("%s.%s",Enum.TEXT_IMG.UNEQUIP,Text.get(IDX.PNG_TYPE))
    self.unEquipSoulButton = ccui.Button:create(imgName,imgName,imgName)
    self.unEquipSoulButton:setPosition(pos2)
    self:addChild(self.unEquipSoulButton)
    self.unEquipSoulButton:setTag(TAG.UNEQUIP_SOUL)
    self.unEquipSoulButton:addTouchEventListener(doButtonAction)

    --炼魂
    imgName = string.format("%s.%s",Enum.TEXT_IMG.SMELTSOUL,Text.get(IDX.PNG_TYPE))
    self.smeltSoulButton = ccui.Button:create(imgName,imgName,imgName)
    self.smeltSoulButton:setPosition(pos1)
    self:addChild(self.smeltSoulButton)
    self.smeltSoulButton:setTag(TAG.MERGE_SOUL)
    self.smeltSoulButton:addTouchEventListener(doButtonAction)

    --获取魂
    imgName = string.format("%s.%s",Enum.TEXT_IMG.GET,Text.get(IDX.PNG_TYPE))
    self.getSoulButton = ccui.Button:create(imgName,imgName,imgName)
    self.getSoulButton:setPosition(pos2)
    self:addChild(self.getSoulButton)
    self.getSoulButton:setTag(TAG.GET_SOUL)
    self.getSoulButton:addTouchEventListener(doButtonAction)

    --锻造武器
    imgName = string.format("%s.%s",Enum.TEXT_IMG.SMELT,Text.get(IDX.PNG_TYPE))
    self.smeltWeaponButton = ccui.Button:create(imgName,imgName,imgName)
    self.smeltWeaponButton:setPosition(pos1)
    self:addChild(self.smeltWeaponButton)
    self.smeltWeaponButton:setTag(TAG.MERGE_WEAPON)
    self.smeltWeaponButton:addTouchEventListener(doButtonAction)

    --获取武器
    imgName = string.format("%s.%s",Enum.TEXT_IMG.GET,Text.get(IDX.PNG_TYPE))
    self.getWeaponButton = ccui.Button:create(imgName,imgName,imgName)
    self.getWeaponButton:setPosition(pos2)
    self:addChild(self.getWeaponButton)
    self.getWeaponButton:setTag(TAG.GET_WEAPON)
    self.getWeaponButton:addTouchEventListener(doButtonAction)

    --购买
    imgName = string.format("%s.%s",Enum.TEXT_IMG.BUY,Text.get(IDX.PNG_TYPE))
    self.buyButton = ccui.Button:create(imgName,imgName,imgName)
    self.buyButton:setPosition(pos1)
    self:addChild(self.buyButton)
    self.buyButton:setTag(TAG.BUY)
    self.buyButton:addTouchEventListener(doButtonAction)

    self.nextTimeLabel = cc.Label:create()
    self.nextTimeLabel:setSystemFontSize(UILib.FONT.MINI)
    -- self.nextTimeLabel:setColor(UILib.UI_COLOR3B.BLACK)
    self.nextTimeLabel:setString("")
    self.nextTimeLabel:setPosition(cc.p(pos1.x + 10, pos1.y))
    self:addChild(self.nextTimeLabel)
    self.nextTimeLabel:setVisible(false)
    
    --恢复(购买)
    imgName = string.format("%s.%s",Enum.TEXT_IMG.RESTORE,Text.get(IDX.PNG_TYPE))
    self.restoreButton = ccui.Button:create(imgName,imgName,imgName)
    local pos3 = cc.p(pos1.x - 10 - self.restoreButton:getContentSize().width,pos1.y)
    self.restoreButton:setPosition(pos3)
    self:addChild(self.restoreButton)
    self.restoreButton:setTag(TAG.RESTORE)
    self.restoreButton:addTouchEventListener(doButtonAction)

    --返回
    imgName = string.format("%s.%s",Enum.TEXT_IMG.BACK,Text.get(IDX.PNG_TYPE))
    self.backButton = ccui.Button:create(imgName,imgName,imgName)
    self.backButton:setPosition(pos2)
    self:addChild(self.backButton)
    self.backButton:setTag(TAG.BACK)
    self.backButton:addTouchEventListener(doButtonAction)

    --卖出
    imgName = string.format("%s.%s",Enum.TEXT_IMG.SELL,Text.get(IDX.PNG_TYPE))
    self.sellButton = ccui.Button:create(imgName,imgName,imgName)
    self.sellButton:setPosition(pos1)
    self:addChild(self.sellButton)
    self.sellButton:setTag(TAG.SELL)
    self.sellButton:addTouchEventListener(doButtonAction)

    local function update(delta)
        -- TODO lua
        self:updateInfomationAnim(delta)
        self:payProcess()
    end

    local function updateNextTime(delta)
        if self.adsFreeGemTimes ~= 0 and self.adsNextTime <= self.adsFreeGemCD and self.adsNextTime > 0 and self.adsTimes >= self.adsFreeGemTimes then
            local nextTime = self.adsFreeGemCD - self.adsNextTime
            local minute = math.floor(nextTime/60)
            local second = nextTime - minute*60
            local time = minute..":"
            if second < 10 then
                time = time.."0"..second
            else
                time = time..second
            end
            self.nextTimeLabel:setString(time)
            self.adsNextTime = self.adsNextTime + 1
            if nextTime == 0 then
                lk.LakooManager:getInstance():resetAds("1")
                self.nextTimeLabel:setVisible(false)
                self.buyButton:setVisible(true)
                self.buyButton:loadTextures("p_202.png", "p_202.png", "p_202.png")
            end
        end
    end

    local function onNodeEvent(event)
        if "enterTransitionFinish" == event then
            self.scheduleID = cc.Director:getInstance():getScheduler():scheduleScriptFunc(update, 0, false)
            self.scheduleTime = cc.Director:getInstance():getScheduler():scheduleScriptFunc(updateNextTime, 1, false)
        end
        if "exit" == event then
            if self.scheduleID ~= nil then
                cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.scheduleID)
            end
            if self.scheduleTime ~= nil then
                cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.scheduleTime)
            end
        end
    end
    self:registerScriptHandler(onNodeEvent)
end

function ItemInfoView:payProcess()
    local payStatus = lk.LakooManager:getInstance():getPayStatus()
    if payStatus == Enum.PAY_STATUS.NONE or payStatus == Enum.PAY_STATUS.PAYING then
        return
    end

    cclog("充值成功或失败")
    if payStatus == Enum.PAY_STATUS.SUCCESS then
        local orderId = lk.LakooManager:getInstance():getPayOrderId()

        local targetPlatform = CCApplication:getInstance():getTargetPlatform()
        if targetPlatform == kTargetIphone or targetPlatform == kTargetIpad then
            local receipt = lk.LakooManager:getInstance():getPayReceipt()
            --IAP 付费成功，提交服务器验证
            NetworkManager.requestAppleVerifyReceipt(orderId,receipt)
        elseif targetPlatform == kTargetAndroid then
            local signature = lk.LakooManager:getInstance():getPaySignature()
            local signedData = lk.LakooManager:getInstance():getPaySignedData()
            signature = string.gsub(signature,"\"","\\\"")--[[服务器需要--]]
            signedData = string.gsub(signedData,"\"","\\\"")--[[服务器需要--]]
            NetworkManager.requestGoogleVerifySignedData(orderId,signature,signedData)
		elseif targetPlatform == 11 then--没有合适的targer定义，hardcode
            -- Window IAP
            local receipt = lk.LakooManager:getInstance():getPayReceipt()
            local thirdOrderId = lk.LakooManager:getInstance():getPaySignature()
            NetworkManager.requestWindowsVerifyReceipt(orderId,receipt,thirdOrderId)
        end
        local itemId = self.item[Enum.IAP_KEY.IAP_ID]
        if itemId == PayUtil.PAY_ITEM_ID.IAP_ITEM_ID5 or itemId == PayUtil.PAY_ITEM_ID.IAP_ITEM_ID6 then
        	lk.LakooManager:getInstance():addUsedItemId(itemId)
            self.updateBagFun(false, nil, true)
        end
        lk.LakooManager:getInstance():onTalkingDataChargeSuccess(tostring(orderId))
    elseif payStatus == Enum.PAY_STATUS.FAIL then
        GamePool.setIsHasLoadingNotice(false)
    end

    lk.LakooManager:getInstance():setPayStatus(Enum.PAY_STATUS.NONE)
end

---卖出东西
function ItemInfoView:doSell()
    local pos = self.item[Enum.ITEM_KEY.POS]
    local result = lk.LakooManager:getInstance():sellItem(pos)
    if result then
        self.updateBagFun(false)
    end
end

---购买商店物品
function ItemInfoView:doBuy()
    if self.item[Enum.IAP_KEY.TYPE] == 99 then
        if self.adsFreeGemTimes ~= 0 and self.adsNextTime <= self.adsFreeGemCD and self.adsNextTime > 0 and self.adsTimes >= self.adsFreeGemTimes then
            local language = lk.LakooManager:getInstance():getCurrentLang()
            local msg = ""
            if language == 2 then
                msg = "The video isn't ready yet!"
            elseif language == 3 then
                msg = "视频还没准备好哦！"
            elseif language == 4 then
                msg = "視頻還沒準備好哦！"
            elseif language == 5 then
                msg = "ビデオはまだ準備していない！"
            end
            lk.LakooManager:getInstance():addPopTip(msg)
        else
            lk.LakooManager:getInstance():showVideoAdsView(Enum.ADMOB_REWARD_VIDEO_LIST.REWARD_STONE)
        end
        return
    end

    if self.item[Enum.IAP_KEY.IAP_ID] == Enum.ITEM_ID.FREE_SOULSTONE_ID then
        lk.LakooManager:getInstance():tapjoyShowOffers()
        return
    end

    --lk.LakooManager:getInstance():getProductIdentifier(self.item[Enum.IAP_KEY.IAP_ID])
    local productId = PayUtil.getProductIdByItemId(self.item[Enum.IAP_KEY.IAP_ID])

    if productId ~= "" and productId ~= nil then
        GamePool.setCurrentBuyIapItemId(self.item[Enum.IAP_KEY.IAP_ID])
        --IAP Pay[从GameServer请求orderId]
        NetworkManager.requestCreateOrder(productId)
        return
    end

    --购买普通物品
    local result = false
    local isAdditionItem = false
    if self.item[Enum.IAP_KEY.TYPE] == Enum.ITEM_TYPE.MEDICINE_DATA and self.item[Enum.IAP_KEY.USETYPE] == Enum.ITEM_USETYPE.INDEX_USETYPE_ADDITION then
        isAdditionItem = true
    end

    if self.bagTag == Enum.BAG_TAG.NPC_SHOP then
        result = lk.LakooManager:getInstance():buyNpcShopItem(self.item[Enum.IAP_KEY.IAP_ID])
    else
        local iapIdx = self.item[Enum.IAP_KEY.IAP_INDEX]
        if iapIdx ~= nil then
            lk.LakooManager:getInstance():buyItem(iapIdx)
            self.updateBagFun(false, self.itemIndex, true)
        end
    end
    if result then
        if isAdditionItem then
            self.updateBagFun(false, nil, true)
        else
            self.updateBagFun(false)
        end
    end
end

---关闭itemInfoView显示
function ItemInfoView:doBack()
    if self.closeCallback ~= nil then
        self.closeCallback()
    end
end

---获取武器
---获取妖魂
function ItemInfoView:doGet()
    self.currentOpType = Enum.ITEM_OP.GET

    local layer = WarningLayer:create()
    GamePool.getCurrentScene():addChild(layer)
    layer:setDelegate(self)
    --layer:setOnActionFunc(self.doGet)
    local msg = lk.LakooManager:getInstance():getString(Enum.TEXT.T_SPEND_STONE_GETING_ITEM)
    msg = string.format(msg,self.item[Enum.PIC_KEY.COST_STONE],self.item[Enum.PIC_KEY.NAME])
    cclog("msg = "..msg)
    cclog("self.item[Enum.PIC_KEY.COST_STONE] = "..self.item[Enum.PIC_KEY.COST_STONE])
    cclog("self.item[Enum.PIC_KEY.NAME] = "..self.item[Enum.PIC_KEY.NAME])
    local okText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_OK)
    local cancelText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_CANCEL)
    layer:setString(msg, okText, cancelText)
end

---来自WarningLayer callBack调用
function ItemInfoView:onActionFunc()
    lk.LakooManager:getInstance():picOp(self.item[Enum.PIC_KEY.ID], self.currentOpType)
end

function ItemInfoView:showMergeConfirm()
    local layer = WarningLayer:create()
    GamePool.getCurrentScene():addChild(layer)
    layer:setDelegate(self)
    local okText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_OK)
    local cancelText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_CANCEL)
    layer:setString("", okText, cancelText)

    local node = cc.Node:create()
    local size = layer:getBgImgSize()
    node:setContentSize(size)
    layer:addChildNode(node)

    local mA_icon = UILib.createClipSprite(self.item[Enum.PIC_KEY.M_A_IMG],self.item[Enum.PIC_KEY.M_A_ICON])
    if mA_icon ~= nil then
        local pos = UILib.offsetWithSuperSize(30,-12,mA_icon:getContentSize().width,mA_icon:getContentSize().height,size.width,size.height,UILib.UI_ANCHOR.TOP_LEFT)
        mA_icon:setPosition(pos)
        node:addChild(mA_icon)
    end

    local mB_icon = UILib.createClipSprite(self.item[Enum.PIC_KEY.M_B_IMG],self.item[Enum.PIC_KEY.M_B_ICON])
    if mB_icon ~= nil then
        pos = UILib.offsetWithSuperSize(0,-12,mB_icon:getContentSize().width,mB_icon:getContentSize().height,size.width,size.height,UILib.UI_ANCHOR.CENTER_TOP)
        mB_icon:setPosition(pos)
        node:addChild(mB_icon)
    end

    local addLab = UILib.textAddStroke("+",UILib.FONT_NAME_NORMAL,UILib.FONT.SMALL,UILib.UI_COLOR3B.WHITE,UILib.UI_COLOR3B.BLACK,1)
    if mA_icon ~= nil and addLab ~= nil then
        pos = cc.p(mA_icon:getPositionX()+(mB_icon:getPositionX()-mA_icon:getPositionX())/2,mA_icon:getPositionY())
        addLab:setPosition(pos)
        node:addChild(addLab)
    end

    local iconImgIndex = self.item[Enum.ITEM_KEY.ICON_IDX]
    local iconInImgIndex = self.item[Enum.ITEM_KEY.ICON_IN_IDX]
    local icon = UILib.createClipSprite(iconImgIndex,iconInImgIndex)
    if icon ~= nil then
        pos = UILib.offsetWithSuperSize(-30,-12,icon:getContentSize().width,icon:getContentSize().height,size.width,size.height,UILib.UI_ANCHOR.TOP_RIGHT)
        icon:setPosition(pos)
        node:addChild(icon)
    end

    local equalLab = UILib.textAddStroke("=",UILib.FONT_NAME_NORMAL,UILib.FONT.SMALL,UILib.UI_COLOR3B.WHITE,UILib.UI_COLOR3B.BLACK,1)
    if mB_icon ~= nil and equalLab ~= nil then
        pos = cc.p(mB_icon:getPositionX()+(icon:getPositionX()-mB_icon:getPositionX())/2,mB_icon:getPositionY())
        equalLab:setPosition(pos)
        node:addChild(equalLab)
    end

    if self.item[Enum.PIC_KEY.M_COST_STONE] > 0 then
        local costLab = UILib.textAddStroke(Text.get(IDX.COST),UILib.FONT_NAME_NORMAL,UILib.FONT.MINI,UILib.UI_COLOR3B.WHITE,UILib.UI_COLOR3B.BLACK,1)
        pos = UILib.offsetWithSuperSize(35,5,costLab:getContentSize().width,costLab:getContentSize().height,size.width,size.height,UILib.UI_ANCHOR.CENTER_LEFT)
        costLab:setPosition(pos)
        node:addChild(costLab)

        local costIcon = UILib.createClipSprite(UILib.IMG_IDX.IMG91,UILib.IN_IMG_IDX.SOULSTONE)
        local px = costLab:getPositionX()+costLab:getContentSize().width/2+costIcon:getContentSize().width/2
        local py = costLab:getPositionY()
        pos = cc.p(px,py)
        costIcon:setPosition(pos)
        node:addChild(costIcon)

        local stoneStr = lk.LakooManager:getInstance():getString(Enum.TEXT.T_SOUL_STONE)
        stoneStr = string.format("%s X %d",stoneStr, self.item[Enum.PIC_KEY.M_COST_STONE])
        local stoneLab = UILib.textAddStroke(stoneStr,UILib.FONT_NAME_NORMAL,UILib.FONT.MINI,UILib.UI_COLOR3B.WHITE,UILib.UI_COLOR3B.BLACK,1)
        px = costIcon:getPositionX()+costIcon:getContentSize().width/2+stoneLab:getContentSize().width/2
        pos = cc.p(px,py)
        stoneLab:setPosition(pos)
        node:addChild(stoneLab)
    end
end

---锻造武器
function ItemInfoView:doMergeWeapon()
    self.currentOpType = Enum.ITEM_OP.SMELT
    self:showMergeConfirm()
end

---炼魂
function ItemInfoView:doMergeSoul()
    self.currentOpType = Enum.ITEM_OP.SMELTSOUL
    self:showMergeConfirm()
end

---卸下妖魂
function ItemInfoView:doUnEquipSoul()
    local isEquipped = self.item[Enum.ITEM_KEY.IS_EQUIPPED]
    if isEquipped == false then
        return
    end
    local fromPos = self.item[Enum.ITEM_KEY.EQ_SOUL_POS]
    local result = lk.LakooManager:getInstance():itemOp(self.bagTag, isEquipped, fromPos, -1, Enum.ITEM_OP.UNEQUIP_SOUL)
    if result then
        self.updateBagFun(true)
    end
end

---打开装备妖魂界面
function ItemInfoView:doEquipSoul()
    cclog("ItemInfoView:doEquipSoul()")
    local wPos = self:getParent():convertToWorldSpace(cc.p(0,0))
    local pos = cc.p(-wPos.x, -wPos.y)
    --local equipLayer = EquipSoulLayer:create(self.item, self.doEquipSoulCallBack)
    local equipLayer = EquipSoulLayer.create(self, self.item)--需要传self,否则回调doEquipSoulCallBack时使用self.XX时无效
    equipLayer:setPosition(pos)
    self:getParent():addChild(equipLayer)
end

function ItemInfoView:doEquipSoulCallBack(_toPos)
    local isEquipped = self.item[Enum.ITEM_KEY.IS_EQUIPPED]
    local fromPos = self.item[Enum.ITEM_KEY.POS]
    if isEquipped then
        fromPos = self.item[Enum.ITEM_KEY.EQ_SOUL_POS]
    end
    local result = lk.LakooManager:getInstance():itemOp(self.bagTag, isEquipped, fromPos, _toPos, Enum.ITEM_OP.EQUIP_SOUL)
    if result then
        self.updateBagFun(true)
    end
end

---装备
function ItemInfoView:doEquip()
    if self.item == nil then
        return
    end
    if self.item[Enum.ITEM_KEY.IS_EQUIPPED] then
        return
    end
    cclog("ItemInfoView:doEquip()")
    local result = lk.LakooManager:getInstance():itemOp(self.bagTag, false, self.item[Enum.ITEM_KEY.POS], -1, Enum.ITEM_OP.EQUIP)
    if result then
        self.updateBagFun(true)
    else
        self.updateBagFun(false,nil,true)
    end
end

---强化
function ItemInfoView:doStrengthen()
    local isEquipped = self.item[Enum.ITEM_KEY.IS_EQUIPPED]
    local pos = self.item[Enum.ITEM_KEY.POS]
    if isEquipped then
        pos = GamePool.getEquipPosByType(self.item[Enum.ITEM_KEY.TYPE])
    end
    cclog("ItemInfoView:doStrengthen()")
    local result = lk.LakooManager:getInstance():itemOp(self.bagTag, isEquipped, pos, -1, Enum.ITEM_OP.STRENGTHEN)
    if result then
        if ItemInfoView.focusAni ~=nil then
            ItemInfoView.focusAni:setVisible(false)
            ItemInfoView.focusAni:removeFromParent(true)
            ItemInfoView.focusAni =nil
        end
        self.updateBagFun(true, self.itemIndex)

    end
end

---使用
function ItemInfoView:doUse()
    if self.item == nil then
        return
    end
    cclog("ItemInfoView:doUse()")
    local result = lk.LakooManager:getInstance():itemOp(self.bagTag, false, self.item[Enum.ITEM_KEY.POS], -1, Enum.ITEM_OP.USE)
    if result then
        self.updateBagFun(false)
    end
end

function ItemInfoView:updateShowButton()
    cclog("updateShowButton!!!!!")
    ItemInfoView.tapTable = {}
    self.equipButton:setVisible(false)
    self.equipSoulButton:setVisible(false)
    self.strengButton:setVisible(false)
    self.useButton:setVisible(false)
    self.unEquipSoulButton:setVisible(false)
    self.smeltSoulButton:setVisible(false)
    self.getSoulButton:setVisible(false)
    self.smeltWeaponButton:setVisible(false)
    self.getWeaponButton:setVisible(false)
    self.buyButton:setVisible(false)
    self.restoreButton:setVisible(false)
    self.backButton:setVisible(false)
    self.sellButton:setVisible(false)

    if self.bagTag == Enum.BAG_TAG.MERGE then
        self.smeltSoulButton:setVisible(true)
        self.getSoulButton:setVisible(true)

        table.insert(ItemInfoView.tapTable,self.smeltSoulButton)
        table.insert(ItemInfoView.tapTable,self.getSoulButton)
        return
    end
    if self.bagTag == Enum.BAG_TAG.TRANSMUTEITEM then
        self.smeltWeaponButton:setVisible(true)
        self.getWeaponButton:setVisible(true)

        table.insert(ItemInfoView.tapTable,self.smeltWeaponButton)
        table.insert(ItemInfoView.tapTable,self.getWeaponButton)
        return
    end

    if self.bagTag == Enum.BAG_TAG.SHOP
        or self.bagTag == Enum.BAG_TAG.NPC_SHOP then
        
        local itemId = self.item[Enum.IAP_KEY.IAP_ID]
        local itemType = self.item[Enum.IAP_KEY.TYPE]
        local osType = lk.LakooManager:getInstance():getOSType()
        if self.bagTag == Enum.BAG_TAG.SHOP 
        and itemId == PayUtil.PAY_ITEM_ID.IAP_ITEM_ID8 
        and osType == Enum.OS_TYPE.IOS then
            self.restoreButton:setVisible(true)
            table.insert(ItemInfoView.tapTable,self.restoreButton)
        end
        
        if itemType == 99 then
            self.buyButton:loadTextures("p_202.png", "p_202.png", "p_202.png")
        else
            local imgName = string.format("%s.%s",Enum.TEXT_IMG.BUY,Text.get(IDX.PNG_TYPE))
            self.buyButton:loadTextures(imgName, imgName, imgName)
        end

        if itemType == 99 and self.adsFreeGemTimes ~= 0 and self.adsTimes >= self.adsFreeGemTimes and self.adsNextTime < self.adsFreeGemCD and self.adsNextTime > 0 then
            self.buyButton:loadTextures("p_203.png", "p_203.png", "p_203.png")
            self.nextTimeLabel:setVisible(true)
        else
            self.nextTimeLabel:setVisible(false)
        end
        self.buyButton:setVisible(true)
        self.backButton:setVisible(true)

        table.insert(ItemInfoView.tapTable,self.buyButton)
        table.insert(ItemInfoView.tapTable,self.backButton)
        return
    end

    if self.bagTag == Enum.BAG_TAG.NPC_SHOP_SELL then
        self.sellButton:setVisible(true)
        table.insert(ItemInfoView.tapTable,self.sellButton)
        return
    end

    local type = self.item[Enum.ITEM_KEY.TYPE]
    --cclog("type=%d,itemId=%d",type,self.item[Enum.ITEM_KEY.ID])
    if type == Enum.ITEM_TYPE.ADORN_DATA
        or type == Enum.ITEM_TYPE.WEAPON
        or type == Enum.ITEM_TYPE.ARMOUR then
        self.equipButton:setVisible(true)

        table.insert(ItemInfoView.tapTable,self.equipButton)
    end

    if self.enableShowStrengBut then
        self.strengButton:setVisible(true)
        table.insert(ItemInfoView.tapTable,self.strengButton)
    end

    if type == Enum.ITEM_TYPE.MEDICINE_DATA then
        self.useButton:setVisible(true)
        table.insert(ItemInfoView.tapTable,self.useButton)
    end

    if type == Enum.ITEM_TYPE.MONSTER_SOUL then
        self.equipSoulButton:setVisible(true)
        table.insert(ItemInfoView.tapTable,self.equipSoulButton)
        if self.item[Enum.ITEM_KEY.IS_EQUIPPED] then
            self.unEquipSoulButton:setVisible(true)
            table.insert(ItemInfoView.tapTable,self.unEquipSoulButton)
        end
    end
end

function ItemInfoView:updateInfo(_item, _itemIndex)
    if _item == nil then
        cclog("ItemInfoView:updateInfo return")
        return
    end
    self.item = _item
    self.itemIndex = _itemIndex

    self:updateInfoView()
end

--TODO:重复代码较多，待优化
function ItemInfoView:updateInfoView()
    self.scrollView:getInnerContainer():removeAllChildren(true)
    self.scrollView:jumpToTop()

    if self.bagTag == Enum.BAG_TAG.SHOP
        or self.bagTag == Enum.BAG_TAG.NPC_SHOP then
        self:updateIapItemInfo()
        return
    end

    if self.item[Enum.ITEM_KEY.TYPE] == Enum.ITEM_TYPE.MONSTER_SOUL then
        self:updateSoulInfo()
        return
    end

    if self.bagTag == Enum.BAG_TAG.MERGE
        or self.bagTag == Enum.BAG_TAG.TRANSMUTEITEM then
        self:updatePicItemInfo()
        return
    end

    local iconImgIndex = self.item[Enum.ITEM_KEY.ICON_IDX]
    local iconInImgIndex = self.item[Enum.ITEM_KEY.ICON_IN_IDX]
    local icon = UILib.createClipSprite(iconImgIndex,iconInImgIndex)
    self.scrollView:getInnerContainer():addChild(icon)

    local itemName = self.item[Enum.ITEM_KEY.NAME]
    local nameLabel = cc.Label:create()
    nameLabel:setString(itemName)
    nameLabel:setSystemFontSize(UILib.FONT.SMALL)
    self.scrollView:getInnerContainer():addChild(nameLabel)

    local height = icon:getContentSize().height

    local detail = self.item[Enum.ITEM_KEY.DETAIL]
    local detailLabel = nil
    if detail ~= "" then
        --detailLabel = cc.Label:create()
        --detailLabel:setString(detail)
        --detailLabel:setSystemFontSize(UILib.FONT.SMALL)
        detailLabel = CCLabelTTF:create(detail, UILib.FONT_NAME_NORMAL, UILib.FONT.SMALL, cc.size(280, 0), kCCVerticalTextAlignmentTop, kCCTextAlignmentLeft)
        self.scrollView:getInnerContainer():addChild(detailLabel)

        height = height + detailLabel:getContentSize().height
    end

    local level = self.item[Enum.ITEM_KEY.LEVEL]
    local lvImgView = nil
    local lvLabel = nil
    if level > 0 then
        local imgName = string.format("%s.%s",Enum.TEXT_IMG.LEVEL,Text.get(IDX.PNG_TYPE))
        lvImgView = ccui.ImageView:create(imgName)
        self.scrollView:getInnerContainer():addChild(lvImgView)

        height = height + lvImgView:getContentSize().height

        lvLabel = cc.LabelAtlas:_create(level, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
        self.scrollView:getInnerContainer():addChild(lvLabel)
    end

    local prop = self.item[Enum.ITEM_KEY.PROPERTY]
    local propTextImgView = nil
    local propImgView = nil
    if prop >= 0 and prop <= #Enum.PROP_IMG_NAME then
        local imgName = string.format("%s.%s",Enum.TEXT_IMG.PROPERTY,Text.get(IDX.PNG_TYPE))
        propTextImgView = ccui.ImageView:create(imgName)
        self.scrollView:getInnerContainer():addChild(propTextImgView)

        if lvImgView == nil then
            height = height + propTextImgView:getContentSize().height
        end

        local imgName = string.format("%s.%s",Enum.PROP_IMG_NAME[prop],Text.get(IDX.PNG_TYPE))
        propImgView = ccui.ImageView:create(imgName)
        self.scrollView:getInnerContainer():addChild(propImgView)
    end

    --强化信息
    self.enableShowStrengBut = false
    local strInfoLabel = nil
    local strengthKind = self.item[Enum.ITEM_KEY.STR_KIND]
    local strengthTimes = self.item[Enum.ITEM_KEY.STR_TIMES]
    if strengthKind > 0 and self.bagTag ~= Enum.BAG_TAG.NPC_SHOP_SELL then
        local strengthValue = self.item[Enum.ITEM_KEY.STR_VAL]
        local tIdx = Enum.STRENGTH_PROP[strengthKind]
        if tIdx > 0 then
            local strengthStr = lk.LakooManager:getInstance():getString(tIdx)
            strengthStr = string.format("%s + %d",strengthStr,strengthValue)
            if strengthTimes <= 0 then
                self.enableShowStrengBut = true

                local pStr = lk.LakooManager:getInstance():getString(Enum.TEXT.T_STRENGTH_GAIN)
                strengthStr = string.format(pStr,strengthStr)
            end
            if strengthStr ~= "" then
                strInfoLabel = cc.Label:create()
                strInfoLabel:setString(strengthStr)
                strInfoLabel:setSystemFontSize(UILib.FONT.SMALL)
                self.scrollView:getInnerContainer():addChild(strInfoLabel)

                height = height + strInfoLabel:getContentSize().height
            end
        end
    end

    --强化所需费用信息
    local costIcon = nil
    local stoneLabel = nil
    local costStoneLabel = nil
    local costStone = self.item[Enum.ITEM_KEY.COST_STONE]
    if costStone > 0 and strengthTimes <= 0 and self.bagTag ~= Enum.BAG_TAG.NPC_SHOP_SELL then
        costIcon = UILib.createClipSprite(UILib.IMG_IDX.IMG91,UILib.IN_IMG_IDX.SOULSTONE)
        self.scrollView:getInnerContainer():addChild(costIcon)

        height = height + costIcon:getContentSize().height

        local stone = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.SOUL_STONE)
        local stoneColor = Enum.DIGITAL.RED
        if stone >= costStone then
            stoneColor = Enum.DIGITAL.BEIGE
        end
        stoneLabel = cc.LabelAtlas:_create(stone, stoneColor, 198/21, 11, string.byte('0'))
        costStoneLabel = cc.LabelAtlas:_create(">"..costStone, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
        self.scrollView:getInnerContainer():addChild(stoneLabel)
        self.scrollView:getInnerContainer():addChild(costStoneLabel)
    end

    --NPC shop卖出物品所能获得的游戏币
    local moneyIcon = nil
    local sellMoneyLabel = nil
    local sellMoney = self.item[Enum.ITEM_KEY.SELL_PRICE]
    if sellMoney > 0 and self.bagTag == Enum.BAG_TAG.NPC_SHOP_SELL then
        moneyIcon = UILib.createClipSprite(UILib.IMG_IDX.IMG91,UILib.IN_IMG_IDX.MONEY)
        self.scrollView:getInnerContainer():addChild(moneyIcon)

        height = height + moneyIcon:getContentSize().height

        sellMoneyLabel = cc.LabelAtlas:_create(sellMoney, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
        self.scrollView:getInnerContainer():addChild(sellMoneyLabel)
    end

    --#################################################
    if height < self.scrollView:getContentSize().height then
        height = self.scrollView:getContentSize().height
    end
    self.scrollView:setInnerContainerSize(cc.size(self.scrollView:getInnerContainerSize().width, height))
    --#################################################
    local drawX = icon:getContentSize().width/2
    local drawY = self.scrollView:getInnerContainer():getContentSize().height-icon:getContentSize().height/2
    local pos = cc.p(drawX, drawY)
    icon:setPosition(pos)
    pos = cc.p(icon:getPositionX()+icon:getContentSize().width/2+nameLabel:getContentSize().width/2,icon:getPositionY())
    nameLabel:setPosition(pos)

    drawX = 0
    drawY = drawY - icon:getContentSize().height/2

    if detailLabel ~= nil then
        drawX = detailLabel:getContentSize().width/2
        drawY = drawY - detailLabel:getContentSize().height/2
        pos = cc.p(drawX, drawY)
        detailLabel:setPosition(pos)

        drawX = 0
        drawY = drawY - detailLabel:getContentSize().height/2
    end

    if lvImgView ~= nil then
        drawX = lvImgView:getContentSize().width/2
        drawY = drawY - lvImgView:getContentSize().height/2
        pos = cc.p(drawX, drawY)
        lvImgView:setPosition(pos)

        drawX = drawX + lvImgView:getContentSize().width/2
        drawY = drawY - lvImgView:getContentSize().height/2

        if lvLabel ~= nil then
            drawX = drawX + lvLabel:getContentSize().width/2
            pos = cc.p(drawX,pos.y-lvLabel:getContentSize().height/2)
            lvLabel:setPosition(pos)

            drawX = drawX + lvLabel:getContentSize().width/2
        end
    end

    if propTextImgView ~= nil then
        drawX = drawX + 50
        local pY = drawY
        if lvImgView ~= nil then
            pY = lvImgView:getPositionY()
        end
        pos = cc.p(drawX, pY)
        propTextImgView:setPosition(pos)

        if lvImgView == nil then
            drawY = drawY - propTextImgView:getContentSize().height/2
        end
        drawX = drawX + propTextImgView:getContentSize().width/2

        if propImgView ~= nil then
            pos = cc.p(drawX+propImgView:getContentSize().width/2,pY)
            propImgView:setPosition(pos)
        end
    end
    drawX = 0

    if strInfoLabel ~= nil then
        drawX = strInfoLabel:getContentSize().width/2
        drawY = drawY - strInfoLabel:getContentSize().height/2
        pos = cc.p(drawX, drawY)
        strInfoLabel:setPosition(pos)

        drawY = drawY - strInfoLabel:getContentSize().height/2
    end

    if costIcon ~= nil then
        drawX = costIcon:getContentSize().width/2
        drawY = drawY - costIcon:getContentSize().height/2
        pos = cc.p(drawX, drawY)
        costIcon:setPosition(pos)

        drawX = drawX + costIcon:getContentSize().width/2
        drawY = drawY - costIcon:getContentSize().height/2

        if stoneLabel ~= nil and costStoneLabel ~= nil then
            pos = cc.p(drawX, drawY+costIcon:getContentSize().height/2-stoneLabel:getContentSize().height/2)
            stoneLabel:setPosition(pos)

            drawX = drawX + stoneLabel:getContentSize().width
            pos = cc.p(drawX, pos.y)
            costStoneLabel:setPosition(pos)
        end
    end

    if moneyIcon ~= nil then
        drawX = moneyIcon:getContentSize().width/2
        drawY = drawY - moneyIcon:getContentSize().height/2
        pos = cc.p(drawX, drawY)
        moneyIcon:setPosition(pos)

        drawX = drawX + moneyIcon:getContentSize().width/2
        drawY = drawY - moneyIcon:getContentSize().height/2

        if sellMoneyLabel ~= nil then
            pos = cc.p(drawX, drawY+moneyIcon:getContentSize().height/2-sellMoneyLabel:getContentSize().height/2)
            sellMoneyLabel:setPosition(pos)
        end
    end

    self:updateShowButton()
end

function ItemInfoView:updateSoulInfo()
    if self.item == nil then
        return
    end

    local iconImgIndex = self.item[Enum.ITEM_KEY.ICON_IDX]
    local iconInImgIndex = self.item[Enum.ITEM_KEY.ICON_IN_IDX]
    local icon = UILib.createClipSprite(iconImgIndex,iconInImgIndex)
    self.scrollView:getInnerContainer():addChild(icon)

    local itemName = self.item[Enum.ITEM_KEY.NAME]
    local nameLabel = cc.Label:create()
    nameLabel:setString(itemName)
    nameLabel:setSystemFontSize(UILib.FONT.SMALL)
    self.scrollView:getInnerContainer():addChild(nameLabel)

    local height = icon:getContentSize().height

    --magicCostMP信息显示
    local magicCostMp = self.item[Enum.ITEM_KEY.MAGIC_COST_MP]
    local imgName = string.format("%s.%s",Enum.TEXT_IMG.MP,Text.get(IDX.PNG_TYPE))
    local mCostMpTextImgView = ccui.ImageView:create(imgName)
    self.scrollView:getInnerContainer():addChild(mCostMpTextImgView)

    local mpLab = cc.LabelAtlas:_create(magicCostMp, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
    self.scrollView:getInnerContainer():addChild(mpLab)

    --"召"信息
    imgName = string.format("%s.%s",Enum.TEXT_IMG.SUMMON,Text.get(IDX.PNG_TYPE))
    local summonImg = ccui.ImageView:create(imgName)
    self.scrollView:getInnerContainer():addChild(summonImg)
    local prop = self.item[Enum.ITEM_KEY.PROPERTY]
    local propTextImgView = nil
    local propImgView = nil
    if prop >= 0 and prop <= #Enum.PROP_IMG_NAME then
        local imgName = string.format("%s.%s",Enum.TEXT_IMG.PROPERTY,Text.get(IDX.PNG_TYPE))
        propTextImgView = ccui.ImageView:create(imgName)
        self.scrollView:getInnerContainer():addChild(propTextImgView)

        local imgName = string.format("%s.%s",Enum.PROP_IMG_NAME[prop],Text.get(IDX.PNG_TYPE))
        propImgView = ccui.ImageView:create(imgName)
        self.scrollView:getInnerContainer():addChild(propImgView)
    end

    imgName = Enum.IMG_NAME_PNGHK.ATK
    local atkImg = ccui.ImageView:create(imgName)
    self.scrollView:getInnerContainer():addChild(atkImg)
    local atkLab = cc.LabelAtlas:_create(self.item[Enum.ITEM_KEY.ATK], Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
    self.scrollView:getInnerContainer():addChild(atkLab)

    imgName = Enum.IMG_NAME_PNGHK.DEF
    local defImg = ccui.ImageView:create(imgName)
    self.scrollView:getInnerContainer():addChild(defImg)
    local defLab = cc.LabelAtlas:_create(self.item[Enum.ITEM_KEY.DEF], Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
    self.scrollView:getInnerContainer():addChild(defLab)

    imgName = Enum.IMG_NAME_PNGHK.AGI
    local agiImg = ccui.ImageView:create(imgName)
    self.scrollView:getInnerContainer():addChild(agiImg)
    local agiLab = cc.LabelAtlas:_create(self.item[Enum.ITEM_KEY.AGI], Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
    self.scrollView:getInnerContainer():addChild(agiLab)

    height = height + summonImg:getContentSize().height + 5

    --"附"信息
    imgName = string.format("%s.%s",Enum.TEXT_IMG.ASSISTANT,Text.get(IDX.PNG_TYPE))
    local assistantImg = ccui.ImageView:create(imgName)
    self.scrollView:getInnerContainer():addChild(assistantImg)

    local assistantDetail = self.item[Enum.ITEM_KEY.ASSISTANT_DETAIL]
    local assistantDetailLab = cc.Label:create()
    assistantDetailLab:setString(assistantDetail)
    assistantDetailLab:setSystemFontSize(UILib.FONT.MINI)
    self.scrollView:getInnerContainer():addChild(assistantDetailLab)

    height = height + assistantImg:getContentSize().height + 5

    --"术"信息
    imgName = string.format("%s.%s",Enum.TEXT_IMG.MAGIC,Text.get(IDX.PNG_TYPE))
    local magicImg = ccui.ImageView:create(imgName)
    self.scrollView:getInnerContainer():addChild(magicImg)

    local magicDetail = self.item[Enum.ITEM_KEY.MAGIC_DETAIL]
    local magicDetailLab = cc.Label:create()
    magicDetailLab:setString(magicDetail)
    magicDetailLab:setSystemFontSize(UILib.FONT.MINI)
    self.scrollView:getInnerContainer():addChild(magicDetailLab)

    height = height + magicImg:getContentSize().height

    --#################################################
    if height < self.scrollView:getContentSize().height then
        height = self.scrollView:getContentSize().height
    end
    self.scrollView:setInnerContainerSize(cc.size(self.scrollView:getInnerContainerSize().width, height))
    --#################################################
    local drawX = icon:getContentSize().width/2
    local drawY = self.scrollView:getInnerContainer():getContentSize().height-icon:getContentSize().height/2
    local pos = cc.p(drawX, drawY)
    icon:setPosition(pos)
    pos = cc.p(icon:getPositionX()+icon:getContentSize().width/2+nameLabel:getContentSize().width/2,icon:getPositionY())
    nameLabel:setPosition(pos)

    drawX = nameLabel:getPositionX()+nameLabel:getContentSize().width/2 + mCostMpTextImgView:getContentSize().width/2
    pos = cc.p(drawX+20,drawY)
    mCostMpTextImgView:setPosition(pos)
    drawX = mCostMpTextImgView:getPositionX() + mCostMpTextImgView:getContentSize().width/2+10
    pos = cc.p(drawX, drawY - mpLab:getContentSize().height/2)
    mpLab:setPosition(pos)

    --summon
    drawX = summonImg:getContentSize().width/2
    drawY = drawY - icon:getContentSize().height/2 - summonImg:getContentSize().height/2
    pos = cc.p(drawX, drawY)
    summonImg:setPosition(pos)

    if propTextImgView ~= nil and propImgView ~= nil then
        drawX = drawX + summonImg:getContentSize().width/2 + propTextImgView:getContentSize().width/2 + 10
        pos = cc.p(drawX, drawY)
        propTextImgView:setPosition(pos)

        drawX = drawX + propTextImgView:getContentSize().width/2 + propImgView:getContentSize().width/2
        pos = cc.p(drawX, drawY)
        propImgView:setPosition(pos)

        drawX = drawX + propImgView:getContentSize().width/2
    end

    drawX = drawX + atkImg:getContentSize().width/2 + 20
    pos = cc.p(drawX, drawY)
    atkImg:setPosition(pos)

    drawX = drawX + atkImg:getContentSize().width/2
    pos = cc.p(drawX, drawY-atkLab:getContentSize().height/2)
    atkLab:setPosition(pos)

    drawX = drawX + atkLab:getContentSize().width + 20 + defImg:getContentSize().width/2
    pos = cc.p(drawX, drawY)
    defImg:setPosition(pos)

    drawX = drawX + atkImg:getContentSize().width/2
    pos = cc.p(drawX, drawY-defLab:getContentSize().height/2)
    defLab:setPosition(pos)

    drawX = drawX + defLab:getContentSize().width + 20 + agiImg:getContentSize().width/2
    pos = cc.p(drawX, drawY)
    agiImg:setPosition(pos)

    drawX = drawX + agiImg:getContentSize().width/2
    pos = cc.p(drawX, drawY-agiLab:getContentSize().height/2)
    agiLab:setPosition(pos)

    --assistant
    drawX = assistantImg:getContentSize().width/2
    drawY = drawY - summonImg:getContentSize().height/2 - 10 - assistantImg:getContentSize().height/2
    pos = cc.p(drawX, drawY)
    assistantImg:setPosition(pos)

    drawX = drawX + assistantImg:getContentSize().width + assistantDetailLab:getContentSize().width/2
    pos = cc.p(drawX, drawY)
    assistantDetailLab:setPosition(pos)

    --magic
    drawX = magicImg:getContentSize().width/2
    drawY = drawY - assistantImg:getContentSize().height/2 - 10 - magicImg:getContentSize().height/2
    pos = cc.p(drawX, drawY)
    magicImg:setPosition(pos)

    drawX = drawX + magicImg:getContentSize().width + magicDetailLab:getContentSize().width/2
    pos = cc.p(drawX, drawY)
    magicDetailLab:setPosition(pos)

    self:updateShowButton()
end

function ItemInfoView:updatePicItemInfo()
    if self.item == nil then
        return
    end

    local iconImgIndex = self.item[Enum.PIC_KEY.ICON_IDX]
    local iconInImgIndex = self.item[Enum.PIC_KEY.ICON_IN_IDX]
    local icon = UILib.createClipSprite(iconImgIndex,iconInImgIndex)
    self.scrollView:getInnerContainer():addChild(icon)

    local itemName = self.item[Enum.PIC_KEY.NAME]
    local nameLabel = cc.Label:create()
    nameLabel:setString(itemName)
    nameLabel:setSystemFontSize(UILib.FONT.SMALL)
    self.scrollView:getInnerContainer():addChild(nameLabel)

    local height = icon:getContentSize().height

    --稀有度
    local rare = self.item[Enum.PIC_KEY.RARE]
    local imgName = string.format("%s.%s",Enum.TEXT_IMG.RARE,Text.get(IDX.PNG_TYPE))
    local rareTextImgView = ccui.ImageView:create(imgName)
    self.scrollView:getInnerContainer():addChild(rareTextImgView)
    local rareLabel = cc.LabelAtlas:_create(rare, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
    self.scrollView:getInnerContainer():addChild(rareLabel)
    --属性
    local prop = self.item[Enum.PIC_KEY.PROPERTY]
    local propTextImgView = nil
    local propImgView = nil
    if prop >= 0 and prop <= #Enum.PROP_IMG_NAME then
        local imgName = string.format("%s.%s",Enum.TEXT_IMG.PROPERTY,Text.get(IDX.PNG_TYPE))
        propTextImgView = ccui.ImageView:create(imgName)
        self.scrollView:getInnerContainer():addChild(propTextImgView)

        local imgName = string.format("%s.%s",Enum.PROP_IMG_NAME[prop],Text.get(IDX.PNG_TYPE))
        propImgView = ccui.ImageView:create(imgName)
        self.scrollView:getInnerContainer():addChild(propImgView)
    end

    height = height + rareTextImgView:getContentSize().height

    --材料一
    local materialA_icon = UILib.createClipSprite(self.item[Enum.PIC_KEY.M_A_IMG],self.item[Enum.PIC_KEY.M_A_ICON])
    local mANameLabel = nil
    if materialA_icon ~= nil then
        self.scrollView:getInnerContainer():addChild(materialA_icon)
        height = height + materialA_icon:getContentSize().height

        local mAName = self.item[Enum.PIC_KEY.M_A_NAME]
        mANameLabel = cc.Label:create()
        mANameLabel:setString(mAName)
        mANameLabel:setSystemFontSize(UILib.FONT.MINI)
        self.scrollView:getInnerContainer():addChild(mANameLabel)
    end

    --材料二
    local materialB_icon = UILib.createClipSprite(self.item[Enum.PIC_KEY.M_B_IMG],self.item[Enum.PIC_KEY.M_B_ICON])
    local mBNameLabel = nil
    if materialB_icon ~= nil then
        self.scrollView:getInnerContainer():addChild(materialB_icon)
        height = height + materialB_icon:getContentSize().height

        local mBName = self.item[Enum.PIC_KEY.M_B_NAME]
        mBNameLabel = cc.Label:create()
        mBNameLabel:setString(mBName)
        mBNameLabel:setSystemFontSize(UILib.FONT.MINI)
        self.scrollView:getInnerContainer():addChild(mBNameLabel)
    end

    --炼魂，锻造所需费用信息
    local costIcon = nil
    local stoneLabel = nil
    local costStoneLabel = nil
    local costStone = self.item[Enum.PIC_KEY.M_COST_STONE]
    if costStone >= 0 then
        costIcon = UILib.createClipSprite(UILib.IMG_IDX.IMG91,UILib.IN_IMG_IDX.SOULSTONE)
        self.scrollView:getInnerContainer():addChild(costIcon)

        height = height + costIcon:getContentSize().height

        local stone = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.SOUL_STONE)
        local stoneColor = Enum.DIGITAL.RED
        if stone >= costStone then
            stoneColor = Enum.DIGITAL.BEIGE
        end
        stoneLabel = cc.LabelAtlas:_create(stone, stoneColor, 198/21, 11, string.byte('0'))
        costStoneLabel = cc.LabelAtlas:_create(">"..costStone, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
        self.scrollView:getInnerContainer():addChild(stoneLabel)
        self.scrollView:getInnerContainer():addChild(costStoneLabel)
    end

    --#################################################
    if height < self.scrollView:getContentSize().height then
        height = self.scrollView:getContentSize().height
    end
    self.scrollView:setInnerContainerSize(cc.size(self.scrollView:getInnerContainerSize().width, height))
    --#################################################
    local drawX = icon:getContentSize().width/2
    local drawY = self.scrollView:getInnerContainer():getContentSize().height-icon:getContentSize().height/2
    local pos = cc.p(drawX, drawY)
    icon:setPosition(pos)
    pos = cc.p(icon:getPositionX()+icon:getContentSize().width/2+nameLabel:getContentSize().width/2,icon:getPositionY())
    nameLabel:setPosition(pos)

    drawY = drawY - icon:getContentSize().height/2

    --稀有度
    drawX = rareTextImgView:getContentSize().width/2
    drawY = drawY - rareTextImgView:getContentSize().height/2
    pos = cc.p(drawX, drawY)
    rareTextImgView:setPosition(pos)

    drawX = drawX + rareTextImgView:getContentSize().width/2 + 5
    pos = cc.p(drawX, drawY-rareLabel:getContentSize().height/2)
    rareLabel:setPosition(pos)

    --属性
    if propTextImgView ~= nil and propImgView ~= nil then
        drawX = drawX + rareLabel:getContentSize().width + 20 + propTextImgView:getContentSize().width/2
        pos = cc.p(drawX, drawY)
        propTextImgView:setPosition(pos)

        drawX = drawX + propTextImgView:getContentSize().width/2 + propImgView:getContentSize().width/2
        pos = cc.p(drawX, drawY)
        propImgView:setPosition(pos)
    end

    drawY = drawY - rareTextImgView:getContentSize().height/2

    --材料一
    if materialA_icon ~= nil then
        drawX = materialA_icon:getContentSize().width/2
        drawY = drawY - materialA_icon:getContentSize().height/2
        pos = cc.p(drawX, drawY)
        materialA_icon:setPosition(pos)

        drawX = drawX + materialA_icon:getContentSize().width/2

        if mANameLabel ~= nil then
            pos = cc.p(drawX + mANameLabel:getContentSize().width/2, drawY)
            mANameLabel:setPosition(pos)
        end

        drawY = drawY - materialA_icon:getContentSize().height/2
    end
    --材料二
    if materialB_icon ~= nil then
        drawX = materialB_icon:getContentSize().width/2
        drawY = drawY - materialB_icon:getContentSize().height/2
        pos = cc.p(drawX, drawY)
        materialB_icon:setPosition(pos)

        drawX = drawX + materialB_icon:getContentSize().width/2

        if mBNameLabel ~= nil then
            pos = cc.p(drawX+mBNameLabel:getContentSize().width/2, drawY)
            mBNameLabel:setPosition(pos)
        end

        drawY = drawY - materialB_icon:getContentSize().height/2
    end

    --炼魂所需费用信息
    if costIcon ~= nil then
        drawX = costIcon:getContentSize().width/2
        drawY = drawY - costIcon:getContentSize().height/2
        pos = cc.p(drawX, drawY)
        costIcon:setPosition(pos)

        drawX = drawX + costIcon:getContentSize().width/2
        drawY = drawY - costIcon:getContentSize().height/2

        if stoneLabel ~= nil and costStoneLabel ~= nil then
            pos = cc.p(drawX, drawY+costIcon:getContentSize().height/2-stoneLabel:getContentSize().height/2)
            stoneLabel:setPosition(pos)

            drawX = drawX + stoneLabel:getContentSize().width
            pos = cc.p(drawX, pos.y)
            costStoneLabel:setPosition(pos)
        end
    end

    self:updateShowButton()
end

function ItemInfoView:updateIapItemInfo()
    if self.item == nil then
        return
    end

    local num = 0
    local icon = nil
    local itemType = self.item[Enum.IAP_KEY.TYPE]
    local iconImgIndex = self.item[Enum.IAP_KEY.IMG]
    local iconInImgIndex = self.item[Enum.IAP_KEY.ICON]
    if itemType == 99 then
        num = 10
        icon = cc.Sprite:create("p_201.png")
    else
        num = self.item[Enum.IAP_KEY.NUM]
        icon = UILib.createClipSprite(iconImgIndex,iconInImgIndex)
    end
    self.scrollView:getInnerContainer():addChild(icon)

    local itemName = self.item[Enum.IAP_KEY.NAME]
    local numAndName = string.format(" X %d %s",num,itemName)
    --Tapjoy物品不用显示数量
    if self.item[Enum.IAP_KEY.IAP_ID] == Enum.ITEM_ID.FREE_SOULSTONE_ID then
        numAndName = itemName
    end
    local nameLabel = cc.Label:create()
    nameLabel:setString(numAndName)
    nameLabel:setSystemFontSize(UILib.FONT.SMALL)
    self.scrollView:getInnerContainer():addChild(nameLabel)

    local height = icon:getContentSize().height

    local detail = self.item[Enum.IAP_KEY.DETAIL]
    local detailLabel = nil
    if detail ~= "" then
        --detailLabel = cc.Label:create()
        --detailLabel:setString(detail)
        --detailLabel:setSystemFontSize(UILib.FONT.SMALL)
        detailLabel = CCLabelTTF:create(detail, UILib.FONT_NAME_NORMAL, UILib.FONT.SMALL, cc.size(280, 0), kCCVerticalTextAlignmentTop, kCCTextAlignmentLeft)
        self.scrollView:getInnerContainer():addChild(detailLabel)

        height = height + detailLabel:getContentSize().height
    end

    local isMonsterItem = self.item[Enum.IAP_KEY.IS_M]

    local summonImg = nil
    local propTextImgView = nil
    local propImgView = nil

    local atkImg = nil
    local atkLab = nil
    local defImg = nil
    local defLab = nil
    local agiImg = nil
    local agiLab = nil

    local assistantImg = nil
    local assistantDetailLab = nil

    local magicImg = nil
    local magicDetailLab = nil

    if isMonsterItem then
        --"召"信息
        local imgName = string.format("%s.%s",Enum.TEXT_IMG.SUMMON,Text.get(IDX.PNG_TYPE))
        summonImg = ccui.ImageView:create(imgName)
        self.scrollView:getInnerContainer():addChild(summonImg)
        local prop = self.item[Enum.IAP_KEY.PROPERTY]
        if prop >= 0 and prop <= #Enum.PROP_IMG_NAME then
            local imgName = string.format("%s.%s",Enum.TEXT_IMG.PROPERTY,Text.get(IDX.PNG_TYPE))
            propTextImgView = ccui.ImageView:create(imgName)
            self.scrollView:getInnerContainer():addChild(propTextImgView)

            local imgName = string.format("%s.%s",Enum.PROP_IMG_NAME[prop],Text.get(IDX.PNG_TYPE))
            propImgView = ccui.ImageView:create(imgName)
            self.scrollView:getInnerContainer():addChild(propImgView)
        end

        imgName = Enum.IMG_NAME_PNGHK.ATK
        atkImg = ccui.ImageView:create(imgName)
        self.scrollView:getInnerContainer():addChild(atkImg)
        atkLab = cc.LabelAtlas:_create(self.item[Enum.IAP_KEY.ATK], Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
        self.scrollView:getInnerContainer():addChild(atkLab)

        imgName = Enum.IMG_NAME_PNGHK.DEF
        defImg = ccui.ImageView:create(imgName)
        self.scrollView:getInnerContainer():addChild(defImg)
        defLab = cc.LabelAtlas:_create(self.item[Enum.IAP_KEY.DEF], Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
        self.scrollView:getInnerContainer():addChild(defLab)

        imgName = Enum.IMG_NAME_PNGHK.AGI
        agiImg = ccui.ImageView:create(imgName)
        self.scrollView:getInnerContainer():addChild(agiImg)
        agiLab = cc.LabelAtlas:_create(self.item[Enum.IAP_KEY.AGI], Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
        self.scrollView:getInnerContainer():addChild(agiLab)

        height = height + summonImg:getContentSize().height + 5

        --"附"信息
        imgName = string.format("%s.%s",Enum.TEXT_IMG.ASSISTANT,Text.get(IDX.PNG_TYPE))
        assistantImg = ccui.ImageView:create(imgName)
        self.scrollView:getInnerContainer():addChild(assistantImg)

        local assistantDetail = self.item[Enum.IAP_KEY.ASSISTANT_DETAIL]
        assistantDetailLab = cc.Label:create()
        assistantDetailLab:setString(assistantDetail)
        assistantDetailLab:setSystemFontSize(UILib.FONT.MINI)
        self.scrollView:getInnerContainer():addChild(assistantDetailLab)

        height = height + assistantImg:getContentSize().height + 5

        --"术"信息
        imgName = string.format("%s.%s",Enum.TEXT_IMG.MAGIC,Text.get(IDX.PNG_TYPE))
        magicImg = ccui.ImageView:create(imgName)
        self.scrollView:getInnerContainer():addChild(magicImg)

        local magicDetail = self.item[Enum.IAP_KEY.MAGIC_DETAIL]
        magicDetailLab = cc.Label:create()
        magicDetailLab:setString(magicDetail)
        magicDetailLab:setSystemFontSize(UILib.FONT.MINI)
        self.scrollView:getInnerContainer():addChild(magicDetailLab)

        height = height + magicImg:getContentSize().height
    end

    --装备所需等级及属性
    local lvImgView = nil
    local lvLabel = nil
    if isMonsterItem == false then
        local level = self.item[Enum.IAP_KEY.LEVEL]
        if level > 0 then
            local imgName = string.format("%s.%s",Enum.TEXT_IMG.LEVEL,Text.get(IDX.PNG_TYPE))
            lvImgView = ccui.ImageView:create(imgName)
            self.scrollView:getInnerContainer():addChild(lvImgView)

            height = height + lvImgView:getContentSize().height

            lvLabel = cc.LabelAtlas:_create(level, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
            self.scrollView:getInnerContainer():addChild(lvLabel)
        end

        local prop = self.item[Enum.IAP_KEY.PROPERTY]
        if prop >= 0 and prop <= #Enum.PROP_IMG_NAME then
            local imgName = string.format("%s.%s",Enum.TEXT_IMG.PROPERTY,Text.get(IDX.PNG_TYPE))
            propTextImgView = ccui.ImageView:create(imgName)
            self.scrollView:getInnerContainer():addChild(propTextImgView)

            if lvImgView == nil then
                height = height + propTextImgView:getContentSize().height
            end

            local imgName = string.format("%s.%s",Enum.PROP_IMG_NAME[prop],Text.get(IDX.PNG_TYPE))
            propImgView = ccui.ImageView:create(imgName)
            self.scrollView:getInnerContainer():addChild(propImgView)
        end
    end

    --购买所需真实货币 todo
    local costRealMoneyLabel = nil
    local costRealMoney = self.item[Enum.IAP_KEY.LOCAL_PRICE]
    if itemType == 99 then
        costRealMoney = ""
    end
    if costRealMoney ~= "" then
        costRealMoneyLabel = cc.Label:create()
        costRealMoneyLabel:setString(costRealMoney)
        costRealMoneyLabel:setSystemFontSize(UILib.FONT.MINI)
        self.scrollView:getInnerContainer():addChild(costRealMoneyLabel)

        height = height + costRealMoneyLabel:getContentSize().height
    end

    --购买所需费用信息
    local moneyIcon = nil
    local moneyLabel = nil
    local costMoneyLabel = nil
    local costMoney = self.item[Enum.IAP_KEY.COST_MONEY]
    if costMoney > 0 and itemType ~= 99 then
        moneyIcon = UILib.createClipSprite(UILib.IMG_IDX.IMG91,UILib.IN_IMG_IDX.MONEY)
        self.scrollView:getInnerContainer():addChild(moneyIcon)

        height = height + moneyIcon:getContentSize().height-- + moneyIcon:getContentSize().height*0.3--*0.3为间隔

        local money = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.MONEY)
        local moneyColor = Enum.DIGITAL.RED
        if money >= costMoney then
            moneyColor = Enum.DIGITAL.GREEN
        end
        moneyLabel = cc.LabelAtlas:_create(money, moneyColor, 198/21, 11, string.byte('0'))
        costMoneyLabel = cc.LabelAtlas:_create(">"..costMoney, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
        self.scrollView:getInnerContainer():addChild(moneyLabel)
        self.scrollView:getInnerContainer():addChild(costMoneyLabel)
    end

    local stoneIcon = nil
    local stoneLabel = nil
    local costStoneLabel = nil
    local costStone = self.item[Enum.IAP_KEY.COST_STONE]
    if costStone > 0 then
        stoneIcon = UILib.createClipSprite(UILib.IMG_IDX.IMG91,UILib.IN_IMG_IDX.SOULSTONE)
        self.scrollView:getInnerContainer():addChild(stoneIcon)

        height = height + stoneIcon:getContentSize().height

        local stone = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.SOUL_STONE)
        local stoneColor = Enum.DIGITAL.RED
        if stone >= costStone then
            stoneColor = Enum.DIGITAL.GREEN
        end
        stoneLabel = cc.LabelAtlas:_create(stone, stoneColor, 198/21, 11, string.byte('0'))
        costStoneLabel = cc.LabelAtlas:_create(">"..costStone, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
        self.scrollView:getInnerContainer():addChild(stoneLabel)
        self.scrollView:getInnerContainer():addChild(costStoneLabel)
    end

    --#################################################
    if height < self.scrollView:getContentSize().height then
        height = self.scrollView:getContentSize().height
    end
    self.scrollView:setInnerContainerSize(cc.size(self.scrollView:getInnerContainerSize().width, height))
    --#################################################
    local drawX = icon:getContentSize().width/2
    local drawY = self.scrollView:getInnerContainer():getContentSize().height-icon:getContentSize().height/2
    local pos = cc.p(drawX, drawY)
    icon:setPosition(pos)
    pos = cc.p(icon:getPositionX()+icon:getContentSize().width/2+nameLabel:getContentSize().width/2,icon:getPositionY())
    nameLabel:setPosition(pos)

    drawY = drawY - icon:getContentSize().height/2
    if detailLabel ~= nil then
        local dx = detailLabel:getContentSize().width/2
        local dy = drawY - detailLabel:getContentSize().height/2
        pos = cc.p(dx, dy)
        detailLabel:setPosition(pos)

        drawY = drawY - detailLabel:getContentSize().height
    end

    if isMonsterItem then
        --summon
        drawX = summonImg:getContentSize().width/2
        drawY = drawY - summonImg:getContentSize().height/2
        pos = cc.p(drawX, drawY)
        summonImg:setPosition(pos)

        if propTextImgView ~= nil and propImgView ~= nil then
            drawX = drawX + summonImg:getContentSize().width/2 + propTextImgView:getContentSize().width/2 + 10
            pos = cc.p(drawX, drawY)
            propTextImgView:setPosition(pos)

            drawX = drawX + propTextImgView:getContentSize().width/2 + propImgView:getContentSize().width/2
            pos = cc.p(drawX, drawY)
            propImgView:setPosition(pos)

            drawX = drawX + propImgView:getContentSize().width/2
        end

        drawX = drawX + atkImg:getContentSize().width/2 + 20
        pos = cc.p(drawX, drawY)
        atkImg:setPosition(pos)

        drawX = drawX + atkImg:getContentSize().width/2
        pos = cc.p(drawX, drawY-atkLab:getContentSize().height/2)
        atkLab:setPosition(pos)

        drawX = drawX + atkLab:getContentSize().width + 20 + defImg:getContentSize().width/2
        pos = cc.p(drawX, drawY)
        defImg:setPosition(pos)

        drawX = drawX + atkImg:getContentSize().width/2
        pos = cc.p(drawX, drawY-defLab:getContentSize().height/2)
        defLab:setPosition(pos)

        drawX = drawX + defLab:getContentSize().width + 20 + agiImg:getContentSize().width/2
        pos = cc.p(drawX, drawY)
        agiImg:setPosition(pos)

        drawX = drawX + agiImg:getContentSize().width/2
        pos = cc.p(drawX, drawY-agiLab:getContentSize().height/2)
        agiLab:setPosition(pos)

        --assistant
        drawX = assistantImg:getContentSize().width/2
        drawY = drawY - summonImg:getContentSize().height/2 - 10 - assistantImg:getContentSize().height/2
        pos = cc.p(drawX, drawY)
        assistantImg:setPosition(pos)

        drawX = drawX + assistantImg:getContentSize().width + assistantDetailLab:getContentSize().width/2
        pos = cc.p(drawX, drawY)
        assistantDetailLab:setPosition(pos)

        --magic
        drawX = magicImg:getContentSize().width/2
        drawY = drawY - assistantImg:getContentSize().height/2 - 10 - magicImg:getContentSize().height/2
        pos = cc.p(drawX, drawY)
        magicImg:setPosition(pos)

        drawX = drawX + magicImg:getContentSize().width + magicDetailLab:getContentSize().width/2
        pos = cc.p(drawX, drawY)
        magicDetailLab:setPosition(pos)

        drawY = drawY - magicImg:getContentSize().height
    end

    if isMonsterItem == false then
        if lvImgView ~= nil then
            drawX = lvImgView:getContentSize().width/2
            drawY = drawY - lvImgView:getContentSize().height/2
            pos = cc.p(drawX, drawY)
            lvImgView:setPosition(pos)

            drawX = drawX + lvImgView:getContentSize().width/2
            drawY = drawY - lvImgView:getContentSize().height/2

            if lvLabel ~= nil then
                drawX = drawX + lvLabel:getContentSize().width/2
                pos = cc.p(drawX,pos.y-lvLabel:getContentSize().height/2)
                lvLabel:setPosition(pos)

                drawX = drawX + lvLabel:getContentSize().width/2
            end
        end

        if propTextImgView ~= nil then
            drawX = drawX + 50
            local pY = drawY
            if lvImgView ~= nil then
                pY = lvImgView:getPositionY()
            end
            pos = cc.p(drawX, pY)
            propTextImgView:setPosition(pos)

            if lvImgView == nil then
                drawY = drawY - propTextImgView:getContentSize().height/2
            end
            drawX = drawX + propTextImgView:getContentSize().width/2

            if propImgView ~= nil then
                pos = cc.p(drawX+propImgView:getContentSize().width/2,pY)
                propImgView:setPosition(pos)
            end
        end
    end

    --真实货币信息
    if costRealMoneyLabel ~= nil then
        drawX = costRealMoneyLabel:getContentSize().width/2
        drawY = drawY - costRealMoneyLabel:getContentSize().height
        pos = cc.p(drawX, drawY)
        costRealMoneyLabel:setPosition(pos)

        drawY = drawY - costRealMoneyLabel:getContentSize().height/2
    end

    --费用信息
    if moneyIcon ~= nil then
        drawX = moneyIcon:getContentSize().width/2
        drawY = drawY - moneyIcon:getContentSize().height/2
        pos = cc.p(drawX, drawY)
        moneyIcon:setPosition(pos)

        drawX = drawX + moneyIcon:getContentSize().width/2 + moneyLabel:getContentSize().width/2
        drawY = drawY - moneyLabel:getContentSize().height/2
        pos = cc.p(drawX, drawY)
        moneyLabel:setPosition(pos)

        drawX = drawX + moneyLabel:getContentSize().width/2 + costMoneyLabel:getContentSize().width/2
        pos = cc.p(drawX, drawY)
        costMoneyLabel:setPosition(pos)

        drawY = drawY - moneyIcon:getContentSize().height/2
    end

    if stoneIcon ~= nil then
        drawX = stoneIcon:getContentSize().width/2
        drawY = drawY - stoneIcon:getContentSize().height/2
        pos = cc.p(drawX, drawY)
        stoneIcon:setPosition(pos)

        drawX = drawX + stoneIcon:getContentSize().width/2 + stoneLabel:getContentSize().width/2
        drawY = drawY - stoneLabel:getContentSize().height/2
        pos = cc.p(drawX, drawY)
        stoneLabel:setPosition(pos)

        drawX = drawX + stoneLabel:getContentSize().width
        pos = cc.p(drawX, drawY)
        costStoneLabel:setPosition(pos)

        drawY = drawY - stoneIcon:getContentSize().height/2
    end

    self:updateShowButton()
end

function ItemInfoView:updateInfomationAnim(delta)
    if self.scrollView == nil then
        return
    end
    local innerHeight = self.scrollView:getInnerContainer():getContentSize().height
    if self.scrollView:getContentSize().height == innerHeight then
        return
    end

    local drawY = self.scrollView:getInnerContainer():getPositionY()
    drawY = drawY + delta*10
    if drawY >= innerHeight then
        drawY = -innerHeight
    end
    self.scrollView:getInnerContainer():setPositionY(drawY)
end

function ItemInfoView:dpadUp()
    if ItemInfoView.itemSign then
        local pos = ItemInfoView.tapSelected - 1
        if pos <1 then
            return
        end
        ItemInfoView.tapSelected = pos
        local btn = ItemInfoView.tapTable[ItemInfoView.tapSelected]
        if btn ~= nil and ItemInfoView.focusAni~= nil then
            local pos = cc.p(btn:getPositionX(),btn:getPositionY())
            ItemInfoView.focusAni:setPosition(pos)
            ItemInfoView.focusAni:setVisible(true)
        end
    end
end

function ItemInfoView:dpadDown()
    if ItemInfoView.itemSign then
        local pos = ItemInfoView.tapSelected + 1
        if pos >2 then
            return
        end
        ItemInfoView.tapSelected = pos
        local btn = ItemInfoView.tapTable[ItemInfoView.tapSelected]
        if btn ~= nil and ItemInfoView.focusAni~= nil then
            local pos = cc.p(btn:getPositionX(),btn:getPositionY())
            ItemInfoView.focusAni:setPosition(pos)
            ItemInfoView.focusAni:setVisible(true)
        end
    end
end

function ItemInfoView:dpadTap(doButtonAction)
    if ItemInfoView.itemSign then
        local pos = ItemInfoView.tapSelected
        cclog("ItemInfoView.dpadTap() pos = "..pos)
        if ItemInfoView.tapTable[pos] ~= nil then
            local btn = ItemInfoView.tapTable[pos]
            if btn:getTag() == TAG.EQUIP then
                if self.item[Enum.ITEM_KEY.IS_EQUIPPED] then
                    return
                end
                ItemInfoView.setItemSignType(false)
            end

            if btn:getTag() == TAG.EQUIP_SOUL or btn:getTag() == TAG.UNEQUIP_SOUL then
                ItemInfoView.setItemSignType(false)
            end

            doButtonAction(ItemInfoView.tapTable[pos], ccui.TouchEventType.ended)
        end
    end
end

function ItemInfoView:dpadBack()
    if ItemInfoView.itemSign then
        local scene = cc.Director:getInstance():getRunningScene()
        local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
        if layer ~= nil then
            cclog("setItemSignType(false) ItemInfoView.dpadBack()")
            layer:setItemSignType(false)
            ItemInfoView.tapSelected = -1
            if ItemInfoView.focusAni ~= nil then
                ItemInfoView.focusAni:setVisible(false)
            end
            ItemInfoView.focusAni = nil
            ItemInfoView.itemSign = false
        end

        local layer2 = scene:getChildByTag(Enum.LAYER_TYPE.SINGLE_POP_MAIN_SHOP)
        if layer2 ~= nil then
            cclog("setItemSignType(false) ItemInfoView.dpadBack()2")
            layer2:setItemSignType(false)
            ItemInfoView.tapSelected = -1
            if ItemInfoView.focusAni ~= nil then
                ItemInfoView.focusAni:setVisible(false)
            end
            ItemInfoView.focusAni = nil
            ItemInfoView.itemSign = false
        end
        local layer3 = scene:getChildByTag(Enum.LAYER_TYPE.SINGLE_POP_NPC_SHOP)
        if layer3 ~= nil then
            cclog("setItemSignType(false) ItemInfoView.dpadBack()3")
            layer3:setItemSignType(false)
            ItemInfoView.tapSelected = -1
            if ItemInfoView.focusAni ~= nil then
                ItemInfoView.focusAni:setVisible(false)
            end
            ItemInfoView.focusAni = nil
            ItemInfoView.itemSign = false
        end
    end
end

function ItemInfoView:registerControllerListener(doButtonAction)
    local function onConnectController(controller, event)
        if controller == nil or controller == self._firstHolder.controller then
            return
        end

        local deviceInfo = string.format("%s id:%d", controller:getDeviceName(), controller:getDeviceId())
        cclog("ItemInfoView deviceInfo ="..deviceInfo)

        if self._firstHolder.controller == nil then
            self._firstHolder.controller = controller
        end
    end

    local function onDisconnectedController(controller, event)
        if self._firstHolder.controller == controller then
            self._firstHolder.controller = nil
        end
    end

    local function showButtonState(controller, keyCode, isPressed)
        onConnectController(controller, nil)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end

        if lk.LakooManager:getInstance():getIsHasPopView() == true then
            return
        end

        local scene = cc.Director:getInstance():getRunningScene()
        local child = scene:getChildren()

        local topSence = child[#child]
        cclog("ItemInfoView topSence:getName() = "..topSence:getName())
        if topSence:getName() ~= "SinglePopLayer" and topSence:getName() ~= "MainPopLayer" then
            cclog("ItemInfoView no top")
            return
        end

        local popLayer = self:getParent():getChildren()
        if popLayer ~= nil then
            for var = 1,#popLayer do
                local topPopLayer = popLayer[var]
                if topPopLayer ~= nil then
                    cclog("name = "..topPopLayer:getName())
                    if topPopLayer:getName() == "EquipSoulLayer" then
                        return
                    else
                    end
                end
            end
        end

        if isPressed then
            cclog("ItemInfoView keyDown ="..keyCode)
            if keyCode == cc.ControllerKey.BUTTON_A then
                self:dpadTap(doButtonAction)
            elseif keyCode == cc.ControllerKey.BUTTON_B then

            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
                self:dpadUp()
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
                self:dpadDown()
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key Down:%d",keyCode)
            end
        else
            cclog("ItemInfoView keyup ="..keyCode)
            if keyCode == cc.ControllerKey.BUTTON_A then

            elseif keyCode == cc.ControllerKey.BUTTON_B then
                self.dpadBack()
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then

            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then

            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key up:%d",keyCode)
            end
        end
    end

    local function onKeyDown(controller, keyCode, event)
        showButtonState(controller, keyCode, true)
    end

    local function onKeyUp(controller, keyCode, event)
        showButtonState(controller, keyCode, false)
    end

    local function onAxisEvent(controller, keyCode, event)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end

        if lk.LakooManager:getInstance():getIsHasPopView() == true then
            return
        end

        local scene = cc.Director:getInstance():getRunningScene()
        local child = scene:getChildren()

        local topSence = child[#child]
        if topSence:getName() ~= "SinglePopLayer" and topSence:getName() ~= "MainPopLayer" then
            return
        end

        local ketStatus = controller:getKeyStatus(keyCode)
        if keyCode == cc.ControllerKey.JOYSTICK_LEFT_X then

        elseif keyCode == cc.ControllerKey.JOYSTICK_LEFT_Y then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_X then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_Y then
        elseif keyCode == cc.ControllerKey.AXIS_LEFT_TRIGGER then
            if(ketStatus.value == 1) then
            end
        elseif keyCode == cc.ControllerKey.AXIS_RIGHT_TRIGGER then
        else
        --do nothing
        end
    end

    local controllerListener = cc.EventListenerController:create()
    controllerListener:registerScriptHandler(onConnectController, cc.Handler.EVENT_CONTROLLER_CONNECTED)
    controllerListener:registerScriptHandler(onDisconnectedController, cc.Handler.EVENT_CONTROLLER_DISCONNECTED)

    controllerListener:registerScriptHandler(onKeyDown, cc.Handler.EVENT_CONTROLLER_KEYDOWN)
    controllerListener:registerScriptHandler(onKeyUp, cc.Handler.EVENT_CONTROLLER_KEYUP)
    controllerListener:registerScriptHandler(onAxisEvent, cc.Handler.EVENT_CONTROLLER_AXIS)

    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(controllerListener, self)
    cc.Controller:startDiscoveryController()
end

--

function ItemInfoView.setItemSign(type)
    ItemInfoView.itemSign = type

    if ItemInfoView.itemSign then
        cclog("ItemInfoView.itemSign = true ,tap = "..ItemInfoView.tapSelected)
        ItemInfoView.tapSelected = 1
        local btn = ItemInfoView.tapTable[ItemInfoView.tapSelected]
        if  btn~= nil then
            local pos = cc.p(btn:getPositionX(),btn:getPositionY())
                if ItemInfoView.focusAni ~=nil then
                    ItemInfoView.focusAni:setVisible(false)
                    ItemInfoView.focusAni:removeFromParent(true)
                end
                ItemInfoView.focusAni = nil
                ccs.ArmatureDataManager:getInstance():addArmatureFileInfo("FocusAni.csb")
                ItemInfoView.focusAni = ccs.Armature:create("FocusAni")
                btn:getParent():addChild(ItemInfoView.focusAni)
                ItemInfoView.focusAni:setLocalZOrder(99)
                ItemInfoView.focusAni:getAnimation():playWithIndex(0)

                ItemInfoView.focusAni:setPosition(pos)
                ItemInfoView.focusAni:setVisible(true)
        end
    else
        cclog("ItemInfoView.itemSign = false")
        ItemInfoView.tapSelected = -1
        if ItemInfoView.focusAni ~= nil then
            ItemInfoView.focusAni:setVisible(false)
        end
        ItemInfoView.tapTable = {}
    end
end

function ItemInfoView.setItemSignType(type)
    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        cclog("setItemSignType(false)")
        layer:setItemSignType(false)
        ItemInfoView.itemSign = false
        local b =  layer:getItemSignType()
        ItemInfoView.focusAni = nil
    end
end

function ItemInfoView.clean()

end

function ItemInfoView:closeCallBack()
    ItemInfoView.dpadBack()
end


function ItemInfoView:cleanHolder()
    self._firstHolder = nil
end
return ItemInfoView
