require "cocos.init"

local UILib = require("common.UILib")
local Enum = require("common.Enum")
local Util = require("common.Util")

local Text = require("conf.Text")
local IDX = require("conf.TextIndex")

local cc = cc
local ccui = ccui
local class = class


local PicInfoView = class("PicInfoView",
    function()
        return cc.Node:create()
    end
)

local ControllerHolder = class("ControllerHolder")
function ControllerHolder:ctor()
    self.controller  = nil
    self._holderNode = nil

    self._leftJoystick = nil
    self._rightJoystick = nil

    self._dpadLeft = nil
    self._dpadRight = nil
    self._dpadUp = nil
    self._dpadDown = nil

    self._buttonX = nil
    self._buttonY = nil
    self._buttonA = nil
    self._buttonB = nil

    self._buttonL1 = nil
    self._buttonR1 = nil
    self._buttonL2 = nil
    self._buttonR2 = nil

    self.deviceLabel = nil
    self.externalKeyLabel = nil
end

PicInfoView.__index = PicInfoView

PicInfoView.bagTag = -1
PicInfoView.currentPicIndex = 0
PicInfoView.picArray = {}
PicInfoView.item = nil
PicInfoView.scrollView = nil

PicInfoView.preButton = nil
PicInfoView.nextButton = nil
PicInfoView.nameLabel = nil
PicInfoView.orderLabel = nil
PicInfoView.rareLabel = nil
PicInfoView.propImg = nil
PicInfoView.detailLabel = nil
PicInfoView.animPanel = nil

function PicInfoView:create(_bagTag, _picArray, _picIndex, _enablePre, _enableNext, _updateFun, _closeCallBackFun)
    local picInfoView = PicInfoView.new()
    picInfoView:init(_bagTag, _picArray, _picIndex, _enablePre, _enableNext, _updateFun, _closeCallBackFun)

    return picInfoView
end

function PicInfoView:init(_bagTag, _picArray, _picIndex, _enablePre, _enableNext, _updateFun, _closeCallBackFun)
    self:setName("PicInfoView")
    self.bagTag = _bagTag
    self.currentPicIndex = _picIndex
    self.picArray = _picArray
    local item = self.picArray[self.currentPicIndex]
    
    local visibleSize = cc.Director:getInstance():getVisibleSize()
    self:setContentSize(visibleSize)
    
    local bgLayer = cc.LayerColor:create(UILib.UI_COLOR4B.GRAY)
    bgLayer:setContentSize(visibleSize)
    self:addChild(bgLayer)
    
    local eventListener = cc.EventListenerTouchOneByOne:create()
    local function onTouchBegan(touch, event)         
        return true
    end
    local function onTouchEnded(touch, event)
    end

    eventListener:registerScriptHandler(onTouchBegan,cc.Handler.EVENT_TOUCH_BEGAN)
    eventListener:registerScriptHandler(onTouchEnded,cc.Handler.EVENT_TOUCH_ENDED)
    eventListener:setSwallowTouches(true)
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(eventListener, self)
    
    
    local bgImgView = ccui.ImageView:create("p_75.pngHK")
    pos = UILib.offset(0,0,bgImgView:getContentSize().width,bgImgView:getContentSize().height,UILib.UI_ANCHOR.CENTER)
    bgImgView:setPosition(pos)
    self:addChild(bgImgView)
    
    --title
    local tName = Enum.TEXT_IMG.WEAPON
    if self.bagTag == Enum.BAG_TAG.ARMOR then
        tName = Enum.TEXT_IMG.ARMOR
    elseif self.bagTag == Enum.BAG_TAG.DEMON then
        tName = Enum.TEXT_IMG.DEMON
    end
    local titleName = string.format("%s.%s",tName,Text.get(IDX.PNG_TYPE))
    local titleImgView = ccui.ImageView:create(titleName)
    pos = UILib.offsetWithSuperSize(0,-5,titleImgView:getContentSize().width,titleImgView:getContentSize().height,bgImgView:getContentSize().width,bgImgView:getContentSize().height,UILib.UI_ANCHOR.CENTER_TOP)
    titleImgView:setPosition(pos)
    bgImgView:addChild(titleImgView)
    
    --(可显示信息图鉴总数/图鉴总数)
    --"A":"("
    --">":"/"
    --"B":")"
    local valStr = string.format("A%d>%dB",self:getHasPisLength(),#self.picArray)
    local valLabel = cc.LabelAtlas:_create(valStr, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
    pos = cc.p(titleImgView:getPositionX()-valLabel:getContentSize().width/2, titleImgView:getPositionY()-titleImgView:getContentSize().height/2-valLabel:getContentSize().height/2-3)
    valLabel:setPosition(pos)
    bgImgView:addChild(valLabel)
    
    --back button
    local function doBackAction(sender, eventType) 
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            self:removeFromParent(true)
            if _closeCallBackFun ~= nil then
            	_closeCallBackFun()               
            end
        end
    end
    local backButton = ccui.Button:create(Enum.IMG_NAME_PNGHK.BACK,Enum.IMG_NAME_PNGHK.BACK,Enum.IMG_NAME_PNGHK.BACK)
    pos = UILib.offsetWithSuperSize(-8,-3,backButton:getContentSize().width,backButton:getContentSize().height,bgImgView:getContentSize().width,bgImgView:getContentSize().height,UILib.UI_ANCHOR.TOP_RIGHT)
    backButton:setPosition(pos)
    bgImgView:addChild(backButton)
    backButton:addTouchEventListener(doBackAction)
    
--    self.scrollView = ccui.ScrollView:create()
--    self.scrollView:setClippingEnabled(true)
--    self.scrollView:setContentSize(cc.size(self:getContentSize().width-15*2,self:getContentSize().height-20))
--    self.scrollView:setPosition(cc.p(15,5))
--    self:addChild(self.scrollView)

    --上一个/下一个按钮
    local function doButtonAction(sender, eventType)
        if eventType == ccui.TouchEventType.began then 
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            _updateFun(sender:getTag()==1, self.currentPicIndex)
        end
    end
    self.preButton = ccui.Button:create(Enum.IMG_NAME_PNGHK.LEFT_ARROW,Enum.IMG_NAME_PNGHK.LEFT_ARROW_S,Enum.IMG_NAME_PNGHK.LEFT_ARROW_UN)   
    local px = self.preButton:getContentSize().width/2
    pos = UILib.offsetWithSuperSize(px,-px*2.5,self.preButton:getContentSize().width,self.preButton:getContentSize().height,bgImgView:getContentSize().width,bgImgView:getContentSize().height,UILib.UI_ANCHOR.TOP_LEFT)
    self.preButton:setPosition(pos)
    bgImgView:addChild(self.preButton)
    self.preButton:setTag(1)
    self.preButton:addTouchEventListener(doButtonAction)
    
    self.nextButton = ccui.Button:create(Enum.IMG_NAME_PNGHK.LEFT_ARROW,Enum.IMG_NAME_PNGHK.LEFT_ARROW_S,Enum.IMG_NAME_PNGHK.LEFT_ARROW_UN)   
    local px = self.preButton:getContentSize().width/2
    pos = UILib.offsetWithSuperSize(-px,-px*2.5,self.nextButton:getContentSize().width,self.nextButton:getContentSize().height,bgImgView:getContentSize().width,bgImgView:getContentSize().height,UILib.UI_ANCHOR.TOP_RIGHT)
    self.nextButton:setPosition(pos)
    self.nextButton:setRotation(180)
    bgImgView:addChild(self.nextButton)
    self.nextButton:setTag(0)
    self.nextButton:addTouchEventListener(doButtonAction)
    
    self:updateTurnButton(_enablePre,_enableNext)
    
    local osType = Enum.OS_TYPE.IOS
    osType = lk.LakooManager:getInstance():getOSType()
    if osType == Enum.OS_TYPE.ANDROID then
        --joe
        self._firstHolder   = ControllerHolder.new()
        self._firstHolder.controller = nil
        self:registerControllerListener(_closeCallBackFun,_updateFun)
        --
    end
    
    --编号
    local orderName = string.format("%s.%s",Enum.TEXT_IMG.ORDER,Text.get(IDX.PNG_TYPE))
    local orderImgView = ccui.ImageView:create(orderName)
    px = orderImgView:getContentSize().width*2
    pos = UILib.offsetWithSuperSize(-px,20,orderImgView:getContentSize().width,orderImgView:getContentSize().height,bgImgView:getContentSize().width,bgImgView:getContentSize().height,UILib.UI_ANCHOR.CENTER_RIGHT)
    orderImgView:setPosition(pos)
    bgImgView:addChild(orderImgView)
    
    self.orderLabel = cc.LabelAtlas:_create(item[Enum.PIC_KEY.ID], Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
    px = orderImgView:getPositionX()+orderImgView:getContentSize().width/2+self.orderLabel:getContentSize().width/2
    pos = cc.p(px, orderImgView:getPositionY()-self.orderLabel:getContentSize().height/2)
    self.orderLabel:setPosition(pos)
    bgImgView:addChild(self.orderLabel)
   
    --稀有
    local rareName = string.format("%s.%s",Enum.TEXT_IMG.RARE,Text.get(IDX.PNG_TYPE))
    local rareImgView = ccui.ImageView:create(rareName)
    pos = cc.p(orderImgView:getPositionX(),orderImgView:getPositionY()-orderImgView:getContentSize().height*1.5)
    rareImgView:setPosition(pos)
    bgImgView:addChild(rareImgView)
    
    self.rareLabel = cc.LabelAtlas:_create(item[Enum.PIC_KEY.RARE], Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
    px = rareImgView:getPositionX()+rareImgView:getContentSize().width/2+self.rareLabel:getContentSize().width/2
    pos = cc.p(px, rareImgView:getPositionY()-self.rareLabel:getContentSize().height/2)
    self.rareLabel:setPosition(pos)
    bgImgView:addChild(self.rareLabel)
    
    --属性
    local propName = string.format("%s.%s",Enum.TEXT_IMG.PROPERTY,Text.get(IDX.PNG_TYPE))
    local propImgView = ccui.ImageView:create(propName)
    pos = cc.p(rareImgView:getPositionX(),rareImgView:getPositionY()-rareImgView:getContentSize().height*1.5)
    propImgView:setPosition(pos)
    bgImgView:addChild(propImgView)
    
    local prop = item[Enum.PIC_KEY.PROPERTY]
    local imgName = string.format("%s.%s",Enum.PROP_IMG_NAME[prop],Text.get(IDX.PNG_TYPE))
    self.propImg = ccui.ImageView:create(imgName)
    pos = cc.p(propImgView:getPositionX()+propImgView:getContentSize().width/2+self.propImg:getContentSize().width,propImgView:getPositionY())
    self.propImg:setPosition(pos)
    bgImgView:addChild(self.propImg)
    
    --init pic name
    local initName = item[Enum.PIC_KEY.NAME]
    self.nameLabel = cc.Label:create()
    self.nameLabel:setString(initName)
    self.nameLabel:setColor(UILib.UI_COLOR3B.BROWN)
    self.nameLabel:setSystemFontSize(UILib.FONT.SMALL)
    pos = cc.p(self.preButton:getPositionX()+(self.nextButton:getPositionX()-self.preButton:getPositionX())/2,self.preButton:getPositionY())
    self.nameLabel:setPosition(pos)
    bgImgView:addChild(self.nameLabel)
    
    --init detail
    local detail = item[Enum.PIC_KEY.DETAIL]
    --self.detailLabel = cc.Label:create()
    self.detailLabel = CCLabelTTF:create(detail, "Marker Felt", UILib.FONT.MINI, cc.size(UILib.FONT.MINI*12, UILib.FONT.MINI*3), kCCTextAlignmentLeft)
    self.detailLabel:setColor(UILib.UI_COLOR3B.BLACK)
    pos = cc.p(bgImgView:getContentSize().width/2, 35)
    self.detailLabel:setPosition(pos)
    bgImgView:addChild(self.detailLabel)
    
    --animation panel
    --self.animPanel = cc.LayerColor:create(UILib.UI_COLOR4B.RED)
    self.animPanel = cc.Node:create()
    self.animPanel:setContentSize(cc.size(90, 90))
    self.animPanel:setPosition(cc.p(10, 75))
    bgImgView:addChild(self.animPanel)
    local aniName = ""
    if item[Enum.PIC_KEY.TYPE] == Enum.ITEM_TYPE.WEAPON_DATA then
    	aniName = Enum.ANI.RPIC
    elseif item[Enum.PIC_KEY.TYPE] == Enum.ITEM_TYPE.ARMOUR_DATA then
        aniName = Enum.ANI.R0
    end
    lk.LakooManager:getInstance():addLKAnimationNode(self.animPanel, aniName, item[Enum.PIC_KEY.TYPE], item[Enum.PIC_KEY.HEAD], item[Enum.PIC_KEY.BODY], item[Enum.PIC_KEY.ANI_NAME])

    local function update()
        self:updateInfomationAnim()
    end
    
    local function onNodeEvent(event)
        if "enterTransitionFinish" == event then
            self.scheduleID = cc.Director:getInstance():getScheduler():scheduleScriptFunc(update, 0, false)
        end
        if "exit" == event then
            if self.scheduleID ~= nil then
                cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.scheduleID)
            end
            self:clean()
        end
    end
    self:registerScriptHandler(onNodeEvent)
end


function PicInfoView:updateInfo(_picIndex, _enablePre, _enableNext)
    self.currentPicIndex = _picIndex
    local item = self.picArray[self.currentPicIndex]
    if item == nil then
    	return
    end
    if self.nameLabel ~= nil then
        local name = item[Enum.PIC_KEY.NAME]
        self.nameLabel:setString(name)
    end
    
    if self.orderLabel ~= nil then
    	local order = item[Enum.PIC_KEY.ID]
    	self.orderLabel:setString(order)
    end
    
    if self.rareLabel ~= nil then
        local rare = item[Enum.PIC_KEY.RARE]
        self.rareLabel:setString(rare)
    end
    
    if self.propImg ~= nil then
        local parent = self.propImg:getParent()
        local pos = cc.p(self.propImg:getPositionX(),self.propImg:getPositionY())
        self.propImg:removeFromParent(true)
        self.propImg = nil
        local prop = item[Enum.PIC_KEY.PROPERTY]
        local imgName = string.format("%s.%s",Enum.PROP_IMG_NAME[prop],Text.get(IDX.PNG_TYPE))
        self.propImg = ccui.ImageView:create(imgName)
        self.propImg:setPosition(pos)
        parent:addChild(self.propImg)
    end
    
    if self.detailLabel ~= nil then
        local detail = item[Enum.PIC_KEY.DETAIL]
        self.detailLabel:setString(detail)
    end
    
    if self.animPanel ~= nil then
    	self.animPanel:removeAllChildren()
        local aniName = ""
        if item[Enum.PIC_KEY.TYPE] == Enum.ITEM_TYPE.WEAPON_DATA then
            aniName = Enum.ANI.RPIC
        elseif item[Enum.PIC_KEY.TYPE] == Enum.ITEM_TYPE.ARMOUR_DATA then
            aniName = Enum.ANI.R0
        end
        lk.LakooManager:getInstance():addLKAnimationNode(self.animPanel, aniName, item[Enum.PIC_KEY.TYPE], item[Enum.PIC_KEY.HEAD], item[Enum.PIC_KEY.BODY], item[Enum.PIC_KEY.ANI_NAME])
    end
    
    self:updateTurnButton(_enablePre,_enableNext)
end

function PicInfoView:updateTurnButton(_enablePre, _enableNext)
    if self.preButton ~= nil then
        if _enablePre then
            self.preButton:setTouchEnabled(true)
            self.preButton:setBright(true)
            self.preButton:setHighlighted(false)
        else
            self.preButton:setTouchEnabled(false)
            self.preButton:setBright(false)
            self.preButton:setHighlighted(true)
        end
    end
    
    if self.nextButton ~= nil then
        if _enableNext then
            self.nextButton:setTouchEnabled(true)
            self.nextButton:setBright(true)
            self.nextButton:setHighlighted(false)
        else
            self.nextButton:setTouchEnabled(false)
            self.nextButton:setBright(false)
            self.nextButton:setHighlighted(true)
        end
    end
end

function PicInfoView:updateInfomationAnim()
    if self.scrollView == nil then
        return
    end
    local innerHeight = self.scrollView:getInnerContainer():getContentSize().height
    if self.scrollView:getContentSize().height == innerHeight then
    	return
    end
    
    local drawY = self.scrollView:getInnerContainer():getPositionY()
    drawY = drawY + 1
    if drawY >= innerHeight then
        drawY = -innerHeight
    end
    self.scrollView:getInnerContainer():setPositionY(drawY)
end

function PicInfoView:getHasPisLength()
    local length = 0
    for key, var in pairs(self.picArray) do
    	if var[Enum.PIC_KEY.IS_HAS] then
    		length = length + 1
    	end
    end
    return length
end

function PicInfoView:clean()
    self.scrollView = nil
    PicInfoView.preButton = nil
    PicInfoView.nextButton = nil
    PicInfoView.nameLabel = nil
    PicInfoView.orderLabel = nil
    PicInfoView.rareLabel = nil
    PicInfoView.propImg = nil
    PicInfoView.detailLabel = nil
    
    for var=1, #self.picArray do
        for v=1, #self.picArray[var] do
        	self.picArray[var][v] = nil
        end
    end
end

function PicInfoView:registerControllerListener(onCloseFunc,updateFun)
    local function onConnectController(controller, event)
        if controller == nil or controller == self._firstHolder.controller then
            return
        end

        local deviceInfo = string.format("%s id:%d", controller:getDeviceName(), controller:getDeviceId())
        cclog("PicInfoView deviceInfo ="..deviceInfo)

        if self._firstHolder.controller == nil then
            self._firstHolder.controller = controller
        end
    end

    if lk.LakooManager:getInstance():getIsHasPopView() == true then
        return
    end

    local function onDisconnectedController(controller, event)
        if self._firstHolder.controller == controller then
            self._firstHolder.controller = nil
        end
    end

    local function showButtonState(controller, keyCode, isPressed)
        onConnectController(controller, nil)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else    
            return
        end

        if lk.LakooManager:getInstance():getIsHasPopView() == true then
            return
        end

        local scene = cc.Director:getInstance():getRunningScene()
        local child = scene:getChildren()

        local topSence = child[#child]
        if topSence:getName() ~= "MainPopLayer" then 
            cclog("PicInfoView no top")
            return
        end
        if isPressed then
            if keyCode == cc.ControllerKey.BUTTON_A then
            elseif keyCode == cc.ControllerKey.BUTTON_B then
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key Down:%d",keyCode)
            end
        else
            cclog("PicInfoView keyup ="..keyCode)
            if keyCode == cc.ControllerKey.BUTTON_A then
            elseif keyCode == cc.ControllerKey.BUTTON_B then
                onCloseFunc()
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then

            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
                if self.preButton ~= nil then
                    if self.preButton:isTouchEnabled() then
                        updateFun(true,self.currentPicIndex)
                    end
                end
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
                if self.nextButton ~= nil then
                    if self.nextButton:isTouchEnabled() then
                        updateFun(false,self.currentPicIndex)
                    end
                end
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key up:%d",keyCode)
            end
        end
    end 

    local function onKeyDown(controller, keyCode, event)
        showButtonState(controller, keyCode, true)
    end

    local function onKeyUp(controller, keyCode, event)
        showButtonState(controller, keyCode, false)
    end

    local function onAxisEvent(controller, keyCode, event)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end

        if lk.LakooManager:getInstance():getIsHasPopView() == true then
            return
        end
        local ketStatus = controller:getKeyStatus(keyCode)
        if keyCode == cc.ControllerKey.JOYSTICK_LEFT_X then
            if(ketStatus.value == -1) then
            elseif(ketStatus.value == 1) then

            end
        elseif keyCode == cc.ControllerKey.JOYSTICK_LEFT_Y then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_X then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_Y then
        elseif keyCode == cc.ControllerKey.AXIS_LEFT_TRIGGER then
        elseif keyCode == cc.ControllerKey.AXIS_RIGHT_TRIGGER then
        else
        --do nothing
        end
    end

    local controllerListener = cc.EventListenerController:create()
    controllerListener:registerScriptHandler(onConnectController, cc.Handler.EVENT_CONTROLLER_CONNECTED)
    controllerListener:registerScriptHandler(onDisconnectedController, cc.Handler.EVENT_CONTROLLER_DISCONNECTED)

    controllerListener:registerScriptHandler(onKeyDown, cc.Handler.EVENT_CONTROLLER_KEYDOWN)
    controllerListener:registerScriptHandler(onKeyUp, cc.Handler.EVENT_CONTROLLER_KEYUP)
    controllerListener:registerScriptHandler(onAxisEvent, cc.Handler.EVENT_CONTROLLER_AXIS)

    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(controllerListener, self)
    cc.Controller:startDiscoveryController()
end

return PicInfoView