require "cocos.init"

local UILib = require("common.UILib")
local Enum = require("common.Enum")
local Util = require("common.Util")

local Text = require("conf.Text")
local IDX = require("conf.TextIndex")

local GamePool = require("model.GamePool")
local Android = require("conf.Android")
local cc = cc
local ccui = ccui
local class = class

local TAG = {
    BUT_UPGRADE = 1,
    BUT_BACK = 2,
}

local ControllerHolder = class("ControllerHolder")
function ControllerHolder:ctor()
    self.controller  = nil
    self._holderNode = nil

    self._leftJoystick = nil
    self._rightJoystick = nil

    self._dpadLeft = nil
    self._dpadRight = nil
    self._dpadUp = nil
    self._dpadDown = nil

    self._buttonX = nil
    self._buttonY = nil
    self._buttonA = nil
    self._buttonB = nil

    self._buttonL1 = nil
    self._buttonR1 = nil
    self._buttonL2 = nil
    self._buttonR2 = nil

    self.deviceLabel = nil
    self.externalKeyLabel = nil
end


local UpgradeView = class("UpgradeView",
    function()
        return cc.Node:create()
    end
)

UpgradeView.__index = UpgradeView

UpgradeView.scrollView = nil
UpgradeView.updateEquipmentFun = nil

function UpgradeView:create(upgradeItemInfo)
    local upgradeView = UpgradeView.new()
    upgradeView:init(upgradeItemInfo)

    return upgradeView
end

function UpgradeView:init(itemInfo)
    self:setName("UpgradeView")
    if itemInfo == nil then
    	return
    end
    --##########[禁止穿透]
    local eventListener = cc.EventListenerTouchOneByOne:create()
    local function onTouchBegan(touch, event)         
        return true
    end
    local function onTouchEnded(touch, event)
    end

    eventListener:registerScriptHandler(onTouchBegan,cc.Handler.EVENT_TOUCH_BEGAN)
    eventListener:registerScriptHandler(onTouchEnded,cc.Handler.EVENT_TOUCH_ENDED)
    eventListener:setSwallowTouches(true)
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(eventListener, self)
    --##########
    
    local visibleSize = cc.Director:getInstance():getVisibleSize()
    self:setContentSize(visibleSize)

    local bgLayer = cc.LayerColor:create(UILib.UI_COLOR4B.GRAY)
    bgLayer:setContentSize(visibleSize)
    self:addChild(bgLayer)
    
    local bgImgView = ccui.ImageView:create("p_121.pngHK")
    local pos = UILib.offset(0,0,bgImgView:getContentSize().width,bgImgView:getContentSize().height,UILib.UI_ANCHOR.CENTER)
    bgImgView:setPosition(pos)
    self:addChild(bgImgView)
    
    local oName = itemInfo[Enum.UPGRADE_KEY.O_NAME]
    
    local nextId = itemInfo[Enum.UPGRADE_KEY.N_ID]
    if nextId == nil then--[[已升到满级--]]
        local lkFullLvStr = lk.LakooManager:getInstance():getString(Enum.TEXT.T_PLAYER_LEVEL_FULL)
        local fullLvTitle = string.format(lkFullLvStr,oName)
        if GamePool.getLocalLang() == Enum.LANG.EN then
            fullLvTitle = "MAX level"
        end
        if GamePool.getLocalLang() == Enum.LANG.JP then
            fullLvTitle = "Lv4レベル上限に達して"
        end
        
        local fullLvLabel = CCLabelTTF:create(fullLvTitle, UILib.FONT_NAME_NORMAL, UILib.FONT.MINI)
        fullLvLabel:setColor(UILib.UI_COLOR3B.WHITE)
        pos = UILib.offsetWithSuperSize(0,-13,fullLvLabel:getContentSize().width,fullLvLabel:getContentSize().height,bgImgView:getContentSize().width,bgImgView:getContentSize().height,UILib.UI_ANCHOR.CENTER_TOP)
        fullLvLabel:setPosition(pos)
        bgImgView:addChild(fullLvLabel)
    else
        local upgradeText = string.format("%s.%s",Enum.TEXT_IMG.UPGRADE,Text.get(IDX.PNG_TYPE))
        local upgradeImgView = ccui.ImageView:create(upgradeText)
        pos = UILib.offsetWithSuperSize(0,-5,upgradeImgView:getContentSize().width,upgradeImgView:getContentSize().height,bgImgView:getContentSize().width,bgImgView:getContentSize().height,UILib.UI_ANCHOR.CENTER_TOP)
        upgradeImgView:setPosition(pos)
        bgImgView:addChild(upgradeImgView)
    end
    
    --scrollView 
    local oIcon = UILib.createClipSprite(itemInfo[Enum.UPGRADE_KEY.O_IMG],itemInfo[Enum.UPGRADE_KEY.O_ICON])
    
    self.scrollView = ccui.ScrollView:create()
    self.scrollView:setClippingEnabled(true)
    self.scrollView:setContentSize(cc.size(bgImgView:getContentSize().width-10,oIcon:getContentSize().height))
    self.scrollView:setPosition(cc.p(5,110))
    bgImgView:addChild(self.scrollView)
    
    oIcon:setPosition(cc.p(oIcon:getContentSize().width/2,oIcon:getContentSize().height/2))
    self.scrollView:getInnerContainer():addChild(oIcon)
    local drawX = oIcon:getPositionX()
    local drawY = oIcon:getPositionY()
    
    local oNameLabel = CCLabelTTF:create(oName, UILib.FONT_NAME_NORMAL, UILib.FONT.MINI,UILib)
    oNameLabel:setColor(UILib.UI_COLOR3B.BLACK)
    drawX = drawX + oIcon:getContentSize().width/2 + oNameLabel:getContentSize().width/2
    local pos = cc.p(drawX,drawY)
    oNameLabel:setPosition(pos)
    self.scrollView:getInnerContainer():addChild(oNameLabel)
    
    local sWidth = oNameLabel:getPositionX()+oNameLabel:getContentSize().width/2
    
    if nextId ~= nil then
        local upgradeImg = ccui.ImageView:create(Enum.IMG_NAME_PNGHK.RIGHT_ARROW)
        drawX = drawX + oNameLabel:getContentSize().width/2 + upgradeImg:getContentSize().width/2
        pos = cc.p(drawX, drawY)
        upgradeImg:setPosition(pos)
        self.scrollView:getInnerContainer():addChild(upgradeImg)

        local nIcon = UILib.createClipSprite(itemInfo[Enum.UPGRADE_KEY.N_IMG],itemInfo[Enum.UPGRADE_KEY.N_ICON])
        drawX = drawX + upgradeImg:getContentSize().width/2 + nIcon:getContentSize().width/2
        pos = cc.p(drawX, drawY)
        nIcon:setPosition(pos)
        self.scrollView:getInnerContainer():addChild(nIcon)

        local nName = itemInfo[Enum.UPGRADE_KEY.N_NAME]
        local nNameLabel = CCLabelTTF:create(nName, UILib.FONT_NAME_NORMAL, UILib.FONT.MINI,UILib)
        nNameLabel:setColor(UILib.UI_COLOR3B.BLACK)
        drawX = drawX + nIcon:getContentSize().width/2 + nNameLabel:getContentSize().width/2
        local pos = cc.p(drawX,drawY)
        nNameLabel:setPosition(pos)
        self.scrollView:getInnerContainer():addChild(nNameLabel)

        sWidth = nNameLabel:getPositionX()+nNameLabel:getContentSize().width/2
    end
    if sWidth < self.scrollView:getContentSize().width then
        sWidth = self.scrollView:getContentSize().width
    end
    self.scrollView:setInnerContainerSize(cc.size(sWidth,self.scrollView:getInnerContainerSize().height))
    
    if nextId ~= nil then
        --cost infomation
        local costIcon = UILib.createClipSprite(UILib.IMG_IDX.IMG91,UILib.IN_IMG_IDX.SOULSTONE)
        pos = UILib.offsetWithSuperSize(20,10,costIcon:getContentSize().width,costIcon:getContentSize().height,bgImgView:getContentSize().width,bgImgView:getContentSize().height,UILib.UI_ANCHOR.CENTER_LEFT)
        costIcon:setPosition(pos)
        bgImgView:addChild(costIcon)

        local costStone = itemInfo[Enum.UPGRADE_KEY.O_COST]
        local stone = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.SOUL_STONE)
        local stoneColor = Enum.DIGITAL.RED
        if stone >= costStone then
            stoneColor = Enum.DIGITAL.BEIGE
        end
        local stoneLabel = cc.LabelAtlas:_create(stone, stoneColor, 198/21, 11, string.byte('0'))
        local costStoneLabel = cc.LabelAtlas:_create(">"..costStone, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
        drawX = costIcon:getPositionX()+costIcon:getContentSize().width/2+stoneLabel:getContentSize().width/2
        drawY = costIcon:getPositionY()-stoneLabel:getContentSize().height/2
        pos = cc.p(drawX, drawY)
        stoneLabel:setPosition(pos)
        bgImgView:addChild(stoneLabel)
        drawX = drawX + stoneLabel:getContentSize().width/2 + costStoneLabel:getContentSize().width/2
        pos = cc.p(drawX, drawY)
        costStoneLabel:setPosition(pos)
        bgImgView:addChild(costStoneLabel)

        --lv infomation
        local lvImg = ccui.ImageView:create(string.format("%s.%s",Enum.TEXT_IMG.LEVEL,Text.get(IDX.PNG_TYPE)))
        drawX = costIcon:getPositionX()
        drawY = costIcon:getPositionY()-costIcon:getContentSize().height/2-lvImg:getContentSize().height/2
        pos = cc.p(drawX, drawY)
        lvImg:setPosition(pos)
        bgImgView:addChild(lvImg)

        local needLv = itemInfo[Enum.UPGRADE_KEY.O_NEED_LV]
        local lv = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.LEVEL)
        local lvColor = Enum.DIGITAL.RED
        if lv >= needLv then
            lvColor = Enum.DIGITAL.BEIGE
        end
        local lvLabel = cc.LabelAtlas:_create(lv, lvColor, 198/21, 11, string.byte('0'))
        local needLvLabel = cc.LabelAtlas:_create(">"..needLv, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
        drawX = lvImg:getPositionX()+lvImg:getContentSize().width/2+lvLabel:getContentSize().width + 5
        drawY = lvImg:getPositionY()-lvLabel:getContentSize().height/2
        pos = cc.p(drawX, drawY)
        lvLabel:setPosition(pos)
        bgImgView:addChild(lvLabel)
        drawX = drawX + lvLabel:getContentSize().width/2 + needLvLabel:getContentSize().width/2 - 5
        pos = cc.p(drawX, drawY)
        needLvLabel:setPosition(pos)
        bgImgView:addChild(needLvLabel)
    end
    
    --effect infomation
    local hpStrImg = ccui.ImageView:create(string.format("%s.%s",Enum.TEXT_IMG.HP,Text.get(IDX.PNG_TYPE)))
    
    local effectPanel = cc.LayerColor:create(UILib.UI_COLOR4B.GRAY2)
    effectPanel:setContentSize(bgImgView:getContentSize().width-10,hpStrImg:getContentSize().height)
    pos = cc.p(5,48)
    effectPanel:setPosition(pos)
    bgImgView:addChild(effectPanel)
    
    drawX = 5
    drawY = effectPanel:getContentSize().height/2
    if itemInfo[Enum.UPGRADE_KEY.O_TYPE] == Enum.ITEM_TYPE.CATCH_DATA then
        local monsterStrImg = ccui.ImageView:create(string.format("%s.%s",Enum.TEXT_IMG.DEMON_BLACK,Text.get(IDX.PNG_TYPE)))
        drawX = monsterStrImg:getContentSize().width/2
        pos = cc.p(drawX, drawY)
        monsterStrImg:setPosition(pos)
        effectPanel:addChild(monsterStrImg)
        
        drawX = monsterStrImg:getPositionX()+monsterStrImg:getContentSize().width/2+5
    end

    
    drawX = drawX+hpStrImg:getContentSize().width/2
    pos = cc.p(drawX, drawY)
    hpStrImg:setPosition(pos)
    effectPanel:addChild(hpStrImg)
    
    drawX = drawX+hpStrImg:getContentSize().width/2+5
    
    local oVal = itemInfo[Enum.UPGRADE_KEY.O_HP_VAL]
    if itemInfo[Enum.UPGRADE_KEY.O_TYPE] == Enum.ITEM_TYPE.BOTTLE_DATA then
        oVal = itemInfo[Enum.UPGRADE_KEY.O_PREHP_VAL].."D"--ASCII"D"->"%"
    end
    local oValLab = cc.LabelAtlas:_create(oVal, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
    pos = cc.p(drawX, drawY-oValLab:getContentSize().height/2)
    oValLab:setPosition(pos)
    effectPanel:addChild(oValLab)
    
    drawX = drawX + oValLab:getContentSize().width
    
    if nextId ~= nil then
        local nVal = itemInfo[Enum.UPGRADE_KEY.N_HP_VAL]
        if itemInfo[Enum.UPGRADE_KEY.O_TYPE] == Enum.ITEM_TYPE.BOTTLE_DATA then
            nVal = itemInfo[Enum.UPGRADE_KEY.N_PREHP_VAL].."D"--ASCII"D"->"%"
        end
        
        local arrowImg = ccui.ImageView:create(Enum.IMG_NAME_PNGHK.RIGHT_ARROW)
        drawX = drawX+arrowImg:getContentSize().width/2
        pos = cc.p(drawX, drawY)
        arrowImg:setPosition(pos)
        effectPanel:addChild(arrowImg)

        drawX = drawX + arrowImg:getContentSize().width/2

        local nValLab = cc.LabelAtlas:_create(nVal, Enum.DIGITAL.BLUE, 198/21, 11, string.byte('0'))
        pos = cc.p(drawX, drawY-nValLab:getContentSize().height/2)
        nValLab:setPosition(pos)
        effectPanel:addChild(nValLab)
    end
    
    local oPreMp = itemInfo[Enum.UPGRADE_KEY.O_PREMP_VAL]
    if oPreMp > 0 then
        local effectMpPanel = cc.LayerColor:create(UILib.UI_COLOR4B.GRAY2)
        effectMpPanel:setContentSize(effectPanel:getContentSize())
        pos = cc.p(effectPanel:getPositionX(),effectPanel:getPositionY()-effectMpPanel:getContentSize().height)
        effectMpPanel:setPosition(pos)
        bgImgView:addChild(effectMpPanel)
        
        local mpStrImg = ccui.ImageView:create(string.format("%s.%s",Enum.TEXT_IMG.MP,Text.get(IDX.PNG_TYPE)))
        drawX = 5+mpStrImg:getContentSize().width/2
        pos = cc.p(drawX, drawY)
        mpStrImg:setPosition(pos)
        effectMpPanel:addChild(mpStrImg)
        
        drawX = drawX+mpStrImg:getContentSize().width/2+5

        local oMpVal = oPreMp.."D"--ASCII"D"->"%"
        local oMpValLab = cc.LabelAtlas:_create(oMpVal, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
        pos = cc.p(drawX, drawY-oMpValLab:getContentSize().height/2)
        oMpValLab:setPosition(pos)
        effectMpPanel:addChild(oMpValLab)

        drawX = drawX + oMpValLab:getContentSize().width
        
        if nextId ~= nil then
            local nMpVal = itemInfo[Enum.UPGRADE_KEY.N_PREMP_VAL].."D"--ASCII"D"->"%"

            local arrowImg = ccui.ImageView:create(Enum.IMG_NAME_PNGHK.RIGHT_ARROW)
            drawX = drawX+arrowImg:getContentSize().width/2
            pos = cc.p(drawX, drawY)
            arrowImg:setPosition(pos)
            effectMpPanel:addChild(arrowImg)

            drawX = drawX + arrowImg:getContentSize().width/2

            local nMpValLab = cc.LabelAtlas:_create(nMpVal, Enum.DIGITAL.BLUE, 198/21, 11, string.byte('0'))
            pos = cc.p(drawX, drawY-nMpValLab:getContentSize().height/2)
            nMpValLab:setPosition(pos)
            effectMpPanel:addChild(nMpValLab)
        end
    end
    
    --upgrade and back button
    local function doButtonAction(sender, eventType)
        local tag = sender:getTag()
        if eventType == ccui.TouchEventType.began then 
            sender:setTitleColor(UILib.UI_COLOR3B.BLUE)
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            sender:setTitleColor(UILib.UI_COLOR3B.RED)
            if tag == TAG.BUT_UPGRADE then
                
                local fromPos = GamePool.getEquipPosByType(itemInfo[Enum.UPGRADE_KEY.O_TYPE])
                local result = lk.LakooManager:getInstance():itemOp(-1, true, fromPos, -1, Enum.ITEM_OP.UPGRADE)
                if result  and self.updateEquipmentFun ~= nil then
                    self.updateEquipmentFun()
                end
            end
            self:removeFromParent(true)
        end
    end
    
    local fontSize = UILib.FONT.SMALL
    local butOffsetY = 10
    if oPreMp > 0 then
    	butOffsetY = 3
    end
    if nextId ~= nil then
        local title = Text.get(IDX.UPGRADE)
        local upgradeBut = ccui.Button:create(Enum.IMG_NAME_PNGHK.BUTTON_BG2,Enum.IMG_NAME_PNGHK.BUTTON_BG2,Enum.IMG_NAME_PNGHK.BUTTON_BG2) 
        upgradeBut:setTitleText(title)
        upgradeBut:setTitleColor(UILib.UI_COLOR3B.RED)
        upgradeBut:setTitleFontSize(fontSize)
        pos = UILib.offsetWithSuperSize(20,butOffsetY,upgradeBut:getContentSize().width,upgradeBut:getContentSize().height,bgImgView:getContentSize().width,bgImgView:getContentSize().height,UILib.UI_ANCHOR.BOTTOM_LEFT)
        upgradeBut:setPosition(pos)
        bgImgView:addChild(upgradeBut)
        upgradeBut:setTag(TAG.BUT_UPGRADE)
        upgradeBut:addTouchEventListener(doButtonAction)
    end
    
    local back = Text.get(IDX.BACK)
    local backBut = ccui.Button:create(Enum.IMG_NAME_PNGHK.BUTTON_BG2,Enum.IMG_NAME_PNGHK.BUTTON_BG2,Enum.IMG_NAME_PNGHK.BUTTON_BG2) 
    backBut:setTitleText(back)
    backBut:setTitleColor(UILib.UI_COLOR3B.RED)
    backBut:setTitleFontSize(fontSize)
    pos = UILib.offsetWithSuperSize(-20,butOffsetY,backBut:getContentSize().width,backBut:getContentSize().height,bgImgView:getContentSize().width,bgImgView:getContentSize().height,UILib.UI_ANCHOR.BOTTOM_RIGHT)
    backBut:setPosition(pos)
    bgImgView:addChild(backBut)
    backBut:setTag(TAG.BUT_BACK)
    backBut:addTouchEventListener(doButtonAction)

    local function update()
        self:updateInfomationAnim()
    end
    
    local function onNodeEvent(event)
        if "enterTransitionFinish" == event then
            self.scheduleID = cc.Director:getInstance():getScheduler():scheduleScriptFunc(update, 0, false)
        end
        if "exit" == event then
            if self.scheduleID ~= nil then
                cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.scheduleID)
            end
            self:clean()
        end
    end
    self:registerScriptHandler(onNodeEvent)
    
    local osType = Enum.OS_TYPE.IOS
    osType = lk.LakooManager:getInstance():getOSType()
    if osType == Enum.OS_TYPE.ANDROID and Android.CHANNEL == 4 then
        --joe
        self._firstHolder   = ControllerHolder.new()
        self._firstHolder.controller = nil
        self:registerControllerListener(itemInfo)
        --
    end
end

function UpgradeView:setUpdateEquipmentFun(_updateEquipmentFun)
    self.updateEquipmentFun = _updateEquipmentFun
end


function UpgradeView:updateInfomationAnim()
    if self.scrollView == nil then
        return
    end
    local innerWidth = self.scrollView:getInnerContainer():getContentSize().width
    if self.scrollView:getContentSize().width == innerWidth then
        return
    end

    local drawX = self.scrollView:getInnerContainer():getPositionX()
    drawX = drawX - 1
    if drawX <= -innerWidth then
        drawX = innerWidth
    end
    self.scrollView:getInnerContainer():setPositionX(drawX)
end



function UpgradeView:clean()

end

function UpgradeView:registerControllerListener(itemInfo)
    local function onConnectController(controller, event)
        if controller == nil or controller == self._firstHolder.controller then
            return
        end

        local deviceInfo = string.format("%s id:%d", controller:getDeviceName(), controller:getDeviceId())
        cclog("UpgradeView deviceInfo ="..deviceInfo)

        if self._firstHolder.controller == nil then
            self._firstHolder.controller = controller
        end
    end

    local function onDisconnectedController(controller, event)
        if self._firstHolder.controller == controller then
            self._firstHolder.controller = nil
        end
    end

    local function showButtonState(controller, keyCode, isPressed)
        onConnectController(controller, nil)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else    
            return
        end

        local scene = cc.Director:getInstance():getRunningScene()
        local child = scene:getChildren()

        local topSence = child[#child]
        if topSence:getName() ~= "MainPopLayer" then 
            cclog("UpgradeView no top")
            return
        end

        if lk.LakooManager:getInstance():getIsHasPopView() == true then
            return
        end

        if isPressed then
            if keyCode == cc.ControllerKey.BUTTON_A then
            elseif keyCode == cc.ControllerKey.BUTTON_B then 
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key Down:%d",keyCode)
            end
        else
            cclog("UpgradeView keyup ="..keyCode)
            if keyCode == cc.ControllerKey.BUTTON_A then

                    local fromPos = GamePool.getEquipPosByType(itemInfo[Enum.UPGRADE_KEY.O_TYPE])
                    local result = lk.LakooManager:getInstance():itemOp(-1, true, fromPos, -1, Enum.ITEM_OP.UPGRADE)
                    if result  and self.updateEquipmentFun ~= nil then
                        self.updateEquipmentFun()
                    end
                self:removeFromParent(true)
            elseif keyCode == cc.ControllerKey.BUTTON_B then
                topSence:setStatusBlock(false)
                self:removeFromParent(true)
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then

            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then

            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key up:%d",keyCode)
            end
        end
    end 

    local function onKeyDown(controller, keyCode, event)
        showButtonState(controller, keyCode, true)
    end

    local function onKeyUp(controller, keyCode, event)
        showButtonState(controller, keyCode, false)
    end

    local function onAxisEvent(controller, keyCode, event)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end

        if lk.LakooManager:getInstance():getIsHasPopView() == true then
            return
        end

        local ketStatus = controller:getKeyStatus(keyCode)
        if keyCode == cc.ControllerKey.JOYSTICK_LEFT_X then
            if(ketStatus.value == -1) then
            elseif(ketStatus.value == 1) then

            end
        elseif keyCode == cc.ControllerKey.JOYSTICK_LEFT_Y then

        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_X then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_Y then
        elseif keyCode == cc.ControllerKey.AXIS_LEFT_TRIGGER then
        elseif keyCode == cc.ControllerKey.AXIS_RIGHT_TRIGGER then
        else
        --do nothing
        end
    end

    local controllerListener = cc.EventListenerController:create()
    controllerListener:registerScriptHandler(onConnectController, cc.Handler.EVENT_CONTROLLER_CONNECTED)
    controllerListener:registerScriptHandler(onDisconnectedController, cc.Handler.EVENT_CONTROLLER_DISCONNECTED)

    controllerListener:registerScriptHandler(onKeyDown, cc.Handler.EVENT_CONTROLLER_KEYDOWN)
    controllerListener:registerScriptHandler(onKeyUp, cc.Handler.EVENT_CONTROLLER_KEYUP)
    controllerListener:registerScriptHandler(onAxisEvent, cc.Handler.EVENT_CONTROLLER_AXIS)

    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(controllerListener, self)

    cc.Controller:startDiscoveryController()
end

return UpgradeView