require "cocos.init"

local UILib = require("common.UILib")
local Enum = require("common.Enum")
local Util = require("common.Util")

local Text = require("conf.Text")
local IDX = require("conf.TextIndex")

local GamePool = require("model.GamePool")
local Android = require("conf.Android")
local cc = cc
local ccui = ccui
local class = class

local TAG = {
}

local ControllerHolder = class("ControllerHolder")
function ControllerHolder:ctor()
    self.controller  = nil
    self._holderNode = nil

    self._leftJoystick = nil
    self._rightJoystick = nil

    self._dpadLeft = nil
    self._dpadRight = nil
    self._dpadUp = nil
    self._dpadDown = nil

    self._buttonX = nil
    self._buttonY = nil
    self._buttonA = nil
    self._buttonB = nil

    self._buttonL1 = nil
    self._buttonR1 = nil
    self._buttonL2 = nil
    self._buttonR2 = nil

    self.deviceLabel = nil
    self.externalKeyLabel = nil
end

local AchievementLayer = class("AchievementLayer",
    function()
        return cc.Node:create()
    end
)

AchievementLayer.__index = AchievementLayer

AchievementLayer.scrollView = nil
AchievementLayer.achievementArray = nil
AchievementLayer.detailLabel = nil
AchievementLayer.focusAni = nil
AchievementLayer.btnTable = {}
AchievementLayer.tapSelected = -1

AchievementLayer.scrollY = -1

function AchievementLayer:create()
    local achievementLayer = AchievementLayer.new()
    achievementLayer:init()

    return achievementLayer
end

function AchievementLayer:init()
    self:setName("AchievementLayer")
    --##########[禁止穿透]
    local eventListener = cc.EventListenerTouchOneByOne:create()
    local function onTouchBegan(touch, event)         
        return true
    end
    local function onTouchEnded(touch, event)
    end

    eventListener:registerScriptHandler(onTouchBegan,cc.Handler.EVENT_TOUCH_BEGAN)
    eventListener:registerScriptHandler(onTouchEnded,cc.Handler.EVENT_TOUCH_ENDED)
    eventListener:setSwallowTouches(true)
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(eventListener, self)
    --##########
    
    local visibleSize = cc.Director:getInstance():getVisibleSize()
    self:setContentSize(visibleSize)

    local bgLayer = cc.LayerColor:create(UILib.UI_COLOR4B.GRAY)
    bgLayer:setContentSize(visibleSize)
    self:addChild(bgLayer)
    
    local bgImgView = ccui.ImageView:create(Enum.IMG_NAME_PNGHK.DETAIL_BG)
    local pos = UILib.offset(0,0,bgImgView:getContentSize().width,bgImgView:getContentSize().height,UILib.UI_ANCHOR.CENTER)
    bgImgView:setPosition(pos)
    self:addChild(bgImgView)
    
    bgImgView:setScale(visibleSize.height/bgImgView:getContentSize().height)
    
    --back button
    local function doBackAction(sender, eventType) 
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            self:removeFromParent(true)
        end
    end
    local backButton = ccui.Button:create(Enum.IMG_NAME_PNGHK.BACK,Enum.IMG_NAME_PNGHK.BACK,Enum.IMG_NAME_PNGHK.BACK)
    local dx = bgImgView:getPositionX()+(bgImgView:getContentSize().width/2-20)*bgImgView:getScale()
    local dy = bgImgView:getPositionY()+(bgImgView:getContentSize().height/2-20)*bgImgView:getScale()
    pos = cc.p(dx,dy)
    backButton:setPosition(pos)
    self:addChild(backButton)
    backButton:addTouchEventListener(doBackAction)
    
    local tAchievement = lk.LakooManager:getInstance():getString(Enum.TEXT.T_ACHIEVEMENT)
    local valAchievement = lk.LakooManager:getInstance():getAchievement()
    local title = string.format("%s：%d/%d",tAchievement,valAchievement,1000)--1000:对应c++部分的MAX_ACHIEVEMENT
    local titleLabel = CCLabelTTF:create(title, UILib.FONT_NAME_NORMAL, UILib.FONT.SMALL_PLUS)
    titleLabel:setColor(UILib.UI_COLOR3B.BLACK)
    pos = cc.p(bgImgView:getPositionX(),backButton:getPositionY())
    titleLabel:setPosition(pos)
    self:addChild(titleLabel)
    
    --achievement detail bg
    local detailBgImg = ccui.ImageView:create(Enum.IMG_NAME_PNGHK.ACHIEVEMENT_BG)
    local scaleX = bgImgView:getContentSize().width*bgImgView:getScaleX()/detailBgImg:getContentSize().width
    detailBgImg:setScaleX(scaleX)
    detailBgImg:setScaleY(0.72)
    dx = bgImgView:getPositionX()
    dy = detailBgImg:getContentSize().height/2*0.72
    pos = cc.p(dx,dy)
    detailBgImg:setPosition(pos)
    self:addChild(detailBgImg)
    
    local sw = (bgImgView:getContentSize().width-10)*bgImgView:getScale()
    
    self.detailLabel = CCLabelTTF:create("Default Text", UILib.FONT_NAME_NORMAL, UILib.FONT.MINI, cc.size(sw, detailBgImg:getContentSize().height*detailBgImg:getScaleY()), kCCTextAlignmentLeft, kCCVerticalTextAlignmentCenter)
    self.detailLabel:setColor(UILib.UI_COLOR3B.WHITE)
    self.detailLabel:setPosition(pos)
    self:addChild(self.detailLabel)
    
    self.scrollView = ccui.ScrollView:create()
    self.scrollView:setClippingEnabled(true)
    local sh = titleLabel:getPositionY()-titleLabel:getContentSize().height/2-5*bgImgView:getScale()-detailBgImg:getContentSize().height/2*0.72-detailBgImg:getPositionY()
    local sSize = cc.size(sw,sh)
    self.scrollView:setContentSize(sSize)
    pos = cc.p(bgImgView:getPositionX()-sw/2,titleLabel:getPositionY()-titleLabel:getContentSize().height/2-sh-5*bgImgView:getScale())
    self.scrollView:setPosition(cc.p(pos))
    self:addChild(self.scrollView)

    self:initAchievementTable()
    
    if self.focusAni == nil then
        ccs.ArmatureDataManager:getInstance():addArmatureFileInfo("FocusAni2.csb")
        self.focusAni = ccs.Armature:create("FocusAni2")
        self.scrollView:addChild(self.focusAni)
        self.focusAni:setLocalZOrder(99)
        self.focusAni:getAnimation():playWithIndex(0)
        self.focusAni:setVisible(false)
    end

    local function update()
--        self:updateInfomationAnim()     
    end
    
    local function onNodeEvent(event)
        if "enterTransitionFinish" == event then
            self.scheduleID = cc.Director:getInstance():getScheduler():scheduleScriptFunc(update, 0, false)
        end
        if "exit" == event then
            if self.scheduleID ~= nil then
                cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.scheduleID)
            end
            self:clean()
        end
    end
    self:registerScriptHandler(onNodeEvent)
    
    if Android.CHANNEL == 4 then
        self.tapSelected = #self.achievementArray
        if self.focusAni ~= nil then
            self.focusAni:setVisible(true)
            self.focusAni:setPosition(cc.p(self.btnTable[self.tapSelected]:getPositionX(),self.btnTable[self.tapSelected]:getPositionY()))
        end
    end
end

--[[
function AchievementLayer:updateInfomationAnim()
    if self.scrollView == nil then
        return
    end
    local innerWidth = self.scrollView:getInnerContainer():getContentSize().width
    if self.scrollView:getContentSize().width == innerWidth then
        return
    end

    local drawX = self.scrollView:getInnerContainer():getPositionX()
    drawX = drawX - 1
    if drawX <= -innerWidth then
        drawX = innerWidth
    end
    self.scrollView:getInnerContainer():setPositionX(drawX)
end
--]]

function AchievementLayer:initAchievementTable()
	if self.scrollView == nil then
		return
	end
	
    local function doTouchAction(sender, eventType) 
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            local tag = sender:getTag()
            self:updateDetailLabel(tag)
            if self.focusAni ~= nil then
            	self.focusAni:setVisible(true)
            	self.focusAni:setPosition(cc.p(sender:getPositionX(),sender:getPositionY()))
            end
        end
    end
	
    local achievementArrayJson = lk.LakooManager:getInstance():getAchievementDataArrayJson()
    --cclog("achievementArrayJson==%s",achievementArrayJson)
    self.achievementArray = Util.json2Table(achievementArrayJson)
    local drawY = 0
    local drawX = 0
    local innerHeight = 0
    local intervalY = 3
    local button1 = nil
    for var=1, #self.achievementArray do
        local achievementData = self.achievementArray[var]
        --bg image is a button
        --local bgImg = ccui.ImageView:create(Enum.IMG_NAME_PNGHK.OPTION_BG)
        local bgImg = ccui.Button:create(Enum.IMG_NAME_PNGHK.OPTION_BG,Enum.IMG_NAME_PNGHK.OPTION_BG,Enum.IMG_NAME_PNGHK.OPTION_BG)
        bgImg:setTag(var)
        bgImg:addTouchEventListener(doTouchAction)
        if var == #self.achievementArray then
        	button1 = bgImg
        end
        bgImg:setScaleX(GamePool.getScaleX()*1.5)
        if var == 1 then
        	drawX = self.scrollView:getContentSize().width/2
        	drawY = bgImg:getContentSize().height/2
        end
        local pos = cc.p(drawX, drawY)
        bgImg:setPosition(pos)
        self.scrollView:getInnerContainer():addChild(bgImg)
        
        local text = string.format("%s+%d",achievementData[Enum.ACH_KEY.NAME],achievementData[Enum.ACH_KEY.ACHIEVEMENT])
        local textLabel = CCLabelTTF:create(text, UILib.FONT_NAME_NORMAL, UILib.FONT.SMALL)
        local textColor = UILib.UI_COLOR3B.GREY
        if achievementData[Enum.ACH_KEY.IS_HAS_CJ] then
            textColor = UILib.UI_COLOR3B.BLACK
        end
        textLabel:setColor(textColor)
        textLabel:setPosition(pos)
        self.scrollView:getInnerContainer():addChild(textLabel)
        
        drawY = drawY + bgImg:getContentSize().height + intervalY
        innerHeight = innerHeight + bgImg:getContentSize().height + intervalY
        
        if Android.CHANNEL == 4 then
            table.insert(self.btnTable,var,bgImg)
            
            if self.scrollY == -1 then
                self.scrollY = bgImg:getContentSize().height + intervalY
            end
        end
    end
    
    local scrollHeight = self.scrollView:getContentSize().height
    if innerHeight < scrollHeight then
    	innerHeight = scrollHeight
    end
    self.scrollView:setInnerContainerSize(cc.size(self.scrollView:getContentSize().width,innerHeight))
    
    --init touch button 1
    doTouchAction(button1, ccui.TouchEventType.ended)
    
    local osType = Enum.OS_TYPE.IOS
    osType = lk.LakooManager:getInstance():getOSType()
    if osType == Enum.OS_TYPE.ANDROID then
        --joe
        self._firstHolder   = ControllerHolder.new()
        self._firstHolder.controller = nil
        self:registerControllerListener(doTouchAction)
        --
    end
end

function AchievementLayer:updateDetailLabel(_tag)
    if self.achievementArray == nil then
    	return
    end
    local achievementData = self.achievementArray[_tag]
    if achievementData == nil then
    	return
    end
    local detail = achievementData[Enum.ACH_KEY.DETAIL]
    local achievement = achievementData[Enum.ACH_KEY.ACHIEVEMENT]
    local tAchievement = lk.LakooManager:getInstance():getString(Enum.TEXT.T_ACHIEVEMENT)
    local detail = string.format("%s\n%s+%d",detail,tAchievement,achievement)
    self.detailLabel:setString(detail) 
end

function AchievementLayer:clean()

end

function AchievementLayer:dpadUp(doTouchAction)
    if self.tapSelected == #self.btnTable then
        return
    end
    
    local pos = self.tapSelected + 1
    self.tapSelected = pos
    
    if self.btnTable~=nil then
        local btn = self.btnTable[pos]
        if btn ~= nil then
            doTouchAction(btn, ccui.TouchEventType.ended)
            
            local innerHeight = self.scrollView:getInnerContainer():getContentSize().height
            if self.scrollView:getContentSize().height == innerHeight then
            end

            local drawY = self.scrollView:getInnerContainer():getPositionY()

            drawY = drawY - self.scrollY

            if drawY >= innerHeight then
                drawY = innerHeight
            end
            self.scrollView:getInnerContainer():setPositionY(drawY)
        end
    end
end

function AchievementLayer:dpadDown(doTouchAction)
    if self.tapSelected == 1 then
        return 
    end
    
    local pos = self.tapSelected - 1
    self.tapSelected = pos

    if self.btnTable~=nil then
        local btn = self.btnTable[pos]
        if btn ~= nil then
            doTouchAction(btn, ccui.TouchEventType.ended)
            
            local innerHeight = self.scrollView:getInnerContainer():getContentSize().height
            if self.scrollView:getContentSize().height == innerHeight then
            end

            local drawY = self.scrollView:getInnerContainer():getPositionY()
            
            drawY = drawY + self.scrollY
            
            if drawY > 0 then
                drawY = 0
            end
            self.scrollView:getInnerContainer():setPositionY(drawY)
            
        end
    end
    
end


function AchievementLayer:registerControllerListener(doTouchAction)
    local function onConnectController(controller, event)
        if controller == nil or controller == self._firstHolder.controller then
            return
        end

        local deviceInfo = string.format("%s id:%d", controller:getDeviceName(), controller:getDeviceId())
        cclog("AchievementLayer deviceInfo ="..deviceInfo)

        if self._firstHolder.controller == nil then
            self._firstHolder.controller = controller
        end
    end

    local function onDisconnectedController(controller, event)
        if self._firstHolder.controller == controller then
            self._firstHolder.controller = nil
        end
    end

    local function showButtonState(controller, keyCode, isPressed)
        onConnectController(controller, nil)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else    
            return
        end

        local scene = cc.Director:getInstance():getRunningScene()
        local child = scene:getChildren()

        local topSence = child[#child]
        if topSence:getName() ~= "MainPopLayer" then 
            cclog("AchievementLayer no top")
            return
        end

        if lk.LakooManager:getInstance():getIsHasPopView() == true then
            return
        end

        if isPressed then
            if keyCode == cc.ControllerKey.BUTTON_A then
            elseif keyCode == cc.ControllerKey.BUTTON_B then
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key Down:%d",keyCode)
            end
        else
            cclog("AchievementLayer keyup ="..keyCode)
            if keyCode == cc.ControllerKey.BUTTON_A then
            elseif keyCode == cc.ControllerKey.BUTTON_B then
                topSence:setSystemBlock(false)
                self:removeFromParent(true)
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
                self:dpadUp(doTouchAction)
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
                self:dpadDown(doTouchAction)
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then

            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then

            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key up:%d",keyCode)
            end
        end
    end 

    local function onKeyDown(controller, keyCode, event)
        showButtonState(controller, keyCode, true)
    end

    local function onKeyUp(controller, keyCode, event)
        showButtonState(controller, keyCode, false)
    end

    local function onAxisEvent(controller, keyCode, event)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end

        if lk.LakooManager:getInstance():getIsHasPopView() == true then
            return
        end

        local ketStatus = controller:getKeyStatus(keyCode)
        if keyCode == cc.ControllerKey.JOYSTICK_LEFT_X then
            if(ketStatus.value == -1) then
            elseif(ketStatus.value == 1) then

            end
        elseif keyCode == cc.ControllerKey.JOYSTICK_LEFT_Y then

        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_X then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_Y then
        elseif keyCode == cc.ControllerKey.AXIS_LEFT_TRIGGER then
        elseif keyCode == cc.ControllerKey.AXIS_RIGHT_TRIGGER then
        else
        --do nothing
        end
    end

    local controllerListener = cc.EventListenerController:create()
    controllerListener:registerScriptHandler(onConnectController, cc.Handler.EVENT_CONTROLLER_CONNECTED)
    controllerListener:registerScriptHandler(onDisconnectedController, cc.Handler.EVENT_CONTROLLER_DISCONNECTED)

    controllerListener:registerScriptHandler(onKeyDown, cc.Handler.EVENT_CONTROLLER_KEYDOWN)
    controllerListener:registerScriptHandler(onKeyUp, cc.Handler.EVENT_CONTROLLER_KEYUP)
    controllerListener:registerScriptHandler(onAxisEvent, cc.Handler.EVENT_CONTROLLER_AXIS)

    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(controllerListener, self)

    cc.Controller:startDiscoveryController()
end

return AchievementLayer