require "cocos.init"
local UILib = require("common.UILib")
local Enum = require("common.Enum")
local GamePool = require("model.GamePool")
local cc = cc
local class = class

local ControllerHolder = class("ControllerHolder")
function ControllerHolder:ctor()
    self.controller  = nil
    self._holderNode = nil

    self._leftJoystick = nil
    self._rightJoystick = nil

    self._dpadLeft = nil
    self._dpadRight = nil
    self._dpadUp = nil
    self._dpadDown = nil

    self._buttonX = nil
    self._buttonY = nil
    self._buttonA = nil
    self._buttonB = nil

    self._buttonL1 = nil
    self._buttonR1 = nil
    self._buttonL2 = nil
    self._buttonR2 = nil

    self.deviceLabel = nil
    self.externalKeyLabel = nil
end

local ControlHelperLayer = class("ControlHelperLayer",
    function()
        return cc.Node:create()
    end
)

ControlHelperLayer.baLayer = nil

function ControlHelperLayer:create()
    local ControlHelperLayer = ControlHelperLayer.new() 
    ControlHelperLayer:init()  
    return ControlHelperLayer
end

function ControlHelperLayer:init()
    self:setName("ControlHelperLayer")
    local visibleSize = cc.Director:getInstance():getVisibleSize()
    self:setContentSize(visibleSize)
    self:setAnchorPoint(UILib.UI_ANCHOR_POINT.CENTER)
    local pos = UILib.offset(0,0,self:getContentSize().width,self:getContentSize().height,UILib.UI_ANCHOR.CENTER)
    self:setPosition(pos)
    
    local bgLayer = cc.LayerColor:create(UILib.UI_COLOR4B.GRAY)
    bgLayer:setContentSize(visibleSize)
    self:addChild(bgLayer)
    
    local bgImg = ccui.ImageView:create(Enum.IMG_NAME_PNGHK.CONTROL_HELP)
    bgImg:setScaleX(GamePool.getScaleX())
    bgImg:setScaleY(GamePool.getScaleY())
    pos = UILib.offsetWithSuperSize(0,0,bgImg:getContentSize().width,bgImg:getContentSize().height,self:getContentSize().width,self:getContentSize().height,UILib.UI_ANCHOR.CENTER)
    bgImg:setPosition(pos)
    self:addChild(bgImg)
    
    local osType = Enum.OS_TYPE.IOS
    osType = lk.LakooManager:getInstance():getOSType()
    if osType == Enum.OS_TYPE.ANDROID then
        --joe
        self._firstHolder   = ControllerHolder.new()
        self._firstHolder.controller = nil
        self:registerControllerListener()
        --
    end
end

function ControlHelperLayer:registerControllerListener()
    local function onConnectController(controller, event)
        if controller == nil or controller == self._firstHolder.controller then
            return
        end

        local deviceInfo = string.format("%s id:%d", controller:getDeviceName(), controller:getDeviceId())
        cclog("ControlHelperLayer deviceInfo ="..deviceInfo)

        if self._firstHolder.controller == nil then
            self._firstHolder.controller = controller
        end
    end

    local function onDisconnectedController(controller, event)
        if self._firstHolder.controller == controller then
            self._firstHolder.controller = nil
        end
    end

    local function showButtonState(controller, keyCode, isPressed)
        onConnectController(controller, nil)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else    
            return
        end

        if lk.LakooManager:getInstance():getIsHasPopView() == true then
            return
        end

        local scene = cc.Director:getInstance():getRunningScene()
        local child = scene:getChildren()

        local topSence = child[#child]
        if topSence:getName() ~= self:getName() then 
            return
        end

        if isPressed then
            if keyCode == cc.ControllerKey.BUTTON_A then
            elseif keyCode == cc.ControllerKey.BUTTON_B then
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_START then
            else
                local ketStatus = string.format("Key Down:%d",keyCode)
            end
        else
            if keyCode == cc.ControllerKey.BUTTON_A then
            elseif keyCode == cc.ControllerKey.BUTTON_B then
                self:removeFromParent(true)
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then

            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then

            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
               
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then

            elseif keyCode == cc.ControllerKey.BUTTON_START then
            else
                local ketStatus = string.format("Key up:%d",keyCode)
            end
        end
    end 

    local function onKeyDown(controller, keyCode, event)
        showButtonState(controller, keyCode, true)
    end

    local function onKeyUp(controller, keyCode, event)
        showButtonState(controller, keyCode, false)
    end

    local function onAxisEvent(controller, keyCode, event)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end

        if lk.LakooManager:getInstance():getIsHasPopView() == true then
            return
        end

        local scene = cc.Director:getInstance():getRunningScene()
        local child = scene:getChildren()

        local topSence = child[#child]
        if topSence:getName() ~= self:getName() then 
            return
        end

        local ketStatus = controller:getKeyStatus(keyCode)
        if keyCode == cc.ControllerKey.JOYSTICK_LEFT_X then

        elseif keyCode == cc.ControllerKey.JOYSTICK_LEFT_Y then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_X then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_Y then
        elseif keyCode == cc.ControllerKey.AXIS_LEFT_TRIGGER then
            if(ketStatus.value == 1) then
            end
        elseif keyCode == cc.ControllerKey.AXIS_RIGHT_TRIGGER then
        else
        --do nothing
        end
    end

    local controllerListener = cc.EventListenerController:create()
    controllerListener:registerScriptHandler(onConnectController, cc.Handler.EVENT_CONTROLLER_CONNECTED)
    controllerListener:registerScriptHandler(onDisconnectedController, cc.Handler.EVENT_CONTROLLER_DISCONNECTED)

    controllerListener:registerScriptHandler(onKeyDown, cc.Handler.EVENT_CONTROLLER_KEYDOWN)
    controllerListener:registerScriptHandler(onKeyUp, cc.Handler.EVENT_CONTROLLER_KEYUP)
    controllerListener:registerScriptHandler(onAxisEvent, cc.Handler.EVENT_CONTROLLER_AXIS)

    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(controllerListener, self)
    cc.Controller:startDiscoveryController()
end
--

return ControlHelperLayer