require "cocos.init"
local UILib = require("common.UILib")
local Enum = require("common.Enum")
local LKConfig = require("conf.LKConfig")

local Text = require("conf.Text")
local IDX = require("conf.TextIndex")

local cc = cc
local ccs = ccs
local class = class

local TAG = {
    ENTER = 22,--攻击 or 事件确认
    SUMMON = 23,--召
    ASSISTANT = 25,--附
    MAGIC_A = 26,--术A
    MAGIC_B = 27,--术B
    CATCH = 34,--捕捉
    CATCH_PANEL = 35,--捕捉容器(包含CATCH)
}

local EquipSoulLayer = class("EquipSoulLayer",
    function()
        local node = cc.Node:create()
        return node
    end
)

local ControllerHolder = class("ControllerHolder")
function ControllerHolder:ctor()
    self.controller  = nil
    self._holderNode = nil

    self._leftJoystick = nil
    self._rightJoystick = nil

    self._dpadLeft = nil
    self._dpadRight = nil
    self._dpadUp = nil
    self._dpadDown = nil

    self._buttonX = nil
    self._buttonY = nil
    self._buttonA = nil
    self._buttonB = nil

    self._buttonL1 = nil
    self._buttonR1 = nil
    self._buttonL2 = nil
    self._buttonR2 = nil

    self.deviceLabel = nil
    self.externalKeyLabel = nil
end

EquipSoulLayer.__index = EquipSoulLayer

EquipSoulLayer.item = nil
EquipSoulLayer.parent = nil
EquipSoulLayer.icon = nil

function EquipSoulLayer.create(_parent, _item)
    local equipSoulLayer = EquipSoulLayer.new()
    equipSoulLayer:init(_parent, _item)

    return equipSoulLayer
end

function EquipSoulLayer:init(_parent, _item)
    self:setName("EquipSoulLayer")
    if _parent == nil then
    	return
    end
    if _item == nil then
    	return
    end
    self.item = _item
    self.parent = _parent
    
    local visibleSize = cc.Director:getInstance():getVisibleSize()
    self:setContentSize(visibleSize)
    
    local eventListener = cc.EventListenerTouchOneByOne:create()
    local function onTouchBegan(touch, event)         
        return true
    end
    local function onTouchEnded(touch, event)
    end

    eventListener:registerScriptHandler(onTouchBegan,cc.Handler.EVENT_TOUCH_BEGAN)
    eventListener:registerScriptHandler(onTouchEnded,cc.Handler.EVENT_TOUCH_ENDED)
    eventListener:setSwallowTouches(true)
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(eventListener, self)
    
    local bgLayer = cc.LayerColor:create(UILib.UI_COLOR4B.GRAY)
    bgLayer:setContentSize(visibleSize)
    self:addChild(bgLayer)
    
    --item icon
    local iconImgIndex = self.item[Enum.ITEM_KEY.ICON_IDX]
    local iconInImgIndex = self.item[Enum.ITEM_KEY.ICON_IN_IDX]
    self.icon = UILib.createClipSprite(iconImgIndex,iconInImgIndex)
    self.icon:setPosition(cc.p(visibleSize.width/2,visibleSize.height/2))
    self:addChild(self.icon)
    
    --back button
    local function doBackAction(sender, eventType) 
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            self:removeFromParent(true)
        end
    end
    local backButton = ccui.Button:create(Enum.IMG_NAME_PNGHK.BACK,Enum.IMG_NAME_PNGHK.BACK,Enum.IMG_NAME_PNGHK.BACK)
    local pos = UILib.offset(0,0,backButton:getContentSize().width,backButton:getContentSize().height,UILib.UI_ANCHOR.TOP_RIGHT)
    backButton:setPosition(pos)
    self:addChild(backButton)
    backButton:addTouchEventListener(doBackAction)
    
    
    --战斗操控控件
    local widget = ccs.GUIReader:getInstance():widgetFromBinaryFile("BattleControllerUI_1/BattleControllerUI_1.csb")
    self:addChild(widget)
    pos = UILib.offset(0,0,widget:getContentSize().width,widget:getContentSize().height,UILib.UI_ANCHOR.BOTTOM_RIGHT)
    widget:setAnchorPoint(UILib.UI_ANCHOR_POINT.CENTER)
    widget:setPosition(pos)

    local summonKey = UILib.getNodeByTag(widget,TAG.SUMMON)
    local assistantKey = UILib.getNodeByTag(widget,TAG.ASSISTANT)
    local magicKeyA = UILib.getNodeByTag(widget,TAG.MAGIC_A)
    local magicKeyB = UILib.getNodeByTag(widget,TAG.MAGIC_B)

    local function doSoulKeyAction(sender, eventType) 
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            local key = sender:getTag()
            self.parent:doEquipSoulCallBack(key)
            self:removeFromParent(true)
        end
    end
    
    

    local keys = {summonKey, assistantKey, magicKeyA, magicKeyB}
    local imgNames = {Enum.TEXT_IMG.SUMMON, Enum.TEXT_IMG.ASSISTANT, Enum.TEXT_IMG.MAGIC, Enum.TEXT_IMG.MAGIC}
    for var=1, #keys do
        if keys[var] ~= nil then
            lk.LakooManager:getInstance():addSoulKeyNode(keys[var], var-1)
            keys[var]:setTag(var-1)
            keys[var]:addTouchEventListener(doSoulKeyAction)

            --添加“召”、“附”、“术”文字图片
            local imgName = string.format("%s.%s",imgNames[var],Text.get(IDX.PNG_TYPE))
            local textImg = ccui.ImageView:create(imgName)
            local pos = UILib.offsetWithSuperSize(5,-5,textImg:getContentSize().width,textImg:getContentSize().height
                ,keys[var]:getContentSize().width,keys[var]:getContentSize().height,UILib.UI_ANCHOR.TOP_LEFT)
            textImg:setPosition(pos)
            keys[var]:addChild(textImg)
        end
    end

    local catchPanel = UILib.getNodeByTag(widget,TAG.CATCH_PANEL)
    if catchPanel ~= nil then
        catchPanel:setVisible(false)
    end 

    --战斗按钮或事件确认按钮
    local enterBut = UILib.getNodeByTag(widget,TAG.ENTER)
    if enterBut ~= nil then
        enterBut:setVisible(false)
    end
    
    self:setName("EquipSoulLayer")
    local osType = Enum.OS_TYPE.IOS
    osType = lk.LakooManager:getInstance():getOSType()
    if osType == Enum.OS_TYPE.ANDROID then
        --joe
        self._firstHolder   = ControllerHolder.new()
        self._firstHolder.controller = nil
        self:registerControllerListener()
        --
    end
end

function EquipSoulLayer:registerControllerListener()
    local function onConnectController(controller, event)
        if controller == nil or controller == self._firstHolder.controller then
            return
        end

        local deviceInfo = string.format("%s id:%d", controller:getDeviceName(), controller:getDeviceId())
        cclog("EquipSoulLayer deviceInfo ="..deviceInfo)

        if self._firstHolder.controller == nil then
            self._firstHolder.controller = controller
        end
    end

    local function onDisconnectedController(controller, event)
        if self._firstHolder.controller == controller then
            self._firstHolder.controller = nil
        end
    end


    local function showButtonState(controller, keyCode, isPressed)
        onConnectController(controller, nil)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else    
            return
        end

        if lk.LakooManager:getInstance():getIsHasPopView() == true then
            return
        end


        local scene = cc.Director:getInstance():getRunningScene()
        local child = scene:getChildren()

        local topSence = child[#child]
        cclog("topSence:getName() = "..topSence:getName())
        if topSence:getName() ~= "MainPopLayer" then
            return
        end
        
        
        if isPressed then
            cclog("EquipSoulLayer keyDown ="..keyCode)
            if keyCode == cc.ControllerKey.BUTTON_A then
            elseif keyCode == cc.ControllerKey.BUTTON_B then
                topSence:setDoingEquipSoul()
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key Down:%d",keyCode)
            end
        else
            cclog("EquipSoulLayer keyup ="..keyCode)
            if keyCode == cc.ControllerKey.BUTTON_A then
            elseif keyCode == cc.ControllerKey.BUTTON_B then
                self.parent:doEquipSoulCallBack(3)
                self:removeFromParent(true)
            elseif keyCode == cc.ControllerKey.BUTTON_X then
                self.parent:doEquipSoulCallBack(2)
                self:removeFromParent(true)
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
                self.parent:doEquipSoulCallBack(1)
                self:removeFromParent(true)
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then

            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then

            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
                self.parent:doEquipSoulCallBack(0)
                self:removeFromParent(true)
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key up:%d",keyCode)
            end
        end
    end 

    local function onKeyDown(controller, keyCode, event)
        showButtonState(controller, keyCode, true)
    end

    local function onKeyUp(controller, keyCode, event)
        showButtonState(controller, keyCode, false)
    end

    local function onAxisEvent(controller, keyCode, event)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end
        
        local scene = cc.Director:getInstance():getRunningScene()
        local child = scene:getChildren()

        local topSence = child[#child]
        if topSence:getName() ~= self:getName() then
            return
        end

        local ketStatus = controller:getKeyStatus(keyCode)
        if keyCode == cc.ControllerKey.JOYSTICK_LEFT_X then
            if(ketStatus.value == -1) then
            elseif(ketStatus.value == 1) then

            end
        elseif keyCode == cc.ControllerKey.JOYSTICK_LEFT_Y then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_X then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_Y then
        elseif keyCode == cc.ControllerKey.AXIS_LEFT_TRIGGER then
            if(ketStatus.value == 0) then
                
            end
        elseif keyCode == cc.ControllerKey.AXIS_RIGHT_TRIGGER then
            if(ketStatus.value == 0) then
            end
        else
        --do nothing
        end
    end

    local controllerListener = cc.EventListenerController:create()
    controllerListener:registerScriptHandler(onConnectController, cc.Handler.EVENT_CONTROLLER_CONNECTED)
    controllerListener:registerScriptHandler(onDisconnectedController, cc.Handler.EVENT_CONTROLLER_DISCONNECTED)

    controllerListener:registerScriptHandler(onKeyDown, cc.Handler.EVENT_CONTROLLER_KEYDOWN)
    controllerListener:registerScriptHandler(onKeyUp, cc.Handler.EVENT_CONTROLLER_KEYUP)
    controllerListener:registerScriptHandler(onAxisEvent, cc.Handler.EVENT_CONTROLLER_AXIS)

    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(controllerListener, self)

    cc.Controller:startDiscoveryController()
end

return EquipSoulLayer