require "cocos.init"

local UILib = require("common.UILib")
local Enum = require("common.Enum")
local Util = require("common.Util")

local Text = require("conf.Text")
local IDX = require("conf.TextIndex")

local cc = cc
local ccui = ccui
local class = class

local TAG = {
}

local ControllerHolder = class("ControllerHolder")
function ControllerHolder:ctor()
    self.controller  = nil
    self._holderNode = nil

    self._leftJoystick = nil
    self._rightJoystick = nil

    self._dpadLeft = nil
    self._dpadRight = nil
    self._dpadUp = nil
    self._dpadDown = nil

    self._buttonX = nil
    self._buttonY = nil
    self._buttonA = nil
    self._buttonB = nil

    self._buttonL1 = nil
    self._buttonR1 = nil
    self._buttonL2 = nil
    self._buttonR2 = nil

    self.deviceLabel = nil
    self.externalKeyLabel = nil
end

local GameDetailLayer = class("GameDetailLayer",
    function()
        return cc.Node:create()
    end
)

GameDetailLayer.__index = GameDetailLayer

GameDetailLayer.scrollView = nil
GameDetailLayer.titleLabel = nil
GameDetailLayer.detailLabel = nil

GameDetailLayer.onPosPercent = 10

function GameDetailLayer:create()
    local detailLayer = GameDetailLayer.new()
    detailLayer:init()

    return detailLayer
end

function GameDetailLayer:init()
    self:setName("GameDetailLayer")
    --##########[禁止穿透]
    local eventListener = cc.EventListenerTouchOneByOne:create()
    local function onTouchBegan(touch, event)         
        return true
    end
    local function onTouchEnded(touch, event)
    end

    eventListener:registerScriptHandler(onTouchBegan,cc.Handler.EVENT_TOUCH_BEGAN)
    eventListener:registerScriptHandler(onTouchEnded,cc.Handler.EVENT_TOUCH_ENDED)
    eventListener:setSwallowTouches(true)
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(eventListener, self)
    --##########
    
    local osType = Enum.OS_TYPE.IOS
    osType = lk.LakooManager:getInstance():getOSType()
    if osType == Enum.OS_TYPE.ANDROID then
        --joe
        self._firstHolder   = ControllerHolder.new()
        self._firstHolder.controller = nil
        self:registerControllerListener()
        --
    end
    
    local visibleSize = cc.Director:getInstance():getVisibleSize()
    self:setContentSize(visibleSize)

    local bgLayer = cc.LayerColor:create(UILib.UI_COLOR4B.GRAY)
    bgLayer:setContentSize(visibleSize)
    self:addChild(bgLayer)
    
    local bgImgView = ccui.ImageView:create(Enum.IMG_NAME_PNGHK.DETAIL_BG)
    local pos = UILib.offset(0,0,bgImgView:getContentSize().width,bgImgView:getContentSize().height,UILib.UI_ANCHOR.CENTER)
    bgImgView:setPosition(pos)
    self:addChild(bgImgView)
    
    bgImgView:setScale(visibleSize.height/bgImgView:getContentSize().height)
    
    --back button
    local function doBackAction(sender, eventType) 
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            self:removeFromParent(true)
        end
    end
    local backButton = ccui.Button:create(Enum.IMG_NAME_PNGHK.BACK,Enum.IMG_NAME_PNGHK.BACK,Enum.IMG_NAME_PNGHK.BACK)
    local dx = bgImgView:getPositionX()+(bgImgView:getContentSize().width/2-20)*bgImgView:getScale()
    local dy = bgImgView:getPositionY()+(bgImgView:getContentSize().height/2-20)*bgImgView:getScale()
    pos = cc.p(dx,dy)
    backButton:setPosition(pos)
    self:addChild(backButton)
    backButton:addTouchEventListener(doBackAction)
    
    self.titleLabel = CCLabelTTF:create("Default Title", UILib.FONT_NAME_NORMAL, UILib.FONT.MIDDLE)
    self.titleLabel:setColor(UILib.UI_COLOR3B.BLACK)
    pos = cc.p(bgImgView:getPositionX(),backButton:getPositionY())
    self.titleLabel:setPosition(pos)
    self:addChild(self.titleLabel)
    
    --[[
    local testNode = cc.LayerColor:create(UILib.UI_COLOR4B.GREEN)
    local sw = (bgImgView:getContentSize().width-10)*bgImgView:getScale()
    local sh = (bgImgView:getContentSize().height-38)*bgImgView:getScale()
    local sSize = cc.size(sw,sh)
    testNode:setContentSize(sSize)
    pos = cc.p(bgImgView:getPositionX()-sw/2,bgImgView:getPositionY()-sh/2-13*bgImgView:getScale())
    testNode:setPosition(pos)
    self:addChild(testNode)
    ]]--
    
    self.scrollView = ccui.ScrollView:create()
    self.scrollView:setClippingEnabled(true)
    local sw = (bgImgView:getContentSize().width-10)*bgImgView:getScale()
    local sh = (bgImgView:getContentSize().height-38)*bgImgView:getScale()
    local sSize = cc.size(sw,sh)
    self.scrollView:setContentSize(sSize)
    pos = cc.p(bgImgView:getPositionX()-sw/2,bgImgView:getPositionY()-sh/2-13*bgImgView:getScale())
    self.scrollView:setPosition(cc.p(pos))
    self:addChild(self.scrollView)

    local function update()
        --self:updateInfomationAnim()
    end
    
    local function onNodeEvent(event)
        if "enterTransitionFinish" == event then
            self.scheduleID = cc.Director:getInstance():getScheduler():scheduleScriptFunc(update, 0, false)
        end
        if "exit" == event then
            if self.scheduleID ~= nil then
                cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.scheduleID)
            end
            self:clean()
        end
    end
    self:registerScriptHandler(onNodeEvent)
end

--[[
function GameDetailLayer:updateInfomationAnim()
    if self.scrollView == nil then
        return
    end
    local innerWidth = self.scrollView:getInnerContainer():getContentSize().width
    if self.scrollView:getContentSize().width == innerWidth then
        return
    end

    local drawX = self.scrollView:getInnerContainer():getPositionX()
    drawX = drawX - 1
    if drawX <= -innerWidth then
        drawX = innerWidth
    end
    self.scrollView:getInnerContainer():setPositionX(drawX)
end
]]--

function GameDetailLayer:setTitle(_title)
    if self.titleLabel == nil then
    	return
    end
    self.titleLabel:setString(_title)
end

function GameDetailLayer:setDetail(_detail)
    if self.scrollView == nil then
    	return
    end
    if self.detailLabel ~= nil then
    	self.detailLabel:removeFromParent(true)
    end
    
    --cc.size(self.scrollView:getContentSize().width, 0):自动调整高度
    self.detailLabel = CCLabelTTF:create(_detail, UILib.FONT_NAME_NORMAL, UILib.FONT.SMALL, cc.size(self.scrollView:getContentSize().width, 0), kCCVerticalTextAlignmentTop, kCCTextAlignmentLeft)
    self.detailLabel:setColor(UILib.UI_COLOR3B.BLACK)
    pos = cc.p(self.detailLabel:getContentSize().width/2,self.detailLabel:getContentSize().height/2)
    self.detailLabel:setPosition(pos)
    self.scrollView:getInnerContainer():addChild(self.detailLabel)
    self.scrollView:setInnerContainerSize(cc.size(self.scrollView:getInnerContainerSize().width,self.detailLabel:getContentSize().height ))
end


function GameDetailLayer:clean()
    
end

function GameDetailLayer:dpadUp()
    if self.scrollView ~= nil then
        local pos = GameDetailLayer.onPosPercent - 10
        if pos <10 then
            return 
        end
        GameDetailLayer.onPosPercent = pos
        self.scrollView:scrollToPercentVertical(pos,0.3,false)
    end
end

function GameDetailLayer:dpadDown()
    cclog("dpadDown") 
    if self.scrollView ~= nil then
        local pos = GameDetailLayer.onPosPercent + 10
        cclog("pos="..pos) 
        if pos >100 then
            return 
        end
        GameDetailLayer.onPosPercent = pos
        self.scrollView:scrollToPercentVertical(pos,0.3,false)
    end
end

function GameDetailLayer:registerControllerListener()
    local function onConnectController(controller, event)
        if controller == nil or controller == self._firstHolder.controller then
            return
        end

        local deviceInfo = string.format("%s id:%d", controller:getDeviceName(), controller:getDeviceId())
        cclog("GameDetailLayer deviceInfo ="..deviceInfo)

        if self._firstHolder.controller == nil then
            self._firstHolder.controller = controller
        end
    end

    local function onDisconnectedController(controller, event)
        if self._firstHolder.controller == controller then
            self._firstHolder.controller = nil
        end
    end

    local function showButtonState(controller, keyCode, isPressed)
        onConnectController(controller, nil)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else    
            return
        end

        local scene = cc.Director:getInstance():getRunningScene()
        local child = scene:getChildren()

        local topSence = child[#child]
        if topSence:getName() ~= "TitleScene" and topSence:getName() ~= "MainPopLayer" then 
            cclog("GameDetailLayer no top")
            return
        end

        if lk.LakooManager:getInstance():getIsHasPopView() == true then
            return
        end

        if isPressed then
            if keyCode == cc.ControllerKey.BUTTON_A then
            elseif keyCode == cc.ControllerKey.BUTTON_B then
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key Down:%d",keyCode)
            end
        else
            cclog("GameDetailLayer keyup ="..keyCode)
            if keyCode == cc.ControllerKey.BUTTON_A then
            elseif keyCode == cc.ControllerKey.BUTTON_B then
                if topSence:getName() == "MainPopLayer" then
                    topSence:setSystemBlock(false)
                end
                self:removeFromParent(true)
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
                self:dpadUp()
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
                self:dpadDown()
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then

            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then

            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key up:%d",keyCode)
            end
        end
    end 

    local function onKeyDown(controller, keyCode, event)
        showButtonState(controller, keyCode, true)
    end

    local function onKeyUp(controller, keyCode, event)
        showButtonState(controller, keyCode, false)
    end

    local function onAxisEvent(controller, keyCode, event)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end

        if lk.LakooManager:getInstance():getIsHasPopView() == true then
            return
        end

        local ketStatus = controller:getKeyStatus(keyCode)
        if keyCode == cc.ControllerKey.JOYSTICK_LEFT_X then
            if(ketStatus.value == -1) then
            elseif(ketStatus.value == 1) then

            end
        elseif keyCode == cc.ControllerKey.JOYSTICK_LEFT_Y then

        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_X then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_Y then
        elseif keyCode == cc.ControllerKey.AXIS_LEFT_TRIGGER then
        elseif keyCode == cc.ControllerKey.AXIS_RIGHT_TRIGGER then
        else
        --do nothing
        end
    end

    local controllerListener = cc.EventListenerController:create()
    controllerListener:registerScriptHandler(onConnectController, cc.Handler.EVENT_CONTROLLER_CONNECTED)
    controllerListener:registerScriptHandler(onDisconnectedController, cc.Handler.EVENT_CONTROLLER_DISCONNECTED)

    controllerListener:registerScriptHandler(onKeyDown, cc.Handler.EVENT_CONTROLLER_KEYDOWN)
    controllerListener:registerScriptHandler(onKeyUp, cc.Handler.EVENT_CONTROLLER_KEYUP)
    controllerListener:registerScriptHandler(onAxisEvent, cc.Handler.EVENT_CONTROLLER_AXIS)

    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(controllerListener, self)

    cc.Controller:startDiscoveryController()
end

return GameDetailLayer