require "cocos.init"
local UILib = require("common.UILib")
local Enum = require("common.Enum")

local WarningLayer = require("gameUI.layer.WarningLayer")

local GamePool = require("model.GamePool")

local cc = cc
local class = class

local TAG = {
    }

local LANG_KEY = {
    EN = 1,
    JP = 2,
    ZH_CN = 3,
    ZH_HK = 4,
}

local ControllerHolder = class("ControllerHolder")
function ControllerHolder:ctor()
    self.controller  = nil
    self._holderNode = nil

    self._leftJoystick = nil
    self._rightJoystick = nil

    self._dpadLeft = nil
    self._dpadRight = nil
    self._dpadUp = nil
    self._dpadDown = nil

    self._buttonX = nil
    self._buttonY = nil
    self._buttonA = nil
    self._buttonB = nil

    self._buttonL1 = nil
    self._buttonR1 = nil
    self._buttonL2 = nil
    self._buttonR2 = nil

    self.deviceLabel = nil
    self.externalKeyLabel = nil
end


local LanguageLayer = class("LanguageLayer",
    function()
        return cc.Node:create()
    end
)

LanguageLayer.__index = LanguageLayer

LanguageLayer.selectKey = LANG_KEY.EN
LanguageLayer.isCallBackClose = false

LanguageLayer.focusAni = nil
LanguageLayer.tapSelected = -1
LanguageLayer._firstHolder = nil
LanguageLayer.btnTable = {}

function LanguageLayer:create()
    local languageLayer = LanguageLayer.new()
    languageLayer:init()
    return languageLayer
end

function LanguageLayer:init()
    self:setName("LanguageLayer")
    local visibleSize = cc.Director:getInstance():getVisibleSize()
    self:setContentSize(visibleSize)
    self:setAnchorPoint(UILib.UI_ANCHOR_POINT.CENTER)
    local pos = UILib.offset(0,0,self:getContentSize().width,self:getContentSize().height,UILib.UI_ANCHOR.CENTER)
    self:setPosition(pos)

    ---#####禁止穿透
    local eventListener = cc.EventListenerTouchOneByOne:create()
    local function onTouchBegan(touch, event)
        return true
    end
    local function onTouchEnded(touch, event)
    end

    eventListener:registerScriptHandler(onTouchBegan,cc.Handler.EVENT_TOUCH_BEGAN)
    eventListener:registerScriptHandler(onTouchEnded,cc.Handler.EVENT_TOUCH_ENDED)
    eventListener:setSwallowTouches(true)
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(eventListener, self)
    ---#####

    local bgImg = ccui.ImageView:create(Enum.IMG_NAME_PNGHK.LANG_BG)
    bgImg:setScaleX(GamePool.getScaleX())
    bgImg:setScaleY(GamePool.getScaleY())
    pos = UILib.offsetWithSuperSize(0,0,bgImg:getContentSize().width,bgImg:getContentSize().height,self:getContentSize().width,self:getContentSize().height,UILib.UI_ANCHOR.CENTER)
    bgImg:setPosition(pos)
    self:addChild(bgImg)

    --back button
    local function doBackAction(sender, eventType)
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            self:removeFromParent(true)
        end
    end
    local backButton = ccui.Button:create(Enum.IMG_NAME_PNGHK.BACK,Enum.IMG_NAME_PNGHK.BACK,Enum.IMG_NAME_PNGHK.BACK)
    pos = UILib.offset(0,0,backButton:getContentSize().width,backButton:getContentSize().height,UILib.UI_ANCHOR.TOP_RIGHT)
    backButton:setPosition(pos)
    self:addChild(backButton)
    backButton:addTouchEventListener(doBackAction)


    --lang option button
    local function doLangAction(sender, eventType)
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            local key = sender:getTag()
            self.selectKey = key
            local popLayer = WarningLayer:create()
            local msgTabel = {"Select Language:English？","日本語に変換しますか？","是否选择简体中文？","是否選擇繁體中文？"}
            local okTable = {"YES","はい","是","是"}
            local cancelTable = {"NO","いいえ","否","否"}
            popLayer:setString(msgTabel[key],okTable[key],cancelTable[key])
            popLayer:setDelegate(self)
            self:addChild(popLayer)
        end
    end
    local strTable = {"Language：English","言語：日本語","语言：简体中文","語言：繁體中文"}
    local drawX = self:getContentSize().width/2
    local drawY = 0
    local intervalY = 5
    for key, var in ipairs(strTable) do
        local langBut = ccui.Button:create(Enum.IMG_NAME_PNGHK.OPTION_BG,Enum.IMG_NAME_PNGHK.OPTION_BG,Enum.IMG_NAME_PNGHK.OPTION_BG)
        langBut:setTitleText(var)
        langBut:setTitleColor(UILib.UI_COLOR3B.BLACK)
        langBut:setTitleFontSize(UILib.FONT.SMALL)
        langBut:setTag(key)
        if key == 1 then
            drawY = self:getContentSize().height/2 + langBut:getContentSize().height*1.5 + intervalY*1.5
        else
            drawY  = drawY - langBut:getContentSize().height - intervalY
        end
        cclog("x = "..drawX..",drawY"..drawY)
        pos = cc.p(drawX,drawY)
        langBut:setPosition(pos)
        self:addChild(langBut)
        langBut:addTouchEventListener(doLangAction)
        table.insert(self.btnTable,key,langBut)
    end

    local function update()
        if self.isCallBackClose then
            self:getParent():getParent():doLangaugeCallBack()--callcack TitleLayer
            self:removeFromParent(true)
            return
        end
    end

    local function onNodeEvent(event)
        if "enterTransitionFinish" == event then
            self.scheduleID = cc.Director:getInstance():getScheduler():scheduleScriptFunc(update, 0, false)
        end
        if "exit" == event then
            if self.scheduleID ~= nil then
                cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.scheduleID)
            end
        end
    end
    self:registerScriptHandler(onNodeEvent)

    local osType = Enum.OS_TYPE.IOS
    osType = lk.LakooManager:getInstance():getOSType()
    if GamePool.isAndroidTV() then
        --joe
        if self._firstHolder == nil then
            self._firstHolder  = ControllerHolder.new()
            self._firstHolder.controller = nil
            self:registerControllerListener(doLangAction)
        end
        --

        if self.focusAni == nil then
            ccs.ArmatureDataManager:getInstance():addArmatureFileInfo("FocusAni2.csb")
            self.focusAni = ccs.Armature:create("FocusAni2")
            self:addChild(self.focusAni)
            self.focusAni:setLocalZOrder(99)
            self.focusAni:getAnimation():playWithIndex(0)
            self.focusAni:setVisible(false)

            if self.btnTable ~= nil then
                local btn = self.btnTable[1]
                self.tapSelected = 1
                if btn ~= nil then
                    local pos = cc.p(btn:getPositionX(),btn:getPositionY())
                    self.focusAni:setPosition(pos)
                    self.focusAni:setVisible(true)
                end
            end
        end
    end
end

---WarningLayer callBack
function LanguageLayer:onActionFunc()
    local langTable = {Enum.LANG.EN,Enum.LANG.JP,Enum.LANG.ZH_CN,Enum.LANG.ZH_HK}
    GamePool.saveLocalLang(langTable[self.selectKey])
    self.isCallBackClose = true
end

function LanguageLayer:dpadTap(doLangAction)
    if self.tapSelected >0 and self.tapSelected <5 then
        local btn = self.btnTable[self.tapSelected]
        
        if btn ~= nil then
            doLangAction(btn,ccui.TouchEventType.ended)
        end
    end
end

function LanguageLayer:dpadUp()
    local pos = self.tapSelected - 1
    if pos < 1 then
        return
    end

    self.tapSelected = pos

    if self.focusAni ~=nil then
        local btn = self.btnTable[pos]
        if btn ~= nil then
            local pos = cc.p(btn:getPositionX(),btn:getPositionY())
            self.focusAni:setPosition(pos)
            self.focusAni:setVisible(true)
        end
    end
end

function LanguageLayer:dpadDown()
    local pos = self.tapSelected + 1
    if pos > 4 then
        return
    end

    self.tapSelected = pos

    if self.focusAni ~=nil then
        local btn = self.btnTable[pos]
        if btn ~= nil then
            local pos = cc.p(btn:getPositionX(),btn:getPositionY())
            self.focusAni:setPosition(pos)
            self.focusAni:setVisible(true)
        end
    end
end

function LanguageLayer:registerControllerListener(doLangAction)
    local function onConnectController(controller, event)
        if controller == nil or controller == self._firstHolder.controller then
            return
        end

        local deviceInfo = string.format("%s id:%d", controller:getDeviceName(), controller:getDeviceId())
        cclog("LanguageLayer deviceInfo ="..deviceInfo)

        if self._firstHolder.controller == nil then
            self._firstHolder.controller = controller
        end
    end

    local function onDisconnectedController(controller, event)
        if self._firstHolder.controller == controller then
            self._firstHolder.controller = nil
        end
    end

    local function showButtonState(controller, keyCode, isPressed)
        onConnectController(controller, nil)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end

        local scene = cc.Director:getInstance():getRunningScene()
        local child = scene:getChildren()

        local topSence = child[#child]
        if topSence:getName() ~= "TitleScene" then
            return
        end

        if isPressed then
            cclog("LanguageLayer keyDown ="..keyCode)
            if keyCode == cc.ControllerKey.BUTTON_A then
                self:dpadTap(doLangAction)
            elseif keyCode == cc.ControllerKey.BUTTON_B then
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
                self:dpadUp()
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
                self:dpadDown()
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key Down:%d",keyCode)
            end
        else
            cclog("LanguageLayer keyup ="..keyCode)
            if keyCode == cc.ControllerKey.BUTTON_A then

            elseif keyCode == cc.ControllerKey.BUTTON_B then
                self:removeFromParent(true)
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then

            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then

            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then

            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then

            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then

            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then

            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then

            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key up:%d",keyCode)
            end
        end
    end

    local function onKeyDown(controller, keyCode, event)
        showButtonState(controller, keyCode, true)
    end

    local function onKeyUp(controller, keyCode, event)
        showButtonState(controller, keyCode, false)
    end

    local function onAxisEvent(controller, keyCode, event)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end

        local ketStatus = controller:getKeyStatus(keyCode)
        if keyCode == cc.ControllerKey.JOYSTICK_LEFT_X then
            if(ketStatus.value == -1) then
            elseif(ketStatus.value == 1) then

            end
        elseif keyCode == cc.ControllerKey.JOYSTICK_LEFT_Y then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_X then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_Y then
        elseif keyCode == cc.ControllerKey.AXIS_LEFT_TRIGGER then
        elseif keyCode == cc.ControllerKey.AXIS_RIGHT_TRIGGER then
        else
        --do nothing
        end
    end

    local controllerListener = cc.EventListenerController:create()
    controllerListener:registerScriptHandler(onConnectController, cc.Handler.EVENT_CONTROLLER_CONNECTED)
    controllerListener:registerScriptHandler(onDisconnectedController, cc.Handler.EVENT_CONTROLLER_DISCONNECTED)

    controllerListener:registerScriptHandler(onKeyDown, cc.Handler.EVENT_CONTROLLER_KEYDOWN)
    controllerListener:registerScriptHandler(onKeyUp, cc.Handler.EVENT_CONTROLLER_KEYUP)
    controllerListener:registerScriptHandler(onAxisEvent, cc.Handler.EVENT_CONTROLLER_AXIS)

    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(controllerListener, self)
    cc.Controller:startDiscoveryController()
end

return LanguageLayer