require "cocos.init"
local UILib = require("common.UILib")
local Enum = require("common.Enum")
local AbsLayer = require("gameUI.layer.AbsLayer")

local Text = require("conf.Text")
local IDX = require("conf.TextIndex")

local PluginStatus = require("gameUI.layer.PluginStatus")
local PluginBag = require("gameUI.layer.PluginBag")
local PluginMap = require("gameUI.layer.PluginMap")
local PluginSystem = require("gameUI.layer.PluginSystem")

local GamePool = require("model.GamePool")
local Android = require("conf.Android")
local cc = cc
local class = class
local table = table

local TAG = {
    BACKGROUND_UI = 10003,
    BACKGROUND = {
        PANEL = 1,
    },
    TITLE_UI = 10004,
    TITLE = {
        STATUS = 7,
        BAG = 8,
        SOUL = 9,
        COLLECTION = 10,
        MAP = 11,
        SYSTEM = 12,
        BACK = 13,
    },
    STATUS_UI = 10005,
    BAG_UI = 10006,
    MAP_UI = 10007,
    SYSTEM_UI = 10008,
}

local MainPopLayer = class("MainPopLayer",
    function()
        return AbsLayer:create()
    end
)

local ControllerHolder = class("ControllerHolder")
function ControllerHolder:ctor()
    self.controller  = nil
    self._holderNode = nil

    self._leftJoystick = nil
    self._rightJoystick = nil

    self._dpadLeft = nil
    self._dpadRight = nil
    self._dpadUp = nil
    self._dpadDown = nil

    self._buttonX = nil
    self._buttonY = nil
    self._buttonA = nil
    self._buttonB = nil

    self._buttonL1 = nil
    self._buttonR1 = nil
    self._buttonL2 = nil
    self._buttonR2 = nil

    self.deviceLabel = nil
    self.externalKeyLabel = nil
end

MainPopLayer.__index = MainPopLayer

MainPopLayer.subUiTable = {}
MainPopLayer.isLoadedSystemUi = false
MainPopLayer.layerType = Enum.LAYER_TYPE.MAIN_POP

MainPopLayer.tapFoucsAni = nil
MainPopLayer.tapSelected = 1
MainPopLayer.inTitleTap = true
MainPopLayer.oneTapfinish = true

MainPopLayer.isDoingEquipSoul = false

function MainPopLayer:create(onCloseFunc, _layerType)
    local mainPopLayer = MainPopLayer.new()
    mainPopLayer:initLayerInfos(onCloseFunc, _layerType)
    return mainPopLayer
end

function MainPopLayer:initLayerInfos(onCloseFunc, _layerType)
    MainPopLayer.inTitleTap = true
    MainPopLayer.tapSelected = 1
    self.layerType = _layerType
    self:setName("MainPopLayer")
    local mainLayerOut = ccs.SceneReader:getInstance():createNodeWithSceneFile("publish/mainPopLayer.csb")
    local backgroundUi = UILib.getNodeByTag(mainLayerOut,TAG.BACKGROUND_UI)
    local panel = UILib.getNodeByTag(mainLayerOut,TAG.BACKGROUND.PANEL)
    local wSize = panel:getContentSize()
    self:setContentSize(wSize)
    self:setAnchorPoint(UILib.UI_ANCHOR_POINT.CENTER)
    self:addChild(mainLayerOut)

    --状态，背包，妖魂，收集，地图，系统
    local textImgNames = {STATUS = "p_14", BAG = "p_15", SOUL = "p_16", COLLECTION = "p_17"
        , MAP = "p_18", SYSTEM = "p_19"}

    local ACTION_SWITCH = {
        [TAG.TITLE.BACK] = function ()  onCloseFunc(self:getTag())  end,
        [TAG.TITLE.STATUS] = function() self:doActionStatus()   end,
        [TAG.TITLE.BAG] = function() self:doActionBag()   end,
        [TAG.TITLE.SOUL] = function() self:doActionSoul()   end,
        [TAG.TITLE.COLLECTION] = function() self:doActionCollection()   end,
        [TAG.TITLE.MAP] = function() self:doActionMap()   end,
        [TAG.TITLE.SYSTEM] = function() self:doActionSystem()   end,
    }

    local buttons = {}
    local butTexts = {}
    local function touchButton(sender, eventType)
        local tag = sender:getTag()
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            self:updateTitleButtons(buttons, butTexts, tag)
            self:updateShowUi(tag)
            local func = ACTION_SWITCH[tag]
            if func == nil then
                return nil
            else
                func()
            end
        end
    end

    local titleUi = UILib.getNodeByTag(backgroundUi,TAG.TITLE_UI)
    for key, var in pairs(TAG.TITLE) do
        local btn = UILib.getNodeByTag(titleUi,var)
        if btn ~= nil then
            btn:addTouchEventListener(touchButton)
            table.insert(buttons,btn)
            
            if Android.CHANNEL == 4 and self.tapFoucsAni == nil then
                ccs.ArmatureDataManager:getInstance():addArmatureFileInfo("FocusAni.csb")
                self.tapFoucsAni = ccs.Armature:create("FocusAni")
                btn:getParent():addChild(self.tapFoucsAni)
                self.tapFoucsAni:setLocalZOrder(99)
                self.tapFoucsAni:getAnimation():playWithIndex(0)
                self.tapFoucsAni:setVisible(false)
            end

            --add button text img
            local imgN = textImgNames[key]
            if imgN ~= nil then
                local imgName = string.format("%s.%s",imgN,Text.get(IDX.PNG_TYPE))
                local textImg = ccui.ImageView:create(imgName)
                local pos = cc.p(btn:getPositionX(),btn:getPositionY() + btn:getContentSize().height)
                textImg:setPosition(pos)
                btn:getParent():addChild(textImg)
                textImg:setTag(var)
                table.insert(butTexts,textImg)
            end
        end
    end

    local statusUi = UILib.getNodeByTag(backgroundUi,TAG.STATUS_UI)
    if statusUi ~= nil then
        table.insert(self.subUiTable,statusUi)
    end
    local bagUi = UILib.getNodeByTag(backgroundUi,TAG.BAG_UI)
    if bagUi ~= nil then
        table.insert(self.subUiTable,bagUi)
    end
    local mapUi = UILib.getNodeByTag(backgroundUi,TAG.MAP_UI)
    if mapUi ~= nil then
        table.insert(self.subUiTable,mapUi)
    end
    local systemUi = UILib.getNodeByTag(backgroundUi,TAG.SYSTEM_UI)
    if systemUi ~= nil then
        table.insert(self.subUiTable,systemUi)
    end

    --默认选择“状态”
    local selectTag = TAG.TITLE.STATUS
    if self.layerType == Enum.LAYER_TYPE.MAIN_POP_SOUL then
        selectTag = TAG.TITLE.SOUL
    end
    touchButton(UILib.getNodeByTag(titleUi,selectTag),ccui.TouchEventType.ended)

    local function update()
        if PluginSystem.isHasCloseMainPopNotice then
            PluginSystem.setIsHasCloseMainPopNotice(false)
            self:removeFromParent(true)
        end
        if self.layerType == Enum.LAYER_TYPE.MAIN_POP_MAP then
            PluginMap.updateMap()
        end
        if PluginBag.getNeedUpdatePluginStatus() then
            self:doActionStatus()--update PluginStatus
            PluginBag.setNeedUpdatePluginStatus(false)
        end
        if self.layerType == Enum.LAYER_TYPE.MAIN_POP
            and lk.LakooManager:getInstance():getEnableDrawLKLayerInAction() then
            PluginStatus.checkInitAnimNode()
        end
        
        if Android.CHANNEL == 4 then
            if MainPopLayer.inTitleTap then
                for key, var in pairs(buttons) do
                    if var ~= nil then
                        if var:isHighlighted() and self.tapFoucsAni ~= nil then
                            local pos = cc.p(var:getPositionX(), var:getPositionY())
                            self.tapFoucsAni:setPosition(pos)
                            self.tapFoucsAni:setVisible(true)
                        end
                    end
                end
            else
                if self.tapFoucsAni ~= nil then
                    self.tapFoucsAni:setVisible(false)
                end     
            end
        end
    end

    local function onNodeEvent(event)
        if "enterTransitionFinish" == event then
            self.scheduleID = cc.Director:getInstance():getScheduler():scheduleScriptFunc(update, 0, false)
        end

        if "exit" == event then
            for var=1, #self.subUiTable do
                self.subUiTable[var] = nil
            end
            PluginStatus.clean()
            PluginBag.clean()
            PluginMap.clean()
            PluginSystem.clean()

            if self.scheduleID ~= nil then
                cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.scheduleID)
            end
            --MainPopLayer是一个scene,cocos2dx机制问题，scene exit时会将音效关闭
            --这里和ComAudio::onExit中的修改一起使用可避免这个问题
            lk.LakooManager:getInstance():setEnableStopMusic(false)
        end
    end
    self:registerScriptHandler(onNodeEvent)

    local osType = Enum.OS_TYPE.IOS
    osType = lk.LakooManager:getInstance():getOSType()
    if osType == Enum.OS_TYPE.ANDROID then
        --joe
        self._firstHolder   = ControllerHolder.new()
        self._firstHolder.controller = nil
        self:registerControllerListener(onCloseFunc,buttons,butTexts,touchButton,titleUi)
        --
    end
end

function MainPopLayer:doActionStatus()
    cclog("doActionStatus!!!")
    self.layerType = Enum.LAYER_TYPE.MAIN_POP
    local statusUi = self:getSubUiByTag(TAG.STATUS_UI)
    if statusUi ~= nil then
        PluginStatus.init(statusUi)
    end
end

function MainPopLayer:doActionBag()
    cclog("doActionBag!!!")
    self.layerType = Enum.LAYER_TYPE.MAIN_BAG
    local bagUi = self:getSubUiByTag(TAG.BAG_UI)
    if bagUi ~= nil then
        PluginBag.init(bagUi, Enum.BAG_TYPE.BAG)
    end
end

function MainPopLayer:doActionSoul()
    cclog("doActionSoul!!!")
    self.layerType = Enum.LAYER_TYPE.MAIN_POP_SOUL
    --[[不知Android TV为何要去掉这个,这里为恢复代码--]]
    if Android.CHANNEL ~= Enum.CHANNEL.ANDROID_TV_GOOGLE then
        PluginBag.clean()
    end
    local isCanSee = lk.LakooManager:getInstance():isCanSeeSoulPanel()
    if isCanSee == false then
        return
    end
    local bagUi = self:getSubUiByTag(TAG.BAG_UI)
    if bagUi ~= nil then
        PluginBag.init(bagUi, Enum.BAG_TYPE.SOUL)
    end
end

function MainPopLayer:doActionCollection()
    cclog("doActionCollection!!!")
    self.layerType = Enum.LAYER_TYPE.MAIN_POP_COLLECTION
    local bagUi = self:getSubUiByTag(TAG.BAG_UI)
    if bagUi ~= nil then
        PluginBag.init(bagUi, Enum.BAG_TYPE.COLLECTION)
    end
end

function MainPopLayer:doActionMap()
    cclog("doActionMap!!!")
    self.layerType = Enum.LAYER_TYPE.MAIN_POP_MAP
    local mapUi = self:getSubUiByTag(TAG.MAP_UI)
    if mapUi ~= nil then
        PluginMap.init(mapUi)
    end
end

function MainPopLayer:doActionSystem()
    cclog("doActionSystem!!!")
    self.layerType = Enum.LAYER_TYPE.MAIN_POP_SYSTEM
    local systemUi = self:getSubUiByTag(TAG.SYSTEM_UI)
    if systemUi == nil then
        return
    end

    if self.isLoadedSystemUi then
        PluginSystem.initListener()
        return
    end

    PluginSystem.init(systemUi)

    self.isLoadedSystemUi = true
end

function MainPopLayer:updateTitleButtons(buttons, butTexts, selectedTag)
    if selectedTag == TAG.TITLE.BACK then
        return
    end
    for key, var in pairs(buttons) do
        if var ~= nil then
            if var:getTag() == selectedTag then
                var:setHighlighted(true)
            else
                var:setHighlighted(false)
            end
        end
    end
    for key, var in pairs(butTexts) do
        if var ~= nil then
            if var:getTag() == selectedTag then
                var:setVisible(true)
            else
                var:setVisible(false)
            end
        end
    end
end

function MainPopLayer:updateShowUi(selectedTag)
    for key, var in pairs(self.subUiTable) do
        if var ~= nil then
            if (selectedTag == TAG.TITLE.STATUS and var:getTag() == TAG.STATUS_UI)
                or ((selectedTag == TAG.TITLE.BAG
                or selectedTag == TAG.TITLE.SOUL
                or selectedTag == TAG.TITLE.COLLECTION) and var:getTag() == TAG.BAG_UI)
                or (selectedTag == TAG.TITLE.MAP and var:getTag() == TAG.MAP_UI)
                or (selectedTag == TAG.TITLE.SYSTEM and var:getTag() == TAG.SYSTEM_UI) then
                var:setVisible(true)
            else
                var:setVisible(false)
            end
        end
    end
end

function MainPopLayer:getSubUiByTag(tag)
    for var=1, #self.subUiTable do
        if self.subUiTable[var] ~= nil and self.subUiTable[var]:getTag() == tag then
            return self.subUiTable[var]
        end
    end
    return nil
end

--joe
function MainPopLayer:registerControllerListener(onCloseFunc,buttons,butTexts,touchButton,titleUi)
    local function onConnectController(controller, event)
        if controller == nil or controller == self._firstHolder.controller then
            return
        end

        local deviceInfo = string.format("%s id:%d", controller:getDeviceName(), controller:getDeviceId())
        cclog("MainPopLayer deviceInfo ="..deviceInfo)

        if self._firstHolder.controller == nil then
            self._firstHolder.controller = controller
        end
    end

    local function onDisconnectedController(controller, event)
        if self._firstHolder.controller == controller then
            self._firstHolder.controller = nil
        end
    end

    local ACTION_SWITCH = {
        [TAG.TITLE.BACK] = function ()  onCloseFunc(self:getTag())  end,
        [TAG.TITLE.STATUS] = function() self:doActionStatus()   end,
        [TAG.TITLE.BAG] = function() self:doActionBag()   end,
        [TAG.TITLE.SOUL] = function() self:doActionSoul()   end,
        [TAG.TITLE.COLLECTION] = function() self:doActionCollection()   end,
        [TAG.TITLE.MAP] = function() self:doActionMap()   end,
        [TAG.TITLE.SYSTEM] = function() self:doActionSystem()   end,
    }

    local function dpadLeft()
        if MainPopLayer.inTitleTap then
            local tap = MainPopLayer.tapSelected -1
            MainPopLayer.tapSelected = tap
            if MainPopLayer.tapSelected == 1 then
                PluginBag.cleanholer()
                PluginMap.cleanholer()
                PluginSystem.cleanholer()
            elseif MainPopLayer.tapSelected >1 and MainPopLayer.tapSelected <5 then
                PluginStatus.cleanholer()
                PluginMap.cleanholer()
                PluginSystem.cleanholer()
            elseif MainPopLayer.tapSelected == 5 then
                PluginBag.cleanholer()
                PluginStatus.cleanholer()
                PluginSystem.cleanholer()
            elseif MainPopLayer.tapSelected == 6 then
                PluginBag.cleanholer()
                PluginStatus.cleanholer()
                PluginMap.cleanholer()
            end
            touchButton(UILib.getNodeByTag(titleUi,tap+6),ccui.TouchEventType.ended)
        end
    end

    local function dpadRight()
        if MainPopLayer.inTitleTap then
            local tap = MainPopLayer.tapSelected +1
            MainPopLayer.tapSelected = tap
            if MainPopLayer.tapSelected == 1 then
                PluginBag.cleanholer()
                PluginMap.cleanholer()
                PluginSystem.cleanholer()
            elseif MainPopLayer.tapSelected >1 and MainPopLayer.tapSelected <5 then
                PluginStatus.cleanholer()
                PluginMap.cleanholer()
                PluginSystem.cleanholer()
            elseif MainPopLayer.tapSelected == 5 then
                PluginBag.cleanholer()
                PluginStatus.cleanholer()
                PluginSystem.cleanholer()
            elseif MainPopLayer.tapSelected == 6 then
                PluginBag.cleanholer()
                PluginStatus.cleanholer()
                PluginMap.cleanholer()
            end
            touchButton(UILib.getNodeByTag(titleUi,tap+6),ccui.TouchEventType.ended)
        end
    end

    local function dpadDown()
        MainPopLayer.inTitleTap = false
    end

    local function showButtonState(controller, keyCode, isPressed)
        onConnectController(controller, nil)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end

        if lk.LakooManager:getInstance():getIsHasPopView() == true then
            return
        end

        local scene = cc.Director:getInstance():getRunningScene()
        local child = scene:getChildren()

        local topSence = child[#child]
        if topSence:getName() ~= self:getName() then
            return
        end
        
        if self:getSystemBlock() or self:getStatusBlock() or self:getBagBlock() then
            return
        end

        if isPressed then
            cclog("MainPopLayer keyDown ="..keyCode)
            if keyCode == cc.ControllerKey.BUTTON_A then
            elseif keyCode == cc.ControllerKey.BUTTON_B then
            if self.isDoingEquipSoul then
                    cclog("self.isDoingEquipSoul = true")
            end
                if PluginBag.getItemUseType() == true or self.isDoingEquipSoul then
                    self.isDoingEquipSoul = false
                    cclog("MainPopLayer PluginBag.getItemUseType() = true")
                    return
                end
                if self.oneTapfinish then
                    self.oneTapfinish = false
                end
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key Down:%d",keyCode)
            end
        else
            cclog("MainPopLayer keyup ="..keyCode)
            if keyCode == cc.ControllerKey.BUTTON_A then

            elseif keyCode == cc.ControllerKey.BUTTON_B then
                if self.oneTapfinish == false then
                    self.oneTapfinish = true
                    onCloseFunc(self:getTag())
                    PluginBag.cleanholer()
                    PluginStatus.cleanholer()
                    PluginMap.cleanholer()
                    PluginSystem.cleanholer()
                    MainPopLayer.tapSelected = 1
                end
                
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            --                dpadDown()
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
                if MainPopLayer.tapSelected == 1 then
                    return
                end
                dpadLeft()
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
                if MainPopLayer.tapSelected == 6 then
                    return
                end
                dpadRight()
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key up:%d",keyCode)
            end
        end
    end

    local function onKeyDown(controller, keyCode, event)
        showButtonState(controller, keyCode, true)
    end

    local function onKeyUp(controller, keyCode, event)
        showButtonState(controller, keyCode, false)
    end

    local function onAxisEvent(controller, keyCode, event)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end

        local ketStatus = controller:getKeyStatus(keyCode)
        if keyCode == cc.ControllerKey.JOYSTICK_LEFT_X then
            if(ketStatus.value == -1) then
            elseif(ketStatus.value == 1) then

            end
        elseif keyCode == cc.ControllerKey.JOYSTICK_LEFT_Y then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_X then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_Y then
        elseif keyCode == cc.ControllerKey.AXIS_LEFT_TRIGGER then
        elseif keyCode == cc.ControllerKey.AXIS_RIGHT_TRIGGER then
        else
        --do nothing
        end
    end

    local controllerListener = cc.EventListenerController:create()
    controllerListener:registerScriptHandler(onConnectController, cc.Handler.EVENT_CONTROLLER_CONNECTED)
    controllerListener:registerScriptHandler(onDisconnectedController, cc.Handler.EVENT_CONTROLLER_DISCONNECTED)

    controllerListener:registerScriptHandler(onKeyDown, cc.Handler.EVENT_CONTROLLER_KEYDOWN)
    controllerListener:registerScriptHandler(onKeyUp, cc.Handler.EVENT_CONTROLLER_KEYUP)
    controllerListener:registerScriptHandler(onAxisEvent, cc.Handler.EVENT_CONTROLLER_AXIS)

    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(controllerListener, self)
    cc.Controller:startDiscoveryController()
end
--

function MainPopLayer:getMainPopLayerTapType()
    return MainPopLayer.inTitleTap
end

function MainPopLayer:setMainPopLayerTap(tap)
    MainPopLayer.inTitleTap = tap
end

function MainPopLayer:getTitleType()
    return MainPopLayer.tapSelected
end

function MainPopLayer:setItemSignType(type)
    cclog("MainPopLayer:setItemSignType ")
    PluginBag.setItemUseType(type)
    PluginBag.showChoice()
end

function MainPopLayer:getItemSignType()
    PluginBag.getItemUseType()
end

function MainPopLayer:setSystemBlock(type)
    PluginSystem:setBlock(type)
end

function MainPopLayer:getSystemBlock()
    return PluginSystem:getBlock()
end

function MainPopLayer:setStatusBlock(type)
    PluginStatus:setBlock(type)
end

function MainPopLayer:getStatusBlock()
    return PluginStatus:getBlock()
end

function MainPopLayer:setBagBlock(type)
    PluginBag:setBlock(type)
end

function MainPopLayer:getBagBlock()
    return PluginBag:getBlock()
end

function MainPopLayer:setDoingEquipSoul()
    cclog("setDoingEquipSoul ")
    self.isDoingEquipSoul = true
end


return MainPopLayer