require "cocos.init"
local UILib = require("common.UILib")
local Enum = require("common.Enum")
local Config = require("conf.LKConfig")
local AbsLayer = require("gameUI.layer.AbsLayer")

local GamePool = require("model.GamePool")

local cc = cc
local class = class
local table = table

local TAG = {
}

local NoticeLayer = class("NoticeLayer",
    function()
        return AbsLayer:create()
    end
)

NoticeLayer.__index = NoticeLayer

NoticeLayer.layerName = "NoticeLayer"

function NoticeLayer:create(onCloseFunc)
    local noticeLayer = NoticeLayer.new() 
    noticeLayer:initLayerInfos(onCloseFunc)  
    return noticeLayer
end

function NoticeLayer:initLayerInfos(onCloseFunc)
    self:setName(NoticeLayer.layerName)
    local visibleSize = cc.Director:getInstance():getVisibleSize()
    self:setContentSize(visibleSize)

    local bgLayer = cc.LayerColor:create(UILib.UI_COLOR4B.GRAY)
    bgLayer:setContentSize(visibleSize)
    self:addChild(bgLayer)
    
    ---#####禁止穿透
    local eventListener = cc.EventListenerTouchOneByOne:create()
    local function onTouchBegan(touch, event)         
        return true
    end
    local function onTouchEnded(touch, event)
    end

    eventListener:registerScriptHandler(onTouchBegan,cc.Handler.EVENT_TOUCH_BEGAN)
    eventListener:registerScriptHandler(onTouchEnded,cc.Handler.EVENT_TOUCH_ENDED)
    eventListener:setSwallowTouches(true)
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(eventListener, self)
    ---#####
    
    local bgImg = ccui.ImageView:create(Enum.IMG_NAME_PNGHK.NOTICE_BG)
    pos = UILib.offsetWithSuperSize(0,0,bgImg:getContentSize().width,bgImg:getContentSize().height,self:getContentSize().width,self:getContentSize().height,UILib.UI_ANCHOR.CENTER)
    bgImg:setPosition(pos)
    self:addChild(bgImg)
    local sX = visibleSize.width/Config.STANDARD_WIDTH*0.45--此图是以@2x为标准的图
    local sY = visibleSize.height/Config.STANDARD_HEIGHT*0.45
    bgImg:setScaleX(sX)
    bgImg:setScaleY(sY)
  
    local webViewWidth = (bgImg:getContentSize().width-40)*sX
    local webViewHeight = (bgImg:getContentSize().height-40)*sY
    local url = "http://www.lakoo.com"
    local webUrlType = GamePool.getWebUrlType()
    if webUrlType == Enum.WEB_URL_TYPE.TITLE_NOTICE then
    	url = GamePool.getNoticeUrl()
    elseif webUrlType == Enum.WEB_URL_TYPE.CONTACT_US then
        url = "http://forum.lakoo.com"
    end
    lk.LakooManager:getInstance():openWebView(url,"",webViewWidth,webViewHeight)
    
    --mail
    local function doBackAction(sender, eventType)
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            self:removeFromParent(true)
            lk.LakooManager:getInstance():closeWebView()
        end
    end
    local backBut = ccui.Button:create(Enum.IMG_NAME_PNGHK.NOTICE_BACK,Enum.IMG_NAME_PNGHK.NOTICE_BACK,Enum.IMG_NAME_PNGHK.NOTICE_BACK)
    pos = UILib.offset(-bgImg:getContentSize().width/2*sX,bgImg:getContentSize().height/2*sY,backBut:getContentSize().width,backBut:getContentSize().height,UILib.UI_ANCHOR.CENTER)
    backBut:setPosition(pos)
    self:addChild(backBut)
    backBut:setScaleX(sX)
    backBut:setScaleY(sY)
    backBut:addTouchEventListener(doBackAction)
    
    local function onNodeEvent(event)
        if "exit" == event then
        end
    end
    self:registerScriptHandler(onNodeEvent)
end


return NoticeLayer