require "cocos.init"
local UILib = require("common.UILib")
local Enum = require("common.Enum")
local Util = require("common.Util")

local Text = require("conf.Text")
local IDX = require("conf.TextIndex")
local Config = require("conf.LKConfig")

local LayerStage = require("manager.LayerStage")
local ItemInfoView = require("gameUI.component.ItemInfoView")
local PicInfoView = require("gameUI.component.PicInfoView")

local GamePool = require("model.GamePool")
local Android = require("conf.Android")
local table = table

local TAG = {
    BUT_1 = 8,
    BUT_2 = 10,
    BUT_3 = 11,
    SCROLLVIEW = 12,
    BAG_PANEL = 2,--Only MainPopLayer BagUi
    SINGLE_BAG_PANEL = 1,--Only singleBag
    SINGLE_BAG_SUBTITLE = 13,--Only singleBag
    CLOSE = 14,--Only singleBag
    SINGLE_BAG_NPC_SHOP_BUY = 15,
    SINGLE_BAG_NPC_SHOP_SELL = 16,
    SINGLE_BAG_TITLE = 999,--Code custom
}

local GRID_QUANTITY = {
    EQUIPMENT = 15,
    POTION = 15,
    QUEST = 15,
    SUMMON = 15,
    MERGE = 60,
    TRANSMUTEITEM = 40,
    WEAPON = 20,
    ARMOR = 20,
    DEMON = 60,
    MAIN_SHOP = 30,
}

local ControllerHolder = class("ControllerHolder")
function ControllerHolder:ctor()
    self.controller  = nil
    self._holderNode = nil

    self._leftJoystick = nil
    self._rightJoystick = nil

    self._dpadLeft = nil
    self._dpadRight = nil
    self._dpadUp = nil
    self._dpadDown = nil

    self._buttonX = nil
    self._buttonY = nil
    self._buttonA = nil
    self._buttonB = nil

    self._buttonL1 = nil
    self._buttonR1 = nil
    self._buttonL2 = nil
    self._buttonR2 = nil

    self.deviceLabel = nil
    self.externalKeyLabel = nil
end

local P = {}

P.bagUi = nil
P.bagType = Enum.BAG_TYPE.BAG
P.scrollView = nil
P.focusAni = nil
P.layerType = Enum.LAYER_TYPE.NONE
P.itemInfoView = nil
P.picInfoView = nil
P.currentBagTag = Enum.BAG_TAG.EQUIPMENT
P.needUpdatePluginStatus = false
P.shopItemIds = ""
_firstHolder = nil

P.titleButs = {}
P.titleImgs = {}
P.itemGrids = {}
P.itemArray = {}

P.previewIconPanel = nil
P.playerMoneyLabel = nil--npc shop display player total money
P.currentSelectedIndex = -1
P.currentSelectedItemNumLabel = nil
P.stoneLabel = nil

P.tapSelected = -1
P.isMainpopLayer = false
P.TitleCount = 0
P.inChoicetap = false
P.tapfocusAni = nil
P.ScrollY = 0
P.itemUseSign = false
P.tapBlock = false
P.itemSelectBack = false

function P.init(_bagUi, _bagType)
    P.showBag(_bagUi, _bagType, 1)--默认首先打开第一项
end

function P.showBag(_bagUi, _bagType, _butIdx)
    local BAG_ACTION = {
        [TAG.BUT_1] = function ()  P.doBag_Equipment()  end,
        [TAG.BUT_2] = function ()  P.doBag_Potion() end,
        [TAG.BUT_3] = function ()  P.doBag_Quest() end,
    }

    local SOUL_ACTION = {
        [TAG.BUT_1] = function ()  P.doSoul_Summon()  end,
        [TAG.BUT_2] = function ()  P.doSoul_Merge() end,
        [TAG.BUT_3] = function ()  P.doSoul_TransmuteItem() end,
    }

    local COLLECTION_ACTION = {
        [TAG.BUT_1] = function ()  P.doCollection_Weapon()  end,
        [TAG.BUT_2] = function ()  P.doCollection_Armor() end,
        [TAG.BUT_3] = function ()  P.doCollection_Demon() end,
    }

    local function doTitleAction(sender, eventType)
        local tag = sender:getTag()
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            --self:updateTitleButtons(buttons, butTexts, tag)
            local func = nil
            if P.bagType == Enum.BAG_TYPE.BAG then
                func = BAG_ACTION[tag]
            elseif P.bagType == Enum.BAG_TYPE.SOUL then
                func = SOUL_ACTION[tag]
            elseif P.bagType == Enum.BAG_TYPE.COLLECTION then
                func = COLLECTION_ACTION[tag]
            end
            if func == nil then
                return nil
            else
                func()
            end
            P.updateTitleButtonsStatus(tag)
            if P.focusAni ~= nil then
                P.focusAni:setVisible(false)
            end
            if P.itemInfoView ~= nil then
                P.itemInfoView:removeFromParent(true)
                P.itemInfoView = nil
            end
        end
    end

    if P.bagUi == nil then
        P.isMainpopLayer = true
        cclog("PluginBag init!!!")
        P.bagUi = _bagUi
        local button = UILib.getNodeByTag(P.bagUi,TAG.BUT_1)
        button:addTouchEventListener(doTitleAction)
        table.insert(P.titleButs,button)
        button = UILib.getNodeByTag(P.bagUi,TAG.BUT_2)
        button:addTouchEventListener(doTitleAction)
        table.insert(P.titleButs,button)
        button = UILib.getNodeByTag(P.bagUi,TAG.BUT_3)
        button:addTouchEventListener(doTitleAction)
        table.insert(P.titleButs,button)

        if P.tapfocusAni == nil then
            ccs.ArmatureDataManager:getInstance():addArmatureFileInfo("FocusAni.csb")
            P.tapfocusAni = ccs.Armature:create("FocusAni")
            button:getParent():addChild(P.tapfocusAni)
            P.tapfocusAni:setLocalZOrder(99)
            P.tapfocusAni:getAnimation():playWithIndex(0)
            P.tapfocusAni:setVisible(false)
        end

        P.scrollView = UILib.getNodeByTag(P.bagUi,TAG.SCROLLVIEW)
        P.scrollView:setClippingEnabled(true)

        if P.focusAni == nil then
            ccs.ArmatureDataManager:getInstance():addArmatureFileInfo("FocusAni.csb")
            P.focusAni = ccs.Armature:create("FocusAni")
            P.scrollView:addChild(P.focusAni)
            P.focusAni:setLocalZOrder(99)
            P.focusAni:getAnimation():playWithIndex(0)
            P.focusAni:setVisible(false)
        end
    end
    P.bagType = _bagType
    local osType = Enum.OS_TYPE.IOS
    osType = lk.LakooManager:getInstance():getOSType()
    if osType == Enum.OS_TYPE.ANDROID then
        --joe
        if _firstHolder == nil then
            _firstHolder  = ControllerHolder.new()
            _firstHolder.controller = nil
            P:registerControllerListener()
            cclog("pluginBag showBag registerControllerListener")
        else
            cclog("pluginBag showBag _firstHolder~=nil")
        end
        --
    end
    cclog("PluginBag update!!!")
    P.updateTitle()
    --initBag(batType)
    doTitleAction(P.titleButs[_butIdx],ccui.TouchEventType.ended)
    if P.tapSelected == -1 then
        P.tapSelected = 1
    end


end

function P.initSingleBag(_bagUi, _layerType)
    local function doCloseAction(sender, eventType)
        local tag = sender:getTag()
        if eventType == ccui.TouchEventType.ended then
            LayerStage.setHideLayerType(P.layerType)
            --lk.LakooManager:getInstance():setIsHasPopView(false)
            lk.LakooManager:getInstance():setIsPopNpcShop(false)--[[不能用setIsHasPopView替代，否刚购买后弹出提示无法显示在最上层--]]
        end
    end

    if P.bagUi == nil then
        P.isMainpopLayer = false
        cclog("PluginBag initSingleBag!!! P.layerType = ".._layerType)
        P.bagUi = _bagUi
        local closeBut = UILib.getNodeByTag(P.bagUi,TAG.CLOSE)
        closeBut:addTouchEventListener(doCloseAction)

        P.scrollView = UILib.getNodeByTag(P.bagUi,TAG.SCROLLVIEW)
        P.scrollView:setClippingEnabled(true)

        if P.focusAni == nil then
            ccs.ArmatureDataManager:getInstance():addArmatureFileInfo("FocusAni.csb")
            P.focusAni = ccs.Armature:create("FocusAni")
            P.scrollView:addChild(P.focusAni)
            P.focusAni:setLocalZOrder(99)
            P.focusAni:getAnimation():playWithIndex(0)
            P.focusAni:setVisible(false)
        end

        local osType = Enum.OS_TYPE.IOS
        osType = lk.LakooManager:getInstance():getOSType()
        if osType == Enum.OS_TYPE.ANDROID then
            --joe
            if _firstHolder == nil then
                _firstHolder  = ControllerHolder.new()
                _firstHolder.controller = nil
                P:registerControllerListener()
                cclog("pluginBag initSingleBag registerControllerListener")
            else
                cclog("pluginBag initSingleBag _firstHolder~=nil")
            end
            --
        end
    end
    P.layerType = _layerType

    if P.focusAni ~= nil then
        P.focusAni:setVisible(false)
    end
    if P.itemInfoView ~= nil then
        P.itemInfoView:removeFromParent(true)
        P.itemInfoView = nil
    end

    if P.layerType == Enum.LAYER_TYPE.SINGLE_POP_NPC_SHOP then
        P.shopItemIds = lk.LakooManager:getInstance():checkShopItemIds()
        lk.LakooManager:getInstance():setShopItemIds("")
        --lk.LakooManager:getInstance():setIsHasPopView(true)
        lk.LakooManager:getInstance():setIsPopNpcShop(true)--[[不能用setIsHasPopView替代，否刚购买后弹出提示无法显示在最上层--]]
    end
    
    if P.layerType == Enum.LAYER_TYPE.SINGLE_POP_MAIN_SHOP then
        lk.LakooManager:getInstance():setIsPopNpcShop(true)
    end

    P.updateSingleBagTitle()

    if P.layerType == Enum.LAYER_TYPE.SINGLE_POP_EQUIPMENT_WEAPON then
        P.doBag_Equipment_weapon()
    elseif P.layerType == Enum.LAYER_TYPE.SINGLE_POP_EQUIPMENT_ARMOUR then
        P.doBag_Equipment_armour()
    elseif P.layerType == Enum.LAYER_TYPE.SINGLE_POP_EQUIPMENT_ADORN_DATA then
        P.doBag_Equipment_adorn()
    elseif P.layerType == Enum.LAYER_TYPE.SINGLE_POP_MAIN_SHOP then
        P.doMainShop()
    end
end

function P.updateSingleBagTitle()
    P.hadTitle = false
    P.tapSelected = -1
    P.TitleCount = 0
    P.inChoicetap = false
    local imgNames = {
        [Enum.LAYER_TYPE.SINGLE_POP_EQUIPMENT_WEAPON] = "p_43",
        [Enum.LAYER_TYPE.SINGLE_POP_EQUIPMENT_ARMOUR] = "p_43",
        [Enum.LAYER_TYPE.SINGLE_POP_EQUIPMENT_ADORN_DATA] = "p_43",
        [Enum.LAYER_TYPE.SINGLE_POP_MAIN_SHOP] = "p_84",
        [Enum.LAYER_TYPE.SINGLE_POP_NPC_SHOP] = "p_84",
    }

    local textImg = UILib.getNodeByTag(P.bagUi,TAG.SINGLE_BAG_TITLE)
    if textImg ~= nil then
        textImg:removeFromParent(true)
    end
    local curTitleName = imgNames[P.layerType]
    if curTitleName ~= nil then
        local curTitleName = string.format("%s.%s",curTitleName,Text.get(IDX.PNG_TYPE))
        textImg = ccui.ImageView:create(curTitleName)
        textImg:setTag(TAG.SINGLE_BAG_TITLE)
        local panel = UILib.getNodeByTag(P.bagUi,TAG.SINGLE_BAG_PANEL)
        local pos = cc.p((panel:getContentSize().width-textImg:getContentSize().width)/2+20,panel:getContentSize().height-textImg:getContentSize().height*2.2)
        if P.layerType == Enum.LAYER_TYPE.SINGLE_POP_MAIN_SHOP then
            pos.y = pos.y + 8
        end
        textImg:setPosition(pos)
        P.bagUi:addChild(textImg)
    end

    local subTitlePanel = UILib.getNodeByTag(P.bagUi,TAG.SINGLE_BAG_SUBTITLE)
    if subTitlePanel == nil then
        return
    end
    subTitlePanel:removeAllChildren(true)
    if P.layerType == Enum.LAYER_TYPE.SINGLE_POP_EQUIPMENT_WEAPON
        or P.layerType == Enum.LAYER_TYPE.SINGLE_POP_EQUIPMENT_ARMOUR
        or P.layerType == Enum.LAYER_TYPE.SINGLE_POP_EQUIPMENT_ADORN_DATA
        or P.layerType == Enum.LAYER_TYPE.SINGLE_POP_MAIN_SHOP then

        local moneyIcon = UILib.createClipSprite(UILib.IMG_IDX.IMG91,1)
        local pos = cc.p(40,15)
        moneyIcon:setPosition(pos)
        subTitlePanel:addChild(moneyIcon)

        local money = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.MONEY)
        local lab = cc.LabelAtlas:_create(money, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
        lab:setPosition(cc.p(moneyIcon:getPositionX()+moneyIcon:getContentSize().width/2,moneyIcon:getPositionY()-lab:getContentSize().height/2))
        subTitlePanel:addChild(lab)

        local stoneIcon = UILib.createClipSprite(UILib.IMG_IDX.IMG91,2)
        local pos = cc.p(subTitlePanel:getContentSize().width/2,15)
        stoneIcon:setPosition(pos)
        subTitlePanel:addChild(stoneIcon)

        local stone = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.SOUL_STONE)
        P.stoneLabel = cc.LabelAtlas:_create(stone, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
        P.stoneLabel:setPosition(cc.p(stoneIcon:getPositionX()+stoneIcon:getContentSize().width/2,stoneIcon:getPositionY()-P.stoneLabel:getContentSize().height/2))
        subTitlePanel:addChild(P.stoneLabel)
    elseif P.layerType == Enum.LAYER_TYPE.SINGLE_POP_NPC_SHOP then
        local moneyIcon = UILib.createClipSprite(UILib.IMG_IDX.IMG91,1)
        local pos = cc.p(subTitlePanel:getContentSize().width/2,15)
        if GamePool.getLocalLang() == Enum.LANG.EN then
            pos = cc.p(subTitlePanel:getContentSize().width/2+60,15)
        end
        moneyIcon:setPosition(pos)
        subTitlePanel:addChild(moneyIcon)

        local money = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.MONEY)
        P.playerMoneyLabel = cc.LabelAtlas:_create(money, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
        P.playerMoneyLabel:setPosition(cc.p(moneyIcon:getPositionX()+moneyIcon:getContentSize().width/2,moneyIcon:getPositionY()-P.playerMoneyLabel:getContentSize().height/2))
        subTitlePanel:addChild(P.playerMoneyLabel)

        if P.previewIconPanel == nil then
            P.previewIconPanel = cc.Node:create()
            P.previewIconPanel:setContentSize(cc.size(moneyIcon:getContentSize().width*1.5,moneyIcon:getContentSize().height))
            pos = cc.p(P.playerMoneyLabel:getPositionX()+P.playerMoneyLabel:getContentSize().width+10,0)
            P.previewIconPanel:setPosition(pos)
            subTitlePanel:addChild(P.previewIconPanel)
        end
    end

    if P.layerType == Enum.LAYER_TYPE.SINGLE_POP_NPC_SHOP then
        local bgName1 = string.format("%s.%s",Enum.IMG_NAME.BUT_BG_1,Text.get(IDX.PNG_TYPE))
        local bgName2 = string.format("%s.%s",Enum.IMG_NAME.BUT_BG_2,Text.get(IDX.PNG_TYPE))

        local isExist = cc.FileUtils:getInstance():isFileExist(bgName1)
        if isExist ~= true then
            bgName1 = string.format("%s.pngHK",Enum.IMG_NAME.BUT_BG_1)
        end
        isExist = cc.FileUtils:getInstance():isFileExist(bgName2)
        if isExist ~= true then
            bgName2 = string.format("%s.pngHK",Enum.IMG_NAME.BUT_BG_2)
        end

        local function doNpcShopTabAction(sender, eventType)
            if eventType == ccui.TouchEventType.ended then
                local tag = sender:getTag()
                sender:setHighlighted(true)
                if tag == TAG.SINGLE_BAG_NPC_SHOP_BUY then
                    local sellBut = UILib.getNodeByTag(P.bagUi,TAG.SINGLE_BAG_NPC_SHOP_SELL)
                    if sellBut ~= nil then
                        sellBut:setHighlighted(false)
                    end
                    P.doNpcShopBuy()
                else
                    local buyBut = UILib.getNodeByTag(P.bagUi,TAG.SINGLE_BAG_NPC_SHOP_BUY)
                    if buyBut ~= nil then
                        buyBut:setHighlighted(false)
                    end
                    P.doNpcShopSell()
                end

                if P.focusAni ~= nil then
                    P.focusAni:setVisible(false)
                end
                if P.itemInfoView ~= nil then
                    P.itemInfoView:removeFromParent(true)
                    P.itemInfoView = nil
                end
                if P.previewIconPanel ~= nil then
                    P.previewIconPanel:removeAllChildren(true)
                end
            end
        end

        local buyBut = ccui.Button:create(bgName1,bgName2,bgName2)
        local pos = cc.p(buyBut:getContentSize().width/2,buyBut:getContentSize().height/2)
        buyBut:setPosition(pos)
        subTitlePanel:addChild(buyBut)
        local textBuy = string.format("%s.%s",Enum.TEXT_IMG.BUY_NOBG,Text.get(IDX.PNG_TYPE))
        local textImg = ccui.ImageView:create(textBuy)
        textImg:setPosition(pos)
        buyBut:addChild(textImg)
        buyBut:setTag(TAG.SINGLE_BAG_NPC_SHOP_BUY)
        buyBut:addTouchEventListener(doNpcShopTabAction)

        local sellBut = ccui.Button:create(bgName1,bgName2,bgName2)
        local px = buyBut:getPositionX()+buyBut:getContentSize().width/2+sellBut:getContentSize().width/2
        local py = buyBut:getPositionY()
        local pos = cc.p(px,py)
        sellBut:setPosition(pos)
        subTitlePanel:addChild(sellBut)
        local textSell = string.format("%s.%s",Enum.TEXT_IMG.SELL_NOBG,Text.get(IDX.PNG_TYPE))
        local textImg = ccui.ImageView:create(textSell)
        textImg:setPosition(cc.p(sellBut:getContentSize().width/2,sellBut:getContentSize().height/2))
        sellBut:addChild(textImg)
        sellBut:setTag(TAG.SINGLE_BAG_NPC_SHOP_SELL)
        sellBut:addTouchEventListener(doNpcShopTabAction)

        P.hadTitle = true
        P.tapSelected = 1
        P.TitleCount = 2
        P.inChoicetap = true
        doNpcShopTabAction(buyBut, ccui.TouchEventType.ended)--默认选择


        if P.tapfocusAni == nil then
            ccs.ArmatureDataManager:getInstance():addArmatureFileInfo("FocusAni.csb")
            P.tapfocusAni = ccs.Armature:create("FocusAni")
            subTitlePanel:addChild(P.tapfocusAni)
            P.tapfocusAni:setLocalZOrder(99)
            P.tapfocusAni:getAnimation():playWithIndex(0)
            P.tapfocusAni:setVisible(false)
        end

    end
end

---与游戏中的NPC对话后弹出的购买商店
function P.doNpcShopBuy()
    P.removeOldItemGrids()
    if P.shopItemIds == "" then
        return
    end
    local jsonStr = "["..P.shopItemIds.."]"
    local itemIdTable = Util.json2Table(jsonStr)
    local gridsCount = (math.floor((#itemIdTable)/5)+1)*5
    P.addItemGrids(gridsCount)
    P.addNpcShopItemIcons()
end

---与游戏中的NPC对话后弹出的卖出商店
function P.doNpcShopSell()
    cclog("doBag_Equipment")
    P.removeOldItemGrids()
    --local gridsCount = (math.floor(lk.LakooManager:getInstance():getSellItemCount()/5)+1)*5
    --P.addItemGrids(gridsCount)
    P.addItemIconsByBagTag(Enum.BAG_TAG.NPC_SHOP_SELL, 0)
end

---点击主界面"金币"&"魂石"action
function P.doMainShop()
    cclog("doMainShop")
    P.removeOldItemGrids()
    P.addItemGrids(GRID_QUANTITY.MAIN_SHOP)
    P.addIapItemIcons()
end

---点击“状态”中的"饰品"装备项action
function P.doBag_Equipment_adorn()
    cclog("doBag_Equipment_adorn")
    P.removeOldItemGrids()
    --P.addItemGrids(GRID_QUANTITY.EQUIPMENT)
    P.addItemIconsByBagTag(Enum.BAG_TAG.EQUIPMENT_ADORN_DATA, GRID_QUANTITY.EQUIPMENT)
end

---点击“状态”中的"防具"装备项action
function P.doBag_Equipment_armour()
    cclog("doBag_Equipment_armour")
    P.removeOldItemGrids()
    --P.addItemGrids(GRID_QUANTITY.EQUIPMENT)
    P.addItemIconsByBagTag(Enum.BAG_TAG.EQUIPMENT_ARMOUR, GRID_QUANTITY.EQUIPMENT)
end

---点击“状态”中的"武器"装备项action
function P.doBag_Equipment_weapon()
    cclog("doBag_Equipment_weapon")
    P.removeOldItemGrids()
    --P.addItemGrids(GRID_QUANTITY.EQUIPMENT)
    P.addItemIconsByBagTag(Enum.BAG_TAG.EQUIPMENT_WEAPON, GRID_QUANTITY.EQUIPMENT)
end

---装备
function P.doBag_Equipment()
    cclog("doBag_Equipment")
    P.removeOldItemGrids()
    --P.addItemGrids(GRID_QUANTITY.EQUIPMENT)
    P.addItemIconsByBagTag(Enum.BAG_TAG.EQUIPMENT, GRID_QUANTITY.EQUIPMENT)
end

---药品
function P.doBag_Potion()
    cclog("doBag_Potion")
    P.removeOldItemGrids()
    --P.addItemGrids(GRID_QUANTITY.POTION)
    P.addItemIconsByBagTag(Enum.BAG_TAG.POTION, GRID_QUANTITY.POTION)
end

---任务
function P.doBag_Quest()
    cclog("doBag_Quest")
    P.removeOldItemGrids()
    --P.addItemGrids(GRID_QUANTITY.QUEST)
    P.addItemIconsByBagTag(Enum.BAG_TAG.QUEST, GRID_QUANTITY.QUEST)
end

---召唤
function P.doSoul_Summon()
    cclog("doSoul_Summon")
    P.removeOldItemGrids()
    --P.addItemGrids(GRID_QUANTITY.SUMMON)
    P.addItemIconsByBagTag(Enum.BAG_TAG.SUMMON, GRID_QUANTITY.SUMMON)
end

---炼魂
function P.doSoul_Merge()
    cclog("doSoul_Merge")
    P.removeOldItemGrids()
    P.addItemGrids(GRID_QUANTITY.MERGE)
    P.addAllPicIconsByBagTag(Enum.BAG_TAG.MERGE, true)
end

---铸造
function P.doSoul_TransmuteItem()
    cclog("doSoul_TransmuteItem")
    P.removeOldItemGrids()
    P.addItemGrids(GRID_QUANTITY.TRANSMUTEITEM)
    P.addAllPicIconsByBagTag(Enum.BAG_TAG.TRANSMUTEITEM, false)
end

---武器
function P.doCollection_Weapon()
    cclog("doCollection_Weapon")
    P.removeOldItemGrids()
    P.addItemGrids(GRID_QUANTITY.WEAPON)
    P.addPicIconsByBagTag(Enum.BAG_TAG.WEAPON)
end

---防具
function P.doCollection_Armor()
    cclog("doCollection_Armor")
    P.removeOldItemGrids()
    P.addItemGrids(GRID_QUANTITY.ARMOR)
    P.addPicIconsByBagTag(Enum.BAG_TAG.ARMOR)
end

---妖怪
function P.doCollection_Demon()
    cclog("doCollection_Demon")
    P.removeOldItemGrids()
    P.addItemGrids(GRID_QUANTITY.DEMON)
    P.addPicIconsByBagTag(Enum.BAG_TAG.DEMON)
end

function P.addItemGrids(quantity)
    if quantity <= 0 then
        return
    end
    if P.scrollView == nil then
        return
    end

    local row = math.ceil(quantity/5)
    --更新加载内容容器的大小
    if row >= 3 then
        row = row + 1--[[修正物品项超过三行时最后一行会被物品详细信息框阻挡无法点击问题--]]
    end
    local innerHeight = row*65
    local contentHeight = P.scrollView:getContentSize().height
    if innerHeight < contentHeight then
        innerHeight = contentHeight
    end
    P.scrollView:setInnerContainerSize(cc.size(P.scrollView:getInnerContainerSize().width, innerHeight))
    P.scrollView:jumpToTop()

    local startX = 65
    local startY = 28
    local posX = startX
    local posY = P.scrollView:getInnerContainerSize().height - startY
    local intervarX = (P.scrollView:getContentSize().width - startX*2)/4--5列
    local intervarY = (P.scrollView:getContentSize().height - startY*2)/2--3行
    P.ScrollY = intervarY
    for var=1, quantity do
        if var % 5 == 1 and var ~= 1 then
            posY = posY - intervarY
        end
        if var % 5 == 1 then
            posX = startX
        else
            posX = posX + intervarX
        end
        local img = ccui.ImageView:create("p_26.pngHK")
        local pos = cc.p(posX,posY)
        img:setPosition(pos)
        P.scrollView:addChild(img)

        table.insert(P.itemGrids,img)
    end
end

---显示图鉴详细信息
--@param picIndex int item在itemArray中的序列号
function P.showPicInfo(picIndex)
    local enablePre = false
    local enableNext = false
    for var=1, picIndex-1 do
        enablePre = P.itemArray[var][Enum.PIC_KEY.IS_HAS]
        if enablePre then
            break
        end
    end
    for var=picIndex+1, #P.itemArray do
        enableNext = P.itemArray[var][Enum.PIC_KEY.IS_HAS]
        if enableNext then
            break
        end
    end

    P.tapBlock = true

    if P.picInfoView == nil then
        local tag = TAG.SINGLE_BAG_PANEL
        if P.layerType == Enum.LAYER_TYPE.NONE then
            tag = TAG.BAG_PANEL
        end
        local panel = UILib.getNodeByTag(P.bagUi,tag)
        if panel == nil then
            return
        end
        P.picInfoView = PicInfoView:create(P.currentBagTag, P.itemArray, picIndex, enablePre, enableNext, P.doUpdatePicAction, P.doClosePicCallBack)
        if P.picInfoView == nil then
            return
        end
        local targetPos = panel:convertToWorldSpace(cc.p(0,0))
        targetPos = cc.p(-targetPos.x,-targetPos.y)
        --        local initPos = cc.p(targetPos.x, targetPos.y-P.picInfoView:getContentSize().height)
        --        P.picInfoView:setPosition(initPos)
        P.picInfoView:setPosition(targetPos)
        P.bagUi:addChild(P.picInfoView)

        --action move
        --        local moveAct = cc.MoveTo:create(0.2, targetPos)
        --        P.picInfoView:runAction(moveAct)
    else
        P.picInfoView:updateInfo(picIndex, enablePre, enableNext)
    end
end

---PicInfoView关闭时回调通知清除P.picInfoView
function P.doClosePicCallBack()
    if Android.CHANNEL == Enum.CHANNEL.ANDROID_TV_GOOGLE then
        P.picInfoView:removeFromParent(true)
        P.tapBlock = false
    end
    P.picInfoView = nil
end

---点击PicInfoView左右按钮时回调
function P.doUpdatePicAction(_isPre, _picIndex)
    local newPicIndx = _picIndex
    if _isPre then
        for var=_picIndex-1, 1, -1 do
            enablePre = P.itemArray[var][Enum.PIC_KEY.IS_HAS]
            if enablePre then
                newPicIndx = var
                break
            end
        end
    else
        for var=_picIndex+1, #P.itemArray do
            enableNext = P.itemArray[var][Enum.PIC_KEY.IS_HAS]
            if enableNext then
                newPicIndx = var
                break
            end
        end
    end
    local itemGrid = P.itemGrids[newPicIndx]
    if itemGrid == nil then
        return
    end
    local itemButton = UILib.getNodeByTag(itemGrid,newPicIndx)
    if itemButton == nil then
        return
    end

    P.doPicAction(itemButton,ccui.TouchEventType.ended)
end

---显示物品详细信息
--@param itemIndex int item在itemArray中的序列号
function P.showItemInfo(itemIndex)
    if P.itemInfoView == nil then
        local tag = TAG.SINGLE_BAG_PANEL
        if P.layerType == Enum.LAYER_TYPE.NONE then
            tag = TAG.BAG_PANEL
        end
        cclog("showItemInfo tag = "..tag)
        local panel = UILib.getNodeByTag(P.bagUi,tag)
        if panel == nil then
            return
        end
        --P.itemInfoView = ccui.ImageView:create("p_46.pngHK")
        P.itemInfoView = ItemInfoView:create(P.currentBagTag, P.updateBag, P.closeItemInfoViewCallback)
        if P.itemInfoView == nil then
            return
        end
        local pos = cc.p(panel:getContentSize().width/2,20)
        P.itemInfoView:setPosition(pos)
        P.itemInfoView:setScale(0.1)
        P.bagUi:addChild(P.itemInfoView)

        --action move scale
        pos = cc.p(P.itemInfoView:getPositionX(),P.itemInfoView:getContentSize().height/2+10)
        local moveAct = cc.MoveTo:create(0.3, pos)
        local scaleAct = cc.ScaleTo:create(0.3, 1)
        local showAct = cc.Spawn:create(moveAct,scaleAct)

        P.itemInfoView:runAction(showAct)
    end

    if P.itemInfoView ~= nil then
        cclog("P.showItemInfo index = "..itemIndex)
        P.itemInfoView:updateInfo(P.itemArray[itemIndex], itemIndex)
    end
end

function P.doItemAction(sender, eventType)
    if eventType == ccui.TouchEventType.ended then
        if P.focusAni ~= nil then
            local pos = cc.p(sender:getParent():getPositionX(),sender:getParent():getPositionY())
            P.focusAni:setPosition(pos)
            P.focusAni:setVisible(true)
        end

        if P.currentBagTag == Enum.BAG_TAG.NPC_SHOP
            or P.currentBagTag == Enum.BAG_TAG.NPC_SHOP_SELL then
            if P.previewIconPanel ~= nil then
                P.previewIconPanel:removeAllChildren(true)

                P.currentSelectedIndex = sender:getTag()
                local item = P.itemArray[sender:getTag()]
                local iconImgIndex = item[Enum.IAP_KEY.IMG]
                local iconInImgIndex = item[Enum.IAP_KEY.ICON]

                local icon = UILib.createClipSprite(iconImgIndex,iconInImgIndex)
                icon:setPosition(cc.p(icon:getContentSize().width/2,icon:getContentSize().height/2))
                P.previewIconPanel:addChild(icon)

                local bagQuantity = 0
                if P.currentBagTag == Enum.BAG_TAG.NPC_SHOP then
                    bagQuantity = item[Enum.IAP_KEY.BAG_NUM]
                else
                    bagQuantity = item[Enum.ITEM_KEY.QUANTITY]
                end

                P.currentSelectedItemNumLabel = cc.LabelAtlas:_create(bagQuantity, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
                P.currentSelectedItemNumLabel:setPosition(cc.p(icon:getContentSize().width,(P.previewIconPanel:getContentSize().height-P.currentSelectedItemNumLabel:getContentSize().height)/2))
                P.previewIconPanel:addChild(P.currentSelectedItemNumLabel)
            end
        end

        P.showItemInfo(sender:getTag())
    end
end

function P.doPicAction(sender, eventType)
    if eventType == ccui.TouchEventType.ended then
        if P.focusAni ~= nil then
            local pos = cc.p(sender:getParent():getPositionX(),sender:getParent():getPositionY())
            P.focusAni:setPosition(pos)
            P.focusAni:setVisible(true)
        end

        if P.currentBagTag == Enum.BAG_TAG.MERGE
            or P.currentBagTag == Enum.BAG_TAG.TRANSMUTEITEM then
            P.showItemInfo(sender:getTag())
        elseif P.currentBagTag == Enum.BAG_TAG.WEAPON
            or P.currentBagTag == Enum.BAG_TAG.ARMOR
            or P.currentBagTag == Enum.BAG_TAG.DEMON then
            P.showPicInfo(sender:getTag())
        end
    end
end

function P.addPropImg(parent, prop)
    if prop < 0 then
        return
    end
    --    local imgN = {
    --        [0] = "p_114",
    --        [1] = "p_115",
    --        [2] = "p_116",
    --        [3] = "p_117",
    --    }
    --#imgN==3
    if prop > #Enum.PROP_IMG_NAME then
        return
    end
    local imgView = UILib.addImageTo(parent,Enum.PROP_IMG_NAME[prop],cc.p(0,0))
    imgView:setPosition(UILib.offsetWithSuperSize(0,0,imgView:getContentSize().width,imgView:getContentSize().height,parent:getContentSize().width,parent:getContentSize().height,UILib.UI_ANCHOR.BOTTOM_RIGHT))
end

function P.addItemIconsByBagTag(bagTag, defaultGridsCount)
    P.currentBagTag = bagTag
    local itemArrayJson = ""
    if bagTag == Enum.BAG_TAG.NPC_SHOP_SELL then
        itemArrayJson = lk.LakooManager:getInstance():getSellItemArrayJson()
    else
        itemArrayJson = lk.LakooManager:getInstance():getItemArrayJsonByTag(bagTag)
    end

    if itemArrayJson == "" then
        for var=1, #P.itemArray do
            P.itemArray[var] = nil
        end
        return
    end
    --cclog("itemArrayJson==%s",itemArrayJson)
    P.itemArray = Util.json2Table(itemArrayJson)
    
    local gridsCount = (math.floor((#P.itemArray)/5)+1)*5
    if gridsCount < defaultGridsCount then
        gridsCount = defaultGridsCount
    end
    P.addItemGrids(gridsCount)
    
    for var=1, #P.itemArray do
        local itemTable = P.itemArray[var]
        local itemId = itemTable[Enum.ITEM_KEY.ID]
        local iconImgIndex = itemTable[Enum.ITEM_KEY.ICON_IDX]
        local iconInImgIndex = itemTable[Enum.ITEM_KEY.ICON_IN_IDX]
        local quantity = itemTable[Enum.ITEM_KEY.QUANTITY]
        local prop = itemTable[Enum.ITEM_KEY.PROPERTY]
        local isEquipped = itemTable[Enum.ITEM_KEY.IS_EQUIPPED]

        local itemGrid = P.itemGrids[var]
        if itemGrid == nil then
            cclog("addItemIconsByBagTag itemGrid == nil!!! itemId:%d",itemId)
        end
        if itemId > 0 and itemGrid ~= nil then
            local button = ccui.Button:create("p_26.pngHK", "p_26.pngHK", "p_26.pngHK")
            button:setContentSize(itemGrid:getContentSize())
            local pos = cc.p(itemGrid:getContentSize().width/2, itemGrid:getContentSize().height/2)
            button:setPosition(pos)
            itemGrid:addChild(button)
            button:addTouchEventListener(P.doItemAction)
            button:setTag(var)

            local icon = UILib.createClipSprite(iconImgIndex,iconInImgIndex)
            icon:setPosition(pos)
            itemGrid:addChild(icon)

            --add item quantity Label
            if bagTag == Enum.BAG_TAG.POTION
                or bagTag == Enum.BAG_TAG.QUEST
                or bgTag == Enum.BAG_TAG.NPC_SHOP_SELL then
                if quantity > 0 then
                    local lab = cc.LabelAtlas:_create(quantity, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
                    lab:setPosition(UILib.offsetWithSuperSize(-15,0,lab:getContentSize().width,lab:getContentSize().height,itemGrid:getContentSize().width,itemGrid:getContentSize().height,UILib.UI_ANCHOR.BOTTOM_RIGHT))
                    itemGrid:addChild(lab)
                end
            end

            if isEquipped then
                if bagTag == Enum.BAG_TAG.SUMMON then
                    local equipSoulPos = itemTable[Enum.ITEM_KEY.EQ_SOUL_POS]
                    local imgNames = {Enum.TEXT_IMG.SUMMON, Enum.TEXT_IMG.ASSISTANT, Enum.TEXT_IMG.MAGIC, Enum.TEXT_IMG.MAGIC}
                    local imgName = imgNames[equipSoulPos+1]
                    imgName = string.format("%s.%s",imgName,Text.get(IDX.PNG_TYPE))
                    local imgView = ccui.ImageView:create(imgName)
                    imgView:setPosition(UILib.offsetWithSuperSize(5,-5,imgView:getContentSize().width,imgView:getContentSize().height,itemGrid:getContentSize().width,itemGrid:getContentSize().height,UILib.UI_ANCHOR.TOP_LEFT))
                    itemGrid:addChild(imgView)
                else
                    local lab = cc.Label:create()
                    lab:setString("E")
                    lab:setSystemFontSize(UILib.FONT.SMALL)
                    lab:setPosition(UILib.offsetWithSuperSize(5,5,lab:getContentSize().width,lab:getContentSize().height,itemGrid:getContentSize().width,itemGrid:getContentSize().height,UILib.UI_ANCHOR.TOP_LEFT))
                    itemGrid:addChild(lab)
                end
            end

            if bagTag ~= Enum.BAG_TAG.NPC_SHOP_SELL then
                P.addPropImg(itemGrid, prop)
            end
        end
    end

    --[[
    for key, var in ipairs(P.itemGrids) do
    local itemId = lk.LakooManager:getInstance():getItemIdByBagTagAndPos(bagTag, key-1)
    if itemId > 0 then
    local button = ccui.Button:create("p_26.pngHK", "p_26.pngHK", "p_26.pngHK")
    button:setContentSize(var:getContentSize())
    local pos = cc.p(var:getContentSize().width/2, var:getContentSize().height/2)
    button:setPosition(pos)
    var:addChild(button)
    button:addTouchEventListener(P.doItemAction)
    button:setTag(itemId)

    local iconImgIndex = lk.LakooManager:getInstance():getIconImageIndexByItemId(itemId)
    local iconInImgIndex = lk.LakooManager:getInstance():getIconInImageIndexByItemId(itemId)
    local icon = UILib.createClipSprite(iconImgIndex,iconInImgIndex)
    icon:setPosition(pos)
    var:addChild(icon)

    --add item quantity Label
    if bagTag == Enum.BAG_TAG.POTION or bagTag == Enum.BAG_TAG.QUEST then
    local quantity = lk.LakooManager:getInstance():getQuantityByItemId(itemId)
    if quantity > 0 then
    local lab = cc.LabelAtlas:_create(quantity, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
    lab:setPosition(UILib.offsetWithSuperSize(-15,0,lab:getContentSize().width,lab:getContentSize().height,var:getContentSize().width,var:getContentSize().height,UILib.UI_ANCHOR.BOTTOM_RIGHT))
    var:addChild(lab)
    end
    end
    end
    end
    --]]
end

function P.addPicIconsByBagTag(bagTag)
    P.currentBagTag = bagTag
    local picArrayJson = lk.LakooManager:getInstance():getPicArrayJsonByTag(bagTag)
    --cclog("picArrayJson==%s",picArrayJson)
    P.itemArray = Util.json2Table(picArrayJson)

    for var=1, #P.itemArray do
        if var <= #P.itemGrids then
            local picTable = P.itemArray[var]
            local picId = picTable[Enum.PIC_KEY.ID]
            local iconImgIndex = picTable[Enum.PIC_KEY.ICON_IDX]
            local iconInImgIndex = picTable[Enum.PIC_KEY.ICON_IN_IDX]
            local isHas = picTable[Enum.PIC_KEY.IS_HAS]
            local prop = picTable[Enum.PIC_KEY.PROPERTY]

            if isHas == false then
                iconImgIndex = UILib.IMG_IDX.IMG91
                iconInImgIndex = UILib.IN_IMG_IDX.QUESTION--"?"图标所在位置
            end
            if picId > 0 then
                local itemGrid = P.itemGrids[var]

                if isHas then
                    local button = ccui.Button:create("p_26.pngHK", "p_26.pngHK", "p_26.pngHK")
                    button:setContentSize(itemGrid:getContentSize())
                    local pos = cc.p(itemGrid:getContentSize().width/2, itemGrid:getContentSize().height/2)
                    button:setPosition(pos)
                    itemGrid:addChild(button)
                    button:addTouchEventListener(P.doPicAction)
                    button:setTag(var)
                end

                local icon = UILib.createClipSprite(iconImgIndex,iconInImgIndex)
                local pos = cc.p(itemGrid:getContentSize().width/2, itemGrid:getContentSize().height/2)
                icon:setPosition(pos)
                itemGrid:addChild(icon)

                if isHas then
                    P.addPropImg(itemGrid,prop)
                end
            end
        end
    end
end

function P.addAllPicIconsByBagTag(bagTag, isCheckHas)
    P.currentBagTag = bagTag
    local picArrayJson = lk.LakooManager:getInstance():getPicArrayJsonByTag(bagTag)
    --cclog("allPicArrayJson==%s",picArrayJson)
    P.itemArray = Util.json2Table(picArrayJson)

    local gridIdx = 1
    for var=1, #P.itemArray do
        if var <= #P.itemGrids then
            local picTable = P.itemArray[var]
            local picId = picTable[Enum.PIC_KEY.ID]
            local iconImgIndex = picTable[Enum.PIC_KEY.ICON_IDX]
            local iconInImgIndex = picTable[Enum.PIC_KEY.ICON_IN_IDX]
            local isHas = picTable[Enum.PIC_KEY.IS_HAS]
            local isCanCreate = picTable[Enum.PIC_KEY.IS_CAN_CREATE]
            local isHasFull = picTable[Enum.PIC_KEY.IS_HAS_FULL]
            local prop = picTable[Enum.PIC_KEY.PROPERTY]
            local checked = true
            if isCheckHas == true and isHas == false then
                checked = false
            end
            if (bagTag == Enum.BAG_TAG.MERGE or bagTag == Enum.BAG_TAG.TRANSMUTEITEM) and isCanCreate == false then
                checked = false
            end
            if picId > 0 and checked then
                local itemGrid = P.itemGrids[gridIdx]
                local button = ccui.Button:create("p_26.pngHK", "p_26.pngHK", "p_26.pngHK")
                button:setContentSize(itemGrid:getContentSize())
                local pos = cc.p(itemGrid:getContentSize().width/2, itemGrid:getContentSize().height/2)
                button:setPosition(pos)
                itemGrid:addChild(button)
                button:addTouchEventListener(P.doPicAction)
                button:setTag(var)

                local icon = UILib.createClipSprite(iconImgIndex,iconInImgIndex)
                local pos = cc.p(itemGrid:getContentSize().width/2, itemGrid:getContentSize().height/2)
                icon:setPosition(pos)
                itemGrid:addChild(icon)

                if bagTag == Enum.BAG_TAG.TRANSMUTEITEM and isHasFull == false then
                    icon:setColor(UILib.UI_COLOR3B.GREY)
                end

                P.addPropImg(itemGrid,prop)

                gridIdx = gridIdx + 1
            end
        end
    end
    --[[
    for key, var in ipairs(P.itemGrids) do
    local picId = lk.LakooManager:getInstance():getPicIdFromBagTagAndPos(bagTag, key-1, false)
    if picId > 0 then
    local button = ccui.Button:create("p_26.pngHK", "p_26.pngHK", "p_26.pngHK")
    button:setContentSize(var:getContentSize())
    local pos = cc.p(var:getContentSize().width/2, var:getContentSize().height/2)
    button:setPosition(pos)
    var:addChild(button)
    button:addTouchEventListener(P.doPicAction)
    button:setTag(picId)

    local iconImgIndex = lk.LakooManager:getInstance():getPicIconImageIndexByPicId(picId)
    local iconInImgIndex = lk.LakooManager:getInstance():getPicIconInImageIndexByPicId(picId)
    local icon = UILib.createClipSprite(iconImgIndex,iconInImgIndex)
    local pos = cc.p(var:getContentSize().width/2, var:getContentSize().height/2)
    icon:setPosition(pos)
    var:addChild(icon)
    end
    end
    --]]
end

function P.addNpcShopItemIcons()
    P.currentBagTag = Enum.BAG_TAG.NPC_SHOP
    local itemArrayJson = lk.LakooManager:getInstance():getNpcShopItemArrayJson(P.shopItemIds)
    P.addPayItemIcons(itemArrayJson)
end

function P.addIapItemIcons()
    P.currentBagTag = Enum.BAG_TAG.SHOP
    local itemArrayJson = lk.LakooManager:getInstance():getIapItemArrayJson()
    P.addPayItemIcons(itemArrayJson)
end

function P.addPayItemIcons(itemArrayJson)
    --cclog("iapItemArrayJson==%s",itemArrayJson)
    if itemArrayJson == "" or itemArrayJson == nil then
        return
    end

    P.itemArray = Util.json2Table(itemArrayJson)

    for var=1, #P.itemArray do
        local itemTable = P.itemArray[var]
        local itemId = itemTable[Enum.IAP_KEY.IAP_ID]
        local iconImgIndex = itemTable[Enum.IAP_KEY.IMG]
        local iconInImgIndex = itemTable[Enum.IAP_KEY.ICON]
        local iconType = itemTable["type"]
        if var == 1 then
            print("Id "..itemId)
            print("Img Index "..iconImgIndex)
            print("Type "..iconType)
        end

        if itemId ~= nil and itemId > 0 then
            local itemGrid = P.itemGrids[var]
            local button = ccui.Button:create("p_26.pngHK", "p_26.pngHK", "p_26.pngHK")
            button:setContentSize(itemGrid:getContentSize())
            local pos = cc.p(itemGrid:getContentSize().width/2, itemGrid:getContentSize().height/2)
            button:setPosition(pos)
            itemGrid:addChild(button)
            button:addTouchEventListener(P.doItemAction)
            button:setTag(var)

            local icon = nil
            if (iconType ~= nil and iconType == 99) then
                icon = cc.Sprite:create("p_201.png")
            else
                icon = UILib.createClipSprite(iconImgIndex,iconInImgIndex)
            end
            icon:setPosition(pos)
            itemGrid:addChild(icon)
        end
    end
end

function P.updateTitleButtonsStatus(selectTag)
    for key, var in ipairs(P.titleButs) do
        local tag = var:getTag()
        if tag == selectTag then
            var:setHighlighted(true)
        else
            var:setHighlighted(false)
        end
    end
end

function P.updateTitle()
    for var=1, #P.titleImgs do
        if P.titleImgs[var] ~= nil then
            P.titleImgs[var]:removeFromParent(true)
            P.titleImgs[var] = nil
        end
    end

    local titleImgNames = {}
    if P.bagType == Enum.BAG_TYPE.BAG then
        table.insert(titleImgNames,"p_43")--[[装备--]]
        table.insert(titleImgNames,"p_42")--[[药品--]]
        table.insert(titleImgNames,"p_45")--[[任务--]]
    elseif P.bagType == Enum.BAG_TYPE.SOUL then
        table.insert(titleImgNames,"p_50")--[[召唤--]]
        table.insert(titleImgNames,"p_52")--[[炼魂--]]
        table.insert(titleImgNames,"p_87")--[[锻造--]]
    elseif P.bagType == Enum.BAG_TYPE.COLLECTION then
        table.insert(titleImgNames,"p_54")--[[武器--]]
        table.insert(titleImgNames,"p_55")--[[防具--]]
        table.insert(titleImgNames,"p_56")--[[妖怪--]]
    end

    local extName = "pngHK"
    local posX = 80
    if GamePool.getLocalLang() == Enum.LANG.EN then
        extName = "pngEN"
        posX = 107
    end
    local imgName1 = string.format("%s.%s",Enum.IMG_NAME.BUT_BG_1,extName)
    local imgName2 = string.format("%s.%s",Enum.IMG_NAME.BUT_BG_2,extName)

    for key, var in ipairs(P.titleButs) do
        var:loadTextures(imgName1,imgName2)

        local imgN = titleImgNames[key]
        if imgN ~= nil then
            local imgName = string.format("%s.%s",imgN,Text.get(IDX.PNG_TYPE))
            local textImg = ccui.ImageView:create(imgName)
            local pos = cc.p(var:getContentSize().width/2,var:getContentSize().height/2)
            textImg:setPosition(pos)
            var:addChild(textImg)

            table.insert(P.titleImgs,textImg)
            if key > 1 then
                posX = posX + var:getContentSize().width + 1
            end
            local posY = var:getPositionY()
            pos = cc.p(posX, posY)
            var:setPosition(pos)
        end
    end
end

---外部更新背包使用@see ItemInfoView:doXXX
--@param bool _needUpdatePluginStatus 是否需要更新PluginStatus
--@param int _updateItemIndex 更新ItemInfoView,默认_updateItemIndex为nil是不更新
--@param bool _removeSelectStatus 去掉选中状态，默认为nil即为false
function P.updateBag(_needUpdatePluginStatus, _updateItemIndex, _removeSelectStatus)
    print("currentBagTag is "..P.currentBagTag)
    if _removeSelectStatus then
        P.currentSelectedIndex = -1

        if P.focusAni ~= nil then
            P.focusAni:setVisible(false)
        end

        if P.itemInfoView ~= nil then
            P.itemInfoView:removeFromParent(true)
            P.itemInfoView = nil
        end
        
        if P.currentSelectedItemNumLabel ~= nil then
            P.currentSelectedItemNumLabel:removeFromParent(true)
            P.currentSelectedItemNumLabel = nil
        end

        if P.previewIconPanel ~= nil then
            P.previewIconPanel:removeAllChildren(true)
        end
    end

    if P.currentBagTag == Enum.BAG_TAG.SHOP then
    	P.doMainShop()
        if _updateItemIndex ~= nil then
            P.setSelectPos(_updateItemIndex)
    	end
    	return
    end

    if P.currentBagTag == Enum.BAG_TAG.NPC_SHOP_SELL
        or P.currentBagTag == Enum.BAG_TAG.NPC_SHOP then

        local money = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.MONEY)
        P.playerMoneyLabel:setString(money)

        if P.currentBagTag == Enum.BAG_TAG.NPC_SHOP_SELL then
            P.doNpcShopSell()

            if P.focusAni ~= nil then
                P.focusAni:setVisible(false)
            end

            if P.itemInfoView ~= nil then
                P.itemInfoView:removeFromParent(true)
                P.itemInfoView = nil
            end

            if P.previewIconPanel ~= nil then
                P.previewIconPanel:removeAllChildren(true)
            end
        else
            P.doNpcShopBuy()

            if P.itemInfoView ~= nil then
                P.itemInfoView:updateInfoView()
            end

            if P.currentSelectedItemNumLabel ~= nil then
                local item = P.itemArray[P.currentSelectedIndex]
                P.currentSelectedItemNumLabel:setString(item[Enum.IAP_KEY.BAG_NUM])
            end
        end

        return
    end
    if P.currentBagTag == Enum.BAG_TAG.EQUIPMENT_WEAPON
        or P.currentBagTag == Enum.BAG_TAG.EQUIPMENT_ARMOUR
        or P.currentBagTag == Enum.BAG_TAG.EQUIPMENT_ADORN_DATA then
        P.initSingleBag(P.bagUi,P.layerType)

        P.needUpdatePluginStatus = _needUpdatePluginStatus
        if  _updateItemIndex ~= nil then
            --P.showItemInfo( _updateItemIndex)
            P.setSelectPos(_updateItemIndex)
        end
        return
    end
    local butIdx = 1
    if P.currentBagTag == Enum.BAG_TAG.POTION
        or P.currentBagTag == Enum.BAG_TAG.MERGE
        or P.currentBagTag == Enum.BAG_TAG.ARMOR then
        butIdx = 2
    end

    if P.currentBagTag == Enum.BAG_TAG.QUEST
        or P.currentBagTag == Enum.BAG_TAG.TRANSMUTEITEM
        or P.currentBagTag == Enum.BAG_TAG.DEMON  then
        butIdx = 3
    end
    
    if Android.CHANNEL == Enum.CHANNEL.ANDROID_TV_GOOGLE then
        if P.currentBagTag == Enum.BAG_TAG.EQUIPMENT then
            P.currentSelectedIndex = -1
        end
    end
    
    P.showBag(P.bagUi,P.bagType,butIdx)

    P.needUpdatePluginStatus = _needUpdatePluginStatus
    if  _updateItemIndex ~= nil then
        P.showItemInfo(_updateItemIndex)
        if Android.CHANNEL == Enum.CHANNEL.ANDROID_TV_GOOGLE then
            P.itemUseSign = true
            P.setItemSelect()
        else
            P.setSelectPos(_updateItemIndex)
        end
    end
end

function P.getNeedUpdatePluginStatus()
    return P.needUpdatePluginStatus
end

function P.setNeedUpdatePluginStatus(_isNeed)
    P.needUpdatePluginStatus = _isNeed
end

function P.updateUIValue()
    if P.stoneLabel ~= nil then
        local stone = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.SOUL_STONE)
        P.stoneLabel:setString(stone)
    end
end

function P.updateData()
    local ads_state = lk.LakooManager:getInstance():getVideoAdsStateInfo()
    if ads_state == 1 or ads_state == 2 then
        lk.LakooManager:getInstance():setVideoAdsStateInfo(0)
        LayerStage.setHideLayerType(P.layerType)
        lk.LakooManager:getInstance():setIsPopNpcShop(false)
        if ads_state == 1 then
            local stoneTip = lk.LakooManager:getInstance():getString(Enum.TEXT.T_ADS_REWARD_TIPS)
            lk.LakooManager:getInstance():addPopTip(stoneTip)
            local saveTips = lk.LakooManager:getInstance():getString(Enum.TEXT.T_TIPS_SAVE)
            lk.LakooManager:getInstance():addPopTip(saveTips)
        end
    end
end


function P.removeOldItemGrids()
    for var=1, #P.itemGrids do
        if P.itemGrids[var] ~= nil then
            P.itemGrids[var]:removeFromParent(true)
            P.itemGrids[var] = nil
        end
    end
end

function P.cleanholer()
    if P.bagUi ~= nil then
        local eventDispatcher = P.bagUi:getEventDispatcher()
        eventDispatcher:removeEventListenersForTarget(P.bagUi, false)
        _firstHolder = nil
        P.tapSelected = -1
        P.currentSelectedIndex = -1
    end
end

function P.clean()
    P.layerType = Enum.LAYER_TYPE.NONE
    if P.itemInfoView ~= nil then
        P.itemInfoView:removeFromParent(true)
        P.itemInfoView = nil
    end
    P.bagUi = nil
    P.scrollView = nil
    cclog("PluginBag clean")
    for var=1, #P.titleButs do
        P.titleButs[var] = nil
    end
    for var=1, #P.titleImgs do
        P.titleImgs[var]:removeFromParent(true)
        P.titleImgs[var] = nil
    end
    for var=1, #P.itemGrids do
        P.itemGrids[var]:removeFromParent(true)
        P.itemGrids[var] = nil
    end
    if P.focusAni ~= nil then
        P.focusAni:getAnimation():stop()
        P.focusAni:removeFromParent(true)
        P.focusAni = nil
    end
    for var=1, #P.itemArray do
        P.itemArray[var] = nil
    end

    if P.previewIconPanel ~= nil then
        P.previewIconPanel:removeFromParent(true)
        P.previewIconPanel = nil
    end

    P.currentSelectedItemNumLabel = nil
    P.stoneLabel = nil

    if P.tapfocusAni ~=nil then
        if P.tapfocusAni:getAnimation() ~= nil then
            P.tapfocusAni:getAnimation():stop()
        end
        P.tapfocusAni:removeFromParent(true)
        P.tapfocusAni = nil
    end

    if P.picInfoView ~= nil then
        P.picInfoView:removeFromParent(true)
        P.picInfoView = nil
    end
    P.tapSelected = -1
    P.hadTitle = false
    P.TitleCount = 0
    P.inChoicetap = false
    P.ScrollY = 0
    P.currentSelectedIndex =-1
    _firstHolder = nil
end

function P.closeItemInfoViewCallback()
    if Android.CHANNEL == 4 then
        if P.itemInfoView ~= nil then
            P.itemInfoView.closeCallBack()
            P.itemSelectBack = true
        end
        return
    end
    if P.focusAni ~= nil then
        P.focusAni:setVisible(false)
    end
    if P.itemInfoView ~= nil then
        P.itemInfoView:removeFromParent(true)
        P.itemInfoView = nil
    end
end

function P.scrollToPercent(pre)
    P.scrollView:scrollToPercentVertical(pre,0.2,false)
end

function P.scrollToTop()
    P.scrollView:scrollToTop(0.2,false)
end

function P.setSelectPos(var)
    cclog("#P.itemGrids = "..#P.itemGrids..",var = "..var..",itemArray ="..#P.itemArray)
    if var <= 0 or var > #P.itemArray then
        return
    end

    if var >10 and #P.itemArray >10 then
        local row = math.ceil(var/5)
        local max = math.ceil(#P.itemArray/5)
        local oneRow = math.ceil(100/max)
        local Percent = oneRow * 1.5+(row-3)*oneRow* 1.5
        P.scrollToPercent(Percent)
    else
        P.scrollToTop()
    end


    if P.focusAni ~= nil then
        local itemGrid = P.itemGrids[var]
        local pos = cc.p(itemGrid:getPositionX(), itemGrid:getPositionY())
        P.focusAni:setPosition(pos)
        P.focusAni:setVisible(true)
    end

    if P.currentBagTag == Enum.BAG_TAG.NPC_SHOP
        or P.currentBagTag == Enum.BAG_TAG.NPC_SHOP_SELL then
        if P.previewIconPanel ~= nil then
            P.previewIconPanel:removeAllChildren(true)

            local item = P.itemArray[var]
            local iconImgIndex = item[Enum.IAP_KEY.IMG]
            local iconInImgIndex = item[Enum.IAP_KEY.ICON]

            local icon = UILib.createClipSprite(iconImgIndex,iconInImgIndex)
            icon:setPosition(cc.p(icon:getContentSize().width/2,icon:getContentSize().height/2))
            P.previewIconPanel:addChild(icon)

            local bagQuantity = 0
            if P.currentBagTag == Enum.BAG_TAG.NPC_SHOP then
                bagQuantity = item[Enum.IAP_KEY.BAG_NUM]
            else
                bagQuantity = item[Enum.ITEM_KEY.QUANTITY]
            end

            P.currentSelectedItemNumLabel = cc.LabelAtlas:_create(bagQuantity, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
            P.currentSelectedItemNumLabel:setPosition(cc.p(icon:getContentSize().width,(P.previewIconPanel:getContentSize().height-P.currentSelectedItemNumLabel:getContentSize().height)/2))
            P.previewIconPanel:addChild(P.currentSelectedItemNumLabel)
        end
    end
    
    P.currentSelectedIndex = var
    cclog("P.currentBagTag = "..P.currentBagTag)
    if P.currentBagTag == Enum.BAG_TAG.WEAPON
        or P.currentBagTag == Enum.BAG_TAG.ARMOR
        or P.currentBagTag == Enum.BAG_TAG.DEMON then
    --        P.showPicInfo(var)
    else
        if P.currentBagTag == Enum.BAG_TAG.TRANSMUTEITEM then
            if var < 19 then
                var = var +1
            else
                var = var +2
            end
        end
        P.showItemInfo(var)
    end
end

function P.setSelectTap()
    local subTitlePanel = UILib.getNodeByTag(P.bagUi,TAG.SINGLE_BAG_SUBTITLE)
    if subTitlePanel == nil then
        cclog("subTitlePanel == nil")
        return
    end

    local buyBut = UILib.getNodeByTag(P.bagUi,TAG.SINGLE_BAG_NPC_SHOP_BUY)
    local sellBut = UILib.getNodeByTag(P.bagUi,TAG.SINGLE_BAG_NPC_SHOP_SELL)
end

function P.dpadUp()
    local pos = 0

    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        if layer:getMainPopLayerTapType() then
            return
        end
    end

    if P.itemUseSign then
        if P.itemInfoView == nil then
            P.itemUseSign = false
        else
            cclog("PluginBag dpadUp second return")
            return
        end
    end

    if P.isMainpopLayer then
        if P.inChoicetap then
            cclog("dpadUp() inChoicetap = true")
            P.inChoicetap = false

            if P.tapfocusAni ~= nil then
                P.tapfocusAni:setVisible(false)
            end

            local scene = cc.Director:getInstance():getRunningScene()
            local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
            if layer ~= nil then
                if layer:getMainPopLayerTapType() == false then
                    layer:setMainPopLayerTap(true)
                    return
                end
            end
        else
            cclog("dpadUp() inChoicetap = false tapselect = "..P.tapSelected)
            if P.currentSelectedIndex > 0 and P.currentSelectedIndex <= 5 then

                P.inChoicetap = true
                local button = UILib.getNodeByTag(P.bagUi,TAG.BUT_1)
                local button2 = UILib.getNodeByTag(P.bagUi,TAG.BUT_2)
                local button3 = UILib.getNodeByTag(P.bagUi,TAG.BUT_3)

                if button ~=nil and button2 ~=nil and button3 ~= nil then
                    if P.tapSelected == 1 then
                        if P.tapfocusAni ~= nil then
                            local pos = cc.p(button:getPositionX(), button:getPositionY())
                            P.tapfocusAni:setPosition(pos)
                            P.tapfocusAni:setVisible(true)
                        end
                    elseif P.tapSelected == 2 then
                        if P.tapfocusAni ~= nil then
                            local pos = cc.p(button2:getPositionX(), button2:getPositionY())
                            P.tapfocusAni:setPosition(pos)
                            P.tapfocusAni:setVisible(true)
                        end
                    elseif P.tapSelected == 3 then
                        if P.tapfocusAni ~= nil then
                            local pos = cc.p(button3:getPositionX(), button3:getPositionY())
                            P.tapfocusAni:setPosition(pos)
                            P.tapfocusAni:setVisible(true)
                        end
                    end
                    if P.focusAni ~= nil then
                        P.focusAni:setVisible(false)
                    end
                    if P.itemInfoView ~= nil then
                        P.itemInfoView:removeFromParent(true)
                        P.itemInfoView = nil
                    end
                    if P.previewIconPanel ~= nil then
                        P.previewIconPanel:removeAllChildren(true)
                    end
                    P.currentSelectedIndex = -1
                end
                return
            end
        end
    end


    if P.layerType == Enum.LAYER_TYPE.SINGLE_POP_NPC_SHOP then
        if P.inChoicetap then
            return
        end
        if P.currentSelectedIndex > 0 and P.currentSelectedIndex <= 5 then
            local buyBut = UILib.getNodeByTag(P.bagUi,TAG.SINGLE_BAG_NPC_SHOP_BUY)
            local sellBut = UILib.getNodeByTag(P.bagUi,TAG.SINGLE_BAG_NPC_SHOP_SELL)

            --
            if P.tapSelected == 1 then
                if P.tapfocusAni ~= nil and buyBut ~=nil then
                    local pos = cc.p(buyBut:getPositionX(), buyBut:getPositionY())
                    P.tapfocusAni:setPosition(pos)
                    P.tapfocusAni:setVisible(true)
                end
            else
                if P.tapfocusAni ~= nil and sellBut ~=nil then
                    local pos = cc.p(sellBut:getPositionX(), sellBut:getPositionY())
                    P.tapfocusAni:setPosition(pos)
                    P.tapfocusAni:setVisible(true)
                end
            end

            if buyBut ~=nil and sellBut ~= nil then
                if P.focusAni ~= nil then
                    P.focusAni:setVisible(false)
                end
                if P.itemInfoView ~= nil then
                    P.itemInfoView:removeFromParent(true)
                    P.itemInfoView = nil
                end
                if P.previewIconPanel ~= nil then
                    P.previewIconPanel:removeAllChildren(true)
                end
                P.currentSelectedIndex = -1
                P.inChoicetap = true
            end
            return
        end
    end
    if P.currentSelectedIndex == -1 then
        P.setSelectPos(1)
        return
    end
    pos = P.currentSelectedIndex -5
    P.setSelectPos(pos)
end

function P.dapdDown()
    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        if layer:getTitleType() <2 or layer:getTitleType()>4 then
            cclog("PluginBag dpaddown() first return")
            return
        end
    end

    if P.itemUseSign then
        if P.itemInfoView == nil then
            P.itemUseSign = false
        else
            cclog("PluginBag dpadDown second return")
            return
        end
    end

    --在一层菜单
    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        if layer:getMainPopLayerTapType() then

            if P.isMainpopLayer then
                if P.bagUi== nil then
                    return
                end

                local button = UILib.getNodeByTag(P.bagUi,TAG.BUT_1)
                local button2 = UILib.getNodeByTag(P.bagUi,TAG.BUT_2)
                local button3 = UILib.getNodeByTag(P.bagUi,TAG.BUT_3)
                layer:setMainPopLayerTap(false)
                P.inChoicetap = true

                if button ~=nil and button2 ~=nil and button3 ~= nil then
                    if P.tapSelected == 1 then
                        button:setHighlighted(true)
                        button2:setHighlighted(false)
                        button3:setHighlighted(false)

                        if P.tapfocusAni ~= nil then
                            local pos = cc.p(button:getPositionX(), button:getPositionY())
                            P.tapfocusAni:setPosition(pos)
                            P.tapfocusAni:setVisible(true)
                        end
                    elseif P.tapSelected == 2 then
                        button2:setHighlighted(true)
                        button:setHighlighted(false)
                        button3:setHighlighted(false)

                        if P.tapfocusAni ~= nil then
                            local pos = cc.p(button2:getPositionX(), button2:getPositionY())
                            P.tapfocusAni:setPosition(pos)
                            P.tapfocusAni:setVisible(true)
                        end
                    elseif P.tapSelected == 3 then
                        button3:setHighlighted(true)
                        button2:setHighlighted(false)
                        button:setHighlighted(false)

                        if P.tapfocusAni ~= nil then
                            local pos = cc.p(button3:getPositionX(), button3:getPositionY())
                            P.tapfocusAni:setPosition(pos)
                            P.tapfocusAni:setVisible(true)
                        end
                    end
                end
                cclog("PluginBag dpaddown() thrid return")
                return
            end
        end
    end

    --在二层菜单
    --三层菜单
    cclog("dpadDown() third")
    if #P.itemArray == 0 then
        return
    end
    
   for var=1, #P.itemArray do
   
   end
    local picTable = P.itemArray[1]
    
    if (P.currentBagTag == Enum.BAG_TAG.MERGE) then
        local isCanCreate = picTable[Enum.PIC_KEY.IS_CAN_CREATE]
        if isCanCreate == false then
            return
        end
    end
    
    local pos = 0
    P.inChoicetap = false
    if P.tapfocusAni ~=nil then
        P.tapfocusAni:setVisible(false)
    end
    if P.currentSelectedIndex == -1 then
        P.setSelectPos(1)
        cclog("PluginBag dpaddown() fouth return")
        return
    end
    pos = P.currentSelectedIndex +5
    P.setSelectPos(pos)
end


function P.dpadLeft()
    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        if layer:getMainPopLayerTapType() then
            return
        end
    end

    if P.itemUseSign then
        if P.itemInfoView == nil then
            P.itemUseSign = false
        else
            cclog("PluginBag dpadLeft second return")
            return
        end
    end
    -- second layer
    if P.isMainpopLayer and P.inChoicetap == true then
        local button = UILib.getNodeByTag(P.bagUi,TAG.BUT_1)
        local button2 = UILib.getNodeByTag(P.bagUi,TAG.BUT_2)
        local button3 = UILib.getNodeByTag(P.bagUi,TAG.BUT_3)

        if button ~=nil and button2 ~=nil and button3 ~= nil then
            if P.tapSelected == 1 then
                return
            end

            local BAG_ACTION = {
                [TAG.BUT_1] = function ()  P.doBag_Equipment()  end,
                [TAG.BUT_2] = function ()  P.doBag_Potion() end,
                [TAG.BUT_3] = function ()  P.doBag_Quest() end,
            }

            local SOUL_ACTION = {
                [TAG.BUT_1] = function ()  P.doSoul_Summon()  end,
                [TAG.BUT_2] = function ()  P.doSoul_Merge() end,
                [TAG.BUT_3] = function ()  P.doSoul_TransmuteItem() end,
            }

            local COLLECTION_ACTION = {
                [TAG.BUT_1] = function ()  P.doCollection_Weapon()  end,
                [TAG.BUT_2] = function ()  P.doCollection_Armor() end,
                [TAG.BUT_3] = function ()  P.doCollection_Demon() end,
            }

            local Nextselect = P.tapSelected-1

            P.tapSelected = Nextselect
            local  tag = TAG.BUT_1
            if P.tapSelected == 1 then
                button:setHighlighted(true)
                button2:setHighlighted(false)
                button3:setHighlighted(false)

                if P.tapfocusAni ~= nil then
                    local pos = cc.p(button:getPositionX(), button:getPositionY())
                    P.tapfocusAni:setPosition(pos)
                    P.tapfocusAni:setVisible(true)
                end
                tag = TAG.BUT_1
            elseif P.tapSelected == 2 then
                button2:setHighlighted(true)
                button3:setHighlighted(false)
                button:setHighlighted(false)

                if P.tapfocusAni ~= nil then
                    local pos = cc.p(button2:getPositionX(), button2:getPositionY())
                    P.tapfocusAni:setPosition(pos)
                    P.tapfocusAni:setVisible(true)
                end
                tag = TAG.BUT_2
            end
            local func = nil
            if P.bagType == Enum.BAG_TYPE.BAG then
                func = BAG_ACTION[tag]
            elseif P.bagType == Enum.BAG_TYPE.SOUL then
                func = SOUL_ACTION[tag]
            elseif P.bagType == Enum.BAG_TYPE.COLLECTION then
                func = COLLECTION_ACTION[tag]
            end
            if func == nil then
                return
            else
                func()
                return
            end
        end

    end

    --single
    local pos = 0
    if P.layerType == Enum.LAYER_TYPE.SINGLE_POP_NPC_SHOP then
        if P.inChoicetap then
            if P.tapSelected ~= 1 then
                local buyBut = UILib.getNodeByTag(P.bagUi,TAG.SINGLE_BAG_NPC_SHOP_BUY)
                local sellBut = UILib.getNodeByTag(P.bagUi,TAG.SINGLE_BAG_NPC_SHOP_SELL)

                if buyBut ~=nil and sellBut ~= nil then

                    buyBut:setHighlighted(true)
                    sellBut:setHighlighted(false)
                    P.doNpcShopBuy()
                    P.tapSelected = 1

                    if P.tapSelected == 1 then
                        if P.tapfocusAni ~= nil and buyBut ~=nil then
                            local pos = cc.p(buyBut:getPositionX(), buyBut:getPositionY())
                            P.tapfocusAni:setPosition(pos)
                            P.tapfocusAni:setVisible(true)
                        end
                    else
                        if P.tapfocusAni ~= nil and sellBut ~=nil then
                            local pos = cc.p(sellBut:getPositionX(), sellBut:getPositionY())
                            P.tapfocusAni:setPosition(pos)
                            P.tapfocusAni:setVisible(true)
                        end
                    end

                    if P.focusAni ~= nil then
                        P.focusAni:setVisible(false)
                    end
                    if P.itemInfoView ~= nil then
                        P.itemInfoView:removeFromParent(true)
                        P.itemInfoView = nil
                    end
                    if P.previewIconPanel ~= nil then
                        P.previewIconPanel:removeAllChildren(true)
                    end
                    P.currentSelectedIndex = -1
                    P.inChoicetap = true
                end
            end
            return
        end
    end
    -- third layer
    if P.currentSelectedIndex == -1 then
        P.setSelectPos(1)
        return
    end
    pos = P.currentSelectedIndex -1
    P.setSelectPos(pos)

end

function P.dpadRight()
    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        if layer:getMainPopLayerTapType() then
            return
        end
    end

    if P.itemUseSign then
        if P.itemInfoView == nil then
            P.itemUseSign = false
        else
            cclog("PluginBag dpadRight second return")
            return
        end
    end
    -- second layer
    if P.isMainpopLayer and P.inChoicetap == true then
        local button = UILib.getNodeByTag(P.bagUi,TAG.BUT_1)
        local button2 = UILib.getNodeByTag(P.bagUi,TAG.BUT_2)
        local button3 = UILib.getNodeByTag(P.bagUi,TAG.BUT_3)

        if button ~=nil and button ~=nil and button ~= nil then
            if P.tapSelected == 3 then
                return
            end

            local BAG_ACTION = {
                [TAG.BUT_1] = function ()  P.doBag_Equipment()  end,
                [TAG.BUT_2] = function ()  P.doBag_Potion() end,
                [TAG.BUT_3] = function ()  P.doBag_Quest() end,
            }

            local SOUL_ACTION = {
                [TAG.BUT_1] = function ()  P.doSoul_Summon()  end,
                [TAG.BUT_2] = function ()  P.doSoul_Merge() end,
                [TAG.BUT_3] = function ()  P.doSoul_TransmuteItem() end,
            }

            local COLLECTION_ACTION = {
                [TAG.BUT_1] = function ()  P.doCollection_Weapon()  end,
                [TAG.BUT_2] = function ()  P.doCollection_Armor() end,
                [TAG.BUT_3] = function ()  P.doCollection_Demon() end,
            }

            local Nextselect = P.tapSelected+1

            P.tapSelected = Nextselect
            local tag = TAG.BUT_1
            if P.tapSelected == 2 then
                button2:setHighlighted(true)
                button:setHighlighted(false)
                button3:setHighlighted(false)

                if P.tapfocusAni ~= nil then
                    local pos = cc.p(button2:getPositionX(), button2:getPositionY())
                    P.tapfocusAni:setPosition(pos)
                    P.tapfocusAni:setVisible(true)
                end
                tag = TAG.BUT_2
            elseif P.tapSelected == 3 then
                button3:setHighlighted(true)
                button2:setHighlighted(false)
                button:setHighlighted(false)

                if P.tapfocusAni ~= nil then
                    local pos = cc.p(button3:getPositionX(), button3:getPositionY())
                    P.tapfocusAni:setPosition(pos)
                    P.tapfocusAni:setVisible(true)
                end
                tag = TAG.BUT_3
            end
            local func = nil
            if P.bagType == Enum.BAG_TYPE.BAG then
                func = BAG_ACTION[tag]
            elseif P.bagType == Enum.BAG_TYPE.SOUL then
                func = SOUL_ACTION[tag]
            elseif P.bagType == Enum.BAG_TYPE.COLLECTION then
                func = COLLECTION_ACTION[tag]
            end
            if func == nil then
                return nil
            else
                func()
                return
            end
        end

    end

    local pos = 0

    if P.layerType == Enum.LAYER_TYPE.SINGLE_POP_NPC_SHOP then
        if P.inChoicetap then
            if P.tapSelected ~= 2 then
                local buyBut = UILib.getNodeByTag(P.bagUi,TAG.SINGLE_BAG_NPC_SHOP_BUY)
                local sellBut = UILib.getNodeByTag(P.bagUi,TAG.SINGLE_BAG_NPC_SHOP_SELL)

                if buyBut ~=nil and sellBut ~= nil then
                    buyBut:setHighlighted(false)
                    sellBut:setHighlighted(true)
                    P.doNpcShopSell()
                    P.tapSelected = 2

                    if P.tapSelected == 1 then
                        if P.tapfocusAni ~= nil and buyBut ~=nil then
                            local pos = cc.p(buyBut:getPositionX(), buyBut:getPositionY())
                            P.tapfocusAni:setPosition(pos)
                            P.tapfocusAni:setVisible(true)
                        end
                    else
                        if P.tapfocusAni ~= nil and sellBut ~=nil then
                            local pos = cc.p(sellBut:getPositionX(), sellBut:getPositionY())
                            P.tapfocusAni:setPosition(pos)
                            P.tapfocusAni:setVisible(true)
                        end
                    end

                    if P.focusAni ~= nil then
                        P.focusAni:setVisible(false)
                    end
                    if P.itemInfoView ~= nil then
                        P.itemInfoView:removeFromParent(true)
                        P.itemInfoView = nil
                    end
                    if P.previewIconPanel ~= nil then
                        P.previewIconPanel:removeAllChildren(true)
                    end
                    P.currentSelectedIndex = -1
                    P.inChoicetap = true
                end
            end
            return
        end
    end

    if P.currentSelectedIndex == -1 then
        P.setSelectPos(1)
        return
    end
    pos = P.currentSelectedIndex +1
    P.setSelectPos(pos)
end

function P.dpadTap()
    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        if layer:getMainPopLayerTapType() then
            return
        end
    end
    
    if P.itemSelectBack then
        P.itemSelectBack = false
        return
    end

    if P.itemUseSign then
        cclog("bag itemUseSign true")
    else
        cclog("bag itemUseSign false")
    end
    cclog("P.currentSelectedIndex = "..P.currentSelectedIndex)
    if P.currentSelectedIndex ~= -1 and P.itemUseSign == false then
        if P.currentBagTag ~= Enum.BAG_TAG.WEAPON
            and P.currentBagTag ~= Enum.BAG_TAG.ARMOR
            and P.currentBagTag ~= Enum.BAG_TAG.DEMON then

            if P.itemInfoView ~= nil then
                P.setItemUseType(true)
                P.setItemSelect()
                
            end
        else
            local picTable = P.itemArray[P.currentSelectedIndex]
            local isHas = picTable[Enum.PIC_KEY.IS_HAS]
            if isHas then 
                P.showPicInfo(P.currentSelectedIndex)
            end
        end
    end
end

function P:registerControllerListener()
    local function onConnectController(controller, event)
        --        if self._firstHolder == nil then
        --            self._firstHolder  = ControllerHolder.new()
        --            self._firstHolder.controller = nil
        --        end
        if _firstHolder == nil then
            return
        end
        if controller == nil or controller ==   _firstHolder.controller then
            return
        end

        local deviceInfo = string.format("%s id:%d", controller:getDeviceName(), controller:getDeviceId())
        cclog("PluginBag deviceInfo ="..deviceInfo)

        if _firstHolder.controller == nil then
            _firstHolder.controller = controller
        end
    end

    local function onDisconnectedController(controller, event)
        if _firstHolder.controller == controller then
            _firstHolder.controller = nil
        end
    end

    local function showButtonState(controller, keyCode, isPressed)
        onConnectController(controller, nil)
        local holder = nil

        if _firstHolder == nil then
            return
        end
        if controller == _firstHolder.controller then
            holder = _firstHolder
        else
            return
        end

        local scene = cc.Director:getInstance():getRunningScene()
        local child = scene:getChildren()

        local topSence = child[#child]
        if topSence:getName() ~= "MainPopLayer" and topSence:getName() ~= "SinglePopLayer" then
            return
        end
        
        if P.bagUi ~= nil then
            local popLayer = P.bagUi:getChildren()
            for var = 1,#popLayer do
                local topPopLayer = popLayer[var]
                if topPopLayer ~= nil then
                    cclog("name = "..topPopLayer:getName())
                    if topPopLayer:getName() == "EquipSoulLayer" then
                        return
                    else
                    end
                end
            end
        end

        if P.picInfoView ~= nil then
            cclog("P.picInfoView ~= nil")
            return
        end

        if isPressed then
            cclog("PluginBag keyDown ="..keyCode)
            if keyCode == cc.ControllerKey.BUTTON_A then
                P.dpadTap()
            elseif keyCode == cc.ControllerKey.BUTTON_B then
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
                P.dpadUp()
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
                P.dapdDown()
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
                P.dpadLeft()
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
                P.dpadRight()
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key Down:%d",keyCode)
            end
        else
            cclog("PluginBag keyup ="..keyCode)
            if keyCode == cc.ControllerKey.BUTTON_A then

            elseif keyCode == cc.ControllerKey.BUTTON_B then

            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then

            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then

            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then

            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then

            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then

            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then

            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then

            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key up:%d",keyCode)
            end
        end
    end

    local function onKeyDown(controller, keyCode, event)
        showButtonState(controller, keyCode, true)
    end

    local function onKeyUp(controller, keyCode, event)
        showButtonState(controller, keyCode, false)
    end

    local function onAxisEvent(controller, keyCode, event)
        local holder = nil

        if controller == _firstHolder.controller then
            holder = _firstHolder
        else
            return
        end

        local ketStatus = controller:getKeyStatus(keyCode)
        if keyCode == cc.ControllerKey.JOYSTICK_LEFT_X then
            if(ketStatus.value == -1) then
            elseif(ketStatus.value == 1) then

            end
        elseif keyCode == cc.ControllerKey.JOYSTICK_LEFT_Y then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_X then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_Y then
        elseif keyCode == cc.ControllerKey.AXIS_LEFT_TRIGGER then
        elseif keyCode == cc.ControllerKey.AXIS_RIGHT_TRIGGER then
        else
        --do nothing
        end
    end

    local controllerListener = cc.EventListenerController:create()
    controllerListener:registerScriptHandler(onConnectController, cc.Handler.EVENT_CONTROLLER_CONNECTED)
    controllerListener:registerScriptHandler(onDisconnectedController, cc.Handler.EVENT_CONTROLLER_DISCONNECTED)

    controllerListener:registerScriptHandler(onKeyDown, cc.Handler.EVENT_CONTROLLER_KEYDOWN)
    controllerListener:registerScriptHandler(onKeyUp, cc.Handler.EVENT_CONTROLLER_KEYUP)
    controllerListener:registerScriptHandler(onAxisEvent, cc.Handler.EVENT_CONTROLLER_AXIS)

    local eventDispatcher = P.bagUi:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(controllerListener, P.bagUi)
    cc.Controller:startDiscoveryController()
end

function P.setItemSelect()
    if P.itemUseSign then
        cclog("P.itemUseSign true")
        if P.focusAni ~= nil then
            P.focusAni:setVisible(false)
            if P.itemInfoView ~= nil then
                P.itemInfoView.setItemSign(true)
            end
        end
    else
        cclog("P.itemUseSign false,pos = "..P.currentSelectedIndex)
        local pos = P.currentSelectedIndex
        if pos ~= -1 then
            P.setSelectPos(pos)
        end
        P.itemInfoView.setItemSign(false)
    end
end

function P.getItemUseType()
    return P.itemUseSign
end

function P.setItemUseType(type)
    P.itemUseSign = type
end

function P.showChoice()
    local pos = P.currentSelectedIndex
    cclog("pos = "..pos)
    if pos ~= -1 then
        P.setSelectPos(pos)
    else
        P.setSelectPos(1)
    end
end

function P:setBlock(type)
    P.tapBlock = type
end

function P:getBlock()
    return P.tapBlock
end

return P
