local UILib = require("common.UILib")
local Enum = require("common.Enum")
local Util = require("common.Util")

local Text = require("conf.Text")
local IDX = require("conf.TextIndex")

local GamePool = require("model.GamePool")

local table = table

local TAG = {
    BUT_MAP = 7,
    SCROLLVIEW = 8,
    BIG_MAP = 10,
    BIG_MAP_BG = 11,
    CLOSE_MAP = 12,
    PLAYER = 13,
    TASK = 14,
}

local ControllerHolder = class("ControllerHolder")
function ControllerHolder:ctor()
    self.controller  = nil
    self._holderNode = nil

    self._leftJoystick = nil
    self._rightJoystick = nil

    self._dpadLeft = nil
    self._dpadRight = nil
    self._dpadUp = nil
    self._dpadDown = nil

    self._buttonX = nil
    self._buttonY = nil
    self._buttonA = nil
    self._buttonB = nil

    self._buttonL1 = nil
    self._buttonR1 = nil
    self._buttonL2 = nil
    self._buttonR2 = nil

    self.deviceLabel = nil
    self.externalKeyLabel = nil
end

local P = {}

P.mapUi = nil
P.scrollView = nil
P.bigMap = nil
P.bigMapBg = nil
P.infoParentY = 0
P.innerContainerHeight = 0
P.touchBeganY = 0
P.bigMapOriginY = 0
P.playerIcon = nil
P.taskIcon = nil
P.taskIconOrginY = 0
P.isTaskIconUpAnim = false

P.titleButs = {}

P._firstHolder = nil

P.focusAni = nil

P.tapSelected = -1

function P.init(_mapUi)
    local function doTitleAction(sender, eventType)
        local tag = sender:getTag()
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            P.updateTitleButtonsStatus(tag)
            P.updateTaskInfomation(tag-1)
        end
    end

    if P.mapUi == nil then
        cclog("PluginBag init!!!")
        P.mapUi = _mapUi

        local mapBut = UILib.getNodeByTag(P.mapUi,TAG.BUT_MAP)

        local drawX = P.mapUi:getPositionX() + 42
        local drawY = 0
        if mapBut ~= nil then
            drawY = mapBut:getPositionY() - mapBut:getContentSize().height/2 - 25
        end

        local extName = "pngHK"
        if GamePool.getLocalLang() == Enum.LANG.EN then
            extName = "pngEN"
        end
        local imgName1 = string.format("%s.%s",Enum.IMG_NAME.BUT_BG_1,extName)
        local imgName2 = string.format("%s.%s",Enum.IMG_NAME.BUT_BG_2,extName)

        local extraNum = lk.LakooManager:getInstance():getQuestQuantity()
        local total = extraNum + 1
        for var=1, total do
            local titleButton = ccui.Button:create(imgName1,imgName2,imgName2)
            local pos = cc.p(drawX + titleButton:getContentSize().width/2, drawY)
            titleButton:setPosition(pos)
            titleButton:setTag(var)
            P.mapUi:addChild(titleButton)

            --add button text img
            local textImgName = "p_58"
            if var == 1 then
                textImgName = "p_57"
            end
            textImgName = string.format("%s.%s",textImgName,Text.get(IDX.PNG_TYPE))
            local textImg = ccui.ImageView:create(textImgName)
            pos = cc.p(titleButton:getContentSize().width/2, titleButton:getContentSize().height/2)
            textImg:setPosition(pos)
            titleButton:addChild(textImg)

            titleButton:addTouchEventListener(doTitleAction)
            table.insert(P.titleButs,titleButton)

            drawX = drawX + titleButton:getContentSize().width
        end

        P.scrollView = UILib.getNodeByTag(P.mapUi,TAG.SCROLLVIEW)
        P.scrollView:setClippingEnabled(true)

        P.bigMap = UILib.getNodeByTag(P.mapUi,TAG.BIG_MAP)

        local function doMapAction(sender, eventType)
            if eventType == ccui.TouchEventType.began then
            end
            if eventType == ccui.TouchEventType.moved then
            end
            if eventType == ccui.TouchEventType.ended then
                if P.bigMap ~= nil then
                    P.bigMap:setVisible(true)
                    P.bigMap:getParent():setLocalZOrder(1)
                    P.bigMapBg:setPositionY(P.bigMapOriginY)--//TODO:角色所在地

                    local playerX = lk.LakooManager:getInstance():getPlayerInMapDataX()
                    local playerY = lk.LakooManager:getInstance():getPlayerInMapDataY()
                    if P.playerIcon ~= nil then
                        P.playerIcon:setPosition(cc.p(playerX+P.playerIcon:getContentSize().width/2, P.bigMapBg:getContentSize().height-playerY-P.playerIcon:getContentSize().height/2))
                    end

                    local taskX = lk.LakooManager:getInstance():getMainQuestInMapDataX()
                    local taskY = lk.LakooManager:getInstance():getMainQuestInMapDataY()
                    if P.taskIcon ~= nil then
                        P.taskIcon:setPosition(cc.p(taskX+P.taskIcon:getContentSize().width/2, P.bigMapBg:getContentSize().height-taskY))
                        P.taskIconOrginY = P.taskIcon:getPositionY()
                    end
                end
            end
        end

        local mapBut = UILib.getNodeByTag(P.mapUi,TAG.BUT_MAP)
        if mapBut ~= nil then
            mapBut:addTouchEventListener(doMapAction)
        end

        P.initBigMap()

        local colseBut = UILib.getNodeByTag(P.mapUi,TAG.CLOSE_MAP)
        local function doMapCloseAction(sender, eventType)
            if eventType == ccui.TouchEventType.began then
            end
            if eventType == ccui.TouchEventType.moved then
            end
            if eventType == ccui.TouchEventType.ended then
                if P.bigMap ~= nil then
                    P.bigMap:setVisible(false)
                    P.bigMap:getParent():setLocalZOrder(-1)
                end
            end
        end
        if colseBut ~= nil then
            colseBut:addTouchEventListener(doMapCloseAction)
        end
    end

    doTitleAction(P.titleButs[1],ccui.TouchEventType.ended)--默认首先打开主线任务

    local osType = Enum.OS_TYPE.IOS
    osType = lk.LakooManager:getInstance():getOSType()
    if osType == Enum.OS_TYPE.ANDROID then
        --joe
        if P._firstHolder == nil then
            P._firstHolder  = ControllerHolder.new()
            P._firstHolder.controller = nil
            P:registerControllerListener(doTitleAction)
        end

        if P.focusAni == nil then
            ccs.ArmatureDataManager:getInstance():addArmatureFileInfo("FocusAni.csb")
            P.focusAni = ccs.Armature:create("FocusAni")
            P.mapUi:addChild(P.focusAni)
            P.focusAni:setLocalZOrder(99)
            P.focusAni:getAnimation():playWithIndex(0)
            P.focusAni:setVisible(false)
        end
        --
    end
end

function P.updateTitleButtonsStatus(selectTag)
    for key, var in ipairs(P.titleButs) do
        local tag = var:getTag()
        if tag == selectTag then
            var:setHighlighted(true)
        else
            var:setHighlighted(false)
        end
    end
end

function P.updateTaskInfomation(taskIndex)
    if P.scrollView == nil then
        return
    end

    local innerHeight = 0
    local detail = nil
    if taskIndex == 0 then
        detail = lk.LakooManager:getInstance():getMainMissionDetail()
    else
        detail = lk.LakooManager:getInstance():getMissionDetailByQuestIndex(taskIndex-1)
    end

    P.scrollView:getInnerContainer():removeAllChildren(true)
    P.scrollView:getInnerContainer():setPositionY(P.infoParentY)

    --local label = CCLabelTTF:create(detail, UILib.FONT_NAME_NORMAL, UILib.FONT.SMALL, cc.size(P.scrollView:getContentSize().width-10, 0), kCCVerticalTextAlignmentTop, kCCTextAlignmentLeft)
    --label:setColor(UILib.UI_COLOR3B.BLACK)
    --pos = cc.p(label:getContentSize().width/2,label:getContentSize().height/2)
    --label:setPosition(pos)
    --P.scrollView:getInnerContainer():addChild(label)

    local uiList = {}

    local list,len = Util.tab_cutText(detail)
    local lx = 0
    local ly = 0
    local DEFAULT_COLOR = UILib.UI_COLOR3B.BLACK
    local rColor = UILib.UI_COLOR3B.RED
    local isRichText = false
    local colorIdx = 0
    local textIdx = 0
    local labelSide = 21--[[avr为"\n"时，label height为"0"，所以需要一个固定值来换行--]]
    for key, var in ipairs(list) do
        if isRichText then
            if colorIdx <= 6 then
                colorIdx = colorIdx + 1
            end
        else
            colorIdx = 0
        end

        if colorIdx == 0 or colorIdx > 6 then--[[排除颜色代码--]]
            local tLabel = cc.Label:create()
            tLabel:setString(var)
            tLabel:setSystemFontSize(UILib.FONT.SMALL)

            if var == "#" then
                if isRichText then
                    isRichText = false
                else
                    isRichText = true
                    local colorCode = ""
                    for i=key+1, key+6, 1 do
                        colorCode = string.format("%s%s",colorCode,list[i])
                    end
                    --cclog("colorCode=%s",colorCode)
                    rColor = Util.getTextColor(colorCode)
                end
            end
            if isRichText then
                tLabel:setColor(rColor)
            else
                tLabel:setColor(DEFAULT_COLOR)
            end

            if textIdx == 0 then
                lx = labelSide/2+5
                ly = labelSide/2
            end
            textIdx = textIdx + 1
            tLabel:setPosition(cc.p(lx, ly))
            if var ~= "\n" and var ~= "#" then
                P.scrollView:getInnerContainer():addChild(tLabel)
                table.insert(uiList, tLabel)
            end
            lx = lx + tLabel:getContentSize().width
            if lx > P.scrollView:getContentSize().width-10 or var == "\n" then
                lx = labelSide/2+5
                ly = ly - labelSide

                innerHeight = innerHeight + labelSide
            end
        end
    end

    innerHeight = innerHeight + labelSide

    --[[
    local label = cc.Label:create()
    label:setString(detail)
    label:setSystemFontSize(UILib.FONT.SMALL)
    label:setColor(UILib.UI_COLOR3B.BLACK)
    label:setPosition(cc.p(label:getContentSize().width/2 + 10, P.scrollView:getContentSize().height - label:getContentSize().height/2))
    P.scrollView:getInnerContainer():addChild(label)
    --]]

    --innerHeight = innerHeight + label:getContentSize().height

    if taskIndex ~= 0 then
        local itemId = lk.LakooManager:getInstance():getItemIdByQuestIndex(taskIndex-1)
        if itemId > 0 then
            local iconImgIndex = lk.LakooManager:getInstance():getIconImageIndexByItemId(itemId)
            local iconInImgIndex = lk.LakooManager:getInstance():getIconInImageIndexByItemId(itemId)
            local icon = UILib.createClipSprite(iconImgIndex,iconInImgIndex)
            local pos = cc.p(icon:getContentSize().width/2 + 10, ly - labelSide/2 - icon:getContentSize().height/2 -5)
            icon:setPosition(pos)
            P.scrollView:getInnerContainer():addChild(icon)

            table.insert(uiList, icon)

            local itemName = lk.LakooManager:getInstance():getItemNameByItemId(itemId)
            local nameLabel = cc.Label:create()
            nameLabel:setString(itemName)
            nameLabel:setSystemFontSize(UILib.FONT.SMALL)
            nameLabel:setColor(UILib.UI_COLOR3B.BLACK)
            nameLabel:setPosition(cc.p(icon:getPositionX()+icon:getContentSize().width/2+nameLabel:getContentSize().width/2+5,icon:getPositionY()))
            P.scrollView:getInnerContainer():addChild(nameLabel)

            table.insert(uiList, nameLabel)

            local num = lk.LakooManager:getInstance():getProgressByQuestIndex(taskIndex-1)
            local numLab = cc.LabelAtlas:_create(num, Enum.DIGITAL.RED, 198/21, 11, string.byte('0'))
            numLab:setPosition(cc.p(nameLabel:getPositionX()+nameLabel:getContentSize().width/2+5,icon:getPositionY()-numLab:getContentSize().height/2))
            P.scrollView:getInnerContainer():addChild(numLab)

            table.insert(uiList, numLab)

            local total = lk.LakooManager:getInstance():getTaskTotalByQuestIndex(taskIndex-1)
            local totalLab = cc.LabelAtlas:_create(">"..total,Enum.DIGITAL.BEIGE,198/21,11,string.byte('0'))
            totalLab:setPosition(cc.p(numLab:getPositionX()+numLab:getContentSize().width, numLab:getPositionY()))
            P.scrollView:getInnerContainer():addChild(totalLab)

            table.insert(uiList, totalLab)

            innerHeight = innerHeight + icon:getContentSize().height
        end
    end

    P.innerContainerHeight = innerHeight
    local contentHeight = P.scrollView:getContentSize().height
    local fixHeight = 0
    if innerHeight < contentHeight then
        innerHeight = contentHeight
        fixHeight = fixHeight + contentHeight
    else
        fixHeight = fixHeight + innerHeight
    end
    P.scrollView:setInnerContainerSize(cc.size(P.scrollView:getInnerContainerSize().width, innerHeight))

    for key, var in ipairs(uiList) do
        local posY = var:getPositionY()
        posY = posY + fixHeight
        var:setPositionY(posY)
    end
end

function P.updateTaskInfomationAnim()
    if P.scrollView == nil then
        return
    end
    local drawY = P.scrollView:getInnerContainer():getPositionY()
    --cclog("drawY:%f,ih:%f,ch:%f",drawY,P.innerContainerHeight,P.scrollView:getContentSize().height)
    drawY = drawY + 1
    if drawY >= P.infoParentY + (P.innerContainerHeight - P.scrollView:getContentSize().height) then
        drawY = P.infoParentY - P.innerContainerHeight
    end
    P.scrollView:getInnerContainer():setPositionY(drawY)
end

function P.updateTaskIcon()
    if P.taskIcon == nil then
        return
    end

    local newY = P.taskIcon:getPositionY()
    if P.isTaskIconUpAnim then
        newY = newY - 1
        if newY <= P.taskIconOrginY - 2 then
            P.isTaskIconUpAnim = false
        end
    else
        newY = newY + 1
        if newY >= P.taskIconOrginY + 2 then
            P.isTaskIconUpAnim = true
        end
    end
    P.taskIcon:setPositionY(newY)
end

function P.updateMap()
    P.updateTaskInfomationAnim()
    P.updateTaskIcon()
end

function P.clean()
    P.mapUi = nil
    for var=1, #P.titleButs do
        P.titleButs[var] = nil
    end
    P.scrollView = nil
    P.bigMap = nil
    P.bigMapBg = nil
    P.innerContainerHeight = 0
    P.bigMapOriginY = 0
    P.playerIcon = nil
    P.taskIcon = nil
    P.taskIconOrginY = 0
    P.isTaskIconUpAnim = false
    
    P.focusAni = nil
end

local function touchBegan(touch, event)
    P.touchBeganY = touch:getLocation().y
    return true
end
local function touchMoved(touch, event)
    local nowY = touch:getLocation().y
    local distanceY = nowY - P.touchBeganY
    local orginY = P.bigMapBg:getPositionY()
    local newY = orginY + distanceY
    local visibleSize = cc.Director:getInstance():getVisibleSize()
    local visibleHeight = visibleSize.height
    local downLimit = P.bigMapOriginY + (P.bigMapBg:getContentSize().height-visibleHeight)/2
    local upLimit = P.bigMapOriginY - (P.bigMapBg:getContentSize().height-visibleHeight)/2
    if newY > downLimit then
        newY = downLimit
    end
    if newY < upLimit then
        newY = upLimit
    end
    P.bigMapBg:setPositionY(newY)
end
local function touchEnded(touch, event)
--TODO:runAction复原到角色所在位置
end

function P.initBigMap()
    if P.bigMap == nil then
        return
    end

    P.bigMapBg = UILib.getNodeByTag(P.bigMap,TAG.BIG_MAP_BG)
    if P.bigMapBg == nil then
        return
    end

    P.playerIcon = UILib.getNodeByTag(P.mapUi,TAG.PLAYER)
    P.taskIcon = UILib.getNodeByTag(P.mapUi,TAG.TASK)

    P.bigMapOriginY = UILib.offset(0,0,P.bigMapBg:getContentSize().width,P.bigMapBg:getContentSize(),UILib.UI_ANCHOR.CENTER).y

    --单点触控
    local eventListener = cc.EventListenerTouchOneByOne:create()
    eventListener:registerScriptHandler(touchBegan, cc.Handler.EVENT_TOUCH_BEGAN)
    eventListener:registerScriptHandler(touchMoved, cc.Handler.EVENT_TOUCH_MOVED)
    eventListener:registerScriptHandler(touchEnded, cc.Handler.EVENT_TOUCH_ENDED)
    eventListener:setSwallowTouches(false)

    local eventDispatcher = P.bigMap:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(eventListener, P.bigMapBg)
end

function P.dpadUp()
    cclog("PluginMap dpadUp()")
    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        if layer:getTitleType() ~= 5 then
            cclog("first return")
            return
        end
    end
    
    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        if layer:getMainPopLayerTapType() then
            return
        else
            layer:setMainPopLayerTap(true)
            if P.focusAni ~=nil then
                P.focusAni:setVisible(false)
            end
        end
    end
end

function P.dpadDown(doTitleAction)
    cclog("PluginMap dpadDown()")
    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        if layer:getTitleType() ~= 5 then
            cclog("first return")
            return
        end
    end

    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        if layer:getMainPopLayerTapType() then
            layer:setMainPopLayerTap(false)
            P.tapSelected = 1
            local btn = P.titleButs[1]
            if btn ~= nil then
                local pos = cc.p(btn:getPositionX(), btn:getPositionY())
                P.focusAni:setPosition(pos)
                P.focusAni:setVisible(true)
                doTitleAction(P.titleButs[1],ccui.TouchEventType.ended)
            end
            cclog("second return")
            return
        end
    end
end

function P.dpadLeft(doTitleAction)
    cclog("PluginMap dpadLeft()")
    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        if layer:getTitleType() ~= 5 then
            cclog("first return")
            return
        end
    end

    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        if layer:getMainPopLayerTapType() then
            return
        end
    end
    
    if P.tapSelected == 1 then
        return
    end
    
    local btn = P.titleButs[1]
    if btn ~= nil then
        P.tapSelected = 1
        local pos = cc.p(btn:getPositionX(), btn:getPositionY())
        P.focusAni:setPosition(pos)
        P.focusAni:setVisible(true)
        doTitleAction(P.titleButs[1],ccui.TouchEventType.ended)
    end
end

function P.dpadRight(doTitleAction)
    cclog("PluginMap dpadRight()")
    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        if layer:getTitleType() ~= 5 then
            cclog("first return")
            return
        end
    end
    
    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        if layer:getMainPopLayerTapType() then
            return
        end
    end

    if P.tapSelected == 2 then
        return
    end

    local btn = P.titleButs[2]
    if btn ~= nil then
        P.tapSelected = 2
        local pos = cc.p(btn:getPositionX(), btn:getPositionY())
        P.focusAni:setPosition(pos)
        P.focusAni:setVisible(true)
        doTitleAction(P.titleButs[2],ccui.TouchEventType.ended)
    end
end

function P.dpadTap()

end

function P:registerControllerListener(doTitleAction)
    local function onConnectController(controller, event)
        if controller == nil or controller == self._firstHolder.controller then
            return
        end

        local deviceInfo = string.format("%s id:%d", controller:getDeviceName(), controller:getDeviceId())
        cclog("PluginStatus deviceInfo ="..deviceInfo)

        if self._firstHolder.controller == nil then
            self._firstHolder.controller = controller
        end
    end

    local function onDisconnectedController(controller, event)
        if self._firstHolder.controller == controller then
            self._firstHolder.controller = nil
        end
    end

    local function showButtonState(controller, keyCode, isPressed)
        onConnectController(controller, nil)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end

        local scene = cc.Director:getInstance():getRunningScene()
        local child = scene:getChildren()

        local topSence = child[#child]
        if topSence:getName() ~= "MainPopLayer" and topSence:getName() ~= "SinglePopLayer" then
            return
        end
        if isPressed then
            if keyCode == cc.ControllerKey.BUTTON_A then
            elseif keyCode == cc.ControllerKey.BUTTON_B then
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key Down:%d",keyCode)
            end
        else
            cclog("PluginStatus keyup ="..keyCode)

            if keyCode == cc.ControllerKey.BUTTON_A then
                P.dpadTap()
            elseif keyCode == cc.ControllerKey.BUTTON_B then

            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then

            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
                P.dpadUp()
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
                P.dpadDown(doTitleAction)
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
                P.dpadLeft(doTitleAction)
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
                P.dpadRight(doTitleAction)
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then

            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then

            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key up:%d",keyCode)
            end
        end
    end

    local function onKeyDown(controller, keyCode, event)
        showButtonState(controller, keyCode, true)
    end

    local function onKeyUp(controller, keyCode, event)
        showButtonState(controller, keyCode, false)
    end

    local function onAxisEvent(controller, keyCode, event)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end

        local ketStatus = controller:getKeyStatus(keyCode)
        if keyCode == cc.ControllerKey.JOYSTICK_LEFT_X then
            if(ketStatus.value == -1) then
            elseif(ketStatus.value == 1) then

            end
        elseif keyCode == cc.ControllerKey.JOYSTICK_LEFT_Y then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_X then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_Y then
        elseif keyCode == cc.ControllerKey.AXIS_LEFT_TRIGGER then
        elseif keyCode == cc.ControllerKey.AXIS_RIGHT_TRIGGER then
        else
        --do nothing
        end
    end

    local controllerListener = cc.EventListenerController:create()
    controllerListener:registerScriptHandler(onConnectController, cc.Handler.EVENT_CONTROLLER_CONNECTED)
    controllerListener:registerScriptHandler(onDisconnectedController, cc.Handler.EVENT_CONTROLLER_DISCONNECTED)

    controllerListener:registerScriptHandler(onKeyDown, cc.Handler.EVENT_CONTROLLER_KEYDOWN)
    controllerListener:registerScriptHandler(onKeyUp, cc.Handler.EVENT_CONTROLLER_KEYUP)
    controllerListener:registerScriptHandler(onAxisEvent, cc.Handler.EVENT_CONTROLLER_AXIS)

    local eventDispatcher = P.mapUi:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(controllerListener, P.mapUi)
    cc.Controller:startDiscoveryController()
end

function P.cleanholer()
    if P.mapUi ~= nil then
        local eventDispatcher = P.mapUi:getEventDispatcher()
        eventDispatcher:removeEventListenersForTarget(P.mapUi, false)
        P._firstHolder = nil
        P.tapSelected = -1
    end
end

return P