local UILib = require("common.UILib")
local Enum = require("common.Enum")
local Util = require("common.Util")

local UpgradeView = require("gameUI.component.UpgradeView")

local LayerStage = require("manager.LayerStage")

local table = table

local TAG = {
    ANIM_BACKGROUND = 6,
    ITEM = {
        B_1 = 7,
        B_2 = 9,
        B_3 = 10,
        B_4 = 11,
        B_5 = 12,
    },
    POINTS = {
        DECREASE_1 = 13,
        DECREASE_2 = 15,
        DECREASE_3 = 16,
        INCREASE_1 = 17,
        INCREASE_2 = 19,
        INCREASE_3 = 20,
        LAB_BG_1 = 21,
        LAB_BG_2 = 23,
        LAB_BG_3 = 24,
    },
    DETAIL = {
        D_1 = 25,
        D_2 = 27,
        D_3 = 28,
        D_4 = 29,
        D_5 = 30,
        D_6 = 31,
    },
}

local ControllerHolder = class("ControllerHolder")
function ControllerHolder:ctor()
    self.controller  = nil
    self._holderNode = nil

    self._leftJoystick = nil
    self._rightJoystick = nil

    self._dpadLeft = nil
    self._dpadRight = nil
    self._dpadUp = nil
    self._dpadDown = nil

    self._buttonX = nil
    self._buttonY = nil
    self._buttonA = nil
    self._buttonB = nil

    self._buttonL1 = nil
    self._buttonR1 = nil
    self._buttonL2 = nil
    self._buttonR2 = nil

    self.deviceLabel = nil
    self.externalKeyLabel = nil
end

local P = {}

P.initAnimNode = false

P.statusUi = nil
P.animBackground = nil
P.hpBarBg2Node = nil
P.hpBar = nil
P.mpBar = nil
P.expBar = nil
P.hpLabel = nil
P.mpLabel = nil
P.expLabel = nil
P.pointsLabel = nil
P.strLable = nil
P.intLabel = nil
P.agiLabel = nil

P.tempStr = 0--临时加上可重新减去的points[力量]
P.tempInt = 0--临时加上可重新减去的points[智慧]
P.tempAgi = 0--临时加上可重新减去的points[敏捷]

P.decBut_1 = nil
P.incBut_1 = nil
P.decBut_2 = nil
P.incBut_2 = nil
P.decBut_3 = nil
P.incBut_3 = nil

P.panelWeapon = nil
P.panelArmor = nil
P.panelAdorn = nil
P.panelBottle = nil
P.panelCatch = nil

P.detailComponets = {}

P._firstHolder = nil

P.focusAni = nil

P.equipSelected = -1
P.pointSelected = -1
P.equipmentRow = false
P.pointRow = false
P.EquipMentTable = {}
P.PointTable = {}
P.oneTapfinish = true
P.tapBlock = false

function P.init(_statusUi)
    if P.statusUi == nil then
        cclog("PluginStatus init!!!")
        P.statusUi = _statusUi
        P.animBackground = UILib.getNodeByTag(P.statusUi,TAG.ANIM_BACKGROUND)

        --name
        local drawX = P.animBackground:getPositionX() + P.animBackground:getContentSize().width
        local drawY = P.animBackground:getPositionY() + P.animBackground:getContentSize().height/2
        local textImg = UILib.addImageTo(P.statusUi,"p_21",cc.p(drawX,drawY))
        drawY = textImg:getPositionY() - textImg:getContentSize().height
        textImg:setPositionY(drawY)
        local textWidth = textImg:getContentSize().width

        --lv
        drawY = drawY - textImg:getContentSize().height
        textImg = UILib.addImageTo(P.statusUi,"p_22",cc.p(drawX,drawY))
        drawX = drawX - (textWidth - textImg:getContentSize().width)/2
        textImg:setPositionX(drawX)
        textWidth = textImg:getContentSize().width

        local lv = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.LEVEL)
        local effect = cc.LabelAtlas:_create(tostring(lv), Enum.DIGITAL.WHITE_ROUND, 253/21, 14, string.byte('0'))
        effect:setPosition(drawX + textWidth/2+10,drawY - textImg:getContentSize().height/2)
        P.statusUi:addChild(effect)

        --point
        drawY = drawY - textImg:getContentSize().height*1.5
        textImg = UILib.addImageTo(P.statusUi,"p_27",cc.p(drawX,drawY))
        drawX = drawX - (textWidth - textImg:getContentSize().width)/2
        textImg:setPositionX(drawX)

        local points = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.POINTS)
        P.pointsLabel = cc.LabelAtlas:_create(tostring(points), Enum.DIGITAL.WHITE_ROUND, 253/21, 14, string.byte('0'))
        P.pointsLabel:setPosition(drawX + textImg:getContentSize().width/2+10,drawY- textImg:getContentSize().height/2 + 2)
        P.statusUi:addChild(P.pointsLabel)

        --HP,MP,EXP
        local startPos = cc.p(255, 223)
        P.initHp_Mp_Exp(startPos)

        --Points
        P.initPoints()
        P.initDetailComponet()
        P.initEquipments()
    end

    P.tempStr = 0
    P.tempInt = 0
    P.tempAgi = 0
    P.updateStatus()
    --P.updateHp_Mp_Exp()
    --P.updateDecAndIncButtonStatus()
    P.updateEquipment()

    local osType = Enum.OS_TYPE.IOS
    osType = lk.LakooManager:getInstance():getOSType()
    if osType == Enum.OS_TYPE.ANDROID then
        --joe
        if P._firstHolder == nil then
            P._firstHolder  = ControllerHolder.new()
            P._firstHolder.controller = nil
            P:registerControllerListener()
        end

        if P.focusAni == nil then
            ccs.ArmatureDataManager:getInstance():addArmatureFileInfo("FocusAni.csb")
            P.focusAni = ccs.Armature:create("FocusAni")
            P.statusUi:addChild(P.focusAni)
            P.focusAni:setLocalZOrder(99)
            P.focusAni:getAnimation():playWithIndex(0)
            P.focusAni:setVisible(false)
        end
        --
    end
end

function P.doWeaponAction()
    local itemCount = lk.LakooManager:getInstance():getItemArrayCountByTag(Enum.BAG_TAG.EQUIPMENT_WEAPON)
    if itemCount > 0 then
        LayerStage.setShowLayerType(Enum.LAYER_TYPE.SINGLE_POP_EQUIPMENT_WEAPON)
    else
        local msg = lk.LakooManager:getInstance():getString(Enum.TEXT.T_NOT_EQUIPMENT_IN_BAG)
        lk.LakooManager:getInstance():addPopTip(msg)
    end
end

function P.doArmorAction()
    local itemCount = lk.LakooManager:getInstance():getItemArrayCountByTag(Enum.BAG_TAG.EQUIPMENT_ARMOUR)
    if itemCount > 0 then
        LayerStage.setShowLayerType(Enum.LAYER_TYPE.SINGLE_POP_EQUIPMENT_ARMOUR)
    else
        local msg = lk.LakooManager:getInstance():getString(Enum.TEXT.T_NOT_EQUIPMENT_IN_BAG)
        lk.LakooManager:getInstance():addPopTip(msg)
    end
end

function P.doAdornAction()
    local itemCount = lk.LakooManager:getInstance():getItemArrayCountByTag(Enum.BAG_TAG.EQUIPMENT_ADORN_DATA)
    if itemCount > 0 then
        LayerStage.setShowLayerType(Enum.LAYER_TYPE.SINGLE_POP_EQUIPMENT_ADORN_DATA)
    else
        local msg = lk.LakooManager:getInstance():getString(Enum.TEXT.T_NOT_ADRON_IN_BAG)
        lk.LakooManager:getInstance():addPopTip(msg)
    end
end

function P.doBottleAction()
    local itemJsonStr = lk.LakooManager:getInstance():getUpgradeItemInfoJsonByPos(Enum.EP.BOTTLE)
    if itemJsonStr == nil or itemJsonStr == "" then
        local msg = lk.LakooManager:getInstance():getString(Enum.TEXT.T_LOCK_TIPS)
        lk.LakooManager:getInstance():addPopTip(msg)
        return
    end
    local upgradeItemInfo = Util.json2Table(itemJsonStr)
    P.showUpgradeView(upgradeItemInfo)
end

function P.doCatchAction()
    local itemJsonStr = lk.LakooManager:getInstance():getUpgradeItemInfoJsonByPos(Enum.EP.CATCH)
    if itemJsonStr == nil or itemJsonStr == "" then
        local msg = lk.LakooManager:getInstance():getString(Enum.TEXT.T_LOCK_TIPS)
        lk.LakooManager:getInstance():addPopTip(msg)
        return
    end
    local upgradeItemInfo = Util.json2Table(itemJsonStr)
    P.showUpgradeView(upgradeItemInfo)
end

function P.showUpgradeView(upgradeItemInfo)
    if P.statusUi == nil then
        return
    end
    local upgradeView = UpgradeView:create(upgradeItemInfo)
    local targetPos = P.statusUi:convertToWorldSpace(cc.p(0,0))
    targetPos = cc.p(-targetPos.x,-targetPos.y)
    local initPos = cc.p(targetPos.x, targetPos.y-upgradeView:getContentSize().height)
    upgradeView:setPosition(initPos)
    P.statusUi:addChild(upgradeView)

    --action move
    local moveAct = cc.MoveTo:create(0.2, targetPos)
    upgradeView:runAction(moveAct)

    upgradeView:setUpdateEquipmentFun(P.updateEquipment)
    P.tapBlock = true
end

function P.initEquipments()
    local butWeapon = UILib.getNodeByTag(P.statusUi,TAG.ITEM.B_1)
    if butWeapon ~= nil then
        P.panelWeapon = cc.Node:create()
        P.panelWeapon:setContentSize(butWeapon:getContentSize())
        butWeapon:addChild(P.panelWeapon)
    end
    table.insert(P.EquipMentTable,1,butWeapon)

    local butArmor = UILib.getNodeByTag(P.statusUi,TAG.ITEM.B_2)
    if butArmor ~= nil then
        P.panelArmor = cc.Node:create()
        P.panelArmor:setContentSize(butArmor:getContentSize())
        butArmor:addChild(P.panelArmor)
    end
    table.insert(P.EquipMentTable,2,butArmor)

    local butAdorn = UILib.getNodeByTag(P.statusUi,TAG.ITEM.B_3)
    if butAdorn ~= nil then
        P.panelAdorn = cc.Node:create()
        P.panelAdorn:setContentSize(butAdorn:getContentSize())
        butAdorn:addChild(P.panelAdorn)
    end
    table.insert(P.EquipMentTable,3,butAdorn)

    local butBottle = UILib.getNodeByTag(P.statusUi,TAG.ITEM.B_4)
    if butBottle ~= nil then
        P.panelBottle = cc.Node:create()
        P.panelBottle:setContentSize(butBottle:getContentSize())
        butBottle:addChild(P.panelBottle)
    end
    table.insert(P.EquipMentTable,4,butBottle)

    local butCatch = UILib.getNodeByTag(P.statusUi,TAG.ITEM.B_5)
    if butCatch ~= nil then
        P.panelCatch = cc.Node:create()
        P.panelCatch:setContentSize(butCatch:getContentSize())
        butCatch:addChild(P.panelCatch)
    end
    table.insert(P.EquipMentTable,5,butCatch)

    local function touchAction(sender, eventType)
        if eventType == ccui.TouchEventType.ended then
            local tag = sender:getTag()
            if tag == TAG.ITEM.B_1 then
                P.doWeaponAction()
            elseif tag == TAG.ITEM.B_2 then
                P.doArmorAction()
            elseif tag == TAG.ITEM.B_3 then
                P.doAdornAction()
            elseif tag == TAG.ITEM.B_4 then
                P.doBottleAction()
            elseif tag == TAG.ITEM.B_5 then
                P.doCatchAction()
            end
        end
    end
    butWeapon:addTouchEventListener(touchAction)
    butArmor:addTouchEventListener(touchAction)
    butAdorn:addTouchEventListener(touchAction)
    butBottle:addTouchEventListener(touchAction)
    butCatch:addTouchEventListener(touchAction)
end

function P.updateEquipment()
    if P.panelWeapon ~= nil then
        P.panelWeapon:removeAllChildren()
        lk.LakooManager:getInstance():addEquipIcon(P.panelWeapon, Enum.EP.WEAPON)
    end
    if P.panelArmor ~= nil then
        P.panelArmor:removeAllChildren()
        lk.LakooManager:getInstance():addEquipIcon(P.panelArmor, Enum.EP.ARMOR)
    end
    if P.panelAdorn ~= nil then
        P.panelAdorn:removeAllChildren()
        lk.LakooManager:getInstance():addEquipIcon(P.panelAdorn, Enum.EP.ADORN)
    end
    if P.panelBottle ~= nil then
        P.panelBottle:removeAllChildren()
        lk.LakooManager:getInstance():addEquipIcon(P.panelBottle, Enum.EP.BOTTLE)
    end
    if P.panelCatch ~= nil then
        P.panelCatch:removeAllChildren()
        lk.LakooManager:getInstance():addEquipIcon(P.panelCatch, Enum.EP.CATCH)
    end
end

--点击力量、智慧、敏捷属性增加或减少时更新相关联的其它组件
function P.updateStatus()
    local points = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.POINTS)
    P.pointsLabel:setString(tostring(points))

    local point = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.STRENGTH)
    P.strLable:setString(tostring(point))
    point = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.INTELLIGENCE)
    P.intLabel:setString(tostring(point))
    point = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.AGILITY)
    P.agiLabel:setString(tostring(point))

    for var=1, #P.detailComponets do
        if P.detailComponets[var] ~= nil then
            local tag = P.detailComponets[var]:getTag()
            local pIdx = P.getPropertyIndexByUITag(tag)
            local value = lk.LakooManager:getInstance():getPlayerProperty(pIdx)
            P.detailComponets[var]:setString(value)
        end
    end

    P.updateHp_Mp_Exp()
    P.updateDecAndIncButtonStatus()
end

---攻击、魔攻、命中、防御、魔防、回避
function P.initDetailComponet()
    local function doDetailTitleAction(sender, eventType)
        if eventType == ccui.TouchEventType.began then
            sender:setOpacity(255)
            local fadeout = cc.FadeOut:create(3)
            sender:runAction(fadeout);
        end
        --if eventType == ccui.TouchEventType.moved then
        --end
        if eventType == ccui.TouchEventType.ended then
        end
    end

    local tPos = 1
    for key, var in pairs(TAG.DETAIL) do
        local imgUi = UILib.getNodeByTag(P.statusUi,var)
        local pIdx = P.getPropertyIndexByUITag(var)
        local value = lk.LakooManager:getInstance():getPlayerProperty(pIdx)
        --cclog("tag:%d,value:%d",var,value)
        local lab = cc.LabelAtlas:_create(value, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
        lab:setTag(var)
        local pos = cc.p(imgUi:getPositionX()+imgUi:getContentSize().width/2,imgUi:getPositionY()-lab:getContentSize().height/2)
        lab:setPosition(pos)
        P.statusUi:addChild(lab)

        table.insert(P.detailComponets,tPos,lab)
        tPos = tPos + 1

        local title = P.getTextByUITag(var)
        local titleBut = ccui.Button:create(Enum.IMG_NAME_PNGHK.BUTTON_BG2,Enum.IMG_NAME_PNGHK.BUTTON_BG2,Enum.IMG_NAME_PNGHK.BUTTON_BG2)
        titleBut:setTitleText(title)
        titleBut:setTitleFontSize(UILib.FONT.SMALL)
        titleBut:setPosition(0, lab:getContentSize().height/2)
        lab:addChild(titleBut)

        titleBut:setOpacity(0)
        titleBut:addTouchEventListener(doDetailTitleAction)
    end
end

function P.getPropertyIndexByUITag(tag)
    local pIdx = Enum.PROPERTY.ATTACK
    if tag == TAG.DETAIL.D_2 then
        pIdx = Enum.PROPERTY.MAGIC_ATTACK
    elseif tag == TAG.DETAIL.D_3 then
        pIdx = Enum.PROPERTY.TO_HIT
    elseif tag == TAG.DETAIL.D_4 then
        pIdx = Enum.PROPERTY.DEFEND
    elseif tag == TAG.DETAIL.D_5 then
        pIdx = Enum.PROPERTY.MAGIC_DEFEND
    elseif tag == TAG.DETAIL.D_6 then
        pIdx = Enum.PROPERTY.AVOID
    end

    return pIdx
end

function P.getTextByUITag(tag)
    local tIdx = Enum.TEXT.T_ATK_STRING
    if tag == TAG.DETAIL.D_2 then
        tIdx = Enum.TEXT.T_MAGIC_ATK_STRING
    elseif tag == TAG.DETAIL.D_3 then
        tIdx = Enum.TEXT.T_TOHIT_STRING
    elseif tag == TAG.DETAIL.D_4 then
        tIdx = Enum.TEXT.T_DEF_STRING
    elseif tag == TAG.DETAIL.D_5 then
        tIdx = Enum.TEXT.T_MAGIC_DEF_STRING
    elseif tag == TAG.DETAIL.D_6 then
        tIdx = Enum.TEXT.T_AVOID_STRING
    end
    local text = lk.LakooManager:getInstance():getString(tIdx)

    return text
end

---力量、智慧、敏捷
function P.initPoints()
    local function doDecAndIncAction(sender, eventType)
        if eventType == ccui.TouchEventType.began then
            local tag = sender:getTag()
            if tag == TAG.POINTS.DECREASE_1 then
                P.tempStr = P.tempStr - 1
                lk.LakooManager:getInstance():changePlayerProperty(Enum.PROPERTY.POINTS, 1)
                lk.LakooManager:getInstance():changePlayerProperty(Enum.PROPERTY.STRENGTH, -1)
            elseif tag == TAG.POINTS.INCREASE_1 then
                P.tempStr = P.tempStr + 1
                lk.LakooManager:getInstance():changePlayerProperty(Enum.PROPERTY.POINTS, -1)
                lk.LakooManager:getInstance():changePlayerProperty(Enum.PROPERTY.STRENGTH, 1)
            elseif tag == TAG.POINTS.DECREASE_2 then
                P.tempInt = P.tempInt - 1
                lk.LakooManager:getInstance():changePlayerProperty(Enum.PROPERTY.POINTS, 1)
                lk.LakooManager:getInstance():changePlayerProperty(Enum.PROPERTY.INTELLIGENCE, -1)
            elseif tag == TAG.POINTS.INCREASE_2 then
                P.tempInt = P.tempInt + 1
                lk.LakooManager:getInstance():changePlayerProperty(Enum.PROPERTY.POINTS, -1)
                lk.LakooManager:getInstance():changePlayerProperty(Enum.PROPERTY.INTELLIGENCE, 1)
            elseif tag == TAG.POINTS.DECREASE_3 then
                P.tempAgi = P.tempAgi - 1
                lk.LakooManager:getInstance():changePlayerProperty(Enum.PROPERTY.POINTS, 1)
                lk.LakooManager:getInstance():changePlayerProperty(Enum.PROPERTY.AGILITY, -1)
            elseif tag == TAG.POINTS.INCREASE_3 then
                P.tempAgi = P.tempAgi + 1
                lk.LakooManager:getInstance():changePlayerProperty(Enum.PROPERTY.POINTS, -1)
                lk.LakooManager:getInstance():changePlayerProperty(Enum.PROPERTY.AGILITY, 1)
            end

            lk.LakooManager:getInstance():setUpdateTipsStatus(Enum.LVUP_TIPS_STATUS.LVUP_NONE)
            P.updateStatus()
        end
        --if eventType == ccui.TouchEventType.moved then
        --end
        if eventType == ccui.TouchEventType.ended then
        end
    end

    P.decBut_1 = UILib.getNodeByTag(P.statusUi,TAG.POINTS.DECREASE_1)
    if P.decBut_1 ~= nil then
        local textImg = UILib.addImageTo(P.statusUi,Enum.TEXT_IMG.STR,cc.p(0,0))
        local drawX = P.decBut_1:getPositionX() - textImg:getContentSize().width
        local drawY = P.decBut_1:getPositionY()
        textImg:setPosition(cc.p(drawX, drawY))
    end
    local pointBg_1 = UILib.getNodeByTag(P.statusUi,TAG.POINTS.LAB_BG_1)
    if pointBg_1 ~= nil then
        local point = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.STRENGTH)
        P.strLable = cc.LabelAtlas:_create(tostring(point), Enum.DIGITAL.BLUE, 198/21, 11, string.byte('0'))
        P.strLable:setPosition((pointBg_1:getContentSize().width - P.strLable:getContentSize().width)/2
            ,(pointBg_1:getContentSize().height - P.strLable:getContentSize().height)/2)
        pointBg_1:addChild(P.strLable)
    end
    P.incBut_1 = UILib.getNodeByTag(P.statusUi,TAG.POINTS.INCREASE_1)

    P.decBut_1:addTouchEventListener(doDecAndIncAction)
    P.incBut_1:addTouchEventListener(doDecAndIncAction)
    table.insert(P.PointTable,1,P.decBut_1)
    table.insert(P.PointTable,2,P.incBut_1)
    --#########################################################################
    P.decBut_2 = UILib.getNodeByTag(P.statusUi,TAG.POINTS.DECREASE_2)
    if P.decBut_2 ~= nil then
        local textImg = UILib.addImageTo(P.statusUi,Enum.TEXT_IMG.INT,cc.p(0,0))
        local drawX = P.decBut_2:getPositionX() - textImg:getContentSize().width
        local drawY = P.decBut_2:getPositionY()
        textImg:setPosition(cc.p(drawX, drawY))
    end
    local pointBg_2 = UILib.getNodeByTag(P.statusUi,TAG.POINTS.LAB_BG_2)
    if pointBg_2 ~= nil then
        local point = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.INTELLIGENCE)
        P.intLabel = cc.LabelAtlas:_create(tostring(point), Enum.DIGITAL.BLUE, 198/21, 11, string.byte('0'))
        P.intLabel:setPosition((pointBg_2:getContentSize().width - P.intLabel:getContentSize().width)/2
            ,(pointBg_2:getContentSize().height - P.intLabel:getContentSize().height)/2)
        pointBg_2:addChild(P.intLabel)
    end
    P.incBut_2 = UILib.getNodeByTag(P.statusUi,TAG.POINTS.INCREASE_2)

    P.decBut_2:addTouchEventListener(doDecAndIncAction)
    P.incBut_2:addTouchEventListener(doDecAndIncAction)
    table.insert(P.PointTable,3,P.decBut_2)
    table.insert(P.PointTable,4,P.incBut_2)
    --#########################################################################
    P.decBut_3 = UILib.getNodeByTag(P.statusUi,TAG.POINTS.DECREASE_3)
    if P.decBut_3 ~= nil then
        local textImg = UILib.addImageTo(P.statusUi,Enum.TEXT_IMG.AGI,cc.p(0,0))
        local drawX = P.decBut_3:getPositionX() - textImg:getContentSize().width
        local drawY = P.decBut_3:getPositionY()
        textImg:setPosition(cc.p(drawX, drawY))
    end
    local pointBg_3 = UILib.getNodeByTag(P.statusUi,TAG.POINTS.LAB_BG_3)
    if pointBg_3 ~= nil then
        local point = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.AGILITY)
        P.agiLabel = cc.LabelAtlas:_create(tostring(point), Enum.DIGITAL.BLUE, 198/21, 11, string.byte('0'))
        P.agiLabel:setPosition((pointBg_3:getContentSize().width - P.agiLabel:getContentSize().width)/2
            ,(pointBg_3:getContentSize().height - P.agiLabel:getContentSize().height)/2)
        pointBg_3:addChild(P.agiLabel)
    end
    P.incBut_3 = UILib.getNodeByTag(P.statusUi,TAG.POINTS.INCREASE_3)

    P.decBut_3:addTouchEventListener(doDecAndIncAction)
    P.incBut_3:addTouchEventListener(doDecAndIncAction)
    table.insert(P.PointTable,5,P.decBut_3)
    table.insert(P.PointTable,6,P.incBut_3)
end

function P.updateDecAndIncButtonStatus()
    if P.decBut_1 == nil
        or P.incBut_1 == nil
        or P.decBut_2 == nil
        or P.incBut_2 == nil
        or P.decBut_3 == nil
        or P.incBut_3 == nil then
        return
    end
    if P.tempStr <= 0 then
        P.setButtonEnabled(P.decBut_1,false)
    else
        P.setButtonEnabled(P.decBut_1,true)
    end
    if P.tempInt <= 0 then
        P.setButtonEnabled(P.decBut_2,false)
    else
        P.setButtonEnabled(P.decBut_2,true)
    end
    if P.tempAgi <= 0 then
        P.setButtonEnabled(P.decBut_3,false)
    else
        P.setButtonEnabled(P.decBut_3,true)
    end

    local points = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.POINTS)
    if points > 0 then
        P.setButtonEnabled(P.incBut_1,true)
        P.setButtonEnabled(P.incBut_2,true)
        P.setButtonEnabled(P.incBut_3,true)
    else
        P.setButtonEnabled(P.incBut_1,false)
        P.setButtonEnabled(P.incBut_2,false)
        P.setButtonEnabled(P.incBut_3,false)
    end
end

function P.setButtonEnabled(button, enable)
    if button == nil then
        return
    end
    if enable then
        button:setTouchEnabled(true)
        button:setBright(true)
        button:setHighlighted(false)
    else
        button:setTouchEnabled(false)
        button:setBright(false)
        button:setHighlighted(false)
    end
end

---layer动画结束后，再加载layer内的LKAnim,否则看起来难看
function P.checkInitAnimNode()
    if P.initAnimNode then
        return
    end
    if P.animBackground ~= nil then
        lk.LakooManager:getInstance():addLKAnimationNode(P.animBackground, "leadingRole")
        P.initAnimNode = true
    end
end

function P.initHp_Mp_Exp(startPos)
    if P.statusUi == nil then
        return
    end
    local drawX = startPos.x
    local drawY = startPos.y
    local INTERVAR_X = 5
    local INTERVAR_Y = 3
    local BAR_BG_WIDTH = 150
    local BAR_BG_HEIGHT = 14
    local textImg = UILib.addImageTo(P.statusUi,"p_24",cc.p(drawX,drawY))
    local barBgNode = cc.LayerColor:create(UILib.UI_COLOR4B.BLACK)
    barBgNode:setPosition(drawX + textImg:getContentSize().width/2 + INTERVAR_X,drawY-BAR_BG_HEIGHT/2)
    barBgNode:setContentSize(cc.size(BAR_BG_WIDTH,BAR_BG_HEIGHT))
    P.statusUi:addChild(barBgNode)

    local hp = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.HP)
    local hpMax = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.MAX_HP)
    local hpColor = UILib.UI_COLOR4B.RED
    local hpBgColor = UILib.UI_COLOR4B.CRIMSON
    if hp >= hpMax then
        hpColor = UILib.UI_COLOR4B.YEELOW
        hpBgColor = UILib.UI_COLOR4B.YEELOW2
    end

    P.hpBarBg2Node = cc.LayerColor:create(hpBgColor)
    P.hpBarBg2Node:setPosition(1.5, 1.5)
    P.hpBarBg2Node:setContentSize(cc.size(barBgNode:getContentSize().width-3,barBgNode:getContentSize().height-3))
    barBgNode:addChild(P.hpBarBg2Node)

    P.hpBar = cc.LayerGradient:create(UILib.UI_COLOR4B.WHITE,hpColor,cc.p(0,1))
    P.hpBar:setContentSize(P.hpBarBg2Node:getContentSize())
    P.hpBarBg2Node:addChild(P.hpBar)

    P.hpLabel = cc.LabelAtlas:_create(hp..">"..hpMax, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
    P.hpBarBg2Node:addChild(P.hpLabel)

    ---#######-----
    drawY = drawY - textImg:getContentSize().height - INTERVAR_Y
    textImg = UILib.addImageTo(P.statusUi,"p_25",cc.p(drawX,drawY))
    barBgNode = cc.LayerColor:create(UILib.UI_COLOR4B.BLACK)
    barBgNode:setPosition(drawX + textImg:getContentSize().width/2 + INTERVAR_X,drawY-BAR_BG_HEIGHT/2)
    barBgNode:setContentSize(cc.size(BAR_BG_WIDTH,BAR_BG_HEIGHT))
    P.statusUi:addChild(barBgNode)

    local barBg2Node = cc.LayerColor:create(UILib.UI_COLOR4B.STEELBULE)
    barBg2Node:setPosition(1.5, 1.5)
    barBg2Node:setContentSize(cc.size(barBgNode:getContentSize().width-3,barBgNode:getContentSize().height-3))
    barBgNode:addChild(barBg2Node)

    P.mpBar = cc.LayerGradient:create(UILib.UI_COLOR4B.WHITE,UILib.UI_COLOR4B.DODGERBLUE,cc.p(0,1))
    P.mpBar:setContentSize(barBg2Node:getContentSize())
    barBg2Node:addChild(P.mpBar)

    local mp = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.MP)
    local mpMax = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.MAX_MP)
    P.mpLabel = cc.LabelAtlas:_create(mp..">"..mpMax, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
    barBg2Node:addChild(P.mpLabel)

    ---#######-----
    drawY = drawY - textImg:getContentSize().height - INTERVAR_Y
    textImg = UILib.addImageTo(P.statusUi,"p_23",cc.p(drawX,drawY))
    barBgNode = cc.LayerColor:create(UILib.UI_COLOR4B.BLACK)
    barBgNode:setPosition(drawX + textImg:getContentSize().width/2 + INTERVAR_X,drawY-BAR_BG_HEIGHT/2)
    barBgNode:setContentSize(cc.size(BAR_BG_WIDTH,BAR_BG_HEIGHT))
    P.statusUi:addChild(barBgNode)

    barBg2Node = cc.LayerColor:create(UILib.UI_COLOR4B.MEDIUMBULE)
    barBg2Node:setPosition(1.5, 1.5)
    barBg2Node:setContentSize(cc.size(barBgNode:getContentSize().width-3,barBgNode:getContentSize().height-3))
    barBgNode:addChild(barBg2Node)

    P.expBar = cc.LayerGradient:create(UILib.UI_COLOR4B.WHITE,UILib.UI_COLOR4B.BLUEVIOLET,cc.p(0,1))
    P.expBar:setContentSize(barBg2Node:getContentSize())
    barBg2Node:addChild(P.expBar)

    local exp = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.EXP)
    local nextExp = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.NEXT_EXP)
    P.expLabel = cc.LabelAtlas:_create(exp..">"..nextExp, Enum.DIGITAL.BEIGE, 198/21, 11, string.byte('0'))
    barBg2Node:addChild(P.expLabel)
end

function P.updateHp_Mp_Exp()
    if P.hpBar ~= nil then
        local hp = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.HP)
        local hpMax = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.MAX_HP)
        local hpColor = UILib.UI_COLOR4B.RED
        local hpBgColor = UILib.UI_COLOR4B.CRIMSON
        if hp >= hpMax then
            hpColor = UILib.UI_COLOR4B.YEELOW
            hpBgColor = UILib.UI_COLOR4B.YEELOW2
        end
        P.hpBar:setEndColor(hpColor)
        if P.hpBarBg2Node ~= nil then
            P.hpBarBg2Node:setColor(hpBgColor)
        end
        
        local hpPer = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.HP_PERCENT)/100
        P.hpBar:setContentSize(P.hpBar:getParent():getContentSize().width*hpPer, P.hpBar:getParent():getContentSize().height)
    end
    if P.hpLabel ~= nil then
        local hp = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.HP)
        local hpMax = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.MAX_HP)
        P.hpLabel:setString(hp..">"..hpMax)
    end
    if P.mpBar ~= nil then
        local mpPer = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.MP_PERCENT)/100
        P.mpBar:setContentSize(P.mpBar:getParent():getContentSize().width*mpPer, P.mpBar:getParent():getContentSize().height)
    end
    if P.mpLabel ~= nil then
        local mp = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.MP)
        local mpMax = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.MAX_MP)
        P.mpLabel:setString(mp..">"..mpMax)
    end
    if P.expBar ~= nil then
        local exp = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.EXP)
        local nextExp = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.NEXT_EXP)
        local expPer = 0
        if nextExp ~= 0 then
            expPer = exp / nextExp
        end
        P.expBar:setContentSize(P.expBar:getParent():getContentSize().width*expPer, P.expBar:getParent():getContentSize().height)
    end
    if P.expLabel ~= nil then
        local exp = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.EXP)
        local nextExp = lk.LakooManager:getInstance():getPlayerProperty(Enum.PROPERTY.NEXT_EXP)
        P.expLabel:setString(exp..">"..nextExp)
    end
end

function P.clean()
    P.initAnimNode = false

    P.statusUi = nil
    P.animBackground = nil
    P.hpBarBg2Node = nil
    P.hpBar = nil
    P.mpBar = nil
    P.expBar = nil
    P.hpLabel = nil
    P.mpLabel = nil
    P.expLabel = nil
    P.pointsLabel = nil
    P.strLable = nil
    P.intLabel = nil
    P.agiLabel = nil

    P.tempStr = 0
    P.tempInt = 0
    P.tempAgi = 0

    P.decBut_1 = nil
    P.incBut_1 = nil
    P.decBut_2 = nil
    P.incBut_2 = nil
    P.decBut_3 = nil
    P.incBut_3 = nil

    P.panelWeapon = nil
    P.panelArmor = nil
    P.panelAdorn = nil
    P.panelBottle = nil
    P.panelCatch = nil

    P.equipSelected = -1
    P.pointSelected = -1
    P.equipmentRow = false
    P.pointRow = false

    if P.focusAni ~= nil then
        if P.focusAni:getAnimation() ~= nil then
            P.focusAni:getAnimation():stop()
        end

        P.focusAni:removeFromParent(true)
        P.focusAni = nil
    end

    for var=1, #P.detailComponets do
        P.detailComponets[var] = nil
    end
end

function P.dpadUp()
    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        if layer:getMainPopLayerTapType() then
            return
        end
    end

    if P.equipmentRow then
        local scene = cc.Director:getInstance():getRunningScene()
        local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
        if layer ~= nil then
            if layer:getMainPopLayerTapType()== false then
                layer:setMainPopLayerTap(true)
                P.equipmentRow = false
                P.equipSelected = -1
                if P.focusAni ~=nil then
                    P.focusAni:setVisible(false)
                end
            end
        end
    end

    if P.pointRow then
        local nextPoint = P.pointSelected-2
        if nextPoint < 1 then
            P.equipmentRow = true
            P.equipSelected = 1
            P.pointSelected = -1
            P.pointRow = false
            local butWeapon = UILib.getNodeByTag(P.statusUi,TAG.ITEM.B_1)
            if butWeapon ~= nil then
                if P.focusAni ~=nil then
                    local pos = cc.p(butWeapon:getPositionX(), butWeapon:getPositionY())
                    P.focusAni:setPosition(pos)
                    P.focusAni:setVisible(true)
                end
            end
            return
        end

        P.pointSelected = nextPoint

        local btn = P.PointTable[P.pointSelected]
        if P.focusAni ~=nil and btn ~= nil then
            local pos = cc.p(btn:getPositionX(), btn:getPositionY())
            P.focusAni:setPosition(pos)
            P.focusAni:setVisible(true)
        end
        return
    end
end

function P.dpadDown()

    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        if layer:getTitleType() ~= 1 then
            return
        end
    end


    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        if layer:getMainPopLayerTapType() then
            layer:setMainPopLayerTap(false)
            if P.equipSelected == -1 then
                P.equipSelected = 1
                local butWeapon = UILib.getNodeByTag(P.statusUi,TAG.ITEM.B_1)
                if butWeapon ~= nil then
                    if P.focusAni ~=nil then
                        local pos = cc.p(butWeapon:getPositionX(), butWeapon:getPositionY())
                        P.focusAni:setPosition(pos)
                        P.focusAni:setVisible(true)
                        P.equipmentRow = true

                    end
                end
            end
            return
        end
    end

    if P.equipmentRow then
        P.equipmentRow = false
        P.equipSelected = -1
        P.pointSelected = 2
        P.pointRow = true
        if P.incBut_1 ~=nil then
            if P.focusAni ~=nil then
                local pos = cc.p(P.incBut_1:getPositionX(), P.incBut_1:getPositionY())
                P.focusAni:setPosition(pos)
                P.focusAni:setVisible(true)
            end
        end
        return
    end

    if P.pointRow then
        local nextPoint = P.pointSelected+2
        if nextPoint >6 then
            return
        end
        P.pointSelected = nextPoint

        local btn = P.PointTable[P.pointSelected]
        if P.focusAni ~=nil and btn ~= nil then
            local pos = cc.p(btn:getPositionX(), btn:getPositionY())
            P.focusAni:setPosition(pos)
            P.focusAni:setVisible(true)
        end
        return
    end

end

function P.dpadLeft()
    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        if layer:getMainPopLayerTapType() then
            return
        end
    end

    if P.equipmentRow then
        local nextEquipment = P.equipSelected -1
        if nextEquipment < 1 then
            return
        end

        P.equipSelected = nextEquipment
        local btn = P.EquipMentTable[P.equipSelected]
        if btn ~=nil then
            if P.focusAni ~=nil then
                local pos = cc.p(btn:getPositionX(), btn:getPositionY())
                P.focusAni:setPosition(pos)
                P.focusAni:setVisible(true)
            end
        end
    end

    if P.pointRow then
        local nextPoint = P.pointSelected-1
        if nextPoint < 1 then
            return
        end
        P.pointSelected = nextPoint

        local btn = P.PointTable[P.pointSelected]
        if P.focusAni ~=nil and btn ~= nil then
            local pos = cc.p(btn:getPositionX(), btn:getPositionY())
            P.focusAni:setPosition(pos)
            P.focusAni:setVisible(true)
        end
        return
    end
end

function P.dpadRight()
    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        if layer:getMainPopLayerTapType() then
            return
        end
    end

    if P.equipmentRow then
        local nextEquipment = P.equipSelected +1
        if nextEquipment > 5 then
            return
        end

        P.equipSelected = nextEquipment
        local btn = P.EquipMentTable[P.equipSelected]
        if btn ~=nil then
            if P.focusAni ~=nil then
                local pos = cc.p(btn:getPositionX(), btn:getPositionY())
                P.focusAni:setPosition(pos)
                P.focusAni:setVisible(true)
            end
        end
    end

    if P.pointRow then
        local nextPoint = P.pointSelected+1
        if nextPoint >6 then
            return
        end
        P.pointSelected = nextPoint

        local btn = P.PointTable[P.pointSelected]
        if P.focusAni ~=nil and btn ~= nil then
            local pos = cc.p(btn:getPositionX(), btn:getPositionY())
            P.focusAni:setPosition(pos)
            P.focusAni:setVisible(true)
        end
        return
    end
end

function P.dpadTap()
    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        if layer:getTitleType() ~= 1 then
            return
        end
    end
    
    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        if layer:getMainPopLayerTapType() then
            return
        end
    end
    
    if P.equipmentRow then
        local tag = P.equipSelected
        if tag == 1 then
            P.doWeaponAction()
        elseif tag == 2 then
            P.doArmorAction()
        elseif tag == 3 then
            P.doAdornAction()
        elseif tag == 4 then
            P.doBottleAction()
        elseif tag == 5 then
            P.doCatchAction()
        end
        return
    end

    if P.pointRow then
        local tag = P.pointSelected

        if tag == 1 then
            local btn = P.PointTable[1]
            if btn ~=nil then
                if btn:isTouchEnabled() then
                    P.tempStr = P.tempStr - 1
                    lk.LakooManager:getInstance():changePlayerProperty(Enum.PROPERTY.POINTS, 1)
                    lk.LakooManager:getInstance():changePlayerProperty(Enum.PROPERTY.STRENGTH, -1)
                end
            end
        elseif tag == 2 then
            local btn = P.PointTable[2]
            if btn ~=nil then
                if btn:isTouchEnabled() then
                    P.tempStr = P.tempStr + 1
                    lk.LakooManager:getInstance():changePlayerProperty(Enum.PROPERTY.POINTS, -1)
                    lk.LakooManager:getInstance():changePlayerProperty(Enum.PROPERTY.STRENGTH, 1)
                end
            end
        elseif tag == 3 then
            local btn = P.PointTable[3]
            if btn ~=nil then
                if btn:isTouchEnabled() then
                    P.tempInt = P.tempInt - 1
                    lk.LakooManager:getInstance():changePlayerProperty(Enum.PROPERTY.POINTS, 1)
                    lk.LakooManager:getInstance():changePlayerProperty(Enum.PROPERTY.INTELLIGENCE, -1)
                end
            end
        elseif tag == 4 then
            local btn = P.PointTable[4]
            if btn ~=nil then
                if btn:isTouchEnabled() then
                    P.tempInt = P.tempInt + 1
                    lk.LakooManager:getInstance():changePlayerProperty(Enum.PROPERTY.POINTS, -1)
                    lk.LakooManager:getInstance():changePlayerProperty(Enum.PROPERTY.INTELLIGENCE, 1)
                end
            end
        elseif tag == 5 then
            local btn = P.PointTable[5]
            if btn ~=nil then
                if btn:isTouchEnabled() then
                    P.tempAgi = P.tempAgi - 1
                    lk.LakooManager:getInstance():changePlayerProperty(Enum.PROPERTY.POINTS, 1)
                    lk.LakooManager:getInstance():changePlayerProperty(Enum.PROPERTY.AGILITY, -1)
                end
            end
        elseif tag == 6 then
            local btn = P.PointTable[6]
            if btn ~=nil then
                if btn:isTouchEnabled() then
                    P.tempAgi = P.tempAgi + 1
                    lk.LakooManager:getInstance():changePlayerProperty(Enum.PROPERTY.POINTS, -1)
                    lk.LakooManager:getInstance():changePlayerProperty(Enum.PROPERTY.AGILITY, 1)
                end
            end
        end

        lk.LakooManager:getInstance():setUpdateTipsStatus(Enum.LVUP_TIPS_STATUS.LVUP_NONE)
        P.updateStatus()
        return
    end
end

function P:registerControllerListener()
    local function onConnectController(controller, event)
        if controller == nil or controller == self._firstHolder.controller then
            return
        end

        local deviceInfo = string.format("%s id:%d", controller:getDeviceName(), controller:getDeviceId())
        cclog("PluginStatus deviceInfo ="..deviceInfo)

        if self._firstHolder.controller == nil then
            self._firstHolder.controller = controller
        end
    end

    local function onDisconnectedController(controller, event)
        if self._firstHolder.controller == controller then
            self._firstHolder.controller = nil
        end
    end

    local function showButtonState(controller, keyCode, isPressed)
        onConnectController(controller, nil)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end
        
        local scene = cc.Director:getInstance():getRunningScene()
        local child = scene:getChildren()

        local topSence = child[#child]
        if topSence:getName() ~= "MainPopLayer" then
            return
        end
        
        if P.statusUi~= nil then
            local popLayer = P.statusUi:getChildren()
            for var = 1,#popLayer do 
                local topPopLayer = popLayer[var]
                if topPopLayer ~= nil then
                    if topPopLayer:getName() == "UpgradeView" then
                        return
                    else
                    end
                end
            end
        end

        if isPressed then
            if keyCode == cc.ControllerKey.BUTTON_A then
                if P.oneTapfinish then
                    P.oneTapfinish = false
                end
            elseif keyCode == cc.ControllerKey.BUTTON_B then
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key Down:%d",keyCode)
            end
        else

            if keyCode == cc.ControllerKey.BUTTON_A then
                if P.oneTapfinish == false then
                    P.dpadTap()
                    P.oneTapfinish = true
                end
            elseif keyCode == cc.ControllerKey.BUTTON_B then

            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then

            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
                P.dpadUp()
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
                P.dpadDown()
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
                P.dpadLeft()
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
                P.dpadRight()
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then

            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then

            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key up:%d",keyCode)
            end
        end
    end

    local function onKeyDown(controller, keyCode, event)
        showButtonState(controller, keyCode, true)
    end

    local function onKeyUp(controller, keyCode, event)
        showButtonState(controller, keyCode, false)
    end

    local function onAxisEvent(controller, keyCode, event)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end

        local ketStatus = controller:getKeyStatus(keyCode)
        if keyCode == cc.ControllerKey.JOYSTICK_LEFT_X then
            if(ketStatus.value == -1) then
            elseif(ketStatus.value == 1) then

            end
        elseif keyCode == cc.ControllerKey.JOYSTICK_LEFT_Y then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_X then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_Y then
        elseif keyCode == cc.ControllerKey.AXIS_LEFT_TRIGGER then
        elseif keyCode == cc.ControllerKey.AXIS_RIGHT_TRIGGER then
        else
        --do nothing
        end
    end

    local controllerListener = cc.EventListenerController:create()
    controllerListener:registerScriptHandler(onConnectController, cc.Handler.EVENT_CONTROLLER_CONNECTED)
    controllerListener:registerScriptHandler(onDisconnectedController, cc.Handler.EVENT_CONTROLLER_DISCONNECTED)

    controllerListener:registerScriptHandler(onKeyDown, cc.Handler.EVENT_CONTROLLER_KEYDOWN)
    controllerListener:registerScriptHandler(onKeyUp, cc.Handler.EVENT_CONTROLLER_KEYUP)
    controllerListener:registerScriptHandler(onAxisEvent, cc.Handler.EVENT_CONTROLLER_AXIS)

    local eventDispatcher = P.statusUi:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(controllerListener, P.statusUi)
    cc.Controller:startDiscoveryController()
end

function P.cleanholer()
    if P.statusUi ~= nil then
        local eventDispatcher = P.statusUi:getEventDispatcher()
        eventDispatcher:removeEventListenersForTarget(P.statusUi, false)
        P._firstHolder = nil
        P.equipSelected = -1
        P.pointSelected = -1
    end
end

function P:setBlock(type)
    P.tapBlock = type
end

function P:getBlock()
    return P.tapBlock
end

return P