local UILib = require("common.UILib")
local Enum = require("common.Enum")

local Android = require("conf.Android")
local Text = require("conf.Text")
local IDX = require("conf.TextIndex")

local GamePool = require("model.GamePool")

local GameDetailLayer = require("gameUI.layer.GameDetailLayer")
local WarningLayer = require("gameUI.layer.WarningLayer")
local SettingLayer = require("gameUI.layer.SettingLayer")
local AchievementLayer = require("gameUI.layer.AchievementLayer")
local ZendeskLayer = require("gameUI.layer.ZendeskLayer")
local ProfileLayer = require("gameUI.layer.ProfileLayer")
local NoticeLayer = require("gameUI.layer.NoticeLayer")

local SceneStage = require("manager.SceneStage")
local LayerStage = require("manager.LayerStage")

local table = table

local TAG = {
    SAVE = 1,
    READ = 2,
    SETTING = 3,
    ACHIEVEMENT = 4,
    GAME_DETAILED = 5,
    BACK_MAIN_MENU = 6,
}

local P = {}

local ControllerHolder = class("ControllerHolder")
function ControllerHolder:ctor()
    self.controller  = nil
    self._holderNode = nil

    self._leftJoystick = nil
    self._rightJoystick = nil

    self._dpadLeft = nil
    self._dpadRight = nil
    self._dpadUp = nil
    self._dpadDown = nil

    self._buttonX = nil
    self._buttonY = nil
    self._buttonA = nil
    self._buttonB = nil

    self._buttonL1 = nil
    self._buttonR1 = nil
    self._buttonL2 = nil
    self._buttonR2 = nil

    self.deviceLabel = nil
    self.externalKeyLabel = nil
end

P.systemUi = nil
P.isHasCloseMainPopNotice = false
P._firstHolder = nil
P.focusAni = nil
P.tapSelected = -1
P.Buts = {}
P.tapBlock = false

function P.init(_systemUi)

    if P.systemUi == nil then
        local ACTION = {
            [TAG.SAVE] = function ()  P.doPopSave()  end,
            [TAG.READ] = function ()  P.doPopRead() end,
            [TAG.SETTING] = function ()  P.doPopSetting() end,
            [TAG.ACHIEVEMENT] = function ()  P.doPopAchievement() end,
            [TAG.GAME_DETAILED] = function ()  P.doGameDetailed() end,
            --[TAG.BACK_MAIN_MENU] = function ()  P.doPopBackMainMenu() end,
            [TAG.BACK_MAIN_MENU] = function ()  P.doPopContactUs() end,
        }

        local function doButtonAction(sender, eventType)
            local tag = sender:getTag()
            if eventType == ccui.TouchEventType.began then
            end
            if eventType == ccui.TouchEventType.moved then
            end
            if eventType == ccui.TouchEventType.ended then
                local func = ACTION[tag]
                if func == nil then
                    return
                else
                    func()
                end
            end
        end

        cclog("PluginSystem init!!!")
        P.systemUi = _systemUi

        --存档，读取，设置，成就，常见问题，联系我们
        local butImgNames = {"p_62","p_63","p_64","p_65","p_230","p_231"}
        local orginX = 150
        local orginY = 200
        local intervalX = 180
        local intervalY = 65
        P.Buts ={}
        for var=1, #butImgNames do
            local name = butImgNames[var]
            if name ~= nil then
                local imgName = string.format("%s.%s",name,Text.get(IDX.PNG_TYPE))
                if imgName ~= nil then
                    local button = ccui.Button:create(imgName,imgName,imgName)
                    local pos = cc.p(orginX, orginY)
                    button:setPosition(pos)
                    button:setTag(var)--对应TAG.SAVE...
                    button:addTouchEventListener(doButtonAction)
                    P.systemUi:addChild(button)
                    table.insert(P.Buts,button)
                    if var%2 == 1 then
                        orginX = orginX + intervalX
                    else
                        orginX = orginX - intervalX
                    end
                    if var%2 == 0 then
                        orginY = orginY - intervalY
                    end
                end
            end
        end
    end
    cclog("System!!!")
    --joe
    local osType = Enum.OS_TYPE.IOS
    osType = lk.LakooManager:getInstance():getOSType()
    if osType == Enum.OS_TYPE.ANDROID then
        if P._firstHolder == nil then
            cclog("System!!!")
            P._firstHolder = ControllerHolder.new()
            P._firstHolder.controller = nil
            P:registerControllerListener()
        end
    end
    --
    if P.focusAni == nil then
        ccs.ArmatureDataManager:getInstance():addArmatureFileInfo("FocusAni.csb")
        P.focusAni = ccs.Armature:create("FocusAni")
        P.systemUi:addChild(P.focusAni)
        P.focusAni:setLocalZOrder(99)
        P.focusAni:getAnimation():playWithIndex(0)
        P.focusAni:setVisible(false)
    end
end

function P.doPopSave()
    local isCanSave = lk.LakooManager:getInstance():isCanSave()
    local isInArena = lk.LakooManager:getInstance():isPlayerInArena()
    if isCanSave and (isInArena == false) then
        local layer = WarningLayer:create()
        GamePool.getCurrentScene():addChild(layer)
        layer:setOnActionFunc(P.doSave)
        local msg = lk.LakooManager:getInstance():getString(Enum.TEXT.T_WHETHER_SAVE)
        local okText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_YES)
        local cancelText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_NO)
        layer:setString(msg, okText, cancelText)
    else
        local msg = lk.LakooManager:getInstance():getString(Enum.TEXT.T_CANNOT_SAVE)
        lk.LakooManager:getInstance():addPopTip(msg)
    end
end

function P.doSave()
    local result = lk.LakooManager:getInstance():doSave(Enum.FILE.GAME_CHUNK_NAME)
    local msg = nil
    if result > 0 then
        msg = lk.LakooManager:getInstance():getString(Enum.TEXT.T_SAVE_OK)
    else
        msg = lk.LakooManager:getInstance():getString(Enum.TEXT.T_SAVE_FAILED)
    end
    if msg ~= nil then
        lk.LakooManager:getInstance():addPopTip(msg)
    end
end

function P.doPopRead()
    local layer = WarningLayer:create()
    GamePool.getCurrentScene():addChild(layer)
    layer:setOnActionFunc(P.doRead)
    local msg = lk.LakooManager:getInstance():getString(Enum.TEXT.T_WHETHER_READ)
    local okText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_YES)
    local cancelText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_NO)
    layer:setString(msg, okText, cancelText)
end

function P.doRead()
    lk.LakooManager:getInstance():saveSlotReplaceGame(Enum.FILE.GAME_CHUNK_NAME)
    P.setIsHasCloseMainPopNotice(true)--关闭mainPopLayer
end

function P.doPopSetting()
    P.tapBlock = true
    if P.focusAni ~= nil then
        P.focusAni:setVisible(false)
    end
    local settingLayer = SettingLayer:create()
    local pos = P.systemUi:convertToWorldSpace(cc.p(0,0))
    pos = cc.p(-pos.x,-pos.y)
    settingLayer:setPosition(pos)
    P.systemUi:addChild(settingLayer)
end

function P.doPopAchievement()
    P.tapBlock = true
    if P.focusAni ~= nil then
        P.focusAni:setVisible(false)
    end

    -- android 使用黙認的成就UI
    if lk.LakooManager:getInstance():getOSType() == Enum.OS_TYPE.ANDROID then
        lk.LakooManager:getInstance():loadGameCenterAchievement()
        return
    end

    local achievementLayer = AchievementLayer:create()
    P.systemUi:addChild(achievementLayer)
    local pos = P.systemUi:convertToWorldSpace(cc.p(0,0))
    pos = cc.p(-pos.x,-pos.y)
    achievementLayer:setPosition(pos)
end

function P.doGameDetailed()
    P.tapBlock = true
    if P.focusAni ~= nil then
        P.focusAni:setVisible(false)
    end
    local detailLayer = GameDetailLayer:create()
    P.systemUi:addChild(detailLayer)
    local title = lk.LakooManager:getInstance():getString(Enum.TEXT.T_GAME_DETAILS)
    detailLayer:setTitle(title)
    local detail = lk.LakooManager:getInstance():getStringFromFile(Enum.DATA.GAME_DETAIL)
    detailLayer:setDetail(detail)
    local pos = P.systemUi:convertToWorldSpace(cc.p(0,0))
    pos = cc.p(-pos.x,-pos.y)
    detailLayer:setPosition(pos)
end

function P.doContactUs()
    local layer = ZendeskLayer:create()
    GamePool.getCurrentScene():addChild(layer)	
end

function P.doPopContactUs()
    local osType = lk.LakooManager:getInstance():getOSType()
    if osType == Enum.OS_TYPE.ANDROID then
        --lk.LakooManager:getInstance():openZendeskWebView()
        GamePool.setWebUrlType(Enum.WEB_URL_TYPE.CONTACT_US)
        LayerStage.setShowLayerType(Enum.LAYER_TYPE.NOTICE)
        return
    end 

    local isHasProfile = GamePool.isHasProfile()
    if isHasProfile then
        P.doContactUs()
    else
        local profileLayer = ProfileLayer:create()
        profileLayer:setOnCallBackFunc(P.doContactUs)
        GamePool.getCurrentScene():addChild(profileLayer)
    end
end

function P.doPopBackMainMenu()
    local layer = GamePool.getCurrentScene():getChildByName(NoticeLayer.layerName)
    if layer ~= nil then
        LayerStage.setHideLayerType(Enum.LAYER_TYPE.NOTICE)
    end

    local layer = GamePool.getCurrentScene():getChildByName(WarningLayer.layerName)
    if layer ~= nil and layer:getTag() == WarningLayer.layerTag.BACK_MAIN_MENU then
    	return
    end
    layer = WarningLayer:create()
    layer:setTag(WarningLayer.layerTag.BACK_MAIN_MENU)
    GamePool.getCurrentScene():addChild(layer)
    layer:setOnActionFunc(P.doBackMainMenu)
    --local msg = lk.LakooManager:getInstance():getString(Enum.TEXT.T_WHETHER_RETURN_TITLE)
    local msg = Text.get(IDX.BACK_TITLE)
    local okText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_YES)
    local cancelText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_NO)
    layer:setString(msg, okText, cancelText)
end

function P.doBackMainMenu()
    SceneStage.setNextType(Enum.SCENE_TYPE.TITLE,UILib.TRANSITION_TYPE.FADE)
end
--[[
function P.doPopExitGame()
    local layer = GamePool.getCurrentScene():getChildByName(WarningLayer.layerName)
    if layer ~= nil and layer:getTag() == WarningLayer.layerTag.EXIT_GAME then
        return
    end
    layer = WarningLayer:create()
    layer:setTag(WarningLayer.layerTag.EXIT_GAME)
    GamePool.getCurrentScene():addChild(layer)
    layer:setOnActionFunc(P.doExitGame)
    local msg = lk.LakooManager:getInstance():getString(Enum.TEXT.T_GAME_EXIT)
    local okText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_YES)
    local cancelText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_NO)
    layer:setString(msg, okText, cancelText)
end

function P.doExitGame()
    os.exit(1)
end
--]]

function P.clean()
    P.systemUi = nil
    P.setIsHasCloseMainPopNotice(false)
    if P.focusAni ~= nil then
        if P.focusAni:getAnimation() ~= nil then
            P.focusAni:getAnimation():stop()
        end

        P.focusAni:removeFromParent(true)
        P.focusAni = nil
    end
    P.tapSelected = -1
end

function P.setIsHasCloseMainPopNotice(flag)
    P.isHasCloseMainPopNotice = flag
end

function P.getIsHasCloseMainPopNotice()
    return P.isHasCloseMainPopNotice
end

function P.dpadUp()
    if P.tapSelected ==1 or P.tapSelected == 2 then
        local scene = cc.Director:getInstance():getRunningScene()
        local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
        if layer ~= nil then
            if layer:getMainPopLayerTapType() == false then
                layer:setMainPopLayerTap(true)
                if P.focusAni ~= nil then
                    if P.focusAni:getAnimation() ~= nil then
                        P.focusAni:getAnimation():stop()
                    end

                    P.focusAni:removeFromParent(true)
                    P.focusAni = nil
                end
                return
            end
        end
    end

    local next = P.tapSelected -2
    if next <1 then
        return
    end

    P.tapSelected = next

    local button = P.Buts[next]
    if P.focusAni ~=nil and button ~=nil then
        local pos = cc.p(button:getPositionX(), button:getPositionY())
        P.focusAni:setPosition(pos)
        P.focusAni:setVisible(true)
    end
end

function P.dpadDown()
    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        if layer:getTitleType() ~= 6 then
            cclog("PluginSystem dpadDown first return")
            return
        end
    end

    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        if layer:getMainPopLayerTapType() then
            layer:setMainPopLayerTap(false)
            local button = P.Buts[1]
            if P.focusAni == nil then
                ccs.ArmatureDataManager:getInstance():addArmatureFileInfo("FocusAni.csb")
                P.focusAni = ccs.Armature:create("FocusAni")
                P.systemUi:addChild(P.focusAni)
                P.focusAni:setLocalZOrder(99)
                P.focusAni:getAnimation():playWithIndex(0)
                P.focusAni:setVisible(false)
            end
            if P.focusAni ~=nil and button ~=nil then
                local pos = cc.p(button:getPositionX(), button:getPositionY())
                P.focusAni:setPosition(pos)
                P.focusAni:setVisible(true)
                P.tapSelected = 1
            end
            cclog("PluginSystem dpadDown second return")
            return
        end
    end

    local next = P.tapSelected +2
    if next >6 then
        cclog("PluginSystem dpadDown thrid return")
        return
    end

    P.tapSelected = next
    local button = P.Buts[next]

    if P.focusAni == nil then
        ccs.ArmatureDataManager:getInstance():addArmatureFileInfo("FocusAni.csb")
        P.focusAni = ccs.Armature:create("FocusAni")
        P.systemUi:addChild(P.focusAni)
        P.focusAni:setLocalZOrder(99)
        P.focusAni:getAnimation():playWithIndex(0)
        P.focusAni:setVisible(false)
    end
    if P.focusAni ~=nil and button ~=nil then
        local pos = cc.p(button:getPositionX(), button:getPositionY())
        P.focusAni:setPosition(pos)
        P.focusAni:setVisible(true)
    end
end

function P.dpadLeft()
    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        if layer:getMainPopLayerTapType() then
            return
        end
    end
    local next = P.tapSelected -1
    if next <1 then
        return
    end

    P.tapSelected = next

    local button = P.Buts[next]
    if P.focusAni ~=nil and button ~=nil then
        local pos = cc.p(button:getPositionX(), button:getPositionY())
        P.focusAni:setPosition(pos)
        P.focusAni:setVisible(true)
    end
end

function P.dpadRight()
    local scene = cc.Director:getInstance():getRunningScene()
    local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
    if layer ~= nil then
        if layer:getMainPopLayerTapType() then
            return
        end
    end
    local next = P.tapSelected +1
    if next >6 then
        return
    end

    P.tapSelected = next

    local button = P.Buts[next]
    if P.focusAni ~=nil and button ~=nil then
        local pos = cc.p(button:getPositionX(), button:getPositionY())
        P.focusAni:setPosition(pos)
        P.focusAni:setVisible(true)
    end
end

function P:registerControllerListener()
    local function onConnectController(controller, event)
        if controller == nil or controller == self._firstHolder.controller then
            return
        end

        local deviceInfo = string.format("%s id:%d", controller:getDeviceName(), controller:getDeviceId())
        cclog("PluginSystem deviceInfo ="..deviceInfo)

        if self._firstHolder.controller == nil then
            self._firstHolder.controller = controller
        end
    end


    local function onDisconnectedController(controller, event)
        if self._firstHolder.controller == controller then
            self._firstHolder.controller = nil
        end
    end

    local function showButtonState(controller, keyCode, isPressed)
        onConnectController(controller, nil)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end

        if lk.LakooManager:getInstance():getIsHasPopView() == true then
            return
        end

        local scene = cc.Director:getInstance():getRunningScene()
        local child = scene:getChildren()

        local topSence = child[#child]
        if topSence:getName() ~= "MainPopLayer" then
            return
        end

        if P.systemUi~= nil then
            local popLayer = P.systemUi:getChildren()
            for var = 1,#popLayer do
                local topPopLayer = popLayer[var]
                if topPopLayer ~= nil then
                    cclog("name = "..topPopLayer:getName())
                    if topPopLayer:getName() == "SettingLayer" or topPopLayer:getName() =="AchievementLayer" or topPopLayer:getName()=="GameDetailLayer" then
                        if P.focusAni ~= nil then
                            P.focusAni:setVisible(false)
                        end
                        return
                    else
                        if P.focusAni ~= nil and P.tapSelected ~= -1 then
                            P.focusAni:setVisible(true)
                        end
                    end
                end
            end
        end

        local ACTION = {
            [TAG.SAVE] = function ()  P.doPopSave()  end,
            [TAG.READ] = function ()  P.doPopRead() end,
            [TAG.SETTING] = function ()  P.doPopSetting() end,
            [TAG.ACHIEVEMENT] = function ()  P.doPopAchievement() end,
            [TAG.GAME_DETAILED] = function ()  P.doGameDetailed() end,
            [TAG.BACK_MAIN_MENU] = function ()  P.doPopBackMainMenu() end,
        }

        if isPressed then
            if keyCode == cc.ControllerKey.BUTTON_A then
                local scene = cc.Director:getInstance():getRunningScene()
                local layer = scene:getChildByTag(Enum.LAYER_TYPE.MAIN_POP)
                if layer ~= nil then
                    if layer:getMainPopLayerTapType() then
                        cclog("PluginSystem BUTTON_A first return")
                        return
                    end
                    if layer:getTitleType() ~= 6 then
                        cclog("PluginSystem BUTTON_A second return")
                        return
                    end
                end

                local func = ACTION[P.tapSelected]
                if func == nil then
                    return
                else
                    cclog("PluginSystem BUTTON_A P.tapSelected ="..P.tapSelected)
                    func()
                end
            elseif keyCode == cc.ControllerKey.BUTTON_B then
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key Down:%d",keyCode)
            end
        else
            cclog("PluginSystem keyup ="..keyCode)

            if keyCode == cc.ControllerKey.BUTTON_A then

            elseif keyCode == cc.ControllerKey.BUTTON_B then

            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then

            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
                P.dpadUp()
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
                P.dpadDown()
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
                P.dpadLeft()
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
                P.dpadRight()
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then

            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then

            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key up:%d",keyCode)
            end
        end
    end

    local function onKeyDown(controller, keyCode, event)
        showButtonState(controller, keyCode, true)
    end

    local function onKeyUp(controller, keyCode, event)
        showButtonState(controller, keyCode, false)
    end

    local function onAxisEvent(controller, keyCode, event)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end

        local ketStatus = controller:getKeyStatus(keyCode)
        if keyCode == cc.ControllerKey.JOYSTICK_LEFT_X then
            if(ketStatus.value == -1) then
            elseif(ketStatus.value == 1) then

            end
        elseif keyCode == cc.ControllerKey.JOYSTICK_LEFT_Y then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_X then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_Y then
        elseif keyCode == cc.ControllerKey.AXIS_LEFT_TRIGGER then
        elseif keyCode == cc.ControllerKey.AXIS_RIGHT_TRIGGER then
        else
        --do nothing
        end
    end

    local controllerListener = cc.EventListenerController:create()
    controllerListener:registerScriptHandler(onConnectController, cc.Handler.EVENT_CONTROLLER_CONNECTED)
    controllerListener:registerScriptHandler(onDisconnectedController, cc.Handler.EVENT_CONTROLLER_DISCONNECTED)

    controllerListener:registerScriptHandler(onKeyDown, cc.Handler.EVENT_CONTROLLER_KEYDOWN)
    controllerListener:registerScriptHandler(onKeyUp, cc.Handler.EVENT_CONTROLLER_KEYUP)
    controllerListener:registerScriptHandler(onAxisEvent, cc.Handler.EVENT_CONTROLLER_AXIS)

    local eventDispatcher = P.systemUi:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(controllerListener, P.systemUi)
    cc.Controller:startDiscoveryController()
end

function P.cleanholer()
    if P.systemUi ~=nil then
        local eventDispatcher = P.systemUi:getEventDispatcher()
        eventDispatcher:removeEventListenersForTarget(P.systemUi, false)
    end
    P._firstHolder = nil
    P.tapSelected = -1
end

function P:setBlock(type)
    P.tapBlock = type
end

function P:getBlock()
    return P.tapBlock
end

function P:initListener()
    local osType = Enum.OS_TYPE.IOS
    osType = lk.LakooManager:getInstance():getOSType()
    if osType == Enum.OS_TYPE.ANDROID then
        if P._firstHolder == nil then
            P._firstHolder = ControllerHolder.new()
            P._firstHolder.controller = nil
            P:registerControllerListener()
        end
    end
end

return P