require "cocos.init"
require "cocos.extension.ExtensionConstants"
local UILib = require("common.UILib")
local Enum = require("common.Enum")
local Text = require("conf.Text")
local TextIndex = require("conf.TextIndex")

local GamePool = require("model.GamePool")

local MessageLayer = require("gameUI.layer.MessageLayer")

local cc = cc
local class = class

local ProfileLayer = class("ProfileLayer",
    function()
        return cc.Node:create()
    end
)

ProfileLayer.__index = ProfileLayer

ProfileLayer.nameEditBox = nil
ProfileLayer.emailEditBox = nil
ProfileLayer.callBackDelegate = nil
ProfileLayer.onCallBackFunc = nil--用于单例如：PluginSystem.lua

function ProfileLayer:create(callBack)
    local profileLayer = ProfileLayer.new()
    profileLayer:init(callBack)
    return profileLayer
end

function ProfileLayer:init(callBack)
    local visibleSize = cc.Director:getInstance():getVisibleSize()
    self:setContentSize(visibleSize)
    self:setAnchorPoint(UILib.UI_ANCHOR_POINT.CENTER)
    local pos = UILib.offset(0,0,self:getContentSize().width,self:getContentSize().height,UILib.UI_ANCHOR.CENTER)
    self:setPosition(pos)

    local bgLayer = cc.LayerColor:create(UILib.UI_COLOR4B.GRAY)
    bgLayer:setContentSize(visibleSize)
    self:addChild(bgLayer)

    ---#####禁止穿透
    local eventListener = cc.EventListenerTouchOneByOne:create()
    local function onTouchBegan(touch, event)
        return true
    end
    local function onTouchEnded(touch, event)
    end

    eventListener:registerScriptHandler(onTouchBegan,cc.Handler.EVENT_TOUCH_BEGAN)
    eventListener:registerScriptHandler(onTouchEnded,cc.Handler.EVENT_TOUCH_ENDED)
    eventListener:setSwallowTouches(true)
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(eventListener, self)
    ---#####

    --back button
    local function doBackAction(sender, eventType)
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            self:removeFromParent(true)
        end
    end
    local backButton = ccui.Button:create(Enum.IMG_NAME_PNGHK.BACK,Enum.IMG_NAME_PNGHK.BACK,Enum.IMG_NAME_PNGHK.BACK)
    local pos = UILib.offset(0,0,backButton:getContentSize().width,backButton:getContentSize().height,UILib.UI_ANCHOR.TOP_RIGHT)
    backButton:setPosition(pos)
    self:addChild(backButton)
    backButton:addTouchEventListener(doBackAction)

    local function editNameBoxTextEventHandle(event)
        if event == "return" then--other:began,ended,changed
            local flag = self:checkName()
            if flag == false then
                local messageLayer = MessageLayer:create(Text.get(TextIndex.INVALID_NAME))
                GamePool.getCurrentScene():addChild(messageLayer)
            end
        end
    end

    local editBoxSize = cc.size(300,50)
	-- cc.EditBox & cc.Scale9Sprite was deprecated
    self.nameEditBox = ccui.EditBox:create(editBoxSize, ccui.Scale9Sprite:create("editBox_bg.png"))
    local pos = UILib.offset(0,-50,self.nameEditBox:getContentSize().width,self.nameEditBox:getContentSize().height,UILib.UI_ANCHOR.CENTER_TOP)
    self.nameEditBox:setPosition(pos)
    self.nameEditBox:setFontSize(25)
    self.nameEditBox:setFontColor(UILib.UI_COLOR3B.WHITE)
    local redeeCodeStr = Text.get(TextIndex.NAME)
    self.nameEditBox:setPlaceHolder(redeeCodeStr)
    self.nameEditBox:setPlaceholderFontColor(UILib.UI_COLOR3B.WHITE)
    self.nameEditBox:setMaxLength(100)
    self.nameEditBox:setReturnType(cc.KEYBOARD_RETURNTYPE_DONE)
    --Handler
    self.nameEditBox:registerScriptEditBoxHandler(editNameBoxTextEventHandle)
    self:addChild(self.nameEditBox)
    
    local function editEmailBoxTextEventHandle(event)
        if event == "return" then--other:began,ended,changed
            local flag = self:checkEmail()
            if flag == false then
                local messageLayer = MessageLayer:create(Text.get(TextIndex.INVALID_EMAIL))
                GamePool.getCurrentScene():addChild(messageLayer)
            end
        end
    end
    
    self.emailEditBox = ccui.EditBox:create(editBoxSize, ccui.Scale9Sprite:create("editBox_bg.png"))
    local pos = cc.p(self.nameEditBox:getPositionX(), self.nameEditBox:getPositionY() - self.nameEditBox:getContentSize().height - 20)
    self.emailEditBox:setPosition(pos)
    self.emailEditBox:setFontSize(25)
    self.emailEditBox:setFontColor(UILib.UI_COLOR3B.WHITE)
    local redeeCodeStr = Text.get(TextIndex.E_MAIL)
    self.emailEditBox:setPlaceHolder(redeeCodeStr)
    self.emailEditBox:setPlaceholderFontColor(UILib.UI_COLOR3B.WHITE)
    self.emailEditBox:setMaxLength(100)
    self.emailEditBox:setReturnType(cc.KEYBOARD_RETURNTYPE_DONE)
    --Handler
    self.emailEditBox:registerScriptEditBoxHandler(editEmailBoxTextEventHandle)
    self:addChild(self.emailEditBox)
    
    --continue button
    local function doContinueAction(sender, eventType)
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            self:doContinue()
        end
    end

    local continueBut = ccui.Button:create(Enum.IMG_NAME_PNGHK.OPTION_BG,Enum.IMG_NAME_PNGHK.OPTION_BG,Enum.IMG_NAME_PNGHK.OPTION_BG)
    continueBut:setTitleText(Text.get(TextIndex.CONTINUE))
    continueBut:setTitleColor(UILib.UI_COLOR3B.BLACK)
    continueBut:setTitleFontSize(UILib.FONT.SMALL)
    local drawX = self.emailEditBox:getPositionX()
    local drawY = self.emailEditBox:getPositionY() - 20 - continueBut:getContentSize().height
    local pos = cc.p(drawX,drawY)
    continueBut:setPosition(pos)
    self:addChild(continueBut)
    continueBut:addTouchEventListener(doContinueAction)
end

function ProfileLayer:checkName()
    return self.nameEditBox:getText() ~= "" and self.nameEditBox:getText() ~= nil
end

function ProfileLayer:checkEmail()
    local email = self.emailEditBox:getText()
    if (email:match("[A-Za-z0-9%.%%%+%-]+@[A-Za-z0-9%.%%%+%-]+%.%w%w%w?%w?")) then
        return true
    end
    
    return false              
end

function ProfileLayer:doContinue()
    local flag = self:checkName()
    if flag == false then
        self.nameEditBox:setText("")
        local messageLayer = MessageLayer:create(Text.get(TextIndex.INVALID_NAME))
        GamePool.getCurrentScene():addChild(messageLayer)
        return
    end
    
    flag = self:checkEmail()
    if flag == false then
        self.emailEditBox:setText("")
        local messageLayer = MessageLayer:create(Text.get(TextIndex.INVALID_EMAIL))
        GamePool.getCurrentScene():addChild(messageLayer)
        return
    end
    
    local zName = self.nameEditBox:getText()
    local zEmail = self.emailEditBox:getText()
    lk.LakooManager:getInstance():initZndeskIdentity(zName, zEmail)
    GamePool.setProfileName(zName)
    GamePool.setProfileEmail(zEmail)
    if self.callBackDelegate ~= nil then
    	self.callBackDelegate:profileLayerCallBackContinue()
    elseif self.onCallBackFunc ~= nil then
        self.onCallBackFunc()
    end
    self:removeFromParent(true)
end

function ProfileLayer:setCallBackDelegate(_delegate)
    self.callBackDelegate = _delegate
end

function ProfileLayer:setOnCallBackFunc(_func)
    self.onCallBackFunc = _func;
end

return ProfileLayer
