require "cocos.init"

local UILib = require("common.UILib")
local Enum = require("common.Enum")
local Util = require("common.Util")

local Text = require("conf.Text")
local IDX = require("conf.TextIndex")

local NetworkManager = require("manager.NetworkManager")
local RedeemCodeLayer = require("gameUI.layer.RedeemCodeLayer")
local WarningLayer = require("gameUI.layer.WarningLayer")
local Android = require("conf.Android")
local GamePool = require("model.GamePool")
local cc = cc
local ccui = ccui
local class = class

local TAG = {
    OPEN_M = 11,
    CLOSE_M = 101,
    OPEN_M_E = 12,
    CLOSE_M_E = 102,
    OPEN_MAP = 13,
    CLOSE_MAP = 103,
    OPEN_SAVE_GC = 14,
    CLOSE_SAVE_GC = 104,
    REDEEM_CODE = 4,
}


local ControllerHolder = class("ControllerHolder")
function ControllerHolder:ctor()
    self.controller  = nil
    self._holderNode = nil

    self._leftJoystick = nil
    self._rightJoystick = nil

    self._dpadLeft = nil
    self._dpadRight = nil
    self._dpadUp = nil
    self._dpadDown = nil

    self._buttonX = nil
    self._buttonY = nil
    self._buttonA = nil
    self._buttonB = nil

    self._buttonL1 = nil
    self._buttonR1 = nil
    self._buttonL2 = nil
    self._buttonR2 = nil

    self.deviceLabel = nil
    self.externalKeyLabel = nil
end

local SettingLayer = class("SettingLayer",
    function()
        return cc.Node:create()
    end
)

SettingLayer.__index = SettingLayer

SettingLayer.focusAni = nil
SettingLayer.tapSelected = -1
SettingLayer._firstHolder = nil
SettingLayer.openbtnTable = {}
SettingLayer.closebtnTable = {}

function SettingLayer:create()
    local settingLayer = SettingLayer.new()
    settingLayer:init()

    return settingLayer
end

function SettingLayer:init()
    self:setName("SettingLayer")
    --##########[禁止穿透]
    local eventListener = cc.EventListenerTouchOneByOne:create()
    local function onTouchBegan(touch, event)         
        return true
    end
    local function onTouchEnded(touch, event)
    end

    eventListener:registerScriptHandler(onTouchBegan,cc.Handler.EVENT_TOUCH_BEGAN)
    eventListener:registerScriptHandler(onTouchEnded,cc.Handler.EVENT_TOUCH_ENDED)
    eventListener:setSwallowTouches(true)
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(eventListener, self)
    --##########
    
    local visibleSize = cc.Director:getInstance():getVisibleSize()
    self:setContentSize(visibleSize)

    local bgLayer = cc.LayerColor:create(UILib.UI_COLOR4B.GRAY)
    bgLayer:setContentSize(visibleSize)
    self:addChild(bgLayer)
    
    local bgImgView = ccui.ImageView:create(Enum.IMG_NAME_PNGHK.DETAIL_BG)
    local pos = UILib.offset(0,0,bgImgView:getContentSize().width,bgImgView:getContentSize().height,UILib.UI_ANCHOR.CENTER)
    bgImgView:setPosition(pos)
    self:addChild(bgImgView)
    
    --back button
    local function doBackAction(sender, eventType) 
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            self:removeFromParent(true)
        end
    end
    local backButton = ccui.Button:create(Enum.IMG_NAME_PNGHK.BACK,Enum.IMG_NAME_PNGHK.BACK,Enum.IMG_NAME_PNGHK.BACK)
    local dx = bgImgView:getPositionX()+(bgImgView:getContentSize().width/2-20)
    local dy = bgImgView:getPositionY()+(bgImgView:getContentSize().height/2-20)
    pos = cc.p(dx,dy)
    backButton:setPosition(pos)
    self:addChild(backButton,1)
    backButton:addTouchEventListener(doBackAction)
    
    local title = lk.LakooManager:getInstance():getString(Enum.TEXT.T_GAME_SETTING)
    local titleLabel = CCLabelTTF:create(title, UILib.FONT_NAME_NORMAL, UILib.FONT.SMALL)
    titleLabel:setColor(UILib.UI_COLOR3B.BLACK)
    pos = cc.p(bgImgView:getPositionX(),backButton:getPositionY())
    titleLabel:setPosition(pos)
    self:addChild(titleLabel)
    
    --music swith
    local mStr = lk.LakooManager:getInstance():getString(Enum.TEXT.T_MUSIC)
    local isMOn = lk.LakooManager:getInstance():isMusicOn()
    
    --music effect swith
    local mEStr = lk.LakooManager:getInstance():getString(Enum.TEXT.T_MUSIC_EFFECT)
    local isEOn = lk.LakooManager:getInstance():isMusicEffectOn()
    
    --mini map swith
    local mapStr = lk.LakooManager:getInstance():getString(Enum.TEXT.T_LITTLE_MAP)
    local isMapOn = lk.LakooManager:getInstance():isMiniMapOn()
    
    --Save to Game Center swith
    local gcStr = Text.get(IDX.SAVE_TO_GAME_CENTER)
    local isGcOn = lk.LakooManager:getInstance():isSaveGameCenterOn()
    
    --redeem code(兑换码)
    local rStr = lk.LakooManager:getInstance():getString(Enum.TEXT.T_BONUS)
    
    local function doButtonAction(sender, eventType)
        if eventType == ccui.TouchEventType.began then
        end
        --if eventType == ccui.TouchEventType.moved then
        --end
        if eventType == ccui.TouchEventType.ended then
            self:doSetting(sender)
        end
    end
    
    local strTable = nil
    local osType = lk.LakooManager:getInstance():getOSType()
    if osType == Enum.OS_TYPE.IOS then
        strTable = {mStr, mEStr, mapStr, gcStr}
        bgImgView:setScaleX(1.5)
        dx = bgImgView:getPositionX() + (backButton:getPositionX() - bgImgView:getPositionX())*1.5
        dy = backButton:getPositionY()
        pos = cc.p(dx, dy)
        backButton:setPosition(pos)
    else
        strTable = {mStr, mEStr, mapStr, rStr}
    end
    local drawX = 0
    local drawY = 0
    local interValY = 0
    local openImgName = string.format("%s.%s",Enum.TEXT_IMG.OPEN,Text.get(IDX.PNG_TYPE))
    local closeImgName = string.format("%s.%s",Enum.TEXT_IMG.CLOSE,Text.get(IDX.PNG_TYPE))
    local inputImgName = string.format("%s.%s",Enum.TEXT_IMG.INPUT,Text.get(IDX.PNG_TYPE))
    for key, var in ipairs(strTable) do
        local label = CCLabelTTF:create(var, UILib.FONT_NAME_NORMAL, UILib.FONT.SMALL)
        label:setColor(UILib.UI_COLOR3B.BLACK)
        if osType == Enum.OS_TYPE.IOS then
            if key == 1 then
                local drawH = label:getContentSize().height
                interValY = drawH*1.5
                drawX = bgImgView:getPositionX() - bgImgView:getContentSize().width*1.5*0.5 + label:getContentSize().width*0.5 + 20
                drawY = bgImgView:getPositionY() + bgImgView:getContentSize().height*0.5 - 60
            else
                drawX = bgImgView:getPositionX() - bgImgView:getContentSize().width*1.5*0.5 + label:getContentSize().width*0.5 + 20
                drawY = pos.y - interValY
            end
            pos = cc.p(drawX, drawY)
            label:setPosition(pos)
            self:addChild(label)
        else
            if key == 1 then
                local drawW = label:getContentSize().width
                local drawH = label:getContentSize().height
                interValY = drawH*1.5
                local beginX = 20

                pos = UILib.offsetWithSuperSize(20,drawH*2-5,drawW,drawH,bgImgView:getContentSize().width,bgImgView:getContentSize().height,UILib.UI_ANCHOR.CENTER_LEFT)
            else
                drawX = label:getContentSize().width/2+20
                drawY = pos.y - interValY
                pos = cc.p(drawX, drawY)
            end
            label:setPosition(pos)
            bgImgView:addChild(label)
        end
        
        --setting button
        local count = 4
        if osType == Enum.OS_TYPE.IOS then
            count = 5
        end
        if key < count then
            local openBut = ccui.Button:create(openImgName,openImgName,openImgName)
            local bPos = cc.p(bgImgView:getContentSize().width-openBut:getContentSize().width*1.5,pos.y)
            if osType == Enum.OS_TYPE.IOS then
                bPos = cc.p(bgImgView:getPositionX()+bgImgView:getContentSize().width*1.5*0.5-openBut:getContentSize().width*0.5-20,pos.y);
            end
            openBut:setPosition(bPos)
            openBut:setTag(10+key)
            if osType == Enum.OS_TYPE.IOS then
                self:addChild(openBut)
            else
                bgImgView:addChild(openBut)
            end
            openBut:addTouchEventListener(doButtonAction)
            
            local closeBut = ccui.Button:create(closeImgName,closeImgName,closeImgName)
            closeBut:setPosition(bPos)
            closeBut:setTag(100+key)
            if osType == Enum.OS_TYPE.IOS then
                self:addChild(closeBut)
            else
                bgImgView:addChild(closeBut)
            end
            closeBut:addTouchEventListener(doButtonAction)
            
            if (key == 1 and isMOn) 
            or (key == 2 and isEOn)
            or (key == 3 and isMapOn)
            or (key == 4 and isGcOn) then
                openBut:setVisible(true)
                closeBut:setVisible(false)
            else
                openBut:setVisible(false)
                closeBut:setVisible(true)
            end
            
            if Android.CHANNEL == 4 then
                table.insert(self.openbtnTable,key,openBut)
                table.insert(self.closebtnTable,key,closeBut)
            end
            
        else
            local inputBut = ccui.Button:create(inputImgName,inputImgName,inputImgName)
            local bPos = cc.p(bgImgView:getContentSize().width-inputBut:getContentSize().width*1.5,pos.y)
            inputBut:setPosition(bPos)
            inputBut:setTag(key)
            inputBut:addTouchEventListener(doButtonAction)
            bgImgView:addChild(inputBut)
        end
    end

    local function update()
        
    end
    
    local function onNodeEvent(event)
        if "enterTransitionFinish" == event then
            self.scheduleID = cc.Director:getInstance():getScheduler():scheduleScriptFunc(update, 0, false)
        end
        if "exit" == event then
            if self.scheduleID ~= nil then
                cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.scheduleID)
            end
            self:clean()
        end
    end
    self:registerScriptHandler(onNodeEvent)
    
    local osType = Enum.OS_TYPE.IOS
    osType = lk.LakooManager:getInstance():getOSType()
    if osType == Enum.OS_TYPE.ANDROID and Android.CHANNEL == 4 then
        --joe
        self._firstHolder   = ControllerHolder.new()
        self._firstHolder.controller = nil
        self:registerControllerListener(doButtonAction)
        --
        
        if self.focusAni == nil then
            ccs.ArmatureDataManager:getInstance():addArmatureFileInfo("FocusAni.csb")
            self.focusAni = ccs.Armature:create("FocusAni")
            bgImgView:addChild(self.focusAni)
            self.focusAni:setLocalZOrder(99)
            self.focusAni:getAnimation():playWithIndex(0)
            self.focusAni:setVisible(false)

            if self.openbtnTable ~= nil then
                local btn = self.openbtnTable[1]
                self.tapSelected = 1
                if btn ~= nil then
                    local pos = cc.p(btn:getPositionX(),btn:getPositionY())
                    self.focusAni:setPosition(pos)
                    self.focusAni:setVisible(true)
                end
            end
        end
    end
end

function SettingLayer:doSetting(sender)
    local tag = sender:getTag()
    if tag == TAG.OPEN_M then
    	sender:setVisible(false)
    	local closeBut = UILib.getNodeByTag(self,TAG.CLOSE_M)
    	closeBut:setVisible(true)
        lk.LakooManager:getInstance():setIsMusicOn(false)
    end
    if tag == TAG.CLOSE_M then
        sender:setVisible(false)
        local openBut = UILib.getNodeByTag(self,TAG.OPEN_M)
        openBut:setVisible(true)
        lk.LakooManager:getInstance():setIsMusicOn(true)
    end 
    if tag == TAG.OPEN_M_E then
        sender:setVisible(false)
        local closeBut = UILib.getNodeByTag(self,TAG.CLOSE_M_E)
        closeBut:setVisible(true)
        lk.LakooManager:getInstance():setIsMusicEffectOn(false)
    end 
    if tag == TAG.CLOSE_M_E then
        sender:setVisible(false)
        local openBut = UILib.getNodeByTag(self,TAG.OPEN_M_E)
        openBut:setVisible(true)
        lk.LakooManager:getInstance():setIsMusicEffectOn(true)
    end 
    if tag == TAG.OPEN_MAP then
        sender:setVisible(false)
        local closeBut = UILib.getNodeByTag(self,TAG.CLOSE_MAP)
        closeBut:setVisible(true)
        lk.LakooManager:getInstance():setIsMiniMapOn(false)
    end 
    if tag == TAG.CLOSE_MAP then
        sender:setVisible(false)
        local openBut = UILib.getNodeByTag(self,TAG.OPEN_MAP)
        openBut:setVisible(true)
        lk.LakooManager:getInstance():setIsMiniMapOn(true)
    end  
    if tag == TAG.OPEN_SAVE_GC then
        self.warningLayer = WarningLayer:create()
        self.warningLayer:setDelegate(self)
        local msg = Text.get(IDX.CLOSE_SAVE_TIPS)
        local okText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_YES)
        local cancelText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_NO)
        self.warningLayer:setString(msg, okText, cancelText)
        GamePool.getCurrentScene():addChild(self.warningLayer)
    end
    if tag == TAG.CLOSE_SAVE_GC then
        sender:setVisible(false)
        local openBut = UILib.getNodeByTag(self,TAG.OPEN_SAVE_GC)
        openBut:setVisible(true)
        lk.LakooManager:getInstance():registerAchievementController()
        lk.LakooManager:getInstance():setIsSaveGameCenterOn(true)
    end
    if tag == TAG.REDEEM_CODE then
    	self:popRedeemCodeInput()
    end
end

function SettingLayer:onActionFunc()
    local sender = UILib.getNodeByTag(self,TAG.OPEN_SAVE_GC)
    sender:setVisible(false)
    local closeBut = UILib.getNodeByTag(self,TAG.CLOSE_SAVE_GC)
    closeBut:setVisible(true)
    lk.LakooManager:getInstance():setIsSaveGameCenterOn(false)
end

---打开兑换码输入界面及输入结束回调后将兑换码发送给服务器验证
function SettingLayer:popRedeemCodeInput()
    local function callBack(redeemCode)
        NetworkManager.requestExchangeRedeemCode(redeemCode)
    end
    local redeemCodeLayer = RedeemCodeLayer:create(callBack)
    self:addChild(redeemCodeLayer,2)
end

function SettingLayer:clean()
end

function SettingLayer:dpadTap(doButtonAction)
    local pos = self.tapSelected
    
    if pos >0 and pos <5 then 
        if pos == 1 then
            local isMOn = lk.LakooManager:getInstance():isMusicOn()
            if isMOn then
                if self.openbtnTable ~= nil then
                    local btn = self.openbtnTable[pos]
                    if btn ~=nil then
                        doButtonAction(btn,ccui.TouchEventType.ended)
                    end
                end
            else
                if self.closebtnTable ~= nil then
                    local btn = self.closebtnTable[pos]
                    if btn ~=nil then
                        doButtonAction(btn,ccui.TouchEventType.ended)
                    end
                end
            end
        elseif pos == 2 then
            local isEOn = lk.LakooManager:getInstance():isMusicEffectOn()
            if isEOn then
                if self.openbtnTable ~= nil then
                    local btn = self.openbtnTable[pos]
                    if btn ~=nil then
                        doButtonAction(btn,ccui.TouchEventType.ended)
                    end
                end
            else
                if self.closebtnTable ~= nil then
                    local btn = self.closebtnTable[pos]
                    if btn ~=nil then
                        doButtonAction(btn,ccui.TouchEventType.ended)
                    end
                end
            end
        elseif pos == 3 then
            local isMapOn = lk.LakooManager:getInstance():isMiniMapOn()
            if isMapOn then
                if self.openbtnTable ~= nil then
                    local btn = self.openbtnTable[pos]
                    if btn ~=nil then
                        doButtonAction(btn,ccui.TouchEventType.ended)
                    end
                end
            else
                if self.closebtnTable ~= nil then
                    local btn = self.closebtnTable[pos]
                    if btn ~=nil then
                        doButtonAction(btn,ccui.TouchEventType.ended)
                    end
                end
            end
        elseif pos == 4 then
        end
    end
    
end

function SettingLayer:dpadUp()
    if self.tapSelected == 1 then
        return
    end
    
    local pos = self.tapSelected - 1
    
    self.tapSelected = pos
    
    if self.openbtnTable ~= nil then
        local btn = self.openbtnTable[pos]
        if btn ~=nil and self.focusAni ~= nil then
            local p = cc.p(btn:getPositionX(),btn:getPositionY())
            self.focusAni:setPosition(p)
            self.focusAni:setVisible(true)
        end
    end
    
end

function SettingLayer:dpadDown()
    if self.tapSelected == 4 then
        return
    end
    
    local pos = self.tapSelected + 1

    self.tapSelected = pos

    if self.openbtnTable ~= nil then
        local btn = self.openbtnTable[pos]
        if btn ~=nil and self.focusAni ~= nil then
            local p = cc.p(btn:getPositionX(),btn:getPositionY())
            self.focusAni:setPosition(p)
            self.focusAni:setVisible(true)
        end
    end
end

function SettingLayer:registerControllerListener(doButtonAction)
    local function onConnectController(controller, event)
        if controller == nil or controller == self._firstHolder.controller then
            return
        end

        local deviceInfo = string.format("%s id:%d", controller:getDeviceName(), controller:getDeviceId())
        cclog("SettingLayer deviceInfo ="..deviceInfo)

        if self._firstHolder.controller == nil then
            self._firstHolder.controller = controller
        end
    end

    local function onDisconnectedController(controller, event)
        if self._firstHolder.controller == controller then
            self._firstHolder.controller = nil
        end
    end

    local function showButtonState(controller, keyCode, isPressed)
        onConnectController(controller, nil)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else    
            return
        end

        local scene = cc.Director:getInstance():getRunningScene()
        local child = scene:getChildren()

        local topSence = child[#child]
        if topSence:getName() ~= "MainPopLayer" then 
            cclog("SettingLayer no top")
            return
        end

        if lk.LakooManager:getInstance():getIsHasPopView() == true then
            return
        end

        if isPressed then
            cclog("SettingLayer keydown ="..keyCode)
            if keyCode == cc.ControllerKey.BUTTON_A then
                self:dpadTap(doButtonAction)
            elseif keyCode == cc.ControllerKey.BUTTON_B then
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key Down:%d",keyCode)
            end
        else
            cclog("SettingLayer keyup ="..keyCode)
            if keyCode == cc.ControllerKey.BUTTON_A then
                
            elseif keyCode == cc.ControllerKey.BUTTON_B then
                topSence:setSystemBlock(false)
                self:removeFromParent(true)
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
                self:dpadUp()
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
                self:dpadDown()
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then

            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then

            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key up:%d",keyCode)
            end
        end
    end 

    local function onKeyDown(controller, keyCode, event)
        showButtonState(controller, keyCode, true)
    end

    local function onKeyUp(controller, keyCode, event)
        showButtonState(controller, keyCode, false)
    end

    local function onAxisEvent(controller, keyCode, event)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end

        if lk.LakooManager:getInstance():getIsHasPopView() == true then
            return
        end

        local ketStatus = controller:getKeyStatus(keyCode)
        if keyCode == cc.ControllerKey.JOYSTICK_LEFT_X then
            if(ketStatus.value == -1) then
            elseif(ketStatus.value == 1) then

            end
        elseif keyCode == cc.ControllerKey.JOYSTICK_LEFT_Y then

        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_X then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_Y then
        elseif keyCode == cc.ControllerKey.AXIS_LEFT_TRIGGER then
        elseif keyCode == cc.ControllerKey.AXIS_RIGHT_TRIGGER then
        else
        --do nothing
        end
    end

    local controllerListener = cc.EventListenerController:create()
    controllerListener:registerScriptHandler(onConnectController, cc.Handler.EVENT_CONTROLLER_CONNECTED)
    controllerListener:registerScriptHandler(onDisconnectedController, cc.Handler.EVENT_CONTROLLER_DISCONNECTED)

    controllerListener:registerScriptHandler(onKeyDown, cc.Handler.EVENT_CONTROLLER_KEYDOWN)
    controllerListener:registerScriptHandler(onKeyUp, cc.Handler.EVENT_CONTROLLER_KEYUP)
    controllerListener:registerScriptHandler(onAxisEvent, cc.Handler.EVENT_CONTROLLER_AXIS)

    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(controllerListener, self)

    cc.Controller:startDiscoveryController()
end

return SettingLayer