require "cocos.init"
local UILib = require("common.UILib")
local Enum = require("common.Enum")
local AbsLayer = require("gameUI.layer.AbsLayer")

local Text = require("conf.Text")
local IDX = require("conf.TextIndex")

local PluginBag = require("gameUI.layer.PluginBag")
local LayerStage = require("manager.LayerStage")
local cc = cc
local class = class
local table = table

local TAG = {
    BACKGROUND_UI = 10021,
    BACKGROUND = {
        PANEL = 1,
    },
    SINGLE_BAG_UI = 10029,
}

local ControllerHolder = class("ControllerHolder")
function ControllerHolder:ctor()
    self.controller  = nil
    self._holderNode = nil

    self._leftJoystick = nil
    self._rightJoystick = nil

    self._dpadLeft = nil
    self._dpadRight = nil
    self._dpadUp = nil
    self._dpadDown = nil

    self._buttonX = nil
    self._buttonY = nil
    self._buttonA = nil
    self._buttonB = nil

    self._buttonL1 = nil
    self._buttonR1 = nil
    self._buttonL2 = nil
    self._buttonR2 = nil

    self.deviceLabel = nil
    self.externalKeyLabel = nil
end

local SinglePopLayer = class("SinglePopLayer",
    function()
        return AbsLayer:create()
    end
)

local closeFunc = nil
SinglePopLayer.__index = SinglePopLayer
SinglePopLayer.oneTapfinish = true

function SinglePopLayer:create(onCloseFunc, layerType)
    local singlePopLayer = SinglePopLayer.new() 
    singlePopLayer:initLayerInfos(onCloseFunc, layerType)  
    return singlePopLayer
end

function SinglePopLayer:initLayerInfos(onCloseFunc, layerType)
    local mainLayerOut = ccs.SceneReader:getInstance():createNodeWithSceneFile("publish/SinglePopLayer.csb")
    local backgroundUi = UILib.getNodeByTag(mainLayerOut,TAG.BACKGROUND_UI)
    local panel = UILib.getNodeByTag(backgroundUi,TAG.BACKGROUND.PANEL)
    local wSize = panel:getContentSize()
    self:setContentSize(wSize)
    self:setAnchorPoint(UILib.UI_ANCHOR_POINT.CENTER)
    self:addChild(mainLayerOut)
    
    local bagUi = UILib.getNodeByTag(backgroundUi,TAG.SINGLE_BAG_UI)
    PluginBag.clean()
    PluginBag.initSingleBag(bagUi, layerType)
    
    self:setName("SinglePopLayer")
    local osType = Enum.OS_TYPE.IOS
    osType = lk.LakooManager:getInstance():getOSType()
    if osType == Enum.OS_TYPE.ANDROID then
        --joe
        self._firstHolder   = ControllerHolder.new()
        self._firstHolder.controller = nil
        self:registerControllerListener(onCloseFunc)
        --
    end
    
    local function update()
        PluginBag.updateUIValue()
    end
    

    local function updateData()
        PluginBag.updateData()
    end

    local function onNodeEvent(event)
        if "enterTransitionFinish" == event then
            self.scheduleID = cc.Director:getInstance():getScheduler():scheduleScriptFunc(update, 0, false)
            self.scheduleTime = cc.Director:getInstance():getScheduler():scheduleScriptFunc(updateData, 0.1, false)
        end

        if "exit" == event then
            PluginBag.clean()

            if self.scheduleID ~= nil then
                cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.scheduleID)
            end
            if self.scheduleTime ~= nil then
                cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.scheduleTime)
            end

            --SinglePopLayer是一个scene,cocos2dx机制问题，scene exit时会将音效关闭
            --这里和ComAudio::onExit中的修改一起使用可避免这个问题
            lk.LakooManager:getInstance():setEnableStopMusic(false)
        end
    end
    self:registerScriptHandler(onNodeEvent)
end

--joe
function SinglePopLayer:registerControllerListener(onCloseFunc)
    local function onConnectController(controller, event)
        if controller == nil or controller == self._firstHolder.controller then
            return
        end

        local deviceInfo = string.format("%s id:%d", controller:getDeviceName(), controller:getDeviceId())
        cclog("SinglePopLayer deviceInfo ="..deviceInfo)

        if self._firstHolder.controller == nil then
            self._firstHolder.controller = controller
        end
    end
    
    if lk.LakooManager:getInstance():getIsHasPopView() == true then
        return
    end

    local function onDisconnectedController(controller, event)
        if self._firstHolder.controller == controller then
            self._firstHolder.controller = nil
        end
    end

    local function showButtonState(controller, keyCode, isPressed)
        onConnectController(controller, nil)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else    
            return
        end
        
        if lk.LakooManager:getInstance():getIsHasPopView() == true then
            return
        end
        
        local scene = cc.Director:getInstance():getRunningScene()
        local child = scene:getChildren()
        local topSence = child[#child]
        if topSence:getName() ~= self:getName() then 
            cclog("SinglePopLayer no top")
            return
        end
        if isPressed then
            if keyCode == cc.ControllerKey.BUTTON_A then
            elseif keyCode == cc.ControllerKey.BUTTON_B then
                if PluginBag.getItemUseType() == true then
                        cclog("SinglePopLayer PluginBag.getItemUseType() = false")
                        return
                end
                self.oneTapfinish = false
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key Down:%d",keyCode)
            end
        else
            cclog("SinglePopLayer keyup ="..keyCode)
            if keyCode == cc.ControllerKey.BUTTON_A then
            elseif keyCode == cc.ControllerKey.BUTTON_B then
                if self.oneTapfinish == false then
                    self.oneTapfinish = true
                    onCloseFunc(self:getTag())
                    LayerStage.setHideLayerType(TAG.SINGLE_BAG_UI)
                    lk.LakooManager:getInstance():setIsPopNpcShop(false)--[[不能用setIsHasPopView替代，否刚购买后弹出提示无法显示在最上层--]]
                end
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
                
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then

            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then

            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key up:%d",keyCode)
            end
        end
    end 

    local function onKeyDown(controller, keyCode, event)
        showButtonState(controller, keyCode, true)
    end

    local function onKeyUp(controller, keyCode, event)
        showButtonState(controller, keyCode, false)
    end

    local function onAxisEvent(controller, keyCode, event)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end

        if lk.LakooManager:getInstance():getIsHasPopView() == true then
            return
        end
        local ketStatus = controller:getKeyStatus(keyCode)
        if keyCode == cc.ControllerKey.JOYSTICK_LEFT_X then
            if(ketStatus.value == -1) then
            elseif(ketStatus.value == 1) then

            end
        elseif keyCode == cc.ControllerKey.JOYSTICK_LEFT_Y then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_X then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_Y then
        elseif keyCode == cc.ControllerKey.AXIS_LEFT_TRIGGER then
        elseif keyCode == cc.ControllerKey.AXIS_RIGHT_TRIGGER then
        else
        --do nothing
        end
    end

    local controllerListener = cc.EventListenerController:create()
    controllerListener:registerScriptHandler(onConnectController, cc.Handler.EVENT_CONTROLLER_CONNECTED)
    controllerListener:registerScriptHandler(onDisconnectedController, cc.Handler.EVENT_CONTROLLER_DISCONNECTED)

    controllerListener:registerScriptHandler(onKeyDown, cc.Handler.EVENT_CONTROLLER_KEYDOWN)
    controllerListener:registerScriptHandler(onKeyUp, cc.Handler.EVENT_CONTROLLER_KEYUP)
    controllerListener:registerScriptHandler(onAxisEvent, cc.Handler.EVENT_CONTROLLER_AXIS)

    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(controllerListener, self)
    cc.Controller:startDiscoveryController()
end
--
function SinglePopLayer:getItemSignType()
    PluginBag.getItemUseType()
end

function SinglePopLayer:setItemSignType(type)
    PluginBag.setItemUseType(type)
    PluginBag.showChoice()
end

return SinglePopLayer