require "cocos.init"
local UILib = require("common.UILib")
local Enum = require("common.Enum")
local GamePool = require("model.GamePool")
local cc = cc
local class = class

local TAG = {
    BG_IMG = 4,
    TEXT = 7,
    BUT_OK = 5,
    BUT_CANCEL = 6,
    BUT_LAB_TAG = 8,
}

-- joe
local ControllerHolder = class("ControllerHolder")
function ControllerHolder:ctor()
    self.controller  = nil
    self._holderNode = nil

    self._leftJoystick = nil
    self._rightJoystick = nil

    self._dpadLeft = nil
    self._dpadRight = nil
    self._dpadUp = nil
    self._dpadDown = nil

    self._buttonX = nil
    self._buttonY = nil
    self._buttonA = nil
    self._buttonB = nil

    self._buttonL1 = nil
    self._buttonR1 = nil
    self._buttonL2 = nil
    self._buttonR2 = nil

    self.deviceLabel = nil
    self.externalKeyLabel = nil
end
--

local warningLayer = nil

local WarningLayer = class("WarningLayer",
    function()
        return cc.Node:create()
    end
)

WarningLayer.__index = WarningLayer

WarningLayer.layerName = "WarningLayer"
WarningLayer.layerTag = {
    BACK_MAIN_MENU = 700001,
    EXIT_GAME = 700002,
    GAME_START = 700003,
    GAME_CONTINUE = 700004,
    GAME_RE_STORAGE_FROM_CLOUD = 700005,
    APPLE_GAME_CENTER = 700006,
}

WarningLayer.callBackDelegate = nil
WarningLayer.onActionFunc = nil
WarningLayer.msgLabel = nil
WarningLayer.cancelBtn = nil
WarningLayer.okBtn = nil
WarningLayer.isfinishTap = true
function WarningLayer:create()
    warningLayer = WarningLayer.new() 
    warningLayer:init()  
    return warningLayer
end

function WarningLayer:init()
    self:setName(WarningLayer.layerName)
    local visibleSize = cc.Director:getInstance():getVisibleSize()
    self:setContentSize(visibleSize)
    self:setAnchorPoint(UILib.UI_ANCHOR_POINT.CENTER)
    local pos = UILib.offset(0,0,self:getContentSize().width,self:getContentSize().height,UILib.UI_ANCHOR.CENTER)
    self:setPosition(pos)

    local bgLayer = cc.LayerColor:create(UILib.UI_COLOR4B.GRAY)
    bgLayer:setContentSize(visibleSize)
    self:addChild(bgLayer)
    
    ---#####禁止穿透
    local eventListener = cc.EventListenerTouchOneByOne:create()
    local function onTouchBegan(touch, event)         
        return true
    end
    local function onTouchEnded(touch, event)
    end

    eventListener:registerScriptHandler(onTouchBegan,cc.Handler.EVENT_TOUCH_BEGAN)
    eventListener:registerScriptHandler(onTouchEnded,cc.Handler.EVENT_TOUCH_ENDED)
    eventListener:setSwallowTouches(true)
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(eventListener, self)
    ---#####

    local osType = Enum.OS_TYPE.IOS
    osType = lk.LakooManager:getInstance():getOSType()
    if osType == Enum.OS_TYPE.ANDROID then
        --joe
        self._firstHolder   = ControllerHolder.new()
        self._firstHolder.controller = nil
        self:registerControllerListener()
        --
    end
    
    local bgImg = ccui.ImageView:create(Enum.IMG_NAME_PNGHK.WARN_BG)
    bgImg:setTag(TAG.BG_IMG)
    pos = UILib.offsetWithSuperSize(0,0,bgImg:getContentSize().width,bgImg:getContentSize().height,self:getContentSize().width,self:getContentSize().height,UILib.UI_ANCHOR.CENTER)
    bgImg:setPosition(pos)
    self:addChild(bgImg)
    
    local function touchButton(sender, eventType) 
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        --cclog("moved---")
        end
        if eventType == ccui.TouchEventType.ended then
            if sender:getTag() == TAG.BUT_OK and (self.onActionFunc ~= nil or self.callBackDelegate ~= nil) then
                if self.callBackDelegate ~= nil then
                    --非单例模式对象调，delegate必需要用名为onActionFunc方法
                    --@see ItemInfoView:doGetWeapon
                    self.callBackDelegate:onActionFunc()
                else
                    self.onActionFunc()--单例模式使用,@see PluginBag P.showPicInfo->create PicInfoView	
                end
            end   
            self:removeFromParent(true)
        end
    end
    
    self.cancelBtn = ccui.Button:create(Enum.IMG_NAME_PNGHK.BUTTON_BG,Enum.IMG_NAME_PNGHK.BUTTON_BG,Enum.IMG_NAME_PNGHK.BUTTON_BG)
    self.cancelBtn:setTag(TAG.BUT_CANCEL)
    pos = UILib.offsetWithSuperSize(-10,25,self.cancelBtn:getContentSize().width,self.cancelBtn:getContentSize().height,bgImg:getContentSize().width,bgImg:getContentSize().height,UILib.UI_ANCHOR.BOTTOM_RIGHT)
    self.cancelBtn:setPosition(pos)
    self:addChild(self.cancelBtn)
    self.cancelBtn:addTouchEventListener(touchButton)
    
    --cancelBut text
    local textLabel = UILib.textAddStroke("NO",UILib.FONT_NAME_NORMAL,UILib.FONT.SMALL,UILib.UI_COLOR3B.WHITE,UILib.UI_COLOR3B.BLACK,1)
    textLabel:setTag(TAG.BUT_LAB_TAG)
    pos = UILib.offsetWithSuperSize(0,0,textLabel:getContentSize().width,textLabel:getContentSize().height,self.cancelBtn:getContentSize().width,self.cancelBtn:getContentSize().height,UILib.UI_ANCHOR.CENTER)
    textLabel:setPosition(pos)
    self.cancelBtn:addChild(textLabel)
    
    self.okBtn = ccui.Button:create(Enum.IMG_NAME_PNGHK.BUTTON_BG,Enum.IMG_NAME_PNGHK.BUTTON_BG,Enum.IMG_NAME_PNGHK.BUTTON_BG)
    self.okBtn:setTag(TAG.BUT_OK)
    pos = UILib.offsetWithSuperSize(15,25,self.okBtn:getContentSize().width,self.okBtn:getContentSize().height,bgImg:getContentSize().width,bgImg:getContentSize().height,UILib.UI_ANCHOR.BOTTOM_LEFT)
    self.okBtn:setPosition(pos)
    self:addChild(self.okBtn)
    self.okBtn:addTouchEventListener(touchButton)
    
    --okBut text
    textLabel = UILib.textAddStroke("YES",UILib.FONT_NAME_NORMAL,UILib.FONT.SMALL,UILib.UI_COLOR3B.WHITE,UILib.UI_COLOR3B.BLACK,1)
    textLabel:setTag(TAG.BUT_LAB_TAG)
    pos = UILib.offsetWithSuperSize(0,0,textLabel:getContentSize().width,textLabel:getContentSize().height,self.okBtn:getContentSize().width,self.okBtn:getContentSize().height,UILib.UI_ANCHOR.CENTER)
    textLabel:setPosition(pos)
    self.okBtn:addChild(textLabel)
    
    self.msgLabel = CCLabelTTF:create("Default Text", UILib.FONT_NAME_NORMAL, UILib.FONT.SMALL, cc.size(bgImg:getContentSize().width-30, UILib.FONT.SMALL*3.5), kCCTextAlignmentCenter)
    --cc.Label:create()
    if self.msgLabel ~= nil then
        self.msgLabel:setColor(UILib.UI_COLOR3B.TREE_GREEN)
    	--self.msgLabel:setString("Default Text")
        pos = UILib.offsetWithSuperSize(0,-15,self.msgLabel:getContentSize().width,self.msgLabel:getContentSize().height,bgImg:getContentSize().width,bgImg:getContentSize().height,UILib.UI_ANCHOR.CENTER_TOP)
        --self.msgLabel:setSystemFontSize(UILib.FONT.SMALL)
    	self.msgLabel:setPosition(pos)
    	self:addChild(self.msgLabel)
    end
    
--    lk.LakooManager:getInstance():setIsHasPopView(true)
end

function WarningLayer:setOnActionFunc(func)
    self.onActionFunc = func
end

function WarningLayer:setDelegate(delegate)
    self.callBackDelegate = delegate
end

function WarningLayer:setString(msg, okBtnText, cancelBtnText)
    local bgImg = UILib.getNodeByTag(self,TAG.BG_IMG)
    local pos = cc.p(0,0)
    if self.msgLabel ~=nil and msg ~= nil then
        self.msgLabel:removeFromParent(true)
        self.msgLabel = CCLabelTTF:create(msg, UILib.FONT_NAME_NORMAL, UILib.FONT.SMALL, cc.size(bgImg:getContentSize().width-30, 0), kCCTextAlignmentCenter)
        local rY = self.msgLabel:getContentSize().height/bgImg:getContentSize().height
        if rY > 0.7 then
            self.msgLabel = CCLabelTTF:create(msg, UILib.FONT_NAME_NORMAL, UILib.FONT.SMALL, cc.size(bgImg:getContentSize().width*1.5-30, 0), kCCTextAlignmentCenter)
        end
        if self.msgLabel ~= nil then
            self.msgLabel:setColor(UILib.UI_COLOR3B.TREE_GREEN)
            pos = UILib.offsetWithSuperSize(0,15,self.msgLabel:getContentSize().width,self.msgLabel:getContentSize().height,self:getContentSize().width,self:getContentSize().height,UILib.UI_ANCHOR.CENTER)
            self.msgLabel:setPosition(pos)
            self:addChild(self.msgLabel)
        end
    end
    
    if self.okBtn ~=nil and okBtnText ~= nil then
        local lab = UILib.getNodeByTag(self.okBtn,TAG.BUT_LAB_TAG)
        if lab ~= nil then
            UILib.setStrokeString(lab,okBtnText)
        	--lab:setString(okBtnText)
        end
    end
    if self.cancelBtn ~=nil and cancelBtnText ~= nil then
        local lab = UILib.getNodeByTag(self.cancelBtn,TAG.BUT_LAB_TAG)
        if lab ~= nil then
            UILib.setStrokeString(lab,cancelBtnText)
            --lab:setString(cancelBtnText)
        end
    end
    
    local bScaleX = (self.msgLabel:getContentSize().width+30)/bgImg:getContentSize().width
    if bScaleX < 1 then
    	bScaleX = 1
    end
    local oHeight = self.msgLabel:getContentSize().height
    local height = oHeight
    if height < bgImg:getContentSize().height then
        height = bgImg:getContentSize().height
    end
    local bScaleY = height/bgImg:getContentSize().height
    if oHeight/bgImg:getContentSize().height > 0.7 then
        bScaleY = bScaleY + 0.4
    end
    bgImg:setScale(bScaleX,bScaleY)
    
    --+self.okBtn:getContentSize().height*0.5
    pos = cc.p(bgImg:getPositionX()-self.okBtn:getContentSize().width*0.5,bgImg:getPositionY()-bgImg:getContentSize().height*bScaleY/2+self.okBtn:getContentSize().height*0.8)
    self.okBtn:setPosition(pos)
    pos = cc.p(self.okBtn:getPositionX()+self.okBtn:getContentSize().width,self.okBtn:getPositionY())
    self.cancelBtn:setPosition(pos)
end

function WarningLayer:addChildNode(node)
    if node == nil then
    	return
    end
    local bgImg = UILib.getNodeByTag(self,TAG.BG_IMG)
    if bgImg == nil then
    	return
    end
    bgImg:addChild(node)
end

function WarningLayer:getBgImgSize()
    local bgImg = UILib.getNodeByTag(self,TAG.BG_IMG)
    if bgImg == nil then
        return
    end
    return bgImg:getContentSize()
end

--joe
function WarningLayer:registerControllerListener()
    local function onConnectController(controller, event)
        if controller == nil or controller == self._firstHolder.controller then
            return
        end

        local deviceInfo = string.format("%s id:%d", controller:getDeviceName(), controller:getDeviceId())
        cclog("WarningLayer deviceInfo ="..deviceInfo)

        if self._firstHolder.controller == nil then
            self._firstHolder.controller = controller
        end
    end

    local function onDisconnectedController(controller, event)
        if self._firstHolder.controller == controller then
            self._firstHolder.controller = nil
        end
    end

    local function showButtonState(controller, keyCode, isPressed)
        onConnectController(controller, nil)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else    
            return
        end
        
        local scene = cc.Director:getInstance():getRunningScene()
        local child = scene:getChildren()
        
        local topSence = child[#child]
        if topSence:getName() ~= self:getName() then
            if  topSence:getName() ~= "TitleScene" then
                return
            end
        end


        if isPressed then
            cclog("WarningLayer keydown ="..keyCode)
            if keyCode == cc.ControllerKey.BUTTON_A then
                if self.isfinishTap == true then
                    self.isfinishTap = false
                end
            elseif keyCode == cc.ControllerKey.BUTTON_B then
                if self.isfinishTap == true then
                    self.isfinishTap = false
                end
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key Down:%d",keyCode)
            end
        else
            cclog("WarningLayer keyup ="..keyCode)
            if keyCode == cc.ControllerKey.BUTTON_A then
                if self.isfinishTap == false then
                    self.isfinishTap = true
                    if self.callBackDelegate ~= nil then
                        --非单例模式对象调，delegate必需要用名为onActionFunc方法
                        --@see ItemInfoView:doGetWeapon
                        self.callBackDelegate:onActionFunc()
                    else
                        self.onActionFunc()--单例模式使用,@see PluginBag P.showPicInfo->create PicInfoView    
                    end
                    self:removeFromParent(true)
                end
            elseif keyCode == cc.ControllerKey.BUTTON_B then
                if self.isfinishTap == false then
                    self.isfinishTap = true
                    self:removeFromParent(true)
                end
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
               
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
                
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key up:%d",keyCode)
            end
        end
    end 

    local function onKeyDown(controller, keyCode, event)
        showButtonState(controller, keyCode, true)
    end

    local function onKeyUp(controller, keyCode, event)
        showButtonState(controller, keyCode, false)
    end

    local function onAxisEvent(controller, keyCode, event)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end

        local ketStatus = controller:getKeyStatus(keyCode)
        if keyCode == cc.ControllerKey.JOYSTICK_LEFT_X then
            if(ketStatus.value == -1) then
            elseif(ketStatus.value == 1) then
                
            end
        elseif keyCode == cc.ControllerKey.JOYSTICK_LEFT_Y then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_X then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_Y then
        elseif keyCode == cc.ControllerKey.AXIS_LEFT_TRIGGER then
        elseif keyCode == cc.ControllerKey.AXIS_RIGHT_TRIGGER then
        else
        --do nothing
        end
    end

    local controllerListener = cc.EventListenerController:create()
    controllerListener:registerScriptHandler(onConnectController, cc.Handler.EVENT_CONTROLLER_CONNECTED)
    controllerListener:registerScriptHandler(onDisconnectedController, cc.Handler.EVENT_CONTROLLER_DISCONNECTED)

    controllerListener:registerScriptHandler(onKeyDown, cc.Handler.EVENT_CONTROLLER_KEYDOWN)
    controllerListener:registerScriptHandler(onKeyUp, cc.Handler.EVENT_CONTROLLER_KEYUP)
    controllerListener:registerScriptHandler(onAxisEvent, cc.Handler.EVENT_CONTROLLER_AXIS)

    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(controllerListener, self)
    cc.Controller:startDiscoveryController()
end
--

return WarningLayer