require "cocos.init"
local UILib = require("common.UILib")
local Enum = require("common.Enum")
local Text = require("conf.Text")
local TextIndex = require("conf.TextIndex")
local LKConfig = require("conf.LKConfig")

local WarningLayer = require("gameUI.layer.WarningLayer")
local ProfileLayer = require("gameUI.layer.ProfileLayer")

local GamePool = require("model.GamePool")
local NetworkManager = require("manager.NetworkManager")

local cc = cc
local class = class

local TAG = {
    }

local ZDK_KEY = {
    NONE = 0,
    KNOWLEDGE = 1,
    RATE_THE_APP = 2,
    CONTACT_US = 3,
}


local ZendeskLayer = class("ZendeskLayer",
    function()
        return cc.Node:create()
    end
)

ZendeskLayer.__index = ZendeskLayer

ZendeskLayer.isCallBackClose = false

ZendeskLayer.focusAni = nil
ZendeskLayer.selectIdx = ZDK_KEY.NONE
ZendeskLayer.btnTable = {}

function ZendeskLayer:create()
    local zendeskLayer = ZendeskLayer.new()
    zendeskLayer:init()
    return zendeskLayer
end

function ZendeskLayer:init()
    local visibleSize = cc.Director:getInstance():getVisibleSize()
    self:setContentSize(visibleSize)
    self:setAnchorPoint(UILib.UI_ANCHOR_POINT.CENTER)
    local pos = UILib.offset(0,0,self:getContentSize().width,self:getContentSize().height,UILib.UI_ANCHOR.CENTER)
    self:setPosition(pos)

    ---#####禁止穿透
    local eventListener = cc.EventListenerTouchOneByOne:create()
    local function onTouchBegan(touch, event)
        return true
    end
    local function onTouchEnded(touch, event)
    end

    eventListener:registerScriptHandler(onTouchBegan,cc.Handler.EVENT_TOUCH_BEGAN)
    eventListener:registerScriptHandler(onTouchEnded,cc.Handler.EVENT_TOUCH_ENDED)
    eventListener:setSwallowTouches(true)
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(eventListener, self)
    ---#####

    local bgImg = ccui.ImageView:create(Enum.IMG_NAME_PNGHK.LANG_BG)
    bgImg:setScaleX(GamePool.getScaleX())
    bgImg:setScaleY(GamePool.getScaleY())
    pos = UILib.offsetWithSuperSize(0,0,bgImg:getContentSize().width,bgImg:getContentSize().height,self:getContentSize().width,self:getContentSize().height,UILib.UI_ANCHOR.CENTER)
    bgImg:setPosition(pos)
    self:addChild(bgImg)

    --back button
    local function doBackAction(sender, eventType)
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            self:removeFromParent(true)
        end
    end
    local backButton = ccui.Button:create(Enum.IMG_NAME_PNGHK.BACK,Enum.IMG_NAME_PNGHK.BACK,Enum.IMG_NAME_PNGHK.BACK)
    pos = UILib.offset(0,0,backButton:getContentSize().width,backButton:getContentSize().height,UILib.UI_ANCHOR.TOP_RIGHT)
    backButton:setPosition(pos)
    self:addChild(backButton)
    backButton:addTouchEventListener(doBackAction)


    --lang option button
    local function doLangAction(sender, eventType)
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            local key = sender:getTag()
            local isHasProfile = GamePool.isHasProfile()
            if isHasProfile then
                if key == ZDK_KEY.KNOWLEDGE then
                    lk.LakooManager:getInstance():showZendeskHelpCenter()
                elseif key == ZDK_KEY.RATE_THE_APP then
                    --lk.LakooManager:getInstance():showFeedback()
                    self:openReview()
                elseif key == ZDK_KEY.CONTACT_US then
                    lk.LakooManager:getInstance():showRequestCreation()
                end
                NetworkManager.putMessageToMailChimp()
            else
                self.selectIdx = key
                self:showProfileLayer()
            end
        end
    end
    local strTable = {Text.get(TextIndex.KNOWLEDGE),Text.get(TextIndex.RATE_THE_APP),Text.get(TextIndex.CONTACT_US)}
    local drawX = self:getContentSize().width/2
    local drawY = 0
    local intervalY = 5
    for key, var in ipairs(strTable) do
        local langBut = ccui.Button:create(Enum.IMG_NAME_PNGHK.OPTION_BG,Enum.IMG_NAME_PNGHK.OPTION_BG,Enum.IMG_NAME_PNGHK.OPTION_BG)
        langBut:setTitleText(var)
        langBut:setTitleColor(UILib.UI_COLOR3B.BLACK)
        langBut:setTitleFontSize(UILib.FONT.SMALL)
        langBut:setTag(key)
        if key == 1 then
            drawY = self:getContentSize().height/2 + langBut:getContentSize().height*1.5 + intervalY*1.5
        else
            drawY  = drawY - langBut:getContentSize().height - intervalY
        end
        cclog("x = "..drawX..",drawY"..drawY)
        pos = cc.p(drawX,drawY)
        langBut:setPosition(pos)
        self:addChild(langBut)
        langBut:addTouchEventListener(doLangAction)
        table.insert(self.btnTable,key,langBut)
    end
    
    local label = cc.Label:create()
    local emailString = string.format(Text.get(TextIndex.YOUR_EMAIL),GamePool.getProfileEmail())
    label:setColor(UILib.UI_COLOR3B.BLACK)
    label:setString(emailString)
    local labelX = label:getContentSize().width*0.5+10
    local labelY = label:getContentSize().height
    local labelPos = cc.p(labelX,labelY)
    label:setPosition(labelPos)
    self:addChild(label)

    local function update()
        lk.LakooManager:getInstance():autoHiddenNavigationBar()
    end

    local function onNodeEvent(event)
        if "enterTransitionFinish" == event then
            self.scheduleID = cc.Director:getInstance():getScheduler():scheduleScriptFunc(update, 0, false)
        end
        if "exit" == event then
            if self.scheduleID ~= nil then
                cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.scheduleID)
            end
        end
    end
    self:registerScriptHandler(onNodeEvent)
end

function ZendeskLayer:showProfileLayer()
    local profileLayer = ProfileLayer:create()
    profileLayer:setCallBackDelegate(self)
    self:addChild(profileLayer)
end

function ZendeskLayer:profileLayerCallBackContinue()
    if self.selectIdx == ZDK_KEY.KNOWLEDGE then
        lk.LakooManager:getInstance():showZendeskHelpCenter()
    elseif self.selectIdx == ZDK_KEY.RATE_THE_APP then
        --lk.LakooManager:getInstance():showFeedback()
        self:openReview()
    elseif self.selectIdx == ZDK_KEY.CONTACT_US then
        lk.LakooManager:getInstance():showRequestCreation()
    end
    NetworkManager.putMessageToMailChimp()
end

function ZendeskLayer.openReview()
    local url = LKConfig.REVIEW_URL_IOS
    if lk.LakooManager:getInstance():getChannel() == Enum.CHANNEL.ANDROID_PHONE_GOOGLE then
        url = LKConfig.REVIEW_URL_GOOGLE_PLAY
    end
    cc.Application:getInstance():openURL(url)
end

return ZendeskLayer