require "cocos.init"
local UILib = require("common.UILib")
local Enum = require("common.Enum")

local Text = require("conf.Text")
local IDX = require("conf.TextIndex")
local Config = require("conf.LKConfig")

local MessageLayer = require("gameUI.layer.MessageLayer")
local WarningLayer = require("gameUI.layer.WarningLayer")
local GameDetailLayer = require("gameUI.layer.GameDetailLayer")
local LanguageLayer = require("gameUI.layer.LanguageLayer")
local GoogleConnectLayer = require("gameUI.layer.GoogleConnectLayer")

local LayerManager = require("manager.LayerManager")

local GamePool = require("model.GamePool")

local LayerStage = require("manager.LayerStage")
local SceneStage = require("manager.SceneStage")
local Android = require("conf.Android")
local cc = cc
local class = class
local table = table
local lk = lk

local ACTION_KEY = {
    CONTINUE = 1,
    START = 2,
    LANGUAGE = 3,
    HELP = 4,
    ABOUT = 5,
}

local TitleScene = class("TitleScene",
    function()
        return cc.Layer:create()
    end
)

-- joe
local ControllerHolder = class("ControllerHolder")
function ControllerHolder:ctor()
    self.controller  = nil
    self._holderNode = nil

    self._leftJoystick = nil
    self._rightJoystick = nil

    self._dpadLeft = nil
    self._dpadRight = nil
    self._dpadUp = nil
    self._dpadDown = nil

    self._buttonX = nil
    self._buttonY = nil
    self._buttonA = nil
    self._buttonB = nil

    self._buttonL1 = nil
    self._buttonR1 = nil
    self._buttonL2 = nil
    self._buttonR2 = nil

    self.deviceLabel = nil
    self.externalKeyLabel = nil
end
--
TitleScene.__index = TitleScene

TitleScene.preButton = nil
TitleScene.nextButton = nil
TitleScene.scrollView = nil
TitleScene.aniTargetX = 0
TitleScene.actionKeyIndex = ACTION_KEY.CONTINUE
TitleScene.isTurnRight = true
TitleScene.isTurning = false
TitleScene.popPanel = nil
TitleScene.isHasChunk = false
TitleScene.aniPanel = nil
TitleScene.isGoogleConnectNotice = false
TitleScene.googleConnectLayer = nil
TitleScene.warningLayer = nil
TitleScene.loadingLayer = nil
TitleScene.isEnterTransitionFinish = false

function TitleScene:create()
    local titleScene = TitleScene.new()
    titleScene:init()

    local scene = cc.Scene:create()--创建场景
    scene:addChild(titleScene)
    return scene
end

--joe
--local osType = Enum.OS_TYPE.IOS
--osType = lk.LakooManager:getInstance():getOSType()
--    if osType == Enum.OS_TYPE.ANDROID then
function TitleScene:registerControllerListener()
    local function onConnectController(controller, event)

        if controller == nil or controller == self._firstHolder.controller then
            return
        end

        local deviceInfo = string.format("%s id:%d", controller:getDeviceName(), controller:getDeviceId())
        cclog("TitleScene deviceInfo ="..deviceInfo)

        if self._firstHolder.controller == nil then
            self._firstHolder.controller = controller
            cclog("onConnectController")
        end
    end

    local function onDisconnectedController(controller, event)
        cclog("onDisconnectedController")
        if self._firstHolder.controller == controller then
            self._firstHolder.controller = nil
        end
    end

    local DO_ACTION = {
        [ACTION_KEY.CONTINUE] = function ()  self:doContinue()  end,
        [ACTION_KEY.START] = function ()  self:doStart()  end,
        [ACTION_KEY.LANGUAGE] = function ()  self:doLanguage()  end,
        [ACTION_KEY.HELP] = function ()  self:doHelp()  end,
    }

    local function showButtonState(controller, keyCode, isPressed)
        onConnectController(controller, nil)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end

        if isPressed then
            cclog("down keyCode = "..keyCode)
            if keyCode == cc.ControllerKey.BUTTON_A then
            elseif keyCode == cc.ControllerKey.BUTTON_B then
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key Down:%d",keyCode)
            end
        else
            cclog("up keyCode = "..keyCode)
            if keyCode == cc.ControllerKey.BUTTON_A then
            local children = self.popPanel:getChildren()
                if children ~= nil then
                    for var=1,#children do
                        local chind = children[var]
                        if chind ~=nil then
                            return
                        end
                    end
                end
                local func = DO_ACTION[self.actionKeyIndex]
                if func == nil then
                    return
                else
                    func()
                end
            elseif keyCode == cc.ControllerKey.BUTTON_B then
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
                if self.isTurning then
                    return
                end
                self.isTurning = true
                if self.actionKeyIndex <=1 then
                    return
                end
                self.actionKeyIndex = self.actionKeyIndex - 1
                local drawX = self.scrollView:getInnerContainer():getPositionX()
                local width = self.scrollView:getContentSize().width
                self.aniTargetX = drawX+width
                self.isTurnRight = false
                local enablePre = true
                local enableNext = true
                local isHasChunk = lk.LakooManager:getInstance():isHasLKData(Enum.FILE.GAME_CHUNK_NAME)
                if self.actionKeyIndex == ACTION_KEY.CONTINUE and isHasChunk then
                    enablePre = false
                end
                if self.actionKeyIndex == ACTION_KEY.START and isHasChunk == false then
                    enablePre = false
                end
                if self.actionKeyIndex == ACTION_KEY.ABOUT then
                    enableNext = false
                end
                self:updateTurnButton(enablePre,enableNext)
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
                if self.isTurning then
                    return
                end
                self.isTurning = true
                if self.actionKeyIndex >=4 then
                    return
                end
                self.actionKeyIndex = self.actionKeyIndex + 1
                local drawX = self.scrollView:getInnerContainer():getPositionX()
                local width = self.scrollView:getContentSize().width
                self.aniTargetX = drawX-width
                self.isTurnRight = true
                local enablePre = true
                local enableNext = true
                local isHasChunk = lk.LakooManager:getInstance():isHasLKData(Enum.FILE.GAME_CHUNK_NAME)
                if self.actionKeyIndex == ACTION_KEY.CONTINUE and isHasChunk then
                    enablePre = false
                end
                if self.actionKeyIndex == ACTION_KEY.START and isHasChunk == false then
                    enablePre = false
                end
                if self.actionKeyIndex == ACTION_KEY.HELP then
                    enableNext = false
                end
                self:updateTurnButton(enablePre,enableNext)
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            else
                local ketStatus = string.format("Key up:%d",keyCode)
            end
        end
    end

    local function onKeyDown(controller, keyCode, event)
        showButtonState(controller, keyCode, true)
    end

    local function onKeyUp(controller, keyCode, event)
        showButtonState(controller, keyCode, false)
    end

    local function onAxisEvent(controller, keyCode, event)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end

        local ketStatus = controller:getKeyStatus(keyCode)
        if keyCode == cc.ControllerKey.JOYSTICK_LEFT_X then
            if(ketStatus.value == -1) then

            elseif(ketStatus.value == 1) then

            end
        elseif keyCode == cc.ControllerKey.JOYSTICK_LEFT_Y then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_X then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_Y then
        elseif keyCode == cc.ControllerKey.AXIS_LEFT_TRIGGER then
        elseif keyCode == cc.ControllerKey.AXIS_RIGHT_TRIGGER then
        else
        --do nothing
        end
    end

    local controllerListener = cc.EventListenerController:create()
    controllerListener:registerScriptHandler(onConnectController, cc.Handler.EVENT_CONTROLLER_CONNECTED)
    controllerListener:registerScriptHandler(onDisconnectedController, cc.Handler.EVENT_CONTROLLER_DISCONNECTED)

    controllerListener:registerScriptHandler(onKeyDown, cc.Handler.EVENT_CONTROLLER_KEYDOWN)
    controllerListener:registerScriptHandler(onKeyUp, cc.Handler.EVENT_CONTROLLER_KEYUP)
    controllerListener:registerScriptHandler(onAxisEvent, cc.Handler.EVENT_CONTROLLER_AXIS)

    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(controllerListener, self)

    cc.Controller:startDiscoveryController()
    cclog(" registerControllerListener done")
end
--
--    end


function TitleScene:init()
    self:setName("TitleScene")
    local visibleSize = cc.Director:getInstance():getVisibleSize()
    self:setContentSize(visibleSize)
    local pos = cc.p(0,0)
    self:setPosition(pos)

    --Util.playMusic("0.mp3",true)
    lk.LakooManager:getInstance():playMusic(0,true)
    local DO_ACTION = {}
    if Android.CHANNEL == 4 then
        DO_ACTION = {
            [ACTION_KEY.CONTINUE] = function ()  self:doContinue()  end,
            [ACTION_KEY.START] = function ()  self:doStart()  end,
            [ACTION_KEY.LANGUAGE] = function ()  self:doLanguage()  end,
            [ACTION_KEY.HELP] = function ()  self:doHelp()  end,
        }
    else
        DO_ACTION = {
            [ACTION_KEY.CONTINUE] = function ()  self:doContinue()  end,
            [ACTION_KEY.START] = function ()  self:doStart()  end,
            [ACTION_KEY.LANGUAGE] = function ()  self:doLanguage()  end,
            [ACTION_KEY.HELP] = function ()  self:doHelp()  end,
            [ACTION_KEY.ABOUT] = function ()  self:doAbout()  end,
        }
    end
    

    ---#####禁止穿透
    local eventListener = cc.EventListenerTouchOneByOne:create()
    local function onTouchBegan(touch, event)
        return true
    end
    local function onTouchEnded(touch, event)
        if touch ~= nil then
            local touchPoint = touch:getLocation()
            if (cc.rectContainsPoint(self.preButton:getBoundingBox(),touchPoint)
                or cc.rectContainsPoint(self.nextButton:getBoundingBox(),touchPoint)) then
                return--[[FIX:左右按钮禁用后会被穿透，左右按钮禁用情况下应无响应--]]
            end
        end

        local func = DO_ACTION[self.actionKeyIndex]
        if func == nil then
            return
        else
            func()
        end
    end

    eventListener:registerScriptHandler(onTouchBegan,cc.Handler.EVENT_TOUCH_BEGAN)
    eventListener:registerScriptHandler(onTouchEnded,cc.Handler.EVENT_TOUCH_ENDED)
    eventListener:setSwallowTouches(true)
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(eventListener, self)
    ---#####
    ------------


    local osType = Enum.OS_TYPE.IOS
    osType = lk.LakooManager:getInstance():getOSType()
    if osType == Enum.OS_TYPE.ANDROID then
        --joe
        self._firstHolder   = ControllerHolder.new()
        self._firstHolder.controller = nil
        self:registerControllerListener()
        --
    end
    self.aniPanel = cc.Node:create()
    self.aniPanel:setContentSize(visibleSize)
    self:addChild(self.aniPanel)
    self:replaceTitleAnim()
    --上一个/下一个按钮
    local function doButtonAction(sender, eventType)
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            if self.isTurning then
                return
            end
            self.isTurning = true
            local tag = sender:getTag()
            if tag == 1 then
                cclog("self.actionKeyIndex = "..self.actionKeyIndex)
                self.actionKeyIndex = self.actionKeyIndex - 1
                local drawX = self.scrollView:getInnerContainer():getPositionX()
                local width = self.scrollView:getContentSize().width
                self.aniTargetX = drawX+width
                self.isTurnRight = false
            else
                cclog("self.actionKeyIndex = "..self.actionKeyIndex)
                self.actionKeyIndex = self.actionKeyIndex + 1
                local drawX = self.scrollView:getInnerContainer():getPositionX()
                local width = self.scrollView:getContentSize().width
                self.aniTargetX = drawX-width
                self.isTurnRight = true
            end
            local enablePre = true
            local enableNext = true
            local isHasChunk = lk.LakooManager:getInstance():isHasLKData(Enum.FILE.GAME_CHUNK_NAME)
            if self.actionKeyIndex == ACTION_KEY.CONTINUE and isHasChunk then
                enablePre = false
            end
            if self.actionKeyIndex == ACTION_KEY.START and isHasChunk == false then
                enablePre = false
            end
            if self.actionKeyIndex == ACTION_KEY.ABOUT then
                enableNext = false
            end
            self:updateTurnButton(enablePre,enableNext)
        end
    end
    self.preButton = ccui.Button:create(Enum.IMG_NAME_PNGHK.LEFT_ARROW,Enum.IMG_NAME_PNGHK.LEFT_ARROW_S,Enum.IMG_NAME_PNGHK.LEFT_ARROW_UN)
    local px = self.preButton:getContentSize().width/2
    pos = UILib.offsetWithSuperSize(-110,35,self.preButton:getContentSize().width,self.preButton:getContentSize().height,self:getContentSize().width,self:getContentSize().height,UILib.UI_ANCHOR.CENTER_BOTTOM)
    self.preButton:setPosition(pos)
    self:addChild(self.preButton)
    self.preButton:setTag(1)
    self.preButton:addTouchEventListener(doButtonAction)

    self.nextButton = ccui.Button:create(Enum.IMG_NAME_PNGHK.LEFT_ARROW,Enum.IMG_NAME_PNGHK.LEFT_ARROW_S,Enum.IMG_NAME_PNGHK.LEFT_ARROW_UN)
    local px = self.preButton:getContentSize().width/2
    pos = UILib.offsetWithSuperSize(110,35,self.preButton:getContentSize().width,self.preButton:getContentSize().height,self:getContentSize().width,self:getContentSize().height,UILib.UI_ANCHOR.CENTER_BOTTOM)
    self.nextButton:setPosition(pos)
    self.nextButton:setRotation(180)
    self:addChild(self.nextButton)
    self.nextButton:setTag(0)
    self.nextButton:addTouchEventListener(doButtonAction)

    self:updateTurnButton(false,true)

    --copyright lakoo
    local lakooImg = ccui.ImageView:create(Enum.IMG_NAME_PNGHK.LAKOO)
    pos = UILib.offset(0,lakooImg:getContentSize().height/2,lakooImg:getContentSize().width,lakooImg:getContentSize().height,UILib.UI_ANCHOR.CENTER_BOTTOM)
    lakooImg:setPosition(pos)
    self:addChild(lakooImg)
    
    if Android.CHANNEL == 4 then
        lakooImg:setVisible(false)
    end

    --notice
    local mailHasBut = nil
    local mailBut = nil
    local function doNoticeAction(sender, eventType)
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            if GamePool.getNoticeUrl() == nil or GamePool.getNoticeUrl() == "" then
                local msg = lk.LakooManager:getInstance():getString(Enum.TEXT.T_NOT_NEWS)
                local messageLayer = MessageLayer:create(msg)
                local scene = cc.Director:getInstance():getRunningScene()
                scene:addChild(messageLayer)
             
                return
            end
            GamePool.setWebUrlType(Enum.WEB_URL_TYPE.TITLE_NOTICE)
            LayerStage.setShowLayerType(Enum.LAYER_TYPE.NOTICE)

            mailHasBut:setVisible(false)
            mailBut:setVisible(true)

            GamePool.refreshNotice()
        end
    end
    mailHasBut = ccui.Button:create(Enum.IMG_NAME_PNGHK.MAIL_HAS,Enum.IMG_NAME_PNGHK.MAIL_HAS,Enum.IMG_NAME_PNGHK.MAIL_HAS)
    pos = UILib.offset(0,0,mailHasBut:getContentSize().width,mailHasBut:getContentSize().height,UILib.UI_ANCHOR.TOP_RIGHT)
    mailHasBut:setPosition(pos)
    self:addChild(mailHasBut)
    mailHasBut:addTouchEventListener(doNoticeAction)

    mailBut = ccui.Button:create(Enum.IMG_NAME_PNGHK.MAIL,Enum.IMG_NAME_PNGHK.MAIL,Enum.IMG_NAME_PNGHK.MAIL)
    mailBut:setPosition(pos)
    self:addChild(mailBut)
    mailBut:addTouchEventListener(doNoticeAction)

    local isNewNotice = GamePool.isHasNewNotice()
    if isNewNotice then
        mailHasBut:setVisible(true)
        mailBut:setVisible(false)
    else
        mailHasBut:setVisible(false)
        mailBut:setVisible(true)
    end

    --show lakoo more game(app)
    local function doShowMoreAction(sender, eventType)
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            lk.LakooManager:getInstance():showChartboostMoreApps()
        end
    end
    local lakooBut = ccui.Button:create(Enum.IMG_NAME_PNGHK.LAKOO_ICON,Enum.IMG_NAME_PNGHK.LAKOO_ICON,Enum.IMG_NAME_PNGHK.LAKOO_ICON)
    pos = cc.p(mailBut:getPositionX()-mailBut:getContentSize().width/2-5-lakooBut:getContentSize().width/2,mailBut:getPositionY()-5)
    lakooBut:setPosition(pos)
    self:addChild(lakooBut)
    lakooBut:addTouchEventListener(doShowMoreAction)

    if Android.CHANNEL == 4 then
        lakooBut:setVisible(false)
    end
    --title select
    self.scrollView = ccui.ScrollView:create()
    self.scrollView:setClippingEnabled(true)
    local sw = self.nextButton:getPositionX()-self.nextButton:getContentSize().width/2-self.preButton:getContentSize().width/2-self.preButton:getPositionX()
    local sh = self.preButton:getContentSize().height
    self.scrollView:setContentSize(cc.size(sw,sh))
    self.scrollView:setPosition(cc.p(self.preButton:getPositionX()+self.preButton:getContentSize().width/2,self.preButton:getPositionY()-self.scrollView:getContentSize().height/2))
    self:addChild(self.scrollView)

    local strTable = {}
    self.isHasChunk = lk.LakooManager:getInstance():isHasLKData(Enum.FILE.GAME_CHUNK_NAME)
    if self.isHasChunk then
        local str = lk.LakooManager:getInstance():getString(Enum.TEXT.T_GAME_CONTINUE)
        table.insert(strTable,ACTION_KEY.CONTINUE,str)
    else
        self.actionKeyIndex = ACTION_KEY.START
    end
    str = lk.LakooManager:getInstance():getString(Enum.TEXT.T_GAME_START)
    table.insert(strTable,ACTION_KEY.START,str)
    str = lk.LakooManager:getInstance():getString(Enum.TEXT.T_LANGUAGE)
    table.insert(strTable,ACTION_KEY.LANGUAGE,str)
    str = lk.LakooManager:getInstance():getString(Enum.TEXT.T_GAME_HELP)
    table.insert(strTable,ACTION_KEY.HELP,str)
    str = lk.LakooManager:getInstance():getString(Enum.TEXT.T_GAME_ABOUT)
    table.insert(strTable,ACTION_KEY.ABOUT,str)

    local function doTouchAction(sender, eventType)
        if eventType == ccui.TouchEventType.began then
        end
        if eventType == ccui.TouchEventType.moved then
        end
        if eventType == ccui.TouchEventType.ended then
            onTouchEnded(nil,eventType)
        end
    end

    local lSize = self.scrollView:getContentSize()
    local dx = lSize.width/2
    local dy = lSize.height/2
    local button = nil
    for var=self.actionKeyIndex, #strTable do
        --lab = CCLabelTTF:create(strTable[var], UILib.FONT_NAME_NORMAL, UILib.FONT.SMALL)
        --lab:setColor(UILib.UI_COLOR3B.BLACK)
        button = ccui.Button:create("titleBut_bg.png","titleBut_bg.png","titleBut_bg.png")--不加图片而直接setSize,button无效，maybe is bug
        button:setTitleText(strTable[var])
        button:setTitleFontSize(UILib.FONT.SMALL)
        button:setTitleColor(UILib.UI_COLOR3B.BLACK)
        button:setTitleFontName(UILib.FONT_NAME_NORMAL)
        button:setPosition(cc.p(dx, dy))
        button:setTag(100+var)
        self.scrollView:getInnerContainer():addChild(button)

        button:addTouchEventListener(doTouchAction)

        dx = dx + lSize.width
    end
    if button ~= nil then
        local iWidth = button:getPositionX()+lSize.width/2
        self.scrollView:setInnerContainerSize(cc.size(iWidth,self.scrollView:getInnerContainerSize().height))
    end

    self.popPanel = cc.Node:create()
    self.popPanel:setContentSize(visibleSize)
    self:addChild(self.popPanel)

    --[[版本号--]]
    local versionName = lk.LakooManager:getInstance():getAppVersionName()
    --cclog("versionName:%s",versionName)
    if versionName ~= nil and versionName ~= "" then
    	versionName = "V"..versionName
        local vLabel = cc.Label:create()
        vLabel:setString(versionName)
        vLabel:setColor(UILib.UI_COLOR3B.WHITE)
        vLabel:setSystemFontSize(UILib.FONT.MINI)
        local pos = UILib.offset(0,0,vLabel:getContentSize().width,vLabel:getContentSize().height,UILib.UI_ANCHOR.BOTTOM_RIGHT)
        vLabel:setPosition(pos)
        self:addChild(vLabel)
    end 
    
    local isGoogleAvailable = lk.LakooManager:getInstance():isGooglePlayServicesAvailable()
    local isGoogleConnected = lk.LakooManager:getInstance():isGoogleApiConnectSuccess()
    local isReGoogleConnect = GamePool.isReGoogleConnect()--[[只有第一次需要玩家授权Google Connect才需要显示这个界面--]]
    if isGoogleAvailable and isGoogleConnected == false and isReGoogleConnect == false then
        self.googleConnectLayer = GoogleConnectLayer:create()
        self.googleConnectLayer:setDelegate(self)
        self:addChild(self.googleConnectLayer)
    end

    if osType == Enum.OS_TYPE.IOS and GamePool.isShowedGameCenter() == false then
        self.warningLayer = WarningLayer:create()
        self.warningLayer:setTag(WarningLayer.layerTag.APPLE_GAME_CENTER)
        self.warningLayer:setDelegate(self)
        local msg = Text.get(IDX.APPLE_GAME_CENTER_TIPS)
        local okText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_YES)
        local cancelText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_NO)
        self.warningLayer:setString(msg, okText, cancelText)
        self.popPanel:addChild(self.warningLayer)
        
        GamePool.setShowedGameCenter(true)
    end

    local connectGameCenterFlag = false
    local function update()
        if osType == Enum.OS_TYPE.ANDROID then
        	self:updateAndroid()
        elseif osType == Enum.OS_TYPE.IOS then
            if self.isEnterTransitionFinish then--[[fix:未加载完TitleScene,在init中show loadingLayer在真机会崩溃--]]
                if lk.LakooManager:getInstance():isSaveGameCenterOn() 
                and lk.LakooManager:getInstance():isUserAuthenticated() == false 
                and connectGameCenterFlag == false then
                    connectGameCenterFlag = true
                    lk.LakooManager:getInstance():registerAchievementController()
                    if self.loadingLayer == nil then
                        self.loadingLayer = LayerManager.showLayerWithType(Enum.LAYER_TYPE.LOADING)
                    end
                end
            end
            self:updateIOSAuth()
            self:updateIOSSnapshot()
        end
        
        self:updateScrollViewAnim()
    end

    local function onNodeEvent(event)
        if "enterTransitionFinish" == event then
            --show Chartboost Interstitial Ads
            if lk.LakooManager:getInstance():getOSType() == Enum.OS_TYPE.ANDROID then
                lk.LakooManager:getInstance():showChartboostInterstitial()
            else
                lk.LakooManager:getInstance():showGADInterstitial()
            end
            self.isEnterTransitionFinish = true
            self.scheduleID = cc.Director:getInstance():getScheduler():scheduleScriptFunc(update, 0, false)
        end
        if "exit" == event then
            if self.scheduleID ~= nil then
                cclog("TitleScene un init scheduleID = "..self.scheduleID)
                cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.scheduleID)
            end
            self:clean()
        end
    end
    self:registerScriptHandler(onNodeEvent)
end

function TitleScene:updateIOSSnapshot()
    local loadFromSnapshotStatus = lk.LakooManager:getInstance():getLoadFromSnapshotStatus()
    if loadFromSnapshotStatus == 2 then--success status
        self:doReStorageFromCloud()
    end

    if (loadFromSnapshotStatus == 2 or loadFromSnapshotStatus == 3) and self.loadingLayer ~= nil then--success or fail status
        LayerManager.closeLayerWithType(Enum.LAYER_TYPE.LOADING)
        self.loadingLayer = nil
    end	
end

function TitleScene:updateIOSAuth()
    local gcAuthStatus = lk.LakooManager:getInstance():getAuthStatus()
    if gcAuthStatus == 0 then
    	return
    end
    
    if gcAuthStatus == 1 then--[[验证中--]]      
        return
    end
    
    if gcAuthStatus == 2 then--[[Game Center登录成功--]]
        lk.LakooManager:getInstance():resetAuthStatus()
        if self.isHasChunk and self.loadingLayer ~= nil then
            LayerManager.closeLayerWithType(Enum.LAYER_TYPE.LOADING)
            self.loadingLayer = nil
            return
        end
        
        --[[没有存档时检查是否有Game Center存档，有的话询问用户是否要从Game Center恢复--]]
        lk.LakooManager:getInstance():loadFromSnapshotInit() 
--        if self.loadingLayer == nil then
--            self.loadingLayer = LayerManager.showLayerWithType(Enum.LAYER_TYPE.LOADING)
--        end
        return       
    end
    
    if gcAuthStatus == 3 then
        lk.LakooManager:getInstance():resetAuthStatus()
        if self.loadingLayer ~= nil then
            LayerManager.closeLayerWithType(Enum.LAYER_TYPE.LOADING)
            self.loadingLayer = nil
        end
        return
    end
end

function TitleScene:updateAndroid()
    if self.isGoogleConnectNotice then
        if self.googleConnectLayer ~= nil then
            self.googleConnectLayer:removeFromParent(true)
            self.googleConnectLayer = nil
        end
        lk.LakooManager:getInstance():googleApiClientConnect()
        self.isGoogleConnectNotice = false
    end
    local isGoogleCon = lk.LakooManager:getInstance():isGoogleApiConnectSuccessFromTheMoment()
    if isGoogleCon then
        if self.isHasChunk then
            self.actionKeyIndex = ACTION_KEY.CONTINUE
            self:goToWorld()
        else
            --self.actionKeyIndex = ACTION_KEY.START 
            --[[没有存档时检查是否有Google Play存档，有的话询问用户是否要从Google Play恢复--]]
            lk.LakooManager:getInstance():loadFromSnapshotInit()
            if self.loadingLayer == nil then
                self.loadingLayer = LayerManager.showLayerWithType(Enum.LAYER_TYPE.LOADING)
            end
        end         
    end

    local loadFromSnapshotStatus = lk.LakooManager:getInstance():getLoadFromSnapshotStatus()
    if loadFromSnapshotStatus == 2 then--success status
        self:doReStorageFromCloud()
    end

    if (loadFromSnapshotStatus == 2 or loadFromSnapshotStatus == 3) and self.loadingLayer ~= nil then--success or fail status
        LayerManager.closeLayerWithType(Enum.LAYER_TYPE.LOADING)
        self.loadingLayer = nil
    end
end

---GoogleConnectLayer被点击后回调
function TitleScene:onConnectNotice()
    self.isGoogleConnectNotice = true
    GamePool.setReGoogleConnect(true)
end

function TitleScene:doReStorageFromCloud()
    self.warningLayer = WarningLayer:create()
    self.warningLayer:setTag(WarningLayer.layerTag.GAME_RE_STORAGE_FROM_CLOUD)
    self.warningLayer:setDelegate(self)
    local msg = Text.get(IDX.RE_STORAGE)
    local osType = lk.LakooManager:getInstance():getOSType()
    if osType == Enum.OS_TYPE.IOS then
    	msg = Text.get(IDX.RE_STORAGE_FROM_GAME_CENTER)
    end
    local okText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_YES)
    local cancelText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_NO)
    self.warningLayer:setString(msg, okText, cancelText)
    self.popPanel:addChild(self.warningLayer)--直接add to self会导致如self.preButton,self.nextButton同级ui会消失，原因不明
end

function TitleScene:doContinue()
    if lk.LakooManager:getInstance():isGoogleApiConnectSuccess() == false then
        lk.LakooManager:getInstance():googleApiClientConnect()
    	return
    end
    
    self.warningLayer = WarningLayer:create()
    self.warningLayer:setTag(WarningLayer.layerTag.GAME_CONTINUE)
    --layer:setOnActionFunc()
    self.warningLayer:setDelegate(self)
    local msg = lk.LakooManager:getInstance():getString(Enum.TEXT.T_WHETHER_READ)
    local okText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_YES)
    local cancelText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_NO)
    self.warningLayer:setString(msg, okText, cancelText)
    self.popPanel:addChild(self.warningLayer)--直接add to self会导致如self.preButton,self.nextButton同级ui会消失，原因不明
end

function TitleScene:doStart()
    if lk.LakooManager:getInstance():isGoogleApiConnectSuccess() == false then
        lk.LakooManager:getInstance():googleApiClientConnect()
        return
    end
    
    if self.isHasChunk == false then
        self:goToWorld()
        return
    end
    self.warningLayer = WarningLayer:create()
    self.warningLayer:setTag(WarningLayer.layerTag.GAME_START)
    --layer:setOnActionFunc()
    self.warningLayer:setDelegate(self)
    local msg = lk.LakooManager:getInstance():getString(Enum.TEXT.T_WHETHER_START)
    local okText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_YES)
    local cancelText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_NO)
    self.warningLayer:setString(msg, okText, cancelText)
    self.popPanel:addChild(self.warningLayer)
end

function TitleScene:onActionFunc()
    if self.warningLayer == nil then
    	return
    end
    local tag = self.warningLayer:getTag()
    if tag == WarningLayer.layerTag.GAME_START or tag == WarningLayer.layerTag.GAME_CONTINUE then
    	self:goToWorld()
    elseif tag == WarningLayer.layerTag.GAME_RE_STORAGE_FROM_CLOUD then
        lk.LakooManager:getInstance():loadFromSnapshot()
        self.isHasChunk = lk.LakooManager:getInstance():isHasLKData(Enum.FILE.GAME_CHUNK_NAME)
        if self.isHasChunk then
            self.actionKeyIndex = ACTION_KEY.CONTINUE
        else
            self.actionKeyIndex = ACTION_KEY.START
        end
        self:goToWorld()
    elseif tag == WarningLayer.layerTag.APPLE_GAME_CENTER then
        --show iOS GameCenter Login
        lk.LakooManager:getInstance():registerAchievementController()
        lk.LakooManager:getInstance():setIsSaveGameCenterOn(true)
        if self.loadingLayer == nil then
            self.loadingLayer = LayerManager.showLayerWithType(Enum.LAYER_TYPE.LOADING)
        end
    end
end

function TitleScene:goToWorld()
    if self.actionKeyIndex == ACTION_KEY.CONTINUE then
        GamePool.setIsContinueGame(true)
    elseif self.actionKeyIndex == ACTION_KEY.START then
        lk.LakooManager:getInstance():restart()
        GamePool.setIsContinueGame(false)
    end

    SceneStage.setNextType(Enum.SCENE_TYPE.WORLD,UILib.TRANSITION_TYPE.FADE)
end

function TitleScene:doLanguage()
    local languageLayer = LanguageLayer:create()
    self.popPanel:addChild(languageLayer)
end

function TitleScene:doHelp()
    local detailLayer = GameDetailLayer:create()
    self.popPanel:addChild(detailLayer)
    local title = lk.LakooManager:getInstance():getString(Enum.TEXT.T_GAME_HELP)
    detailLayer:setTitle(title)
    local detail = lk.LakooManager:getInstance():getStringFromFile(Enum.DATA.HELP)
    detailLayer:setDetail(detail)
end

function TitleScene:doAbout()
    local detailLayer = GameDetailLayer:create()
    self.popPanel:addChild(detailLayer)
    local title = lk.LakooManager:getInstance():getString(Enum.TEXT.T_GAME_ABOUT)
    detailLayer:setTitle(title)
    local detail = lk.LakooManager:getInstance():getStringFromFile(Enum.DATA.ABOUT)
    detailLayer:setDetail(detail)
end

---语言重新选择后更新动画及相关文字
function TitleScene:doLangaugeCallBack()
    self:replaceTitleAnim()

    --upate title button text
    local indexTable = {Enum.TEXT.T_GAME_CONTINUE,Enum.TEXT.T_GAME_START,Enum.TEXT.T_LANGUAGE,Enum.TEXT.T_GAME_HELP,Enum.TEXT.T_GAME_ABOUT}
    for key, var in pairs(ACTION_KEY) do
        local button = UILib.getNodeByTag(self,100+var)
        if button ~= nil then
            local str = lk.LakooManager:getInstance():getString(indexTable[var])
            button:setTitleText(str)
        end
    end
end

function TitleScene:replaceTitleAnim()
    if self.aniPanel == nil then
        return
    end
    self.aniPanel:removeAllChildren(true)

    local visibleSize = cc.Director:getInstance():getVisibleSize()
    local aniName = Text.get(IDX.TITLE_ANI_NAME)
    local sX = visibleSize.width/Config.STANDARD_WIDTH
    local sY = visibleSize.height/Config.STANDARD_HEIGHT
    lk.LakooManager:getInstance():addLKAnimationNode(self.aniPanel, aniName, -1, -1, -1, -1, 0, 0, sX, sY)
end

function TitleScene:updateTurnButton(_enablePre, _enableNext)
    if self.preButton ~= nil then
        if _enablePre then
            self.preButton:setTouchEnabled(true)
            self.preButton:setBright(true)
            self.preButton:setHighlighted(false)
        else
            self.preButton:setTouchEnabled(false)
            self.preButton:setBright(false)
            self.preButton:setHighlighted(true)
        end
    end

    if self.nextButton ~= nil then
        if _enableNext then
            self.nextButton:setTouchEnabled(true)
            self.nextButton:setBright(true)
            self.nextButton:setHighlighted(false)
        else
            self.nextButton:setTouchEnabled(false)
            self.nextButton:setBright(false)
            self.nextButton:setHighlighted(true)
        end
    end
end

function TitleScene:updateScrollViewAnim()
    if self.scrollView == nil then
        return
    end

    if self.isTurning == false then
        return
    end

    local drawX = self.scrollView:getInnerContainer():getPositionX()
    if self.isTurnRight then
        if drawX-1 <= self.aniTargetX then--float存在误差，所以要-1才行
            self.isTurning = false
            return
        end
        drawX = drawX - self.scrollView:getContentSize().width/5
        if drawX < self.aniTargetX then
            drawX = self.aniTargetX
        end
    else
        if drawX+1 >= self.aniTargetX then--float存在误差，所以要＋1才行
            self.isTurning = false
            return
        end
        drawX = drawX + self.scrollView:getContentSize().width/5
        if drawX > self.aniTargetX then
            drawX = self.aniTargetX
        end
    end
    self.scrollView:getInnerContainer():setPositionX(drawX)
end

function TitleScene:clean()
    self.preButton = nil
    self.nextButton = nil
    self.scrollView = nil
    self.aniTargetX = -1
    self.googleConnectLayer = nil
    self.warningLayer = nil
    self.loadingLayer = nil
end

return TitleScene
