require "cocos.init"
local SceneStage = require("manager.SceneStage")
local UILib = require("common.UILib")
local Enum = require("common.Enum")
local Util = require("common.Util")

local Text = require("conf.Text")
local IDX = require("conf.TextIndex")
local Config = require("conf.LKConfig")

local DirControllerUI = require("gameUI.component.DirControllerUI")
local BattleControllerUI = require("gameUI.component.BattleControllerUI")
local HeadUI = require("gameUI.component.HeadUI")

local MessageLayer = require("gameUI.layer.MessageLayer")
local WarningLayer = require("gameUI.layer.WarningLayer")

local GamePool = require("model.GamePool")

local LayerManager = require("manager.LayerManager")
local NetWorkManager = require("manager.NetworkManager")

local cc = cc
local class = class
local lk = lk

local TAG = {
    TITLE = 300000,
}

local WorldScene = class("WorldScene",
    function()
        return cc.Layer:create()
    end
)

-- joe
local ControllerHolder = class("ControllerHolder")
function ControllerHolder:ctor()
    self.controller  = nil
    self._holderNode = nil

    self._leftJoystick = nil
    self._rightJoystick = nil

    self._dpadLeft = nil
    self._dpadRight = nil
    self._dpadUp = nil
    self._dpadDown = nil

    self._buttonX = nil
    self._buttonY = nil
    self._buttonA = nil
    self._buttonB = nil

    self._buttonL1 = nil
    self._buttonR1 = nil
    self._buttonL2 = nil
    self._buttonR2 = nil

    self.deviceLabel = nil
    self.externalKeyLabel = nil
end
--

WorldScene.__index = WorldScene

WorldScene.headUI = nil
WorldScene.dirControllerUI = nil
WorldScene.battleControllerUI = nil
WorldScene.fastButton = nil
WorldScene.loadingLayer = nil
WorldScene.lastHitCount = 0
WorldScene.isShowTiping = false

function WorldScene:create()
    local worldScene = WorldScene.new()
    worldScene:init()

    local scene = cc.Scene:create()--创建场景
    scene:addChild(worldScene)
    return scene
end

function WorldScene:init()
    self:setName("WorldScene")
    lk.LakooManager:getInstance():initMapLayer(self, 3, 11, 11)
    lk.LakooManager:getInstance():loadPlayerData(Enum.DATA.PLAYER)

    local isHasChunk = lk.LakooManager:getInstance():isHasLKData(Enum.FILE.GAME_CHUNK_NAME)
    if isHasChunk and GamePool.getIsContinueGame() then
        --前缘再续
        lk.LakooManager:getInstance():saveSlotReplaceGame(Enum.FILE.GAME_CHUNK_NAME)
    else
        lk.LakooManager:getInstance():setEventPoint(5, 1)--"初涉江湖":重新开始游戏
    end
    
    local uid = GamePool.getUid()
    if uid ~= nil and uid ~= "" then
        lk.LakooManager:getInstance():setTalkingDataAccountId(uid)--[[重新开始游戏后等级会有变化更新talkingData--]]
    end
    
    --ios gameCenter:加载成就信息(需要加载完player信息后再加载成就)
    if lk.LakooManager:getInstance():getOSType() == Enum.OS_TYPE.IOS then
        lk.LakooManager:getInstance():loadGameCenterAchievement()
    end
    self.headUI = HeadUI.create()
    local hUISize = self.headUI:getContentSize()
    local hPoint = UILib.offset(0,0,hUISize.width,hUISize.height,UILib.UI_ANCHOR.TOP_LEFT)
    self.headUI:setPosition(hPoint)
    self:addChild(self.headUI)
    self.headUI:setVisible(false)
    
    --方向操控控件
    self.dirControllerUI = DirControllerUI.create()
    self:addChild(self.dirControllerUI)
    self.dirControllerUI:setVisible(false)
    
    --战斗操控控件
    self.battleControllerUI = BattleControllerUI.create()
    local bUISize = self.battleControllerUI:getContentSize()
    local bPoint = UILib.offset(0,0,bUISize.width,bUISize.height,UILib.UI_ANCHOR.BOTTOM_RIGHT)
    self.battleControllerUI:setPosition(bPoint)
    self:addChild(self.battleControllerUI)
    self.battleControllerUI:setVisible(false)
    
    --加速按钮
    self.fastButton = ccui.Button:create("p_81.png","p_81.png","p_81.png",ccui.TextureResType.localType)
    local point = UILib.offset(0,0,self.fastButton:getContentSize().width,self.fastButton:getContentSize().height,UILib.UI_ANCHOR.TOP_RIGHT)
    self.fastButton:setPosition(point)
    self:addChild(self.fastButton)
    self.fastButton:setVisible(false)
    
    local osType = Enum.OS_TYPE.IOS
    osType = lk.LakooManager:getInstance():getOSType()
    if osType == Enum.OS_TYPE.ANDROID then
        self._firstHolder   = ControllerHolder.new()
        self._firstHolder.controller = nil
        self:registerControllerListener()
    end
    
    local function touchButton(sender, eventType)
        if eventType == ccui.TouchEventType.began then
            lk.LakooManager:getInstance():setIsFastFPS(true)
        end
        --if eventType == ccui.TouchEventType.moved then
        --end
        if eventType == ccui.TouchEventType.ended 
            or eventType == ccui.TouchEventType.canceled then
            lk.LakooManager:getInstance():setIsFastFPS(false)
        end
    end
    self.fastButton:addTouchEventListener(touchButton)
    
    --[[请求每日奖励--]]
    lk.LakooManager:getInstance():setEnableReqDailyReward(true)
    
    local function update()
        local isEvent = lk.LakooManager:getInstance():isExecuteEvent()
        if isEvent then
            self.headUI:setVisible(false)
        	self.dirControllerUI:setVisible(false)
        	self.battleControllerUI:setVisible(false)
            self.fastButton:setVisible(true)
            self.fastButton:setLocalZOrder(1000)
        else
            self.headUI:setVisible(true)
            self.dirControllerUI:setVisible(true)
            self.battleControllerUI:setVisible(true)
            self.fastButton:setVisible(false)
            --Fix:事件结束，加速按钮未松开导致异常问题
            lk.LakooManager:getInstance():setIsFastFPS(false)
        end
        
        --[[请求每日奖励--]]
        if isEvent == false and lk.LakooManager:getInstance():getEnableReqDailyReward() then
            NetWorkManager.requestDailyReward()
            lk.LakooManager:getInstance():setEnableReqDailyReward(false)
        end
 
        local isHasLoadingNotice = GamePool.getIsHasLoadingNotice()
        if isHasLoadingNotice and self.loadingLayer == nil then
            self.loadingLayer = LayerManager.showLayerWithType(Enum.LAYER_TYPE.LOADING)
        elseif isHasLoadingNotice == false and self.loadingLayer ~= nil then
            LayerManager.closeLayerWithType(Enum.LAYER_TYPE.LOADING)
            self.loadingLayer = nil
        end
        local isHasOpenFailedMenuNotice = lk.LakooManager:getInstance():getIsHasOpenFailedMenuNotice()
        if isHasOpenFailedMenuNotice then
            LayerManager.closeLayerWithType(Enum.LAYER_TYPE.NOTICE)
            lk.LakooManager:getInstance():closeWebView()
            LayerManager.showLayerWithType(Enum.LAYER_TYPE.MULTIP_CHOICE)
            lk.LakooManager:getInstance():setIsHasOpenFailedMenuNotice(false)
        end
        local isHasBackTitleNotice = lk.LakooManager:getInstance():getIsHasBackTitleNotice()
        if isHasBackTitleNotice then
            SceneStage.setNextType(Enum.SCENE_TYPE.TITLE,UILib.TRANSITION_TYPE.FADE)
        	lk.LakooManager:getInstance():setIsHasBackTitleNotice(false)
        end
        
        --检查是否有需要弹出npc商店界面
        local shopItemIds = lk.LakooManager:getInstance():checkShopItemIds()
        if shopItemIds ~= "" then
            LayerManager.showLayerWithType(Enum.LAYER_TYPE.SINGLE_POP_NPC_SHOP)
        end
        
        self:processNoticeAction()
        
        self:showTip()
        self:showHitCount()
    end

    local function onNodeEvent(event)
        if "enterTransitionFinish" == event then
            self.scheduleID = cc.Director:getInstance():getScheduler():scheduleScriptFunc(update, 0, false)
        end
        if "exit" == event then
            if self.scheduleID ~= nil then
                cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.scheduleID)
            end
        end
    end
    self:registerScriptHandler(onNodeEvent)
end

function WorldScene:showTip()
    if lk.LakooManager:getInstance():getIsHasPopTip() then
    	return
    end
    
    if lk.LakooManager:getInstance():getIsHasPopView() then
        return
    end
    
    if self.isShowTiping then
    	return
    end

    local tip = lk.LakooManager:getInstance():getFirstTip()
    if tip ~= "" then
        self.isShowTiping = true
        
        local label = UILib.textAddStroke(tip,UILib.FONT_NAME_NORMAL,UILib.FONT.SMALL_PLUS,UILib.UI_COLOR4B.WHITE,UILib.UI_COLOR4B.BLACK,1)
        label:setPosition(UILib.offset(0,-50,label:getContentSize().width,label:getContentSize().height,UILib.UI_ANCHOR.CENTER))
        self:addChild(label)
        
        local targetPos = UILib.offset(0,50,label:getContentSize().width,label:getContentSize().height,UILib.UI_ANCHOR.CENTER)
        local moveAct = cc.MoveTo:create(0.6, targetPos)
        local fadeInAct = cc.FadeIn:create(0.6)
        local showAct = cc.Spawn:create(moveAct,fadeInAct)
        local callBackAct = cc.CallFunc:create(function()
            local fadeOutAct = cc.FadeOut:create(0.4)
            local callBackAct2 = cc.CallFunc:create(function()
                label:removeFromParent()
            end)
            local seqAct = cc.Sequence:create(showAct, callBackAct2)
            label:runAction(seqAct)
            self.isShowTiping = false
        end)
        local seqAct = cc.Sequence:create(showAct, callBackAct)
        label:runAction(seqAct)
        
        return
    end
    
    local popTip = lk.LakooManager:getInstance():getFirstPopTip()
    if popTip ~= "" then
        local messageLayer = MessageLayer:create(popTip)
        local scene = cc.Director:getInstance():getRunningScene()
        scene:addChild(messageLayer)
        return
    end
end

---连击效果显示
function WorldScene:showHitCount()
    local hitCount = lk.LakooManager:getInstance():getPlayerHitCount()
    if hitCount <= 1 then
        lastHitCount = hitCount;
    	return
    end
    
    if lastHitCount == hitCount then
    	return
    end
    lastHitCount = hitCount
    
    local hitCountLabel = cc.LabelAtlas:_create(tostring(hitCount), Enum.DIGITAL.CRIT_NUMBER, 211/10, 27, string.byte('0')) 
    
    local hitEffectNode = cc.Node:create()
    local continueImg = UILib.addImageTo(hitEffectNode, Enum.TEXT_IMG.CONTINUE, cc.p(0,0))
    hitEffectNode:setContentSize(cc.size(hitCountLabel:getContentSize().width+continueImg:getContentSize().width,continueImg:getContentSize().height))
    local pos = UILib.offsetWithSuperSize(-211/10*4,0,hitEffectNode:getContentSize().width,hitEffectNode:getContentSize().height,self:getContentSize().width,self:getContentSize().height,UILib.UI_ANCHOR.CENTER_RIGHT)
    hitEffectNode:setPosition(pos)
    self:addChild(hitEffectNode)
 
    pos = cc.p(0,0)
    hitCountLabel:setPosition(pos)
    pos = cc.p(hitCountLabel:getContentSize().width+continueImg:getContentSize().width*0.8,continueImg:getContentSize().height/2)
    continueImg:setPosition(pos)
    hitEffectNode:addChild(hitCountLabel)
    
    hitEffectNode:setScale(0.1)
    
    local scaleAct = cc.ScaleTo:create(0.2,1.2)
    local fadeInAct = cc.FadeIn:create(0.2)
    local showAct = cc.Spawn:create(scaleAct,fadeInAct)
    local callBackAct = cc.CallFunc:create(function()
        local fadeOutAct = cc.FadeOut:create(0.1)
        local callBackAct2 = cc.CallFunc:create(function()
            hitEffectNode:removeFromParent(true)
        end)
        local seqAct = cc.Sequence:create(showAct, callBackAct2)
        hitCountLabel:runAction(seqAct)
    end)
    local seqAct = cc.Sequence:create(showAct, callBackAct)
    hitEffectNode:runAction(seqAct)
end

function WorldScene:processNoticeAction()
    local noticeType = lk.LakooManager:getInstance():getFirstNotice()
    if noticeType == Enum.NOTICE_TYPE.NONE then
    	return
    end
    
    if noticeType == Enum.NOTICE_TYPE.OPEN_REVIEW then
        local channel = lk.LakooManager:getInstance():getChannel()
        if (channel == Enum.CHANNEL.IOS_APPSTORE or channel == Enum.CHANNEL.ANDROID_PHONE_GOOGLE)
        and GamePool.getReviewSwitch() and GamePool.getReviewTimes() <= 0 then
            local layer = WarningLayer:create()
            GamePool.getCurrentScene():addChild(layer)
            layer:setOnActionFunc(self.openReview)
            local msg = Text.get(IDX.REVIEW_CONTENT)
            local okText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_YES)
            local cancelText = lk.LakooManager:getInstance():getString(Enum.TEXT.T_NO)
            layer:setString(msg, okText, cancelText)
        end
    end
end

function WorldScene.openReview()
    local url = Config.REVIEW_URL_IOS
    if lk.LakooManager:getInstance():getChannel() == Enum.CHANNEL.ANDROID_PHONE_GOOGLE then
    	url = Config.REVIEW_URL_GOOGLE_PLAY
    end
    cc.Application:getInstance():openURL(url)
    
    GamePool.plugReviewTimes()
end

--joe
function WorldScene:registerControllerListener()
    local function onConnectController(controller, event)
        if controller == nil or controller == self._firstHolder.controller then
            return
        end

        local deviceInfo = string.format("%s id:%d", controller:getDeviceName(), controller:getDeviceId())
        cclog("deviceInfo ="..deviceInfo)

        if self._firstHolder.controller == nil then
            self._firstHolder.controller = controller
        end
    end

    local function onDisconnectedController(controller, event)
        if self._firstHolder.controller == controller then
            self._firstHolder.controller = nil
        end
    end

    local function showButtonState(controller, keyCode, isPressed)
        onConnectController(controller, nil)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else    
            return
        end

        if isPressed then
            if keyCode == cc.ControllerKey.BUTTON_A then
            elseif keyCode == cc.ControllerKey.BUTTON_B then
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_START then
                
            elseif keyCode == cc.ControllerKey.BUTTON_SELECT then
                lk.LakooManager:getInstance():setIsFastFPS(true)
            else
                local ketStatus = string.format("Key Down:%d",keyCode)
            end
        else
            if keyCode == cc.ControllerKey.BUTTON_A then
            elseif keyCode == cc.ControllerKey.BUTTON_B then
            elseif keyCode == cc.ControllerKey.BUTTON_X then
            elseif keyCode == cc.ControllerKey.BUTTON_Y then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_UP then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_DOWN then
            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_LEFT then

            elseif keyCode == cc.ControllerKey.BUTTON_DPAD_RIGHT then

            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_SHOULDER then
            elseif keyCode == cc.ControllerKey.BUTTON_LEFT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_RIGHT_THUMBSTICK then
            elseif keyCode == cc.ControllerKey.BUTTON_START then
            elseif keyCode == cc.ControllerKey.BUTTON_SELECT then
                lk.LakooManager:getInstance():setIsFastFPS(false)
            else
                local ketStatus = string.format("Key up:%d",keyCode)
            end
        end
    end 

    local function onKeyDown(controller, keyCode, event)
        showButtonState(controller, keyCode, true)
    end

    local function onKeyUp(controller, keyCode, event)
        showButtonState(controller, keyCode, false)
    end

    local function onAxisEvent(controller, keyCode, event)
        local holder = nil

        if controller == self._firstHolder.controller then
            holder = self._firstHolder
        else
            return
        end

        local ketStatus = controller:getKeyStatus(keyCode)
        if keyCode == cc.ControllerKey.JOYSTICK_LEFT_X then
            if(ketStatus.value == -1) then
            elseif(ketStatus.value == 1) then

            end
        elseif keyCode == cc.ControllerKey.JOYSTICK_LEFT_Y then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_X then
        elseif keyCode == cc.ControllerKey.JOYSTICK_RIGHT_Y then
        elseif keyCode == cc.ControllerKey.AXIS_LEFT_TRIGGER then
        elseif keyCode == cc.ControllerKey.AXIS_RIGHT_TRIGGER then
        else
        --do nothing
        end
    end

    local controllerListener = cc.EventListenerController:create()
    controllerListener:registerScriptHandler(onConnectController, cc.Handler.EVENT_CONTROLLER_CONNECTED)
    controllerListener:registerScriptHandler(onDisconnectedController, cc.Handler.EVENT_CONTROLLER_DISCONNECTED)

    controllerListener:registerScriptHandler(onKeyDown, cc.Handler.EVENT_CONTROLLER_KEYDOWN)
    controllerListener:registerScriptHandler(onKeyUp, cc.Handler.EVENT_CONTROLLER_KEYUP)
    controllerListener:registerScriptHandler(onAxisEvent, cc.Handler.EVENT_CONTROLLER_AXIS)

    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(controllerListener, self)
    cc.Controller:startDiscoveryController()
end
--

return WorldScene
