local Config = require("conf.LKConfig")
local NetWork = require("model.Network")
local Text = require("conf.Text")
local PayUtil = require("common.PayUtil")
local Enum = require("common.Enum")
local json = require("json")

local GamePool = require("model.GamePool")

local P = {}

local PARAM_KEY = {
    REQID = "reqId",
    UID = "uid",
    MSG = "msg",
    ORDERID = "orderId",
	THIRD_ORDERID = "thirdOrderId",
    ITEMID = "itemId",
    AMOUNT = "amount",--quantity
    PRODUCTID = "productId",
    RECEIPT = "receipt",
    SIGNATURE = "signature",
    SIGNED_DATA = "signedData",
    REWARDS = "rewards",--[[多个物品--]]
    REWARD = "reward",--[[单个物品--]]
    PAID_EXT_REWARD = "paidExtReward",--[[每日奖励附加奖励(单个物品)--]]
    ITEMS = "items",
    CODE = "code",--兑换码
    GAME = "game",--游戏代码
    LANG = "lang",
    NOTICE_TIME = "noticeTime",--公告时间截
    NOTICE_URL = "noticeUrl",--公告URL
    TAPJOY_SWITCH = "tapjoySwitch",--Tapjoy开关
    CHANNEL = "channel",--Device Type:1[IOS Appstore],2[WP store],3[Android Phone Google],4[Android TV]
    PAY_TYPE = "payType",
    REVIEW_SWITCH = "commentSwitch",--[[是否允许弹出评论引导--]]
}

local netWork = nil

function P.init()
    if netWork == nil then
        netWork = NetWork:create()
        netWork:retain()--TODO:要找个地方release
    end

    P.requestListConfigs()
    P.putMessageToMailChimp()
end

---only use in IOS
function P.putMessageToMailChimp()
    P.putMessageToMailChimpWithPayFlag(false)
end

---only use in IOS
function P.putMessageToMailChimpWithPayFlag(_flag)
    local isHasProfile = GamePool.isHasProfile()
    if isHasProfile == false then
    	return
    end

    local lastMailChimpDate = GamePool.getLastMailChimpDate()
    
    local nowDate = os.date("%m/%d/%Y", os.time())
    if _flag == false and lastMailChimpDate == nowDate then
    	return
    end
    GamePool.setLastMailChimpDate(nowDate)

    local email = GamePool.getProfileEmail()
    local fName = GamePool.getProfileName()
    local screenRes = lk.LakooManager:getInstance():getScreenSizeString()
    local osVersion = lk.LakooManager:getInstance():getOSVersion();
    local lang = lk.LakooManager:getInstance():getLanguageString()
    local timeZone = os.difftime(os.time(), os.time(os.date("!*t", os.time())))/3600
    local appVersion = lk.LakooManager:getInstance():getAppVersionShortString()
    
    local mailChimpTable = {
        ["email_address"] = email,
        ["status"] = "subscribed",
        ["merge_fields"] = {
            ["FNAME"] = fName,
            --["LNAME"] = lName,
            ["IOS"] = osVersion,
            ["IOSRES"] = screenRes,
            ["TZ_GAME"] = timeZone,
            ["LANG_GAME"] = lang,
            ["LADATE"] = nowDate,
            --["GENDER"] = "",
            ["IOS_APPVER"] = appVersion,
        },
    }
    
    local isFirstPay = GamePool.isFirstPay()
    if isFirstPay and _flag then
        mailChimpTable["merge_fields"]["FPAIDDATE"] = nowDate
        GamePool.setFirstPayFlag(false)
    end
    
    if _flag then
        mailChimpTable["merge_fields"]["LPAIDDATE"] = nowDate
    end
    
    netWork:putMessageToMailChimp(email, json.encode(mailChimpTable))
end

function P.requestDailyReward()
    local paramJson = PARAM_KEY.GAME..":"..Config.GAME_CODE

    local uid = GamePool.getUid()
    if uid ~= nil and uid ~= "" then
        paramJson = paramJson..","..PARAM_KEY.UID..":\""..uid.."\""
    end

    P.request(Config.PROTOCOL.DAILY_REWARD, paramJson, false)
end

function P.responseDailyReward(readDataTable)
    local reward = readDataTable[PARAM_KEY.REWARD]   
    local paidExtReward = readDataTable[PARAM_KEY.PAID_EXT_REWARD]
    
    lk.LakooManager:getInstance():generateDailyReward(reward, paidExtReward)
end

--[[获取Client Configs--]]
function P.requestListConfigs()
    local channel = lk.LakooManager:getInstance():getChannel()

    local paramJson = PARAM_KEY.CHANNEL..":"..channel

    P.request(Config.PROTOCOL.CONFIG, paramJson, false)
end

function P.responseListConfigs(readDataTable)
    local noticeTime = readDataTable[PARAM_KEY.NOTICE_TIME]
    local noticeUrl = readDataTable[PARAM_KEY.NOTICE_URL]
    local tapjoySwitch = readDataTable[PARAM_KEY.TAPJOY_SWITCH]
    local reviewSwitch = readDataTable[PARAM_KEY.REVIEW_SWITCH]

    GamePool.setNewNoticeTime(noticeTime)
    GamePool.setNoticeUrl(noticeUrl)
    GamePool.setTapjoySwitch(tapjoySwitch)
    GamePool.setReviewSwitch(reviewSwitch)
end

function P.requestExchangeRedeemCode(_code)
    if _code == nil or _code == "" then
    	return
    end
    local paramJson = PARAM_KEY.CODE..":\"".._code.."\","..PARAM_KEY.GAME..":"..Config.GAME_CODE

    local uid = GamePool.getUid()
    if uid ~= nil and uid ~= "" then
        paramJson = paramJson..","..PARAM_KEY.UID..":\""..uid.."\""
    end

    P.request(Config.PROTOCOL.EXCHANGE, paramJson, true)
end

function P.responseExchangeRedeemCode(readDataTable)
    local rewards = readDataTable[PARAM_KEY.REWARDS]
    lk.LakooManager:getInstance():generateItems(rewards, Config.GEN_REASON_ERC)
end

function P.requestAppleVerifyReceipt(_orderId, _receipt)
    if _orderId == nil or _orderId == "" or _receipt == nil or _receipt == "" then
    	return
    end
    local paramJson = PARAM_KEY.ORDERID..":\"".._orderId.."\","..PARAM_KEY.RECEIPT..":\"".._receipt.."\""
    P.request(Config.PROTOCOL.APPLE_VERIFY, paramJson, true)
end

function P.responseAppleVerifyReceipt(readDataTable)
    local items = readDataTable[PARAM_KEY.ITEMS]
    lk.LakooManager:getInstance():generateItems(items, Config.GEN_REASON_PAY)
    P.putMessageToMailChimpWithPayFlag(true)
end

function P.requestGoogleVerifySignedData(_orderId, _signature, _signedData)
    if _orderId == nil or _orderId == "" or _signature == nil or _signature == "" then
        return
    end
    if _signedData == nil or _signedData == "" then
    	return
    end
    local paramJson = PARAM_KEY.ORDERID..":\"".._orderId.."\","..PARAM_KEY.SIGNATURE..":\"".._signature.."\","..PARAM_KEY.SIGNED_DATA..":\"".._signedData.."\""
    P.request(Config.PROTOCOL.GOOGLE_VERIFY, paramJson, true)
end

function P.responseGoogleVerifySignedData(readDataTable)
    local items = readDataTable[PARAM_KEY.ITEMS]
    lk.LakooManager:getInstance():generateItems(items, Config.GEN_REASON_PAY)
end

function P.requestWindowsVerifyReceipt(_orderId, _receipt, _thirdOrderId)
    if _orderId == nil or _orderId == "" or _receipt == nil or _receipt == "" then
        return
    end
    local paramJson = PARAM_KEY.ORDERID..":\"".._orderId.."\","..PARAM_KEY.RECEIPT..":\"".._receipt.."\","..PARAM_KEY.THIRD_ORDERID..":\"".._thirdOrderId.."\""
    P.request(Config.PROTOCOL.WINDOW_VERIFY, paramJson, true)
end

function P.responseWindowsVerifyReceipt(readDataTable)
    local items = readDataTable[PARAM_KEY.ITEMS]
    lk.LakooManager:getInstance():generateItems(items, Config.GEN_REASON_PAY)
end

function P.requestRegister()
    P.request(Config.PROTOCOL.REGISTER, nil, true)
end

function P.responseRegister(readDataTable)
    local uid = readDataTable[PARAM_KEY.UID]
    GamePool.setUid(uid)

    P.requestCreateOrder()
end

function P.requestCreateOrder(productId)
    if productId == nil then
    	return
    end
    local paramJson = PARAM_KEY.PRODUCTID..":\""..productId.."\","..PARAM_KEY.GAME..":"..Config.GAME_CODE

    local uid = GamePool.getUid()
    if uid ~= nil and uid ~= "" then
        paramJson = paramJson..","..PARAM_KEY.UID..":\""..uid.."\""
    end
    
    local channel = lk.LakooManager:getInstance():getChannel()
    local payType = PayUtil.getPayType(channel)
    
    if payType ~= nil then
        paramJson = paramJson..","..PARAM_KEY.PAY_TYPE..":\""..payType.."\""
    end

    P.request(Config.PROTOCOL.CREATE_ORDER, paramJson, true)
end

function P.responseCreateOrder(readDataTable)
    local orderId = readDataTable[PARAM_KEY.ORDERID]
    --request apple pay
    lk.LakooManager:getInstance():buyIap(orderId,GamePool.getCurrentBuyIapItemId())
    
    local iapItemId = GamePool.getCurrentBuyIapItemId()
    local currency = PayUtil.getCurrencyByItemId(iapItemId)
    local channel = lk.LakooManager:getInstance():getChannel()
    local payType = PayUtil.getPayType(channel)
    local virtualCurrency = PayUtil.getVirtualCurrencyByItemId(iapItemId)
    lk.LakooManager:getInstance():onTalkingDataChargeRequest(tostring(orderId), tostring(iapItemId), currency, "USD", virtualCurrency, payType)
end

function P.request(_protocol,_paramJson,_isShowLoading)
    if netWork == nil then
        return
    end

    if _paramJson == nil then
        _paramJson = PARAM_KEY.LANG..":"..Text.getCurrentLang()
    else
        _paramJson = _paramJson..","..PARAM_KEY.LANG..":"..Text.getCurrentLang()
    end
    _paramJson = "{".._paramJson.."}"

    GamePool.setIsHasLoadingNotice(_isShowLoading)
    netWork:request(_protocol,_paramJson,P.callback)
end

function P.callback(readDataTable)
    local uid = readDataTable[PARAM_KEY.UID]
    if uid ~= nil and uid ~= "" then
        GamePool.setUid(uid)
    end

    local msg = readDataTable[PARAM_KEY.MSG]
    if msg ~= "" and msg ~= nil then
        GamePool.setIsHasLoadingNotice(false)
        lk.LakooManager:getInstance():addPopTip(msg)
    	return
    end

    local reqId = readDataTable[PARAM_KEY.REQID]
    if reqId == 0 then
        P.responseRegister(readDataTable)
    elseif reqId == 1 then
    	P.responseCreateOrder(readDataTable)
    elseif reqId == 2 then
        P.responseAppleVerifyReceipt(readDataTable)
        --GamePool.setIsHasLoadingNotice(false)
        lk.LakooManager:getInstance():setEnableReqDailyReward(true)
    elseif reqId == 3 then
        P.responseExchangeRedeemCode(readDataTable)
        GamePool.setIsHasLoadingNotice(false)
    elseif reqId == 6 then
        P.responseListConfigs(readDataTable)
    elseif reqId == 7 then
        P.responseDailyReward(readDataTable)
        GamePool.setIsHasLoadingNotice(false)
    elseif reqId == 8 then
        P.responseGoogleVerifySignedData(readDataTable)
        --GamePool.setIsHasLoadingNotice(false)
        lk.LakooManager:getInstance():setEnableReqDailyReward(true)
	elseif reqId == 9 then
        P.responseWindowsVerifyReceipt(readDataTable)
        --GamePool.setIsHasLoadingNotice(false)
        lk.LakooManager:getInstance():setEnableReqDailyReward(true)
    end
    
    if reqId == 2 or reqId == 8 or reqId == 9 then
        local saveTips = lk.LakooManager:getInstance():getString(Enum.TEXT.T_TIPS_SAVE)
        lk.LakooManager:getInstance():addPopTip(saveTips)
    end
end

return P
