local SceneStage = require("manager.SceneStage")
local UILib = require("common.UILib")
local Enum = require("common.Enum")
local LKConfig = require("conf.LKConfig")
local GamePool = require("model.GamePool")

local WorldScene = require("gameUI.scene.WorldScene")
local LogoScene = require("gameUI.scene.LogoScene")
local TitleScene = require("gameUI.scene.TitleScene")

local PluginSystem = require("gameUI.layer.PluginSystem")

local error = error
local cclog = cclog
local cc = cc


local P = {}

local SCENE_SWITCH = {
    [Enum.SCENE_TYPE.WORLD] = function () return WorldScene.create() end,
    [Enum.SCENE_TYPE.LOGO] = function () return LogoScene.create() end,
    [Enum.SCENE_TYPE.TITLE] = function () return TitleScene.create() end,
}


function P.getSceneByType()
    local f = SCENE_SWITCH[SceneStage.getCurType()];
    if f == nil then
    	return nil
    end
    return f()
end


local TRANSITION_SWITCH = {
    [UILib.TRANSITION_TYPE.FADE] = function (scene) return cc.TransitionFade:create(LKConfig.TRANSITION_SCENE_DURATION,scene) end,
    [UILib.TRANSITION_TYPE.FADE_BL] = function (scene) return cc.TransitionFadeBL:create(LKConfig.TRANSITION_SCENE_DURATION,scene) end,
    [UILib.TRANSITION_TYPE.FLIP_X] = function (scene) return cc.TransitionZoomFlipX:create(LKConfig.TRANSITION_SCENE_DURATION,scene) end
}
local function getTransitionSceneByType(type, scene)
    return TRANSITION_SWITCH[type](scene)
end

local function replaceScene()
    SceneStage.setCurType(SceneStage.getNextType())
    local nScene = P.getSceneByType()
    if nScene == nil then
        cclog("SceneManger replaceScene next scene is nil!")
    	return
    end
    
    GamePool.setCurrentScene(nScene)
    --TODO:暂时处理replaceScene之后原LayerManager update会停止
    local layerManager = require("manager.LayerManager")
    layerManager.init()
    
    local tScene = getTransitionSceneByType(SceneStage.getTransitionType(), nScene)
    cc.Director:getInstance():replaceScene(tScene)
end

local function doSceneLogic()
    local isKeyCodeBack = lk.LakooManager:getInstance():isKeyCodeBackFromTheMoment()
    if isKeyCodeBack then
        local sceneType = SceneStage.getCurType()
        if sceneType == Enum.SCENE_TYPE.TITLE then
            --PluginSystem.doPopExitGame()
            os.exit(1)
        elseif sceneType ~= Enum.SCENE_TYPE.LOGO then
            PluginSystem.doPopBackMainMenu()	
        end
    end
end

local function update()
    doSceneLogic()
    if SceneStage.getNextType() ~= SceneStage.getCurType() and SceneStage.getNextType() ~= Enum.SCENE_TYPE.NO_CHANGE then
        replaceScene()
    end
end

function P.init()
    SceneStage.setCurType(Enum.SCENE_TYPE.LOGO)
--    SceneStage.setCurType(Enum.SCENE_TYPE.TITLE)
    SceneStage.setNextType(Enum.SCENE_TYPE.NO_CHANGE)
    local director = cc.Director:getInstance()
    local scene = P.getSceneByType()

    if scene == nil then
        error("can't get scene by getSceneByType")
    end

    GamePool.setCurrentScene(scene)
    
    if director:getRunningScene() then
        director:replaceScene(scene)
    else
        director:runWithScene(scene)
    end

    local scheduleID = director:getScheduler():scheduleScriptFunc(update, 0, false)
    cclog("success run with scene type "..SceneStage.getCurType() .. "!".." scheduleID:"..scheduleID)
    
--    local function onNodeEvent(event)
--        cclog("SceneManager Event:"..event)
--        if "exit" == event then
--            --cc.Director:getInstance():getScheduler():unscheduleScriptEntry(scheduleID)
--        end
--    end
--    scene:registerScriptHandler(onNodeEvent)
end

return P