local Enum = require("common.Enum")
local Config = require("conf.LKConfig")

local IOS = require("conf.IOS")
local WP = require("conf.WP")
local Android = require("conf.Android")

local SAVE_KEY = {
    UID = "uid",
    NOTICE_TIME = "noticeTime",
    NOTICE_URL = "noticeURL",
    TAPJOY_SWITCH = "tapjoySwitch",
    REVIEW_TIMES = "reviewTimes",
    RE_GOOGLE_CONNECT = "googleConnect",
    PROFILE_NAME = "profileName",
    PROFILE_E_MAIL = "profileEmail",
    LAST_MAILCHIMP_DATE = "LAST_MAILCHIMP_DATE",
    FIRST_PAY_KEY = "FIRST_PAY_KEY",
    SHOWED_GAME_CENTER = "showedGameCenter",
}

local P = {}

local localLang = Enum.LANG.NONE--玩家可能未自行设定语言，会根据系统语言作显示
local currentScene = nil
local isHasLoadingNotice = false
local uid = nil
local currentBuyIapItemId = 0
local isContinueGame = false

local noticeTime = ""
local newNoticeTime = ""
local noticeUrl = Config.NOTICE_URL
local tapjoySwitch = false
local reviewSwitch = false
local reviewTimes = 0
local reGoogleConnect = false
local profileName = ""
local profileEmail = ""
local lastMailChimpDate = ""
local firstPayFlag = true
local showedGameCenter = false
local webUrlType = Enum.WEB_URL_TYPE.NONE

function P.init()
    localLang = lk.LakooManager:getInstance():getCurrentLang()
    lk.LakooManager:getInstance():loadGameText(Enum.DATA.TEXT)
    uid = cc.UserDefault:getInstance():getStringForKey(SAVE_KEY.UID)
    lk.LakooManager:getInstance():setLakooUID(uid)
    noticeTime = cc.UserDefault:getInstance():getStringForKey(SAVE_KEY.NOTICE_TIME)
    local url = cc.UserDefault:getInstance():getStringForKey(SAVE_KEY.NOTICE_URL)
    if url ~= nil or url ~= "" then
    	noticeUrl = url
    end
    reviewTimes = cc.UserDefault:getInstance():getIntegerForKey(SAVE_KEY.REVIEW_TIMES)
    
    local channel = Enum.CHANNEL.IOS_APPSTORE
    local targetPlatform = CCApplication:getInstance():getTargetPlatform()
    if targetPlatform == kTargetIphone or targetPlatform == kTargetIpad then
        channel = IOS.CHANNEL
    elseif targetPlatform == kTargetWindows then
        channel = WP.CHANNEL
    elseif targetPlatform == kTargetAndroid then
        channel = Android.CHANNEL
    end 
    lk.LakooManager:getInstance():setChannel(channel)
    
    reGoogleConnect = cc.UserDefault:getInstance():getBoolForKey(SAVE_KEY.RE_GOOGLE_CONNECT, false)
    profileName = cc.UserDefault:getInstance():getStringForKey(SAVE_KEY.PROFILE_NAME)
    profileEmail = cc.UserDefault:getInstance():getStringForKey(SAVE_KEY.PROFILE_E_MAIL)
    lastMailChimpDate = cc.UserDefault:getInstance():getStringForKey(SAVE_KEY.LAST_MAILCHIMP_DATE, "")
    firstPayFlag = cc.UserDefault:getInstance():getBoolForKey(SAVE_KEY.FIRST_PAY_KEY, true)
    showedGameCenter = cc.UserDefault:getInstance():getBoolForKey(SAVE_KEY.SHOWED_GAME_CENTER, false)
end

function P.isHasNewNotice()
    if newNoticeTime == "" or newNoticeTime == "0" then
    	return false
    end
    
    return noticeTime ~= newNoticeTime
end

function P.setNewNoticeTime(_newNoticeTime)
    newNoticeTime = tostring(_newNoticeTime)
end

function P.setNoticeUrl(_noticeUrl)
    noticeUrl = _noticeUrl
    cc.UserDefault:getInstance():setStringForKey(SAVE_KEY.NOTICE_URL,noticeUrl)
end

function P.getNoticeUrl()
    return noticeUrl
end

function P.refreshNotice()
    noticeTime = newNoticeTime
    cc.UserDefault:getInstance():setStringForKey(SAVE_KEY.NOTICE_TIME,noticeTime)
end

function P.setTapjoySwitch(flag)
    tapjoySwitch = flag
    lk.LakooManager:getInstance():setEnableTapjoy(tapjoySwitch)
end

function P.getTapjoySwitch()
    tapjoySwitch = lk.LakooManager:getInstance():getEnableTapjoy()
    return tapjoySwitch
end

function P.setReviewSwitch(flag)
    reviewSwitch = flag
end

function P.getReviewSwitch()
    return reviewSwitch
end

function P.getLocalLang()
    return localLang
end

function P.getReviewTimes()
    return reviewTimes
end

function P.plugReviewTimes()
    reviewTimes = reviewTimes + 1
    cc.UserDefault:getInstance():setIntegerForKey(SAVE_KEY.REVIEW_TIMES,reviewTimes)
    cc.UserDefault:getInstance():flush()
end

function P.setReGoogleConnect(flag)
    reGoogleConnect = flag
    cc.UserDefault:getInstance():setBoolForKey(SAVE_KEY.RE_GOOGLE_CONNECT, flag)
    cc.UserDefault:getInstance():flush()
end

function P.isReGoogleConnect()
	return reGoogleConnect
end

function P.setProfileName(_profileName)
	profileName = _profileName
    cc.UserDefault:getInstance():setStringForKey(SAVE_KEY.PROFILE_NAME,profileName)
    cc.UserDefault:getInstance():flush()
end

function P.getProfileName()
	return profileName
end

function P.setProfileEmail(_profileEmail)
	profileEmail = _profileEmail
    cc.UserDefault:getInstance():setStringForKey(SAVE_KEY.PROFILE_E_MAIL,profileEmail)
    cc.UserDefault:getInstance():flush()
end

function P.getProfileEmail()
	return profileEmail
end

function P.isHasProfile()
    return profileName ~= nil and profileName ~= "" and profileEmail ~= nil and profileEmail ~= ""
end

function P.setLastMailChimpDate(_dateString)
    lastMailChimpDate = _dateString
    cc.UserDefault:getInstance():setStringForKey(SAVE_KEY.LAST_MAILCHIMP_DATE,lastMailChimpDate)
    cc.UserDefault:getInstance():flush()
end

function P.getLastMailChimpDate()
    return lastMailChimpDate
end

function P.setFirstPayFlag(_flag)
    firstPayFlag = _flag
    cc.UserDefault:getInstance():setBoolForKey(SAVE_KEY.FIRST_PAY_KEY, _flag)
    cc.UserDefault:getInstance():flush()
end

function P.isShowedGameCenter()
    return showedGameCenter
end

function P.setShowedGameCenter(_flag)
    showedGameCenter = _flag
    cc.UserDefault:getInstance():setBoolForKey(SAVE_KEY.SHOWED_GAME_CENTER, _flag)
    cc.UserDefault:getInstance():flush()
end

function P.isFirstPay()
    return firstPayFlag
end

function P.saveLocalLang(lang)
    localLang = lang
	lk.LakooManager:getInstance():saveGameLang(lang)
    lk.LakooManager:getInstance():loadGameText(Enum.DATA.TEXT)
end

function P.setCurrentScene(scene)
    currentScene = scene
end

function P.getCurrentScene()
    return currentScene
end

function P.setIsHasLoadingNotice(flag)
    isHasLoadingNotice = flag
end

function P.getIsHasLoadingNotice()
    return isHasLoadingNotice
end

function P.getUid()
    return uid
end

function P.setUid(_uid)
    if _uid == uid then
    	return
    end
    uid = _uid
    cc.UserDefault:getInstance():setStringForKey(SAVE_KEY.UID,uid)
    cc.UserDefault:getInstance():flush()
    
    lk.LakooManager:getInstance():setLakooUID(uid)
    lk.LakooManager:getInstance():setTalkingDataAccountId(uid)
end

function P.getCurrentBuyIapItemId()
    return currentBuyIapItemId
end

function P.setCurrentBuyIapItemId(_currentBuyIapItemId)
    currentBuyIapItemId = _currentBuyIapItemId
end

function P.getIsContinueGame()
    return isContinueGame
end

function P.setIsContinueGame(flag)
    isContinueGame = flag
end

function P.setWebUrlType(_urlType)
    webUrlType = _urlType
end

function P.getWebUrlType()
    return webUrlType
end

function P.getEquipPosByType(type)
    local itemType = type
    if itemType == Enum.ITEM_TYPE.WEAPON then
        return Enum.EP.WEAPON
    end
    if itemType == Enum.ITEM_TYPE.ARMOUR then
        return Enum.EP.ARMOR
    end
    if itemType == Enum.ITEM_TYPE.ADORN_DATA then
        return Enum.EP.ADORN
    end
    if itemType == Enum.ITEM_TYPE.BOTTLE_DATA then
        return Enum.EP.BOTTLE
    end
    if itemType == Enum.ITEM_TYPE.CATCH_DATA then
        return Enum.EP.CATCH
    end

    return -1
end

function P.getScaleX()
    local visibleSize = cc.Director:getInstance():getVisibleSize()
    return visibleSize.width/Config.STANDARD_WIDTH
end
function P.getScaleY()
    local visibleSize = cc.Director:getInstance():getVisibleSize()
    return visibleSize.height/Config.STANDARD_HEIGHT
end

function P.isAndroidTV()
    local targetPlatform = CCApplication:getInstance():getTargetPlatform()
    if targetPlatform ~= kTargetAndroid then
        return false
    end
    
    if Android.CHANNEL == Enum.CHANNEL.ANDROID_TV_GOOGLE then
    	return true
    end
    
    --TODO:other android TV channel
    
    return false
end

return P
